//************************************************************************
//* Este archivo contiene algunos ejemplos de la flexibilidad que nos    *
//* permiten las estrategias de recorrido para movernos sobre una imagen *
//* con un iterador.                                                     *
//************************************************************************  

#include "basic_fun.cpp"  

void main()
{
  unsigned filas = 6, columnas = 4;
  float escala = 1;
  int default_pixel = 0;

  // Primeramente creo una imagen 2D Array de enteros 
  bicotiImageImplementation2DArray< int > * ptr_ima;
  ptr_ima = new bicotiImageImplementation2DArray< int >(default_pixel , columnas , filas , escala , single);

  // Le seteo valores aleatorios y luego la muestro.
  SetRand( ptr_ima );
  Show( ptr_ima , " Imagen ");

  // Ahora creo un iterador para esta imagen 

  bicotiImageIterator2DArray< int > * ptr_iterator;
  ptr_iterator = (bicotiImageIterator2DArray< int > * ) ptr_ima->CreateIterator();
  // hay que hacer un casting porque la funcin CreateIterator devuelve un 
  // puntero a la clase base.

  //**************************************************************************
  // Recorremos la imagen de la forma mas usaual y vemos la salida.
  // por defecto step = 1 

  cout<<endl;
  cout<<"Recorrido por defecto. De izquierda a derecha bajando."<<endl;
  cout<<"step = 1 en x0"<<endl;
  cout<<endl;
  for ( ptr_iterator->First() ; !(ptr_iterator->IsDone()); (*ptr_iterator)++ )
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  cout<<endl;
  cout<<"De derecha a izquierda subiendo."<<endl;
  cout<<"step = 1 en x0"<<endl;
  cout<<endl;
  for ( ptr_iterator->Last() ; !(ptr_iterator->IsDone()); (*ptr_iterator)-- )
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  //**************************************************************************
  // Defino una nueva estrategia con step = 2 en x0 y se la asigno al iterador  
  // me voy a saltear una fila.

  bicotiImageBorderStrategy2DX0<int> x0_strat(2);
  ptr_iterator->UseBorderStrategyX0( x0_strat );

  cout<<endl;
  cout<<"De izquierda a derecha bajando, fila por medio."<<endl;
  cout<<"step = 2 en x0"<<endl;
  cout<<endl;
  for ( ptr_iterator->First() ; !(ptr_iterator->IsDone()); (*ptr_iterator)++ )
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  cout<<endl;
  cout<<"De derecha a izquierda subiendo, fila por medio."<<endl;
  cout<<"step = 2 en x0"<<endl;
  cout<<endl;
  for ( ptr_iterator->Last() ; !(ptr_iterator->IsDone()); (*ptr_iterator)-- )
    cout<<(**ptr_iterator)<<" ";
  cout<<endl<<endl;

  Pausa();
  Show( ptr_ima , " Imagen ");


  //**************************************************************************
  // Le cambio el paso a la estrategia , pongo step = 0
  // solo una fila

  x0_strat.UseStep(0);
  ptr_iterator->UseBorderStrategyX0( x0_strat );

  cout<<endl;
  cout<<"De izquierda a derecha bajando, solo una fila."<<endl;
  cout<<"step = 0 en x0"<<endl;
  cout<<endl;
  for ( ptr_iterator->First() ; !(ptr_iterator->IsDone()); (*ptr_iterator)++ )
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  cout<<endl;
  cout<<"De derecha a izquierda subiendo, solo una fila."<<endl;
  cout<<"step = 0 en x0"<<endl;
  cout<<endl;
  for ( ptr_iterator->Last() ; !(ptr_iterator->IsDone()); (*ptr_iterator)-- )
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;


  //**************************************************************************
  // Ahora pongo step = -1 y hago el recorrido de abajo hacia arriba

  x0_strat.UseStep(-1);
  ptr_iterator->UseBorderStrategyX0( x0_strat );

  cout<<endl;
  cout<<"De izquierda a derecha subiendo."<<endl;
  cout<<"step = -1 en x0"<<endl;
  cout<<endl;
  for ( ptr_iterator->Locate(0,filas-1) ; !(ptr_iterator->IsDone()); (*ptr_iterator)++ )
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  cout<<endl;
  cout<<"De derecha a izquierda bajando."<<endl;
  cout<<"step = -1 en x0"<<endl;
  cout<<endl;
  for ( ptr_iterator->Locate(columnas-1,0) ; !(ptr_iterator->IsDone()); (*ptr_iterator)-- )
    cout<<(**ptr_iterator)<<" ";
  cout<<endl<<endl;

  Pausa();
  Show( ptr_ima , " Imagen ");

  //**************************************************************************
  // Recorrida vertical.
  // por defecto step = 1 

  cout<<endl;
  cout<<"Recorrido vertical por defecto. Bajando, de izquierda a derecha."<<endl;
  cout<<"step = 1 en x1"<<endl;
  cout<<endl;
  for ( ptr_iterator->First() ; !(ptr_iterator->IsDone()); ptr_iterator->X1Increment())
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  cout<<endl;
  cout<<"Subiendo, de derecha a izquierda."<<endl;
  cout<<"step = 1 en x1"<<endl;
  cout<<endl;
  for ( ptr_iterator->Last() ; !(ptr_iterator->IsDone()); ptr_iterator->X1Decrement())
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  //**************************************************************************
  // Recorrida vertical, columna por medio
  // step = 2 en x1 

  bicotiImageBorderStrategy2DX1<int> x1_strat(2);
  ptr_iterator->UseBorderStrategyX1( x1_strat );

  cout<<endl;
  cout<<"Bajando de izquierda a derecha. Columna por medio"<<endl;
  cout<<"step = 2 en x1"<<endl;
  cout<<endl;
  for ( ptr_iterator->First() ; !(ptr_iterator->IsDone()); ptr_iterator->X1Increment())
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  cout<<endl;
  cout<<"Subiendo, de derecha a izquierda.Columna por medio"<<endl;
  cout<<"step = 2 en x1"<<endl;
  cout<<endl;
  for ( ptr_iterator->Last() ; !(ptr_iterator->IsDone()); ptr_iterator->X1Decrement())
    cout<<(**ptr_iterator)<<" ";
  cout<<endl<<endl;
  Pausa();
  Show( ptr_ima , " Imagen ");


  //**************************************************************************
  // Recorrida vertical, solo una columna
  // step = 0 en x1 

  x1_strat.UseStep(0);
  ptr_iterator->UseBorderStrategyX1( x1_strat );

  cout<<endl;
  cout<<"Bajando. Una sola columna"<<endl;
  cout<<"step = 0 en x1"<<endl;
  cout<<endl;
  for ( ptr_iterator->First() ; !(ptr_iterator->IsDone()); ptr_iterator->X1Increment())
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  cout<<endl;
  cout<<"Subiendo. Una sola columna"<<endl;
  cout<<"step = 0 en x1"<<endl;
  cout<<endl;
  for ( ptr_iterator->Last() ; !(ptr_iterator->IsDone()); ptr_iterator->X1Decrement())
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  //**************************************************************************
  // Ahora pongo step = -1 y hago el recorrido de izquierda a derecha.

  x1_strat.UseStep(-1);
  ptr_iterator->UseBorderStrategyX1( x1_strat );

  cout<<endl;
  cout<<"Bajando, de derecha a izquierda"<<endl;
  cout<<"step = -1 en x1"<<endl;
  cout<<endl;
  for ( ptr_iterator->Locate(columnas-1,0) ; !(ptr_iterator->IsDone()); ptr_iterator->X1Increment())
    cout<<(**ptr_iterator)<<" ";
  cout<<endl;

  cout<<"Subiendo, de izquierda a derecha"<<endl;
  cout<<"step = -1 en x1"<<endl;
  cout<<endl;
  for ( ptr_iterator->Locate(0,filas-1) ; !(ptr_iterator->IsDone()); ptr_iterator->X1Decrement())
    cout<<(**ptr_iterator)<<" ";
  cout<<endl<<endl;

}

