/*
   Este ejemplo muestra como crear algunos tipos de curvas
   usando la fachada de Dexel de la imagen.
*/

#define IMAGE_2D_ARRAY_RGB_CHAR 
#include "../mostrar_lib.hpp"

typedef bicotiImage2DArrayRGBChar ImaT;
typedef ImaT::Pixel PixT;
typedef ImaT::RealCoordinate CoordT;

void DrawPoint(  ImaT& image , int x_0, int x_1 )
{
  PixT rojo(255,0,0);

  image.GetImplementation()->SetPixel(x_0-2,x_1,rojo);
  image.GetImplementation()->SetPixel(x_0-1,x_1,rojo);
  image.GetImplementation()->SetPixel(x_0,x_1,rojo);
  image.GetImplementation()->SetPixel(x_0+1,x_1,rojo);
  image.GetImplementation()->SetPixel(x_0+2,x_1,rojo);

  image.GetImplementation()->SetPixel(x_0,x_1-2,rojo);
  image.GetImplementation()->SetPixel(x_0,x_1-1,rojo);
  image.GetImplementation()->SetPixel(x_0,x_1,rojo);
  image.GetImplementation()->SetPixel(x_0,x_1+1,rojo);
  image.GetImplementation()->SetPixel(x_0,x_1+2,rojo);
};
 

void DrawPoints(  ImaT& image , vector< CoordT >& vec )
{
  unsigned i;
  for( i=0 ; i < vec.size() ; i++ )
    DrawPoint( image, (int)vec[i].X0(), (int)vec[i].X1() );
};


void main()
{

  ImaT ima;
  PixT rojo(255,0,0);
  PixT verde(0,255,0);
  PixT azul(0,0,255);

  LeerImagenColor( ima.GetImplementation(), "../Pictures/grid.bmp" );

  // Vector de puntos por donde pasa la curva

  vector< CoordT > coord_vect(5);

  coord_vect[0].X0()=110;
  coord_vect[0].X1()=110;
  
  coord_vect[1].X0()=302;
  coord_vect[1].X1()=80;

  coord_vect[2].X0()=210;
  coord_vect[2].X1()=210;

  coord_vect[3].X0()=290;
  coord_vect[3].X1()=330;

  coord_vect[4].X0()=60;
  coord_vect[4].X1()=290;

  DrawPoints( ima, coord_vect );

  // Creo la fachada de Dexel
  ImaT::Dexel * ptr_dex;
  ptr_dex = ima.CreateDexel();

  // Creo una curva lineal Cerrada
  ptr_dex->BuildNewCurveClosedLinear( coord_vect );
  ptr_dex->GetCurveClosed(0)->DrawCurve(verde);

  // Creo una curva Simpson Cerrada
  ptr_dex->BuildNewCurveClosedSimpson( coord_vect );
  ptr_dex->GetCurveClosed(1)->DrawCurve(azul);

  // Creo una curva Spline Cerrada
  ptr_dex->BuildNewCurveClosedSpline( coord_vect );
  ptr_dex->GetCurveClosed(2)->DrawCurve(rojo);

  MostrarImagenColor( ima.GetImplementation());
  
  delete ptr_dex;

  // Borro la implementacin y vuelvo a leer la grilla 
  ima.DestroyImplementation();
  ima.BuildImplementation(0,0);
  LeerImagenColor( ima.GetImplementation(), "../Pictures/grid.bmp" );

  ptr_dex = ima.CreateDexel();

  // Creo una curva lineal Abierta
  ptr_dex->BuildNewCurveOpenLinear( coord_vect );
  ptr_dex->GetCurveOpen(0)->DrawCurve(verde);

  // Creo una curva Simpson Abierta
  ptr_dex->BuildNewCurveOpenSimpson( coord_vect );
  ptr_dex->GetCurveOpen(1)->DrawCurve(azul);

  // Creo una curva Spline Abierta
  ptr_dex->BuildNewCurveOpenSpline( coord_vect );
  ptr_dex->GetCurveOpen(2)->DrawCurve(rojo);

  MostrarImagenColor( ima.GetImplementation());
  
  delete ptr_dex;
};
