
// Hay que definir este macro para que sea visible la fachada correspondiente
// Ver el archivo bicoti_basic.hpp para saber todos los macros posible
#define IMAGE_2D_ARRAY_CHAR
#include "../../bicoti.hpp" 
 
//*******************************COMENTARIOS GENERALES*****************************
// La imagen abstracta esta compuesta de punteros a objetos que agrupan las 
// operaciones, implementaciones propiedades, etc. de la imagen. 
// La implementacion se crea automaticamente al construir la imagen y todos 
// los otros objetos se crean y destruyen por demanda del usuario usando alguna 
// funcion de la clase imagen del tipo "Create.." o "Destroy.." la imagen 
// controla los new's y delete's de todos sus objetos  
//**************************************EJEMPLOS************************************
typedef unsigned char PixelType;  // definimos el tipo de pixel 
 
void main() 
{ 
  const unsigned filas = 5; 
  const unsigned columnas = 6; 
 
  // Declaracion del objeto imagen abstracta ( nro_columnas, nro_filas ) 
  // La dimension, implementacion y el tipo de pixel quedan definidas en 
  // la declaracion de la Clase 
  // EJ: Image2DArrayChar -> imagen 2 dimensiones, pixel Char e implementacion Array 

  bicotiImage2DArrayChar imagen1( columnas, filas ); 
 
  // Seteo de la Implementacion de la imagen 
  // OJO: El orden es columna, fila y estas van de cero a tamao - 1 
  // imagen.GetImplementation() devuelve un puntero al objeto que tiene la 
  // implentacion de la imagen. 
  // De este objeto mostramos algunas de las operaciones bsicas 
  // Para setear (Set) o leer (Get) pixels de la imagen se usan las funciones 
  // SetPixel( columna, fila, pixel )  
  // y GetPixel( columna, fila ) retorna un pixel. 

  unsigned columna, fila; 
  for( fila = 0; fila < filas; fila++ ) 
    for( columna = 0; columna < columnas; columna++ ) 
      imagen1.GetImplementation()->SetPixel( columna, fila, rand()%10 ); 
   
  cout << "Datos en imagen" << endl; 
  for( fila = 0; fila < filas; fila++ ) 
    {for( columna = 0; columna < columnas; columna++ ) 
      cout << (int)imagen1.GetImplementation()->GetPixel( columna, fila ) << " "; 
    cout << endl;} 
   
  // Tambien es posible preguntarle las dimensiones a la implementacion 
  // con las funciones GetX0Size() y GetX1Size(). 
  cout << "Nmero de columnas de la imagen = " << imagen1.GetImplementation()->GetX0Size() << endl; 
  cout << "Nmero de filas de la imagen    = " << imagen1.GetImplementation()->GetX1Size() << endl; 
  cout << endl; 
 
  // Para copiar los datos de imagen en imagen2 uso CopyMeFrom. 

  bicotiImage2DArrayChar imagen2; 
  imagen2.GetImplementation()->CopyMeFrom( imagen1.GetImplementation() ); 
 
  // Las operaciones punto a punto de varias imgenes en otra, estn agrupadas 
  // en el OperatorMultiple. 
  // Estas incluyen Addition() ( suma ), Prod() ( producto ), 
  // Subtraction() ( resta ), Div( cero, inf ). 
  // En el ltimo caso hay que pasar cual es el cero del tipo pixel 
  // ( no es trivial en RGB por ejemplo ) y que valor se espera al dividir por este.
  // El resultado es devuelto en la primera de las imgenes 
  // A continuacin voy a sumar imagen1 e imagen2 cargando el resultado en la primera 
  imagen1.BuildOperatorMultiple(); // Creo objeto OperatorMultiple 
  imagen1.GetOperatorMultiple()->Append( imagen2.GetImplementation() ); 
  // Agrego una a una las implementaciones 
  // a sumar, en este caso solo la de image2 

  imagen1.GetOperatorMultiple()->Addition(); // sumo las imagenes 
  // si ahora hiciera imagen1.GetOperatorMultiple()->Product() multiplicara el 
  // resultado anterior y sub_imagen2 
  imagen1.DestroyOperatorMultiple();  // Borro el operador

  cout << "Despues de sumar" << endl; 
  cout << "Datos en imagen1" << endl; 
  for( fila = 0; fila < imagen1.GetImplementation()->GetX1Size(); fila++ ) 
    {for( columna = 0; columna < imagen1.GetImplementation()->GetX0Size(); columna++ ) 
      cout << (int)imagen1.GetImplementation()->GetPixel( columna, fila ) << " "; 
    cout << endl;} 
  cout << endl; 
  cout << "Datos en imagen2" << endl; 
  for( fila = 0; fila < imagen2.GetImplementation()->GetX1Size(); fila++ ) 
    {for( columna = 0; columna < imagen2.GetImplementation()->GetX0Size(); columna++ ) 
      cout << (int)imagen2.GetImplementation()->GetPixel( columna, fila ) << " "; 
    cout << endl;} 
};
               
