#include "panel.hpp"


Panel :: Panel( CurrentImageType :: EventLoopManager * ptr_a )
{
  CurrentImageType :: HomologousFactory factory;
  ptr_app = ptr_a;
  
  ptr_panel = new bicotiActorGraphic2DPanel( "panel" );
  ptr_panel->SetMainActor( true );
  
  ptr_load_interface = NULL;
  
  ptr_image = new CurrentImageType( 1, 1, "imagen" );
  ptr_image->GetActor()->SetVisible( false );
  ptr_image->GetActor()->SetPosition( 0, 30 );
  ptr_image->GetActor()->AddListenerGeometry( this );

  ptr_file = new bicotiActorGraphicMenuPopup( "Menu File" );
  ptr_file->AddItem( "&Load" );
  ptr_file->AddItem( "&Save" );
  ptr_file->AddSeparator();
  ptr_file->AddItem( "&Quit" );
  ptr_file->SetVisible( false );
  factory.CreateHomologous( ptr_file );
  
  ptr_algorithms = new bicotiActorGraphicMenuPopup( "Menu Algorithms" );
  factory.CreateHomologous( ptr_algorithms );

  ptr_menu = new bicotiActorGraphicMenuBar( "Menu Bar" );
  ptr_menu->AddItem( "&File", ptr_file );
  ptr_menu->AddItem( "&Algorithms", ptr_algorithms );
  ptr_menu->AddSeparator();
  ptr_menu->AddItem( "&Help" );
  
  ptr_panel->SetChild( ptr_menu );
  ptr_panel->SetChild( ptr_image->GetActor() );
  ptr_panel->SetFixedSize( 750, 30 );
  ptr_panel->SetBackgroundColor( bicotiRGBTools :: CreateRGBA( 0, 128, 0 ) );
  factory.RecursiveCreateHomologous( ptr_panel );
  
  ptr_panel->SetPosition( 200, 200 );

  ptr_file->AddListenerMenu( this, "&Load" );
  ptr_file->AddListenerMenu( this, "&Save" );
  ptr_file->AddListenerMenu( this, "&Quit" );
  
  ptr_menu->AddListenerMenu( this, "&Help" );

  image_started = true;
}


Panel :: ~Panel()
{
  delete ptr_algorithms;
  delete ptr_file;
  delete ptr_menu;
  delete ptr_image;
  delete ptr_panel;
  //if( ptr_load_interface != NULL ) delete ptr_load_interface;
}


void Panel :: ItemSelected( bicotiActor * ptr_actor, char * item )
{
  if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "Menu Bar"  ) )
    {if( !strcmp( "&Help", item ) )
      {cout << "Es el item HELP" << endl;}} // ITEM HELP
  else
    {if( !strcmp( "&Load", item ) ) // ITEM LOAD
      {if( ptr_load_interface == NULL )
	ptr_load_interface = new LoadInterface( ptr_image );
      ptr_load_interface->SetVisible();}
    
    if( !strcmp( "&Save", item ) ) // ITEM SAVE
      cout << "Es el item SAVE" << endl;
    
    if( !strcmp( "&Quit", item ) ) // ITEM QUIT
      ptr_app->Stop();

    if( !strcmp( "Neighbour Filter M&inimum", item ) ) // NF Min
      {ptr_image->BuildNeighbourFilter();
      ptr_image->GetNeighbourFilter()->BuildMinimumFilter( 3 );
      ptr_image->GetNeighbourFilter()->UseExtrapolationStrategyEqual();
      ptr_image->GetNeighbourFilter()->Filter();
      ptr_image->GetImplementation()->CopyMeFrom( ptr_image->GetNeighbourFilterResult() );
      ptr_image->DestroyNeighbourFilter();
      ptr_image->GetActor()->RefreshImage();}
    
    if( !strcmp( "Neighbour Filter M&aximum", item ) ) // NF Max
      {ptr_image->BuildNeighbourFilter();
      ptr_image->GetNeighbourFilter()->BuildMaximumFilter( 3 );
      ptr_image->GetNeighbourFilter()->UseExtrapolationStrategyEqual();
      ptr_image->GetNeighbourFilter()->Filter();
      ptr_image->GetImplementation()->CopyMeFrom( ptr_image->GetNeighbourFilterResult() );
      ptr_image->DestroyNeighbourFilter();
      ptr_image->GetActor()->RefreshImage();}}
}


void Panel :: ActorResized( bicotiActor * ptr_actor, const bicotiEventGeometry& evento )
{
  if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "imagen" ) )
    {ptr_panel->SetFixedSize( evento.GetX(), evento.GetY() + menu_height );}

  if( image_started )
    {image_started = false;
    ptr_algorithms->AddItem( "Neighbour Filter M&inimum" );
    ptr_algorithms->AddItem( "Neighbour Filter M&aximum" );
    ptr_algorithms->AddListenerMenu( this, "Neighbour Filter M&inimum" );
    ptr_algorithms->AddListenerMenu( this, "Neighbour Filter M&aximum" );}
}


void Panel :: ActorMoved( bicotiActor * ptr_actor, const bicotiEventGeometry& evento )
{}
