#include "main_window.hpp"


MainWindow :: MainWindow( CurrentImageType :: EventLoopManager * ptr_el )
{
  ptr_event_loop = ptr_el;

  CreateMe();
}


MainWindow :: ~MainWindow()
{
  ResetMe();
}


void MainWindow :: CreateMe()
{
  ptr_panel = new bicotiActorGraphic2DPanel( "main_panel" );
  ptr_panel->SetSize( main_panel_width, main_panel_height );
  ptr_panel->SetBackgroundColor( bicotiRGBTools :: CreateRGBA( background_red, background_green, background_blue ) );
  ptr_panel->SetMainActor( true );
  ptr_panel->AddListenerGeometry( this );

  ptr_imagen = new CurrentImageType( 1, 1, "imagen" );
  ptr_imagen->GetActor()->SetPosition( 0, main_menu_height );
  ptr_imagen->GetActor()->SetVisible( false );
  
  ptr_menu_manager = new MenuManager( this );
  ptr_panel->SetChild( ptr_menu_manager->GetMainMenu() ); 
  ptr_panel->SetChild( ptr_imagen->GetActor() );
 
  factory.RecursiveCreateHomologous( ptr_panel );
  ptr_panel->SetPosition( 200, 200 );

  agregando_puntos = false;
  sacando_puntos = false;
  moviendo_puntos = false;

  ptr_curve_manager = NULL;
  selected_cruz = NULL;
}


void MainWindow :: ResetMe()
{
  bicotiActorGraphic * ptr_local_actor;
  
  delete ptr_menu_manager;

  if( ptr_curve_manager != NULL )
    {delete ptr_curve_manager;
    ptr_curve_manager = NULL;}
  
  for( int i = ptr_imagen->GetActor()->CountProgenies() - 1; i >= 0; i-- )
    {ptr_local_actor = ptr_imagen->GetActor()->GetProgeny( i );
    if( !strcmp( ptr_local_actor->GetActorDescriptor().GetActorName(), "cruz" ) )
      delete ptr_local_actor;}
  
  delete ptr_imagen;
  delete ptr_panel;
}


void MainWindow :: Recreate()
{
  ResetMe();
  CreateMe();
}


void MainWindow :: ActorResized( bicotiActor * ptr_actor, const bicotiEventGeometry& event )
{
  if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "main_panel" ) )
    {ptr_menu_manager->GetMainMenu()->SetVisible( false ); 
    ptr_menu_manager->GetMainMenu()->SetVisible( true );}
}


void MainWindow :: ActorMoved( bicotiActor * ptr_actor, const bicotiEventGeometry& event )
{
}


void MainWindow :: MousePressed( bicotiActor * ptr_actor, const bicotiEventMouse& event )
{
  if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "imagen" ) && agregando_puntos )
    {Cruz * ptr_local_cruz = new Cruz( event.GetX(), event.GetY() );
    ptr_imagen->GetActor()->SetChild( ptr_local_cruz );
    cruz_factory.CreateHomologous( ptr_local_cruz );
    if( ptr_curve_manager != NULL )
      {ptr_local_cruz->AddListenerGeometry( ptr_curve_manager );
      ptr_curve_manager->RefreshCurve();
      ptr_curve_manager->RefreshPerfil();}}
  
  if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "cruz" ) )
    {if( sacando_puntos )
      {delete ptr_actor;
      if( ptr_curve_manager != NULL )
	{if( !ptr_curve_manager->IsCurrentPossible() )
	  DestroyCurveManager(); // OJO Quilombo con los listeners
	else
	  {ptr_curve_manager->RefreshCurve();
	  ptr_curve_manager->RefreshPerfil();}}}
    
    if( moviendo_puntos )
      {selected_cruz = ((Cruz*)ptr_actor);
      selected_cruz->AddListenerMouseMotion( this );
      ptr_imagen->GetActor()->AddListenerMouseMotion( this );
      ptr_imagen->GetActor()->AddListenerMouse( this );}}
}


void MainWindow :: MouseReleased( bicotiActor * ptr_actor, const bicotiEventMouse& event )
{
  if( moviendo_puntos )
    {if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "cruz" ) )
      selected_cruz->Move( selected_cruz->GetX() + event.GetX(), selected_cruz->GetY() + event.GetY() );
    
    if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "imagen" ) )
      selected_cruz->Move( event.GetX(), event.GetY() );

    selected_cruz->RemoveListenerMouseMotion( this );
    ptr_imagen->GetActor()->RemoveListenerMouseMotion( this );
    ptr_imagen->GetActor()->RemoveListenerMouse( this );
    selected_cruz = NULL;
    
    if( ptr_curve_manager != NULL )
      ptr_curve_manager->RefreshPerfil();}
}


void MainWindow :: MouseMoved( bicotiActor * ptr_actor, const bicotiEventMouseMotion& event )
{
  if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "cruz" ) )
    selected_cruz->Move( selected_cruz->GetX() + event.GetX(), selected_cruz->GetY() + event.GetY() );

  if( !strcmp( ptr_actor->GetActorDescriptor().GetActorName(), "imagen" ) )
    selected_cruz->Move( event.GetX(), event.GetY() );
}

void MainWindow :: Stop()
{
  ptr_event_loop->Stop();
}


void MainWindow :: LoadImage( const char * filename ) //DEBO HACER RESET ANTES DE LEER
{
  // LEO IMAGEN (COLOR) DE DISCO
  ptr_imagen->BuildMapper();
  ptr_imagen->GetMapper()->BuildDeviceFile( filename );
  ptr_imagen->GetMapper()->BuildMapperJPEGIJG();
  ptr_imagen->GetMapper()->Read();
  ptr_imagen->DestroyMapper();
  
  // ACTUALIZO TAMAO DE LA IMAGEN Y EL PANEL
  ptr_imagen->GetActor()->SetSize( ptr_imagen->GetImplementation()->GetX0Size(), ptr_imagen->GetImplementation()->GetX1Size() );
  ptr_imagen->GetActor()->RefreshImage();
  ptr_imagen->GetActor()->SetVisible( true );
  ptr_panel->SetFixedSize( ptr_imagen->GetImplementation()->GetX0Size(), ptr_imagen->GetImplementation()->GetX1Size() + main_menu_height );
}


void MainWindow :: DisableAddPoints()
{
  if( agregando_puntos )
    {agregando_puntos = false;
    ptr_imagen->GetActor()->RemoveListenerMouse( this );}
}


void MainWindow :: DisableRemovePoints()
{
  if( sacando_puntos )
    {sacando_puntos = false;
    for( int i = ptr_imagen->GetActor()->CountProgenies() - 1; i >= 0; i-- )
      if( !strcmp( ptr_imagen->GetActor()->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
	ptr_imagen->GetActor()->GetProgeny( i )->RemoveListenerMouse( this );}
}


void MainWindow :: DisableMovePoints()
{
  if( moviendo_puntos )
    {moviendo_puntos = false;
    for( int i = ptr_imagen->GetActor()->CountProgenies() - 1; i >= 0; i-- )
      if( !strcmp( ptr_imagen->GetActor()->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
	ptr_imagen->GetActor()->GetProgeny( i )->RemoveListenerMouse( this );}
}


void MainWindow :: AddPoints()
{
  DisableRemovePoints();
  DisableMovePoints();
  
  if( !agregando_puntos )
    {agregando_puntos = true;
    ptr_imagen->GetActor()->AddListenerMouse( this );}
}


void MainWindow :: RemovePoints()
{
  DisableAddPoints();
  DisableMovePoints();

  if( !sacando_puntos )
    {sacando_puntos = true;
    for( int i = ptr_imagen->GetActor()->CountProgenies() - 1; i >= 0; i-- )
      if( !strcmp( ptr_imagen->GetActor()->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
	ptr_imagen->GetActor()->GetProgeny( i )->AddListenerMouse( this );}
}


void MainWindow :: MovePoints()
{
  DisableAddPoints();
  DisableRemovePoints();
  
  if( !moviendo_puntos )
    {moviendo_puntos = true;
    for( int i = ptr_imagen->GetActor()->CountProgenies() - 1; i >= 0; i-- )
      if( !strcmp( ptr_imagen->GetActor()->GetProgeny( i )->GetActorDescriptor().GetActorName(), "cruz" ) )
	ptr_imagen->GetActor()->GetProgeny( i )->AddListenerMouse( this );}
}


void MainWindow :: DestroyCurveManager()
{
  if( ptr_curve_manager != NULL )
    {delete ptr_curve_manager;
    ptr_curve_manager = NULL;}
}


void MainWindow :: CreateCurveLinear()
{
  if( CurveManager :: IsLinearPossible( ptr_imagen ) )
    {if( ptr_curve_manager == NULL )
      ptr_curve_manager = new CurveManager( ptr_imagen );
    ptr_curve_manager->CreateLinear();}
}


void MainWindow :: CreateCurveSimpson()
{
  if( CurveManager :: IsSimpsonPossible( ptr_imagen ) )
    {if( ptr_curve_manager == NULL )
      ptr_curve_manager = new CurveManager( ptr_imagen );
    ptr_curve_manager->CreateSimpson();}
}


void MainWindow :: CreateCurveSpline()
{
  if( CurveManager :: IsSplinePossible( ptr_imagen ) )
    {if( ptr_curve_manager == NULL )
      ptr_curve_manager = new CurveManager( ptr_imagen );
    ptr_curve_manager->CreateSpline();}
}
