#include <actor_image_2D_rgb_char_qt.hpp>
#include <histogram_painter.hpp>
 
typedef bicotiActorImage2DRGBCharQT ImaT;
typedef bicotiHistogramPainter< unsigned char, ImaT :: Pixel > HisT;


void main( int argc, char ** argv )  
{ 
  ImaT imagen;
  imagen.BuildMapper();
  imagen.GetMapper()->BuildDeviceFile( "histo4.jpg" );
  imagen.GetMapper()->BuildMapperJPEGIJG();
  imagen.GetMapper()->Read();
  imagen.GetActor()->SetSize( imagen.GetImplementation()->GetX0Size(), imagen.GetImplementation()->GetX1Size() );
  imagen.DestroyMapper();
  
  ImaT :: HomologousFactory factory;
  ImaT :: EventLoopManager event_loop( argc, argv );
  imagen.GetActor()->SetMainActor( true );
  factory.CreateHomologous( imagen.GetActor() );
  
  imagen.BuildProperty();
  imagen.GetProperty()->BuildHistogramExtended();
  
  HisT hist_tool;
  HisT :: actor_histogram histograma( ImaT :: Pixel ( 0, 0, 0 ), 500, 450 );
  histograma.SetSize( histograma.GetX0Size(), histograma.GetX1Size() );
  hist_tool.SetActorImage( &histograma );
  hist_tool.SetPropertyHistogram( imagen.GetProperty()->GetHistogram() );

  bicotiVerticalStrategyLinear vert_strat_r, vert_strat_g, vert_strat_b;

  bicotiHorizontalStrategyLinear< unsigned char > horiz_strat_r, horiz_strat_g, horiz_strat_b;
  horiz_strat_r.SetRange( 255 );
  horiz_strat_g.SetRange( 255 );
  horiz_strat_b.SetRange( 255 );

  bicotiColorStrategyConstant< unsigned char, ImaT :: Pixel > color_strat_r, color_strat_g, color_strat_b;
  color_strat_r.SetPencil( ImaT :: Pixel ( 255, 0, 0 ) );
  color_strat_g.SetPencil( ImaT :: Pixel ( 0, 255, 0 ) );
  color_strat_b.SetPencil( ImaT :: Pixel ( 0, 0, 255 ) );

  hist_tool.AddLayoutInformation( &vert_strat_r, &horiz_strat_r, &color_strat_r, 1, 1, 499, 148 );
  hist_tool.AddLayoutInformation( &vert_strat_g, &horiz_strat_g, &color_strat_g, 1, 151, 499, 298 );
  hist_tool.AddLayoutInformation( &vert_strat_b, &horiz_strat_b, &color_strat_b, 1, 301, 499, 448 );

  hist_tool.PaintBar();
  hist_tool.PaintAxes( ImaT :: Pixel ( 255, 255, 255 ) );
  
  factory.CreateHomologous( &histograma );
  
  event_loop.Start();
  histograma.SetVisible( false );

  // -----------------
  /*
  vector< bicotiPixelOperatorUnary< unsigned char > * > operators_list;
  operators_list.push_back( new bicotiPixelOperatorUnaryUniform< ImaT :: IntegerCoordinate, unsigned char, ImaT :: Pixel >( ( ( bicotiPropertyHistogramExtended< ImaT :: IntegerCoordinate, unsigned char, ImaT :: Pixel > * )(imagen.GetProperty()->GetHistogram()))->GetDexel(0)) );
  operators_list.push_back( new bicotiPixelOperatorUnaryUniform< ImaT :: IntegerCoordinate, unsigned char, ImaT :: Pixel >( ( ( bicotiPropertyHistogramExtended< ImaT :: IntegerCoordinate, unsigned char, ImaT :: Pixel > * )(imagen.GetProperty()->GetHistogram()))->GetDexel(1)) );
  operators_list.push_back( new bicotiPixelOperatorUnaryUniform< ImaT :: IntegerCoordinate, unsigned char, ImaT :: Pixel >( ( ( bicotiPropertyHistogramExtended< ImaT :: IntegerCoordinate, unsigned char, ImaT :: Pixel > * )(imagen.GetProperty()->GetHistogram()))->GetDexel(2)) );
  //  operators_list.push_back( new bicotiPixelOperatorUnaryAbsoluteValue< unsigned char > );
  //operators_list.push_back( new bicotiPixelOperatorUnarySegmentation< unsigned char >( 128, 1, 255 ) );
  
  bicotiHistogramEqualizator< ImaT :: IntegerCoordinate, unsigned char, ImaT :: Pixel >  equalizator( (bicotiPropertyHistogramExtended< ImaT :: IntegerCoordinate, unsigned char, ImaT :: Pixel >* )(imagen.GetProperty()->GetHistogram()) , operators_list );
  
  equalizator();
  
  for( unsigned col_index = 0; col_index < operators_list.size(); col_index++ )
    delete operators_list[ col_index ];
  */
  // -----------------

  imagen.GetProperty()->EqualizateHistogramSegmentation( ImaT :: Pixel( 128, 128, 128 ), ImaT :: Pixel( 0, 0, 0 ), ImaT :: Pixel( 255, 255, 255 ) );


  imagen.GetActor()->RefreshImage();
  imagen.GetActor()->SetVisible( true );
  
  histograma.Resize( 500, 450 );
  hist_tool.PaintBar();
  hist_tool.PaintAxes( ImaT :: Pixel ( 255, 255, 255 ) );
  histograma.RefreshImage();
  histograma.SetVisible( true );
  
  event_loop.Start();
}
