#include <actor_image_2D_double_qt.hpp> 
 

typedef bicotiActorImage2DDoubleQT ImaT; 
typedef ImaT :: Pixel PixT; 
typedef bicotiPixelComplex< PixT > ComT; 
typedef bicotiImageImplementation2DArray< ComT > ImpComT;


void main( int argc, char ** argv ) 
{ 
  ComT zero(0, 0); 
 
  ImaT imagen;
  imagen.BuildMapper();
  imagen.GetMapper()->BuildDeviceFile( "fourier2.jpg" );
  imagen.GetMapper()->BuildMapperJPEGIJG();
  imagen.GetMapper()->Read();
  imagen.GetActor()->SetSize( imagen.GetImplementation()->GetX0Size(), imagen.GetImplementation()->GetX1Size() );

  ImaT :: HomologousFactory factory;
  ImaT :: EventLoopManager event_loop( argc, argv );
  imagen.GetActor()->SetMainActor( true );
  factory.CreateHomologous( imagen.GetActor() );
  event_loop.Start();
  
  ImpComT * ptr_trans = new ImpComT( zero, 1, 1 );
  
  bicotiTransformationFourier2D< PixT > fourier;
  fourier.SetImplementations( ptr_trans, imagen.GetImplementation() );

  bicotiImageExtrapolationStrategy< PixT > * ptr_dir_ext;
  ptr_dir_ext = new bicotiImageExtrapolationStrategy2DConstant< PixT >( 0 );
  bicotiImageExtrapolationStrategy< ComT > * ptr_inv_ext;
  ptr_inv_ext = new bicotiImageExtrapolationStrategy2DConstant< ComT >( zero );
  fourier.SetDirectExtrapolationStrategy( ptr_dir_ext );
  fourier.SetInverseExtrapolationStrategy( ptr_inv_ext );

  cout << "Transformando" << endl;
  fourier.Transform();
  cout << "Transformacion terminada" << endl;
  
  ImaT modulo;
  modulo.SetImplementation( (ImaT::Implementation *)(fourier.GetTransformedModule()) );
  
  cout << "Adaptando transformada a visualizacion" << endl;
  modulo.BuildOperatorUnary();
  modulo.GetOperatorUnary()->Addition( 1 );
  modulo.GetOperatorUnary()->Logarithm();
  modulo.BuildProperty();
  modulo.GetProperty()->BuildStatistical();
  PixT maximo = modulo.GetProperty()->GetStatistical()->MaximumValue();
  modulo.DestroyProperty();
  modulo.GetOperatorUnary()->Product( 255 / maximo );
  modulo.DestroyOperatorUnary();

  modulo.GetActor()->SetMainActor( true );
  modulo.GetActor()->SetSize( modulo.GetImplementation()->GetX0Size(), modulo.GetImplementation()->GetX1Size() );
  factory.CreateHomologous( modulo.GetActor() );
  event_loop.Start();
  
  cout << "Antitransformando" << endl;
  fourier.Antitransform();
  cout << "Antitransformacion terminada" << endl;
  imagen.GetActor()->SetMainActor( true );
  imagen.GetActor()->RefreshImage();
  imagen.GetActor()->SetVisible( true );
  imagen.GetActor()->SetSize( imagen.GetActor()->GetX0Size(), imagen.GetActor()->GetX1Size() );
  event_loop.Start();
    
  delete ptr_dir_ext;
  delete ptr_inv_ext;
  delete ptr_trans;
};
