#include <source/actor/actor_graphic.hpp>


//------------------IMPLEMENTATIONS--------------------
//----------------bicotiActorGraphic-------------------
bicotiActorGraphic :: bicotiActorGraphic( char * act_name ) : bicotiActor( act_name )
{  
  actor_type = _graphic;
  ptr_parent = NULL;
  is_visible = true;
  is_hidden = false;
  is_tracking_on = false;
  main_actor = false;

  background_color = 0xB4B4B4FF;
  foreground_color = 0x000000FF;
}


bicotiActorGraphic :: ~bicotiActorGraphic()
{
  while( progenies.size() != 0 )
    RemoveChild( progenies.back() );
  
  if( ptr_parent != NULL )
    ptr_parent->RemoveChild( this );
}


void bicotiActorGraphic :: DestroyHomologous()
{
  if( ptr_homologous != NULL )
    {for( int i = 0; i < progenies.size(); i++ )
      if( progenies[ i ]->ptr_homologous != NULL )
	((bicotiHomologousGraphic*)ptr_homologous)->RemoveChild( ((bicotiHomologousGraphic*)(progenies[ i ]->ptr_homologous)) );
    
    if( ptr_parent != NULL )
      if( ptr_parent->ptr_homologous != NULL )
	((bicotiHomologousGraphic*)(ptr_parent->ptr_homologous))->RemoveChild( ((bicotiHomologousGraphic*)ptr_homologous) );
    
    delete ptr_homologous;
    ptr_homologous = NULL;}
}


void bicotiActorGraphic :: Hide()
{
  if( is_visible )
    {for( int i = 0; i < progenies.size(); i++ )
      progenies[ i ]->Hide();
    is_visible = false;
    is_hidden = true;

    if( ptr_homologous != NULL )
      ((bicotiHomologousGraphic*)ptr_homologous)->SetVisible( false );}
}

void bicotiActorGraphic :: UnHide()
{
  if( is_hidden )
    {for( int i = 0; i < progenies.size(); i++ )
      progenies[ i ]->UnHide();
    is_visible = true;
    is_hidden = false;

    if( ptr_homologous != NULL )
      ((bicotiHomologousGraphic*)ptr_homologous)->SetVisible( true );}
}


void bicotiActorGraphic :: SetChild( bicotiActorGraphic * ptr_actor )
{
  if ( ptr_actor != NULL )
    {if ( ptr_actor->ptr_parent != NULL )
      {vector< bicotiActorGraphic * > :: iterator iter;
      iter = find( ptr_actor->ptr_parent->progenies.begin(), ptr_actor->ptr_parent->progenies.end(), ptr_actor );
      ptr_actor->ptr_parent->progenies.erase( iter );}
    
    ptr_actor->ptr_parent = this;
    
    progenies.push_back( ptr_actor );
    
    if( ( ptr_homologous != NULL ) && ( ptr_actor->ptr_homologous != NULL ) )
      ((bicotiHomologousGraphic*)ptr_homologous)->SetChild( ((bicotiHomologousGraphic*)ptr_actor->ptr_homologous) );}
  else
    throw bicotiException( "bicotiActorGraphic", "SetChild", 4 );
}


void bicotiActorGraphic :: RemoveChild( bicotiActorGraphic * ptr_actor )
{
  // Chequear que ptr_actor era hijo realmente.

  vector< bicotiActorGraphic * > :: iterator iter;
  iter = find( progenies.begin(), progenies.end(), ptr_actor );
  progenies.erase( iter );
  ptr_actor->ptr_parent = NULL;
  
  if ( ptr_actor->ptr_homologous != NULL )
    if ( ptr_homologous != NULL )
      ((bicotiHomologousGraphic*)ptr_homologous)->RemoveChild( ((bicotiHomologousGraphic*)ptr_actor->ptr_homologous) );
}


bicotiActorGraphic * bicotiActorGraphic :: GetParent()
{return ptr_parent;}


bicotiActorGraphic * bicotiActorGraphic :: GetProgeny( int index )
{return progenies[ index ];}


int bicotiActorGraphic :: CountProgenies()
{return progenies.size();}


void bicotiActorGraphic :: SetBackgroundColor( bicotiPixelRGBA color )
{
  background_color = color;
  
  if( ptr_homologous != NULL )
    ((bicotiHomologousGraphic*)ptr_homologous)->SetBackgroundColor( color );
}


bicotiPixelRGBA bicotiActorGraphic :: GetBackgroundColor()
{return background_color;}


void bicotiActorGraphic :: SetForegroundColor( bicotiPixelRGBA color )
{
  foreground_color = color;
  
  if( ptr_homologous != NULL )
    ((bicotiHomologousGraphic*)ptr_homologous)->SetForegroundColor( color );
}


bicotiPixelRGBA bicotiActorGraphic :: GetForegroundColor()
{return foreground_color;}


void bicotiActorGraphic :: SetVisible( bool visible )
{
  if( is_visible != visible )
    {is_visible = visible;
    int i;
    if( !is_visible )
      {for( i = 0; i < progenies.size(); i++ )
	progenies[ i ]->Hide();}
    else
      {for( i = 0; i < progenies.size(); i++ )
	progenies[ i ]->UnHide();}

    if( ptr_homologous != NULL )
      ((bicotiHomologousGraphic*)ptr_homologous)->SetVisible( is_visible );}
}


bool bicotiActorGraphic :: GetVisible()
{return is_visible;}

  
void bicotiActorGraphic :: SetMainActor( bool ma )
{
  main_actor = ma;
}


bool bicotiActorGraphic :: GetMainActor()
{return main_actor;}

  
void bicotiActorGraphic :: MousePress( int x, int y, char mask )
{
  if ( ptr_homologous == NULL )
    FireMousePressed( this, x, y, mask );
  else 
    ((bicotiHomologousGraphic*)ptr_homologous)->MousePress( x, y, mask );
}


void bicotiActorGraphic :: MouseRelease( int x, int y, char mask )
{
  if ( ptr_homologous == NULL )
    FireMouseReleased( this, x, y, mask );
  else 
    ((bicotiHomologousGraphic*)ptr_homologous)->MouseRelease( x, y, mask );
}


void bicotiActorGraphic :: ActivateMouseMotionTracking()
{
  if( ptr_homologous != NULL )
    ((bicotiHomologousGraphic *)ptr_homologous)->ActivateMouseMotionTracking();
  
  is_tracking_on = true;
}


void bicotiActorGraphic :: DeactivateMouseMotionTracking()
{
  if( ptr_homologous != NULL )
    ((bicotiHomologousGraphic *)ptr_homologous)->DeactivateMouseMotionTracking();

  is_tracking_on = false;
}


bool bicotiActorGraphic :: IsTrackingOn()
{
  return is_tracking_on;
}


void bicotiActorGraphic :: SetFont( bicotiFont font )
{
  current_font = font;

  if( ptr_homologous != NULL )
    ((bicotiHomologousGraphic *)ptr_homologous)->SetFont( current_font );  
}


bicotiFont bicotiActorGraphic :: GetFont()
{
  return current_font;
}


//----------bicotiHomologousGraphic-----------
bicotiHomologousGraphic :: ~bicotiHomologousGraphic()
{}
