#ifndef _bicoti_actor_graphic_2D_file_dialog
#define _bicoti_actor_graphic_2D_file_dialog


#include <source/actor/actor_graphic_2D.hpp>
#include <source/interaction/interaction_file_dialog.hpp>

/**
 * Proporciona un dialogo de archivo
 *
 * Este actor permite seleccionar un archivo para leer o guardar, navegando por la estructura de archivos o digitando su nombre. Como todo actor grfico 2D se puede fijar su tamao y posicin. Puede ser hijo de otro actor grfico. Es fuente de eventos de dialogo de archivo.
 *@see bicotiHomologousGraphic2DFileDialog
 */

//------------------bicotiActorGraphic2DFileDialog-----------------
class bicotiActorGraphic2DFileDialog : public bicotiActorGraphic2D, public bicotiInteractionSourceFileDialog
{
private:
  char * current_directory;
  char * current_filter;
  bool mode_existing_file;

protected:
  bool CheckCurrentDirectory( const char * );
  
public:

  /** 
   * Constructor
   *
   * Construye un nuevo bicotiActorGraphic2DScrollBar
   *@param act_name nombre del actor. 
   */
  bicotiActorGraphic2DFileDialog( char * act_name = NULL );

  /** Destructor */
  ~bicotiActorGraphic2DFileDialog();

  /**
   * Fija el directorio inicial
   *
   * Fija el directorio de partida para la navegacin
   *@param direct  nombre del directorio inicial
   */
  void SetDirectory( const char * direct );
  
  /**
   * Fija un filtro de archivos
   * 
   * Fija un filtro de archivos. Muestra los archivos que cuyo nombre corresponden con el patrn indicado. Se pude poner un breve descripcin del tipo de archivo seguido por el patrn entre parntesis, en este caso solo se toma la expresin parentizada como filtro (por ejemplo "Archivo Postcript (*.ps)" equivale a "*.ps" )
   *@param filter especificacin que deben cumplir los archivos mostrados por bicotiActorGraphic2DFileDialog
   */
  void SetFilter( const char * filter );

  /**
   * Fija el modo del dilogo de archivo
   *
   * Fija el modo del dialgo de archivo. SetExistingFile( true ) solo permite recuperar nombres de archivos existentes. SetExistingFile( false ) permite devolver un nombre de archivo aunque este no exista, este modo sirve basicamente para guardar un nuevo archivo.
   *@param mode  si es true el dialogo funciona en modo archivo existente
   */
  void SetExistingFile( bool mode );

  /** 
   * Recupera el directorio actual
   *
   * Recupera el directorio actual, es decir el ltimo directorio visitado durante la navegacin.
   *@returns nobre del directorio actual
   *@see #SetDirectory
   */ 
  const char * GetDirectory();

  /** 
   * Recupera el filtro de archivo actual.
   *
   * Recupera el filtro de archivo actual
   *@returns patrn de filtro 
   *@see #SetFilter
   */ 
const char * GetFilter();

  /**
   * Devuelve el modo del dilogo de archivo
   *
   * Devuelve true si el dilogo de archivo esta en modo de archivo existente o false en caso contrario
   *@returns modo del dilogo de archivo
   *@see #SetExistingFile
   */
  bool GetExistingFile();
};



//----------------bicotiHomologousGraphic2DFileDialog--------------

/**
 * Homlogo del bicotiActorGraphic2DFileDialog
 *
 *@see bicotiActorGraphic2DFileDialog
 */
class bicotiHomologousGraphic2DFileDialog : public bicotiHomologousGraphic2D
{
public:
  /** 
   * Fija el directorio Inicial
   *
   *@param direct nombre del archivo inicial
   *@see bicotiActorGraphic2DFileDialog::SetDirectory
   */
  virtual void SetDirectory( const char * direct)=0;

  /**
   * Fija un filtro de archivos
   *
   *@param filter
   *@see  bicotiActorGraphic2DFileDialog::SetFilter
   */
  virtual void SetFilter( const char * filter)=0;

  /** 
   * Fija el modo de dilgo de archivo
   *
   *@param mode modo de dilogo de archivo
   *@see bicotiActorGraphic2DFileDialog::SetExistingFile
   */
  virtual void SetExistingFile( bool mode )=0;
};

#endif
