#ifndef _bicoti_actor_graphic_2D_label
#define _bicoti_actor_graphic_2D_label


#include <source/actor/actor_graphic_2D.hpp>


//---------------bicotiActorGraphic2DLabel---------------

/**
 * Proporciona una etiqueta
 *
 * La etiquete permite mostrar un texto. Opcionalmente puede tener un borde.
 *@see bicotiHomologousGraphic2DLabel
 */
class bicotiActorGraphic2DLabel : public bicotiActorGraphic2D 
{
private:
  const char * ptr_label_text;
  bool has_frame;

public:

  /** 
   * Constructor
   *
   * Construye una nueva bicotiActorGraphic2DLabel
   *@param act_name nombre del actor
   */
  bicotiActorGraphic2DLabel( char * act_name = NULL );
  
  /** Destructor */  
  ~bicotiActorGraphic2DLabel();
  
  /**
   * Fija el texto de la etiqueta
   *
   *@param ptr_text texto que mostrar la etiqueta
   */
  void SetText( const char * ptr_text );

  /**
   * Crea un borde alrededor de la etiqueta
   *
   *@param h_frame true para crear un borde, false en caso contrario
   */
  void SetFrame( bool h_frame);
 
  /**
   * Indica si la etiquete tiene borde
   *
   *@returns true si tiene borde false en caso contrario
   *@see #SetFrame
   */
  bool GetFrame();
};


//-----------bicotiHomologousGraphic2DLabel----------

/**
 * Homlogo para bicotiActorGraphic2DLabel
 */
class bicotiHomologousGraphic2DLabel : public bicotiHomologousGraphic2D
{
public:
  
  /** Destructor */
  virtual ~bicotiHomologousGraphic2DLabel();
  
  /** 
   * Fija el texto de la etiqueta
   *
   *@param ptr_text texto que mostrar la etiqueta
   *@see bicotiActorGraphic2DLabel::SetText   
*/ 
  virtual void SetText(const char * ptr_text) = 0;
 
  /**
   * Crea un borde alrededor de la etiqueta
   *
   *@param h_frame true para crear un borde, false en caso contrario
   *@see bicotiActorGraphic2DLabel::SetFrame
   */
  virtual void SetFrame( bool h_frame) = 0; 
};


#endif
