#ifndef _bicoti_actor_timer
#define _bicoti_actor_timer

#include <source/actor/actor_basic.hpp>
#include <source/interaction/interaction_timer.hpp>


//--------------------DECLARATIONS---------------------
//-----------------bicotiActorTimer------------------

/**
 * Proporciona un timer que produce un evento de timeout (tic) cada tantos milesimos de segundo.
 *@see bicotiHomologousTimer
 */
class bicotiActorTimer : public bicotiActor, public bicotiInteractionSourceTimer
{
protected:
  int period;
  
public:

  /** 
   * Constructor
   *
   * Construye un nuevo bicotiActorTimer
   *@param act_name nombre del actor
   */
  bicotiActorTimer( char * act_name = NULL );
  
  /** Destructor */
  virtual ~bicotiActorTimer();

  /** Destruye el homologo del actor */
 void DestroyHomologous();

  /** 
   * Fija el perodo 
   *
   * Fija  cuantos milisegundos debe haber entre dos tics consecutivos
   *@param per cantidad de milisegundos entre dos tics consecutivos
   */  
  void SetPeriod( int per);
  
  /** 
   * Devuelve el perodo
   *
   *@returns perodo en milisegundos entre dos tics consecutivos 
   */
  int GetPeriod();
};


//---------------bicotiHomologousTimer---------------

/** Homlogo de bicotiActorGraphic */

class bicotiHomologousTimer : public bicotiHomologous
{
public:
 
  /** Destructor */
  virtual ~bicotiHomologousTimer();

  /**
   * Fija el perodo
   *
   *@param cantidad de milisegundos entre dos tics consecutivos
   *@see bicotiActorTimer::SetPeriod
   */ 
  virtual void SetPeriod( int per)=0;
};



#endif
