#ifndef _bicoti_font
#define _bicoti_font


//------------Definiciones de propiedades---------

/**
 * Tipos de fuente soportados por bicoti
 *
 *<pre>
 *    Tipo                    Descripcin
 *    ===============         ==========================================
 *    _elvetica               Helvetica
 *    _times                  Times
 *    _courier                Courier
 *    _old_english            Old English
 *    _system                 System
 *</pre>
 */ 
typedef enum { _helvetica, _times, _courier, _old_english, _system } FontType;

/**
 * Tamaos de fuente soportados por bicoti  
 *
 *<pre>
 *   Nombre         Tamao
 *   =======        ======
 *   _light            5
 *   _normal          12
 *   _big             20
 *   _large           30
 *   _huge            40   
 *</pre>
 */  
typedef enum { _light = 5, _normal = 12, _big = 20, _large = 30, _huge = 40 } FontSize;

/** 
 * Estilos de fuente soportados por bicoti. 
 *
 *Pueden ser combinados haciendo un OR entre ellos.
 *<pre>
 *    Estilo                Descripcin
 *    =======               ==============================
 *    _bold                 Negrita (Bold)
 *    _italic               Itlica (Italic)
 *    _underline            Subrallado (Underline)
 *</pre>
 */
typedef enum { _bold = 1, _italic = 2, _underline = 4 } FontStyleMask;


/**
 * Fuentes de bicoti
 *
 * Se puede utilizar para indicar la fuente que deben tener los textos en bicoti
 */

class bicotiFont
{
private:
  FontType font_type;
  int font_size;
  char font_style;

public:
  
  /**
   * Constructor
   *
   * Construye un nuevo bicotiFont
   *@param ft tipo de fuente
   *@param sz tamao de la fuente
   *@param st estilo de la fuente
   */
  bicotiFont( FontType ft = _helvetica, int sz = _normal, char st = 0 );
  
  /** Destructor */
  ~bicotiFont();

  /** 
   * Constructor de copia 
   *
   *@param font_src fuente original
   */
  bicotiFont( const bicotiFont& font_src );
  
  /**
   * Operador =
   *
   * Asigna una copia de la bicotiFont a la derecha
   *@param font_src fuente original
   */
  void operator=( const bicotiFont& font_src);

  /**
   * Fija el tipo de fuente
   *
   *@param ft tipo de la fuente
   */
  void SetType( FontType ft);

  /**
   * Devuelve el tipo de la fuente
   *
   *@returns el tipo actual de la fuente
   *@see #SetType
   */  
  FontType GetType();

  /**
   * Fija el tamao de la fuente
   *
   *@param sz tamao de la fuente 
   */ 
  void SetSize( int sz );

  /** 
   * Devuelve el tamao de la fuente
   *
   *@returns el tamao actual de la fuente 
   *@see #SetSize
   */  
  int GetSize();
  
  /**
   * Fija el estilo de la fuente
   *
   *@param st estilo de la fuente. Puede ser una combinacin de los valores de FontStyleMask 
   */
  void SetStyle( char st );
  
  /**
   * Devuelve el estilo de la fuente
   *
   *@returns el estilo actual  de la fuente
   *@see #SetStyle
   */
  char GetStyle();
};


#endif
