//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################

#ifndef _coordinate_3D
#define _coordinate_3D


#include <source/coordinate/coordinate_basic.hpp>



//----------------------Declarations-----------------------
template<class CoordComponentType >
class bicotiCoordinate3D : public bicotiCoordinate< CoordComponentType >
{
private:
  CoordComponentType  x0;
  CoordComponentType  x1;
  CoordComponentType  x2;
  
public:
  bicotiCoordinate3D( CoordComponentType = 0 ,  CoordComponentType = 0,  CoordComponentType = 0 );
  bicotiCoordinate3D( const bicotiCoordinate3D< CoordComponentType >& );
  
  void operator=( const bicotiCoordinate3D< CoordComponentType >& );
  
  bool operator==( const bicotiCoordinate3D< CoordComponentType >& );
  
  bicotiCoordinate3D< CoordComponentType > operator+( double );
  bicotiCoordinate3D< CoordComponentType > operator-( double );
  bicotiCoordinate3D< CoordComponentType > operator*( double );
  bicotiCoordinate3D< CoordComponentType > operator/( double );

  void operator+=( double );
  void operator-=( double );
  void operator*=( double );
  void operator/=( double );
  
  bicotiCoordinate3D< CoordComponentType > operator+( const bicotiCoordinate3D< CoordComponentType >& );
  bicotiCoordinate3D< CoordComponentType > operator-( const bicotiCoordinate3D< CoordComponentType >& );
  bicotiCoordinate3D< CoordComponentType > operator*( const bicotiCoordinate3D< CoordComponentType >& ); 
  bicotiCoordinate3D< CoordComponentType > operator/( const bicotiCoordinate3D< CoordComponentType >& ); 
  
  void operator+=( const bicotiCoordinate3D< CoordComponentType >& );
  void operator-=( const bicotiCoordinate3D< CoordComponentType >& );
  void operator*=( const bicotiCoordinate3D< CoordComponentType >& );
  void operator/=( const bicotiCoordinate3D< CoordComponentType >& );
  
  REAL ScalarProduct( const bicotiCoordinate3D< CoordComponentType >& );
  bicotiCoordinate3D< CoordComponentType > VectorialProduct( const bicotiCoordinate3D< CoordComponentType >& );
  REAL Norm();
  
  bicotiCoordinate3D< CoordComponentType > operator^( int );
  
  CoordComponentType& operator() ( int );
  
  unsigned GetDimension();
  
  CoordComponentType& X0();
  CoordComponentType& X1();
  CoordComponentType& X2();
  
  operator bicotiCoordinate3D< INTEGER >();
  operator bicotiCoordinate3D< REAL >();
};



//---------------------Implementations---------------------
template< class  CoordComponentType >
bicotiCoordinate3D< CoordComponentType > :: bicotiCoordinate3D( CoordComponentType x_0,  CoordComponentType x_1,  CoordComponentType x_2 )
{
  x0 = x_0;
  x1 = x_1;
  x2 = x_2;
};


template< class  CoordComponentType >
bicotiCoordinate3D< CoordComponentType > :: bicotiCoordinate3D( const bicotiCoordinate3D< CoordComponentType >& coord_src )
{
  x0 = coord_src.x0;
  x1 = coord_src.x1;
  x2 = coord_src.x2;
};


template< class  CoordComponentType >
void bicotiCoordinate3D< CoordComponentType > :: operator=( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  x0 = coord.x0;
  x1 = coord.x1;
  x2 = coord.x2;
};
 

template< class CoordComponentType >
bool bicotiCoordinate3D< CoordComponentType > :: operator==( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  return( ( x0 == coord.x0 ) && ( x1 == coord.x1 ) && ( x2 == coord.x2 ) );
};


template< class CoordComponentType >
bicotiCoordinate3D< CoordComponentType > bicotiCoordinate3D< CoordComponentType > :: operator+( double scalar )
{
  bicotiCoordinate3D< CoordComponentType > temp;
  
  temp.x0=( CoordComponentType ) ( x0 + scalar );
  temp.x1=( CoordComponentType ) ( x1 + scalar );
  temp.x2=( CoordComponentType ) ( x2 + scalar );

  return( temp );
};


template< class CoordComponentType >
bicotiCoordinate3D< CoordComponentType > bicotiCoordinate3D< CoordComponentType > :: operator-( double scalar )
{
  bicotiCoordinate3D< CoordComponentType > temp;
  
  temp.x0=( CoordComponentType ) ( x0 - scalar );
  temp.x1=( CoordComponentType ) ( x1 - scalar );
  temp.x2=( CoordComponentType ) ( x2 - scalar );

  return( temp );
};


template< class CoordComponentType >
bicotiCoordinate3D< CoordComponentType > bicotiCoordinate3D< CoordComponentType > :: operator*( double scalar )
{
  bicotiCoordinate3D< CoordComponentType > temp;

  temp.x0=( CoordComponentType ) ( x0 * scalar );
  temp.x1=( CoordComponentType ) ( x1 * scalar );
  temp.x2=( CoordComponentType ) ( x2 * scalar );
  
  return( temp );
};


template< class CoordComponentType >
bicotiCoordinate3D< CoordComponentType > bicotiCoordinate3D< CoordComponentType > :: operator/( double scalar )
{
  bicotiCoordinate3D< CoordComponentType > temp;

  temp.x0=( CoordComponentType ) ( x0 / scalar );
  temp.x1=( CoordComponentType ) ( x1 / scalar );
  temp.x2=( CoordComponentType ) ( x2 / scalar );

  return( temp );
};


template< class CoordComponentType >
void bicotiCoordinate3D< CoordComponentType > :: operator+=( double scalar )
{
  x0=( CoordComponentType ) ( x0 + scalar );
  x1=( CoordComponentType ) ( x1 + scalar );
  x2=( CoordComponentType ) ( x2 + scalar );
};


template< class CoordComponentType >
void bicotiCoordinate3D< CoordComponentType > :: operator-=( double scalar )
{
  x0=( CoordComponentType ) ( x0 - scalar );
  x1=( CoordComponentType ) ( x1 - scalar );
  x2=( CoordComponentType ) ( x2 - scalar );
};


template< class CoordComponentType >
void bicotiCoordinate3D< CoordComponentType > :: operator*=( double scalar )
{
  x0=( CoordComponentType ) ( x0 * scalar );
  x1=( CoordComponentType ) ( x1 * scalar );
  x2=( CoordComponentType ) ( x2 * scalar );
};


template< class CoordComponentType >
void bicotiCoordinate3D< CoordComponentType > :: operator/=( double scalar )
{
  x0=( CoordComponentType ) ( x0 / scalar );
  x1=( CoordComponentType ) ( x1 / scalar );
  x2=( CoordComponentType ) ( x2 / scalar );
};


template< class  CoordComponentType >
bicotiCoordinate3D< CoordComponentType > bicotiCoordinate3D< CoordComponentType > :: operator+( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  bicotiCoordinate3D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = x0 + coord.x0;
  coord_temp.x1 = x1 + coord.x1;
  coord_temp.x2 = x2 + coord.x2;
  
  return( coord_temp );
};


template<class  CoordComponentType >
bicotiCoordinate3D< CoordComponentType > bicotiCoordinate3D< CoordComponentType > :: operator-( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  bicotiCoordinate3D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = x0 - coord.x0;
  coord_temp.x1 = x1 - coord.x1;
  coord_temp.x2 = x2 - coord.x2;
  
  return( coord_temp );
};


template<class  CoordComponentType >
bicotiCoordinate3D< CoordComponentType > bicotiCoordinate3D< CoordComponentType > :: operator*( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  bicotiCoordinate3D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = x0 * coord.x0;
  coord_temp.x1 = x1 * coord.x1;
  coord_temp.x2 = x2 * coord.x2;
  
  return( coord_temp );
};


template<class  CoordComponentType >
bicotiCoordinate3D< CoordComponentType > bicotiCoordinate3D< CoordComponentType > :: operator/( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  bicotiCoordinate3D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = x0 / coord.x0;
  coord_temp.x1 = x1 / coord.x1;
  coord_temp.x2 = x2 / coord.x2;
  
  return( coord_temp );
};


template< class CoordComponentType >
void bicotiCoordinate3D< CoordComponentType > :: operator+=( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  x0 = x0 + coord.x0;
  x1 = x1 + coord.x1;
  x2 = x2 + coord.x2;
};


template< class CoordComponentType >
void bicotiCoordinate3D< CoordComponentType > :: operator-=( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  x0 = x0 - coord.x0;
  x1 = x1 - coord.x1;
  x2 = x2 - coord.x2;
};


template< class CoordComponentType >
void bicotiCoordinate3D< CoordComponentType > :: operator*=( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  x0 = x0 * coord.x0;
  x1 = x1 * coord.x1;
  x2 = x2 * coord.x2;
};


template< class CoordComponentType >
void bicotiCoordinate3D< CoordComponentType > :: operator/=( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  x0 = x0 / coord.x0;
  x1 = x1 / coord.x1;
  x2 = x2 / coord.x2;
};


template< class  CoordComponentType >
REAL bicotiCoordinate3D< CoordComponentType > :: ScalarProduct( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  return( (REAL) ((x0 * coord.x0) + (x1 * coord.x1) + (x2 * coord.x2)) );
};


template< class  CoordComponentType >
bicotiCoordinate3D< CoordComponentType >  bicotiCoordinate3D< CoordComponentType > :: VectorialProduct( const bicotiCoordinate3D< CoordComponentType >& coord )
{
  bicotiCoordinate3D< REAL > coord_aux;
  
  coord_aux.x0 = ( x1 * coord.x2 ) - ( x2 * coord.x1 );
  coord_aux.x1 = ( x2 * coord.x0 ) - ( x0 * coord.x2 );
  coord_aux.x2 = ( x0 * coord.x1 ) - ( x1 * coord.x0 );
  
  return( coord_aux );
};


template<class  CoordComponentType >
bicotiCoordinate3D< CoordComponentType > bicotiCoordinate3D< CoordComponentType > :: operator^( int exponent )
{
  bicotiCoordinate3D< CoordComponentType > coord_temp;
  
  coord_temp.x0 = ( CoordComponentType ) pow( x0, exponent );
  coord_temp.x1 = ( CoordComponentType ) pow( x1, exponent );
  coord_temp.x2 = ( CoordComponentType ) pow( x2, exponent );
  
  return( coord_temp );
};


template< class CoordComponentType >
REAL bicotiCoordinate3D< CoordComponentType > :: Norm()
{
  return ( (REAL) sqrt( x0*x0 + x1*x1 + x2*x2 ) );
};


template<class  CoordComponentType >
inline CoordComponentType & bicotiCoordinate3D< CoordComponentType > :: operator() ( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( 2 < index) )
    throw bicotiException( "bicotiCoordinate3D", "operator()", 1 );
#endif
  
  switch( index )
    {case 0:
      {return ( x0 );
      break;}
    case 1:
      {return ( x1 );
      break;}
    case 2:
      {return ( x2 );
      break;}}
};


template< class CoordComponentType >
unsigned bicotiCoordinate3D< CoordComponentType > :: GetDimension()
{
  return 3;
};


template< class  CoordComponentType >
CoordComponentType & bicotiCoordinate3D< CoordComponentType > :: X0()
{
  return x0;
};


template< class  CoordComponentType >
CoordComponentType & bicotiCoordinate3D< CoordComponentType > :: X1()
{
  return x1;
};


template< class  CoordComponentType >
CoordComponentType & bicotiCoordinate3D< CoordComponentType > :: X2()
{
  return x2;
};


template< class CoordComponentType >
bicotiCoordinate3D< CoordComponentType > :: operator bicotiCoordinate3D< INTEGER >()
{
  bicotiCoordinate3D< INTEGER > coord_aux( ( INTEGER ) x0, ( INTEGER ) x1, ( INTEGER ) x2 );
  return ( coord_aux );
};


template< class CoordComponentType >
bicotiCoordinate3D< CoordComponentType > :: operator bicotiCoordinate3D< REAL >()
{
  bicotiCoordinate3D< REAL > coord_aux( ( REAL ) x0, ( REAL ) x1, ( REAL ) x2 );
  return ( coord_aux );
};                


#endif
