//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################

#ifndef _coordinate_basic
#define _coordinate_basic



#include <source/bicoti_basic.hpp>



//----------------------Declarations-----------------------
template< class CoordComponentType >
class bicotiCoordinate2D;

template< class CoordComponentType >
class bicotiCoordinate3D;

template< class CoordComponentType >
class bicotiCoordinate
{
private:
  CoordComponentType * ptr_components;
  unsigned dimension;
  
public:
  bicotiCoordinate( int = 0 );
  ~bicotiCoordinate();
  bicotiCoordinate( const bicotiCoordinate< CoordComponentType >& );
  
  void operator=( const bicotiCoordinate< CoordComponentType >& );
  
  bool operator==( const bicotiCoordinate< CoordComponentType >& );
  
  bicotiCoordinate< CoordComponentType > operator+( double );
  bicotiCoordinate< CoordComponentType > operator-( double );
  bicotiCoordinate< CoordComponentType > operator*( double );
  bicotiCoordinate< CoordComponentType > operator/( double );
  
  void operator+=( double );
  void operator-=( double );
  void operator*=( double );
  void operator/=( double );
  
  virtual bicotiCoordinate< CoordComponentType >  operator+( const bicotiCoordinate< CoordComponentType >& );
  virtual bicotiCoordinate< CoordComponentType >  operator-( const bicotiCoordinate< CoordComponentType >& );
  virtual bicotiCoordinate< CoordComponentType >  operator*( const bicotiCoordinate< CoordComponentType >& );
  virtual bicotiCoordinate< CoordComponentType >  operator/( const bicotiCoordinate< CoordComponentType >& );
  
  virtual void operator+=( const bicotiCoordinate< CoordComponentType >& );
  virtual void operator-=( const bicotiCoordinate< CoordComponentType >& );
  virtual void operator*=( const bicotiCoordinate< CoordComponentType >& );
  virtual void operator/=( const bicotiCoordinate< CoordComponentType >& );
  
  virtual REAL ScalarProduct( const bicotiCoordinate< CoordComponentType >& );
  virtual REAL Norm();

  bicotiCoordinate< CoordComponentType > operator^( int );
  
  virtual CoordComponentType& operator() ( int );
  virtual unsigned GetDimension();
  
  virtual operator bicotiCoordinate< INTEGER >();
  virtual operator bicotiCoordinate< REAL >();
  virtual operator bicotiCoordinate2D< INTEGER >();
  virtual operator bicotiCoordinate3D< INTEGER >();
  virtual operator bicotiCoordinate2D< REAL >();
  virtual operator bicotiCoordinate3D< REAL >();
};


REAL fabs( bicotiCoordinate< REAL >& );


//----------------------Implementation-----------------------
template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > :: bicotiCoordinate( int dim )
{
#ifdef TOTAL_CHECK
  if ( dim < 0 )
    throw bicotiException( "bicotiCoordinate", "Constructor", 2 );
#endif
  
  dimension = ( unsigned ) dim;
  
  if ( dimension > 0 ) 
    ptr_components = new CoordComponentType [ dimension ];
  
  for( unsigned index = 0; index < dimension; index ++ )
    ptr_components[ index ] = 0;
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > :: ~bicotiCoordinate()
{
  if ( dimension > 0 ) 
    delete [] ptr_components;
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > :: bicotiCoordinate( const bicotiCoordinate< CoordComponentType >& coord_src ) 
{
  unsigned index;
  
  dimension = coord_src.dimension;
  
  ptr_components = new CoordComponentType [ dimension ];
  
  for( index = 0; index < dimension; index++ )
    ptr_components[ index ] = coord_src.ptr_components[ index ];
};



template< class CoordComponentType >
void bicotiCoordinate< CoordComponentType > :: operator=( const bicotiCoordinate< CoordComponentType >& coord_src )
{
  unsigned index;
  
  if ( dimension == 0 ) 
    {dimension = coord_src.dimension;
    ptr_components = new CoordComponentType [ dimension ];}
  else
    {if( dimension != coord_src.dimension ) 
      {delete [] ptr_components;
      dimension = coord_src.dimension;
      ptr_components = new CoordComponentType [ dimension ];}}
  
  for( index = 0; index < dimension; index++ )
    ptr_components[ index ] = coord_src.ptr_components[ index ];
};


template< class CoordComponentType >
bool bicotiCoordinate< CoordComponentType > :: operator==( const bicotiCoordinate< CoordComponentType >& coord_src )
{
  bool is_equal;
  
  if ( dimension != coord_src.dimension )
    is_equal = false;
  else
    {is_equal = true;
    for( unsigned index = 0; ( (index < dimension ) && is_equal ); index++ )
      is_equal = ( ptr_components[ index ] == coord_src.ptr_components[ index ] );}
  
  return is_equal;
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > bicotiCoordinate< CoordComponentType > :: operator+( double scalar )
{
  bicotiCoordinate< CoordComponentType > coord_temp( dimension );
  
  for ( unsigned index = 0; index < dimension; index++ )
    coord_temp.ptr_components[ index ] = ( CoordComponentType ) ( ptr_components[ index ] + scalar );
  
  return ( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > bicotiCoordinate< CoordComponentType > :: operator-( double scalar )
{
  bicotiCoordinate< CoordComponentType > coord_temp( dimension );
  
  for ( unsigned index = 0; index < dimension; index++ )
    coord_temp.ptr_components[ index ] = ( CoordComponentType ) ( ptr_components[ index ] - scalar );
  
  return ( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > bicotiCoordinate< CoordComponentType > :: operator*( double scalar )
{
  bicotiCoordinate< CoordComponentType > coord_temp( dimension );
  
  for ( unsigned index = 0; index < dimension; index++ )
    coord_temp.ptr_components[ index ] = ( CoordComponentType ) ( ptr_components[ index ] * scalar );

  return ( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > bicotiCoordinate< CoordComponentType > :: operator/( double scalar )
{
  bicotiCoordinate< CoordComponentType > coord_temp( dimension );
  
  for ( unsigned index = 0; index < dimension; index++ )
    coord_temp.ptr_components[ index ] = ( CoordComponentType ) ( ptr_components[ index ] / scalar );
  
  return ( coord_temp );
};


template< class CoordComponentType >
void bicotiCoordinate< CoordComponentType > :: operator+=( double scalar )
{
  for ( unsigned index = 0; index < dimension; index++ )
    ptr_components[ index ] = ( CoordComponentType ) ( ptr_components[ index ] + scalar );
};


template< class CoordComponentType >
void bicotiCoordinate< CoordComponentType > :: operator-=( double scalar )
{
  for ( unsigned index = 0; index < dimension; index++ )
    ptr_components[ index ] = ( CoordComponentType ) ( ptr_components[ index ] - scalar );
};


template< class CoordComponentType >
void bicotiCoordinate< CoordComponentType > :: operator*=( double scalar )
{
  for ( unsigned index = 0; index < dimension; index++ )
    ptr_components[ index ] = ( CoordComponentType ) ( ptr_components[ index ] * scalar );
};


template< class CoordComponentType >
void bicotiCoordinate< CoordComponentType > :: operator/=( double scalar )
{
  for ( unsigned index = 0; index < dimension; index++ )
    ptr_components[ index ] = ( CoordComponentType ) ( ptr_components[ index ] / scalar );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > bicotiCoordinate< CoordComponentType > :: operator+( const bicotiCoordinate< CoordComponentType >& coord_src )
{
  bicotiCoordinate< CoordComponentType > coord_temp( dimension );
  
#ifdef TOTAL_CHECK
  if ( dimension != coord_src.dimension )
    throw bicotiException( "bicotiCoordinate", "operator+", 3 );
#endif
  
  for( unsigned index = 0; index < dimension; index++ )
    coord_temp.ptr_components[ index ] = ptr_components[ index ] + coord_src.ptr_components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > bicotiCoordinate< CoordComponentType > :: operator-( const bicotiCoordinate< CoordComponentType >& coord_src )
{
  bicotiCoordinate< CoordComponentType > coord_temp( dimension );
  
#ifdef TOTAL_CHECK
  if ( dimension != coord_src.dimension )
    throw bicotiException( "bicotiCoordinate", "operator-", 3 );
#endif
  
  for( unsigned index = 0; index < dimension; index++ )
    coord_temp.ptr_components[ index ] = ptr_components[ index ] - coord_src.ptr_components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > bicotiCoordinate< CoordComponentType > :: operator*( const bicotiCoordinate< CoordComponentType >& coord_src )
{
  bicotiCoordinate< CoordComponentType > coord_temp( dimension );
  
#ifdef TOTAL_CHECK
  if ( dimension != coord_src.dimension )
    throw bicotiException( "bicotiCoordinate", "operator*", 3 );
#endif
  
  for( unsigned index = 0; index < dimension; index++ )
    coord_temp.ptr_components[ index ] = ptr_components[ index ] * coord_src.ptr_components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > bicotiCoordinate< CoordComponentType > :: operator/( const bicotiCoordinate< CoordComponentType >& coord_src )
{
  bicotiCoordinate< CoordComponentType > coord_temp( dimension );
  
#ifdef TOTAL_CHECK
  if ( dimension != coord_src.dimension )
    throw bicotiException( "bicotiCoordinate", "operator/", 3 );
#endif
  
  for( unsigned index = 0; index < dimension; index++ )
    coord_temp.ptr_components[ index ] = ptr_components[ index ] / coord_src.ptr_components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType >
void bicotiCoordinate< CoordComponentType > :: operator+=( const bicotiCoordinate< CoordComponentType >& coord_src )
{
#ifdef TOTAL_CHECK
  if ( dimension != coord_src.dimension )
    throw bicotiException( "bicotiCoordinate", "operator+=", 3 );
#endif
  
  for( unsigned index = 0; index < dimension; index++ )
    ptr_components[ index ] = ptr_components[ index ] + coord_src.ptr_components[ index ];
};


template< class CoordComponentType >
void bicotiCoordinate< CoordComponentType > :: operator-=( const bicotiCoordinate< CoordComponentType >& coord_src )
{
#ifdef TOTAL_CHECK
  if ( dimension != coord_src.dimension )
    throw bicotiException( "bicotiCoordinate", "operator-=", 3 );
#endif
  
  for( unsigned index = 0; index < dimension; index++ )
    ptr_components[ index ] = ptr_components[ index ] - coord_src.ptr_components[ index ];
};


template< class CoordComponentType >
void bicotiCoordinate< CoordComponentType > :: operator*=( const bicotiCoordinate< CoordComponentType >& coord_src )
{
#ifdef TOTAL_CHECK
  if ( dimension != coord_src.dimension )
    throw bicotiException( "bicotiCoordinate", "operator*=", 3 );
#endif
  
  for( unsigned index = 0; index < dimension; index++ )
    ptr_components[ index ] = ptr_components[ index ] * coord_src.ptr_components[ index ];
};


template< class CoordComponentType >
void bicotiCoordinate< CoordComponentType > :: operator/=( const bicotiCoordinate< CoordComponentType >& coord_src )
{
#ifdef TOTAL_CHECK
  if ( dimension != coord_src.dimension )
    throw bicotiException( "bicotiCoordinate", "operator/=", 3 );
#endif
  
  for( unsigned index = 0; index < dimension; index++ )
    ptr_components[ index ] = ( CoordComponentType )( ptr_components[ index ] / coord_src.ptr_components[ index ] );
};


template< class CoordComponentType >
REAL bicotiCoordinate< CoordComponentType > :: ScalarProduct( const bicotiCoordinate< CoordComponentType >& coord_src )
{
#ifdef TOTAL_CHECK
  if ( dimension != coord_src.dimension )
    throw bicotiException( "bicotiCoordinate", "ScalarProduct", 3 );
#endif
  
  REAL result = 0;
  
  for( unsigned index = 0; index < dimension; index++ )
    result += ptr_components[ index ] * coord_src.ptr_components[ index ];
  
  return ( result );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > bicotiCoordinate< CoordComponentType > :: operator^( int exponent )
{
  bicotiCoordinate< CoordComponentType > coord_temp( dimension );
  
  for( unsigned index = 0; index < dimension; index++ )
    coord_temp.ptr_components[ index ] = ( CoordComponentType ) pow( ptr_components[ index ], exponent );
  
  return( coord_temp );
};


template< class CoordComponentType >
REAL bicotiCoordinate< CoordComponentType > :: Norm()
{
  REAL result = 0;
  
  for( unsigned index = 0; index < dimension; index++ )
    result += ptr_components[ index ] * ptr_components[ index ];
  
  return ( (REAL) sqrt( result ) );
};


template< class CoordComponentType >
inline CoordComponentType& bicotiCoordinate< CoordComponentType > :: operator() ( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( dimension <= (unsigned) index) )
    throw bicotiException( "bicotiCoordinate", "operator()", 1 );
#endif
  
  return ( ptr_components[ ( unsigned )index ] );
};


template< class CoordComponentType >
unsigned bicotiCoordinate< CoordComponentType > :: GetDimension()
{
  return( dimension );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > :: operator bicotiCoordinate< INTEGER >()
{
  bicotiCoordinate< INTEGER > coord_temp( dimension );

  for( unsigned index = 0; index < dimension; index++ )
    coord_temp( index ) = ( INTEGER ) ptr_components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType >
bicotiCoordinate< CoordComponentType > :: operator bicotiCoordinate< REAL >()
{
  bicotiCoordinate< REAL > coord_temp( dimension );
  
  for( unsigned index = 0; index < dimension; index++ )
    coord_temp( index ) = ( REAL ) ptr_components[ index ];
  
  return( coord_temp );
};


template< class CoordComponentType >
inline bicotiCoordinate< CoordComponentType > :: operator bicotiCoordinate2D< INTEGER >()
{
  bicotiCoordinate2D< INTEGER > coord_temp;
  
#ifdef TOTAL_CHECK
  if ( GetDimension() != 2 ) 
    throw bicotiException( "bicotiCoordinate", "Casting bicotiCoordinate2D< INTEGER >", 3 );
#endif
  
  coord_temp.X0() = ( INTEGER ) operator () ( 0 );
  coord_temp.X1() = ( INTEGER ) operator () ( 1 );
  
  return( coord_temp );
};


template< class CoordComponentType >
inline bicotiCoordinate< CoordComponentType > :: operator bicotiCoordinate3D< INTEGER >()
{
  bicotiCoordinate3D< INTEGER > coord_temp;
  
#ifdef TOTAL_CHECK
  if ( GetDimension() != 3 ) 
    throw bicotiException( "bicotiCoordinate", "Casting bicotiCoordinate3D< INTEGER >", 3 );
#endif
  
  coord_temp.X0() = ( INTEGER ) operator() ( 0 );
  coord_temp.X1() = ( INTEGER ) operator() ( 1 );
  coord_temp.X2() = ( INTEGER ) operator() ( 2 );
  
  return( coord_temp );
};


template< class CoordComponentType >
inline bicotiCoordinate< CoordComponentType > :: operator bicotiCoordinate2D< REAL >()
{
  bicotiCoordinate2D< REAL > coord_temp;
  
#ifdef TOTAL_CHECK
  if ( GetDimension() != 2 ) 
    throw bicotiException( "bicotiCoordinate", "Casting bicotiCoordinate2D< REAL >", 3 );
#endif
  
  coord_temp.X0() = ( REAL ) operator() ( 0 );
  coord_temp.X1() = ( REAL ) operator() ( 1 );
  
  return( coord_temp );
};


template< class CoordComponentType >
inline bicotiCoordinate< CoordComponentType > :: operator bicotiCoordinate3D< REAL >()
{
  bicotiCoordinate3D< REAL > coord_temp;
  
#ifdef TOTAL_CHECK
  if ( GetDimension() != 3 ) 
    throw bicotiException( "bicotiCoordinate", "Casting bicotiCoordinate3D< REAL >", 3 );
#endif
  
  coord_temp.X0() = ( REAL ) operator() ( 0 );
  coord_temp.X1() = ( REAL ) operator() ( 1 );
  coord_temp.X2() = ( REAL ) operator() ( 2 );
  
  return( coord_temp );
};




#endif            
