#ifndef _bicoti_device_file
#define _bicoti_device_file

#include <source/device/device_basic.hpp>

//--------------------DECLARATIONS---------------------
//------------------bicotiDeviceFile-------------------

/**
 * Este device representa un archivo de disco. 
 *
 * Este device representa un archivo de disco, implementando los mtodos abstractos de su clase madre
 */
class bicotiDeviceFile : public bicotiDevice
{
private:

  FILE * ptr_file;
  char file_name[MAX_FILE_NAME_SIZE];
  
public:

  /**
   * Constructor por defecto
   *
   *@param name nombre del archivo
   */ 

  bicotiDeviceFile( char * name);
  
  /**
   * Destructor
   */
~bicotiDeviceFile();
  /**
   * cambia el nobre del archivo
   *
   *@param name nobre del archivo
   */
  void SetFileName( const char * name);
  /**
   * abre el archivo
   * 
   *@param mode modo
   *@exception bicotiException si no puede abrir el archivo
   */
  void Open( const char * mode);

  /**
   * cierra el archivo
   */
  void Close();
  /**
   * Indica si el device esta abierto
   *@return true si el device esta abierto false sin
   */  
bool IsOpened();
  
/**
 * Devuelve el tamao en bytes de la imagen
 *
 *@returns tamao de la imagen en bytes
 *@exeption bicotiException si no hay archivo
 */
  long GetImageSize();
  
   /**
   * Lee secuencialmente bytes del device
   *
   * Lee BlockSize() o los que queden (al final del archivo) del device.
   *@param blob contiene los bytes leidos
   *@returns la cantidad de bytes leidos
   *@exception bicotiException si hay un error de I/O
   */
   
  int Get( char * blob);

 /**
   * Escribe secuencialmente bytes en el device
   *
   * Escribe BlockSize() bytes en el device
   *@param blob contien los bytes a escribir
   *@exception bicotiException si hay un error de I/O
   */
  void Put( const char * blob );
};

#endif
