//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _coordinate_interpolator_1D_closed_spline
#define _coordinate_interpolator_1D_closed_spline



#include <source/dexel/dexel_curves/coordinate_interpolator_1D.hpp>
#include <source/dexel/dexel_curves/coordinate_interpolator_function_spline.hpp>



//*******************************DECLARATIONS***********************************
//----------------bicotiCoordinateInterpolator1DClosedSpline--------------------
template< class CoordType >
class bicotiCoordinateInterpolator1DClosedSpline : public bicotiCoordinateInterpolator1DClosed< CoordType >
{
private:
  vector< bicotiCoordinateInterpolatorFunctionSpline< CoordType > > list_functions;
  bicotiNumericalAlgorithms< double > algorithms;
  bicotiCoordinateInterpolatorFunctionDistanceSpline< CoordType > * ptr_distance_function;

  double FindInternalParameter( double, int );
  
public:
  bicotiCoordinateInterpolator1DClosedSpline();
  ~bicotiCoordinateInterpolator1DClosedSpline();

  void Initiate( const vector< CoordType >& );

  CoordType operator()( double );
  CoordType GetDerivate( double );
  CoordType GetDerivate2( double );
  CoordType GetDerivate3( double );
};




//******************************IMPLEMENTATIONS*********************************
//------------------bicotiCoordinateInterpolator1DClosedSpline------------------
template< class CoordType >
double bicotiCoordinateInterpolator1DClosedSpline< CoordType > :: FindInternalParameter( double relative_offset, int index )
{
  double t, s;
  
  if( ( relative_offset == 0 ) || ( relative_offset == 1 ) )
    { t = 0; }
  else
    { s = ( relative_offset - list_relative_lengths[ index ] ) * length ;
    
    bicotiCoordinateInterpolatorFunctionSpline< CoordType > * ptr_fun;
    ptr_fun = & (list_functions[ index ]);
    
    ptr_distance_function->SetFunction( ptr_fun );
    ptr_distance_function->SetFunVal( s );
    
    t = algorithms.Newton( ptr_distance_function, ptr_fun , 0.5 );};
  
  return( t );
};


template< class CoordType >
bicotiCoordinateInterpolator1DClosedSpline< CoordType > :: bicotiCoordinateInterpolator1DClosedSpline()
{
  ptr_distance_function = new bicotiCoordinateInterpolatorFunctionDistanceSpline< CoordType >;
  ptr_distance_function->SetFirstVal( 0 );
};


template< class CoordType >
bicotiCoordinateInterpolator1DClosedSpline< CoordType > :: ~bicotiCoordinateInterpolator1DClosedSpline()
{
  delete ptr_distance_function;
};


template< class CoordType >
void bicotiCoordinateInterpolator1DClosedSpline< CoordType > :: Initiate( const vector< CoordType >& coords )
{
  unsigned index;
  CoordType y0, y1, y2, y3;

  bicotiCoordinateInterpolatorFunctionSpline< CoordType > fun_aux;
  bicotiCoordinateInterpolatorFunctionSpline< CoordType > * ptr_fun_aux;

  double k_3_0 = - 18.0 / 30;
  double k_3_1 = 42.0 / 30;
  double k_3_2 = - 42.0 / 30;
  double k_3_3 = 18.0 / 30;

  double k_2_0 = 42.0 / 30;
  double k_2_1 = -78.0 / 30;
  double k_2_2 = 48.0 / 30;
  double k_2_3 = -12.0 / 30;

  double k_1_0 = -24.0 / 30;
  double k_1_1 = 6.0 / 30;
  double k_1_2 = 24.0 / 30;
  double k_1_3 = -6.0 / 30;

  list_coords = coords;


  if ( list_coords.size() <= 3 )
    throw bicotiException( "bicotiCoordinateInterpolator1DClosedSpline", "Initiate()", 4 );

  length = 0;

  while( list_functions.size() != 0 )
    list_functions.pop_back();

  while( list_relative_lengths.size() != 0 )
    list_relative_lengths.pop_back();

  list_relative_lengths.push_back( 0 );

  y0 = list_coords.back();
  y1 = list_coords[ 0 ];
  y2 = list_coords[ 1 ];
  y3 = list_coords[ 2 ];
  
  fun_aux.x_3_coeff = ( y0 * k_3_0 ) + ( y1 * k_3_1 ) + ( y2 * k_3_2 ) + ( y3 * k_3_3 );
  fun_aux.x_2_coeff = ( y0 * k_2_0 ) + ( y1 * k_2_1 ) + ( y2 * k_2_2 ) + ( y3 * k_2_3 );
  fun_aux.x_1_coeff = ( y0 * k_1_0 ) + ( y1 * k_1_1 ) + ( y2 * k_1_2 ) + ( y3 * k_1_3 );
  fun_aux.x_0_coeff =  y1;
  
  
  list_functions.push_back( fun_aux );
  ptr_fun_aux = & ( list_functions.back() );

  ptr_distance_function->SetFunction( ptr_fun_aux );
  length += ptr_distance_function->Distance( 0, 1.0 );
  list_relative_lengths.push_back( length );
  
  for( index = 1; index < list_coords.size() - 2; index++ )
    {y0 = list_coords[ index - 1 ];
    y1 = list_coords[ index ];
    y2 = list_coords[ index + 1 ];
    y3 = list_coords[ index + 2 ];

    fun_aux.x_3_coeff = ( y0 * k_3_0 ) + ( y1 * k_3_1 ) + ( y2 * k_3_2 ) + ( y3 * k_3_3 );
    fun_aux.x_2_coeff = ( y0 * k_2_0 ) + ( y1 * k_2_1 ) + ( y2 * k_2_2 ) + ( y3 * k_2_3 );
    fun_aux.x_1_coeff = ( y0 * k_1_0 ) + ( y1 * k_1_1 ) + ( y2 * k_1_2 ) + ( y3 * k_1_3 );
    fun_aux.x_0_coeff =  y1;
    
    list_functions.push_back( fun_aux );
    ptr_fun_aux = & ( list_functions.back() );

    ptr_distance_function->SetFunction( ptr_fun_aux );
    length += ptr_distance_function->Distance( 0, 1.0 );
    list_relative_lengths.push_back( length );};
  
  y0 = list_coords[ index - 1];
  y1 = list_coords[ index ];
  y2 = list_coords[ index + 1];
  y3 = list_coords[ 0 ];
  
  fun_aux.x_3_coeff = ( y0 * k_3_0 ) + ( y1 * k_3_1 ) + ( y2 * k_3_2 ) + ( y3 * k_3_3 );
  fun_aux.x_2_coeff = ( y0 * k_2_0 ) + ( y1 * k_2_1 ) + ( y2 * k_2_2 ) + ( y3 * k_2_3 );
  fun_aux.x_1_coeff = ( y0 * k_1_0 ) + ( y1 * k_1_1 ) + ( y2 * k_1_2 ) + ( y3 * k_1_3 );
  fun_aux.x_0_coeff =  y1;
  
  list_functions.push_back( fun_aux );
  ptr_fun_aux = & ( list_functions.back() );

  ptr_distance_function->SetFunction( ptr_fun_aux );
  length += ptr_distance_function->Distance( 0, 1.0 );
  list_relative_lengths.push_back( length );
  
  index++;
  y0 = list_coords[ index - 1];
  y1 = list_coords[ index ];
  y2 = list_coords[ 0 ];
  y3 = list_coords[ 1 ];
  
  fun_aux.x_3_coeff = ( y0 * k_3_0 ) + ( y1 * k_3_1 ) + ( y2 * k_3_2 ) + ( y3 * k_3_3 );
  fun_aux.x_2_coeff = ( y0 * k_2_0 ) + ( y1 * k_2_1 ) + ( y2 * k_2_2 ) + ( y3 * k_2_3 );
  fun_aux.x_1_coeff = ( y0 * k_1_0 ) + ( y1 * k_1_1 ) + ( y2 * k_1_2 ) + ( y3 * k_1_3 );
  fun_aux.x_0_coeff =  y1;
  
  list_functions.push_back( fun_aux );
  ptr_fun_aux = & ( list_functions.back() );

  ptr_distance_function->SetFunction( ptr_fun_aux );
  length += ptr_distance_function->Distance( 0, 1.0 );
  list_relative_lengths.push_back( length );
  
  for( index = 1; index < list_relative_lengths.size(); index++ )
    list_relative_lengths[ index ] /= length;
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DClosedSpline< CoordType > :: operator()( double relative_offset )
{
  int index;
  double t;

  index = FindIndex( relative_offset );
  t = FindInternalParameter( relative_offset, index );
  
  return( list_functions[ index ].Evaluate( t ) );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DClosedSpline< CoordType > :: GetDerivate( double relative_offset )
{
  int index;
  double t, der_norm;
  CoordType aux_coord;

  index = FindIndex( relative_offset );
  t = FindInternalParameter( relative_offset, index );
  
  if ( relative_offset ==  0 )
    t += EPSILON_MACHINE;

  der_norm = ( list_functions[ index ] ).EvaluateDerNorm( t );
  aux_coord = ( ( list_functions[ index ] ).EvaluateDer( t ) ) * ( length / der_norm );

  return( aux_coord );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DClosedSpline< CoordType > :: GetDerivate2( double relative_offset )
{
  int index;
  double t, der_norm_2, der_der2;
  CoordType aux_coord, der, der2;

  index = FindIndex( relative_offset );
  t = FindInternalParameter( relative_offset, index );

  if ( relative_offset == 0 )
    t += EPSILON_MACHINE;

  der_norm_2 = ( list_functions[ index ] ).EvaluateDerNorm( t );
  der_norm_2 *= der_norm_2;

  der = ( list_functions[ index ] ).EvaluateDer( t );
  der2 = ( list_functions[ index ] ).EvaluateDer2( t );
  der_der2 = der.ScalarProduct( der2 );

  aux_coord = der2 - der * ( der_der2 / der_norm_2 );

  aux_coord = aux_coord * ( ( length * length ) / der_norm_2 );

  return( aux_coord );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DClosedSpline< CoordType > :: GetDerivate3( double relative_offset )
{
  int index;
  double t, dist;

  index = FindIndex( relative_offset );
  t = FindInternalParameter( relative_offset, index );

  if ( relative_offset == 0 )
    t += EPSILON_MACHINE;

  dist = ( ( ( list_functions[ index ] ).EvaluateDer3( t ) ).Norm() ) / ( length * length * length );

  return( ( ( list_functions[ index ] ).EvaluateDer3( t ) ) * ( 1 / dist ) );
};



#endif
