//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _coordinate_interpolator_1D_open_simpson
#define _coordinate_interpolator_1D_open_simpson


#include <source/dexel/dexel_curves/coordinate_interpolator_1D.hpp>
#include <source/dexel/dexel_curves/coordinate_interpolator_function_simpson.hpp>



//*******************************DECLARATIONS***********************************
//-----------------bicotiCoordinateInterpolator1DOpenSimpson--------------------
template< class CoordType >
class bicotiCoordinateInterpolator1DOpenSimpson : public bicotiCoordinateInterpolator1DOpen< CoordType >
{
private:
  vector< bicotiCoordinateInterpolatorFunctionSimpson< CoordType > > list_functions;
  bicotiNumericalAlgorithms< double > algorithms;
  bicotiCoordinateInterpolatorFunctionDistanceSimpson< CoordType > * ptr_distance_function;
  
  double FindInternalParameter( double, int );

public:
  bicotiCoordinateInterpolator1DOpenSimpson();
  ~bicotiCoordinateInterpolator1DOpenSimpson();

  void Initiate( const vector< CoordType >& );

  CoordType operator() ( double );
  CoordType GetDerivate( double );
  CoordType GetDerivate2( double );
  CoordType GetDerivate3( double );

  unsigned GetPointsNumber();
  CoordType GetPoint( int );
  double GetOffset( int );
};




//******************************IMPLEMENTATIONS*********************************
//-----------------bicotiCoordinateInterpolator1DOpenSimpson--------------------
template< class CoordType >
double bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: FindInternalParameter( double relative_offset, int index )
{
  double t, s, first_t, last_t;
  
  if( relative_offset == 0 )
    { t = 0; }
  else
    { if ( ( index == ( list_functions.size() - 1 ) ) && ( ( list_coords.size() % 2 ) == 0 ) )
      {first_t = 1.0;
      last_t = 2.0;}
    else
      {first_t = 0;
      last_t = 2.0;};
    
    bicotiCoordinateInterpolatorFunctionSimpson< CoordType > * ptr_fun;
    ptr_fun = & (list_functions[ index ]);
    
    ptr_distance_function->SetFunction( ptr_fun );
  
    s = ( relative_offset - list_relative_lengths[ index ] ) * length;
    
    ptr_distance_function->SetFirstVal( first_t );
    ptr_distance_function->SetFunVal( s );
    
    t = algorithms.Newton( ptr_distance_function, ptr_fun, ( first_t + last_t ) / 2 );};
  
  return( t );
};


template< class CoordType >
bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: bicotiCoordinateInterpolator1DOpenSimpson()
{
  ptr_distance_function = new bicotiCoordinateInterpolatorFunctionDistanceSimpson< CoordType > ;
};


template< class CoordType >
bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: ~bicotiCoordinateInterpolator1DOpenSimpson()
{
  delete ptr_distance_function;
};


template< class CoordType >
void bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: Initiate( const vector< CoordType >& coords )
{
  CoordType y0, y1, y2;
  unsigned index;

  bicotiCoordinateInterpolatorFunctionSimpson< CoordType > fun_aux;
  bicotiCoordinateInterpolatorFunctionSimpson< CoordType > * ptr_fun_aux;


  list_coords = coords;

  if ( list_coords.size() <= 2 )
    throw bicotiException( "bicotiCoordinateInterpolator1DOpenSimpson", "Initiate()", 4 );

  length = 0;
  while( list_functions.size() != 0 )
    list_functions.pop_back();

  while( list_relative_lengths.size() != 0 )
    list_relative_lengths.pop_back();

  list_relative_lengths.push_back( 0 );
  for( index = 0; index < list_coords.size() - 2 ; index += 2 )
    {y0 = list_coords[ index ];
    y1 = list_coords[ index + 1 ];
    y2 = list_coords[ index + 2 ];
    fun_aux.x_2_coeff = ( ( y0 * .5 ) - y1 + ( y2 * .5 ) );
    fun_aux.x_1_coeff = ( ( y0 * -1.5 ) + ( y1 * 2 ) - ( y2 * .5 ) );
    fun_aux.x_0_coeff = y0;
    fun_aux.ComputeConsts();

    list_functions.push_back( fun_aux );
    ptr_fun_aux = & ( list_functions.back() );
    
    ptr_distance_function->SetFunction( ptr_fun_aux );
    length += ptr_distance_function->Distance( 0, 2.0 );
    list_relative_lengths.push_back( length );}

  if ( index == ( list_coords.size() - 2 ) )
    {y0 = list_coords[ index - 1];
    y1 = list_coords[ index ];
    y2 = list_coords[ index + 1];
    fun_aux.x_2_coeff = ( ( y0 * .5 ) - y1 + ( y2 * .5 ) );
    fun_aux.x_1_coeff = ( ( y0 * -1.5 ) + ( y1 * 2 ) - ( y2 * .5 ) );
    fun_aux.x_0_coeff = y0;
    fun_aux.ComputeConsts();
    
    list_functions.push_back( fun_aux );
    ptr_fun_aux = & ( list_functions.back() );
    
    ptr_distance_function->SetFunction( ptr_fun_aux );
    length += ptr_distance_function->Distance( 1.0, 2.0 );
    list_relative_lengths.push_back( length );};

  for( index = 1; index < list_relative_lengths.size(); index++ )
    list_relative_lengths[ index ] /= length;
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: operator()( double relative_offset )
{
  int index;
  double t;

  index = FindIndex( relative_offset );
  t = FindInternalParameter( relative_offset, index );

  return( list_functions[ index ].Evaluate( t ) );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: GetDerivate( double relative_offset )
{
  int index;
  double t, der_norm;
  CoordType aux_coord;

  index = FindIndex( relative_offset );
  t = FindInternalParameter( relative_offset, index );
  
  if ( relative_offset ==  0 )
    t += EPSILON_MACHINE;

  der_norm = ( list_functions[ index ] ).EvaluateDerNorm( t );
  aux_coord = ( ( list_functions[ index ] ).EvaluateDer( t ) ) * ( length / der_norm );

  return( aux_coord );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: GetDerivate2( double relative_offset )
{
  int index;
  double t, der_norm_2, der_der2;
  CoordType aux_coord, der, der2;

  index = FindIndex( relative_offset );
  t = FindInternalParameter( relative_offset, index );

  if ( relative_offset == 0 )
    t += EPSILON_MACHINE;

  der_norm_2 = ( list_functions[ index ] ).EvaluateDerNorm( t );
  der_norm_2 *= der_norm_2;

  der = ( list_functions[ index ] ).EvaluateDer( t );
  der2 = ( list_functions[ index ] ).EvaluateDer2( t );
  der_der2 = der.ScalarProduct( der2 );

  aux_coord = der2 - der * ( der_der2 / der_norm_2 );

  aux_coord = aux_coord * ( ( length * length ) / der_norm_2 );

  return( aux_coord );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: GetDerivate3( double relative_offset )
{
  int index;
  double t, dist;

  index = FindIndex( relative_offset );
  t = FindInternalParameter( relative_offset, index );

  if ( relative_offset == 0 )
    t += EPSILON_MACHINE;

  dist = ( ( ( list_functions[ index ] ).EvaluateDer3( t ) ).Norm() ) / ( length * length * length );

  return( ( ( list_functions[ index ] ).EvaluateDer3( t ) ) * ( 1 / dist ) );
};


template< class CoordType >
unsigned bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: GetPointsNumber()
{
  return ( list_relative_lengths.size() );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: GetPoint( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( index >=  list_coords.size() ) ) 
    throw bicotiException( "bicotiCoordinateInterpolator1DOpenSimpson", "GetPoint()", 1 );
#endif

  if ( ( index == ( list_relative_lengths.size() - 1 ) ) && ( ( list_relative_lengths.size() % 2 ) == 0 ) )
    return ( list_coords[ 2 * index - 1 ] );
  else
    return ( list_coords[ 2 * index ] );
};


template< class CoordType >
double bicotiCoordinateInterpolator1DOpenSimpson< CoordType > :: GetOffset( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( index >=  list_coords.size() ) ) 
    throw bicotiException( "bicotiCoordinateInterpolator1DOpenSimpson", "GetOffset()", 1 );
#endif

  return ( list_relative_lengths[ index ] );
};  


#endif
