//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _coordinate_interpolator_function_spline
#define _coordinate_interpolator_function_spline


#include <source/dexel/dexel_curves/coordinate_interpolator_function_basic.hpp>



//*******************************DECLARATIONS***********************************
//------------------bicotiCoordinateInterpolatorFunctionSpline------------------
template< class CoordType >
class bicotiCoordinateInterpolatorFunctionSpline : public bicotiCoordinateInterpolatorFunction< CoordType >
{
public:
  bicotiCoordinateInterpolatorFunctionSpline(){};
  bicotiCoordinateInterpolatorFunctionSpline( CoordType, CoordType, CoordType, CoordType );

  double operator()( double );
  CoordType Evaluate( double );
  CoordType EvaluateDer( double );
  double EvaluateDerNorm( double );
  CoordType EvaluateDer2( double );
  CoordType EvaluateDer3( double );

  CoordType x_3_coeff,x_2_coeff, x_1_coeff, x_0_coeff;
};


//-----------------bicotiCoordinateInterpolatorFunctionDistanceSpline----------------
template< class CoordType >
class bicotiCoordinateInterpolatorFunctionDistanceSpline : public bicotiCoordinateInterpolatorFunctionDistance< CoordType >
{
public:
  bicotiCoordinateInterpolatorFunctionDistanceSpline(){};

  double operator()( double);
  double Distance( double, double );
};




//******************************IMPLEMENTATIONS*********************************
//---------------bicotiCoordinateInterpolatorFunctionSpline---------------------
template< class CoordType >
bicotiCoordinateInterpolatorFunctionSpline< CoordType > :: bicotiCoordinateInterpolatorFunctionSpline( CoordType x_3, CoordType x_2, CoordType x_1, CoordType x_0 )
{
  x_3_coeff = x_3;
  x_2_coeff = x_2;
  x_1_coeff = x_1;
  x_0_coeff = x_0;
};


template< class CoordType >
double bicotiCoordinateInterpolatorFunctionSpline< CoordType > :: operator()( double t )
{
  double temp_value;

  temp_value = EvaluateDerNorm( t );

  return( temp_value );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolatorFunctionSpline< CoordType > :: Evaluate( double t )
{
  return( x_3_coeff * t * t * t + x_2_coeff * t * t + x_1_coeff * t + x_0_coeff );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolatorFunctionSpline< CoordType > :: EvaluateDer( double t )
{
  return( ( x_3_coeff * t * t ) * 3 + ( x_2_coeff * t ) * 2 + x_1_coeff );
};


template< class CoordType >
double bicotiCoordinateInterpolatorFunctionSpline< CoordType > :: EvaluateDerNorm( double t )
{
  CoordType aux_coord;
  
  aux_coord = EvaluateDer( t );
  
  return(  aux_coord.Norm() );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolatorFunctionSpline< CoordType > :: EvaluateDer2( double t )
{
  return( ( x_3_coeff * t ) * 6 + ( x_2_coeff * 2 ) );
};


template< class CoordType >
CoordType bicotiCoordinateInterpolatorFunctionSpline< CoordType > :: EvaluateDer3( double t )
{
  return( x_3_coeff * 6 );
};


//----------------bicotiCoordinateInterpolatorFunctionDistanceSpline-----------------------
template< class CoordType >
double bicotiCoordinateInterpolatorFunctionDistanceSpline< CoordType > :: operator()( double t )
{
  bicotiNumericalAlgorithms< double > algorithms;
  
  return( algorithms.Quadrature( ptr_function, first_val, t ) - fun_val );
};


template< class CoordType >
double bicotiCoordinateInterpolatorFunctionDistanceSpline< CoordType > :: Distance( double first_t, double t )
{
  bicotiNumericalAlgorithms< double > algorithms;
  
  return( algorithms.Quadrature( ptr_function, first_t, t ) );
};



#endif       
