//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _dexel_curves_properties
#define _dexel_curves_properties


#include <source/dexel/dexel_curves/dexel_curves_basic.hpp>



//********************************DECLARATIONS***********************************
//----------------------------bicotiCurveProperties------------------------------
template< class CoordType, class PixelType >
class bicotiCurveProperties
{
protected:
  bicotiDexelCurve< CoordType, PixelType > * ptr_curve;
  bicotiNumericalAlgorithms< CoordType > algorithms;
  double const_normalization;

public:
  bicotiCurveProperties( bicotiDexelCurve< CoordType, PixelType > * = NULL );
  virtual ~bicotiCurveProperties(){};

  void SetCurve( bicotiDexelCurve< CoordType, PixelType > * );

  double GetCurvature( double );
  CoordType GetTangent( double );
  CoordType GetNormal( double );
  double GetProjection( CoordType );
  double GetDistance( CoordType );
};



//-----------------------------bicotiCurveProperties2D---------------------------------
template< class PixelType >
class bicotiCurveProperties2D : public bicotiCurveProperties< bicotiCoordinate2D< REAL >, PixelType >
{
public:
  bicotiCurveProperties2D( bicotiDexelCurve< bicotiCoordinate2D< REAL >, PixelType > * = NULL );
  
  double GetTorsion( double );
};



//-----------------------------bicotiCurveProperties3D---------------------------------
template< class PixelType >
class bicotiCurveProperties3D : public bicotiCurveProperties< bicotiCoordinate3D< REAL >, PixelType >
{
public:
  bicotiCurveProperties3D( bicotiDexelCurve< bicotiCoordinate3D< REAL >, PixelType > * = NULL );
  
  double GetTorsion( double );
  bicotiCoordinate3D< REAL > GetBinormal( double );
};



//--------------------------bicotiCurveProperties2DClosed------------------------------
template< class PixelType >
class bicotiCurveProperties2DClosed : public bicotiCurveProperties2D< PixelType >
{
public:
  bicotiCurveProperties2DClosed( bicotiDexelCurveClosed< bicotiCoordinate2D< REAL >, PixelType > * = NULL );
  
  double GetArea();
  bool IsInside( bicotiCoordinate2D< REAL > );
};




//******************************IMPLEMENTATIONS**********************************
//---------------------------bicotiCurveProperties-------------------------------
template< class CoordType, class PixelType >
bicotiCurveProperties< CoordType, PixelType > :: bicotiCurveProperties( bicotiDexelCurve< CoordType, PixelType > * ptr_cur )
{
  ptr_curve = ptr_cur;
  const_normalization = 1.0 / ptr_curve->GetLength();
};


template< class CoordType, class PixelType >
void bicotiCurveProperties< CoordType, PixelType > :: SetCurve( bicotiDexelCurve< CoordType, PixelType > * ptr_cur )
{
  ptr_curve = ptr_cur;
  const_normalization = 1.0 / ptr_curve->GetLength();
};


template< class CoordType, class PixelType >
double bicotiCurveProperties< CoordType, PixelType > :: GetCurvature( double relative_offset )
{
  return( ( ptr_curve->GetDerivate2( relative_offset ) ).Norm() * ( const_normalization * const_normalization ) );
};


template< class CoordType, class PixelType >
CoordType bicotiCurveProperties< CoordType, PixelType > :: GetTangent( double relative_offset )
{
  return( ( ptr_curve->GetDerivate( relative_offset ) ) * const_normalization );
};


template< class CoordType, class PixelType >
CoordType bicotiCurveProperties< CoordType, PixelType > :: GetNormal( double relative_offset )
{
  double norm = ptr_curve->GetDerivate2( relative_offset ).Norm();
  return( ptr_curve->GetDerivate2( relative_offset ) ) * ( 1 / norm );
};


template< class CoordType, class PixelType >
double bicotiCurveProperties< CoordType, PixelType > :: GetProjection( CoordType coord )
{
  return( 0 );
};


template< class CoordType, class PixelType >
double bicotiCurveProperties< CoordType, PixelType > :: GetDistance( CoordType coord )
{
  return( 0 );
};



//---------------------------bicotiCurveProperties2D-----------------------------
template< class PixelType >
bicotiCurveProperties2D< PixelType > :: bicotiCurveProperties2D( bicotiDexelCurve< bicotiCoordinate2D< REAL >, PixelType > * ptr_cur )
{
  ptr_curve = ptr_cur;
};


template< class PixelType >
double bicotiCurveProperties2D< PixelType > :: GetTorsion( double relative_offset )
{
  return ( 0 );
};



//-----------------------------bicotiCurveProperties3D---------------------------------
template< class PixelType >
bicotiCurveProperties3D< PixelType > :: bicotiCurveProperties3D( bicotiDexelCurve< bicotiCoordinate3D< REAL >, PixelType > * ptr_cur )
{
  ptr_curve = ptr_cur;
};


template< class PixelType >
double bicotiCurveProperties3D< PixelType > :: GetTorsion( double relative_offset )
{
  return ( 0 );
};


template< class PixelType >
bicotiCoordinate3D< REAL > bicotiCurveProperties3D< PixelType > :: GetBinormal( double relative_offset )
{
  bicotiCoordinate3D< REAL > coord_aux;

  return( coord_aux );
};



//--------------------------bicotiCurveProperties2DClosed------------------------------
template< class PixelType >
bicotiCurveProperties2DClosed< PixelType > :: bicotiCurveProperties2DClosed( bicotiDexelCurveClosed< bicotiCoordinate2D< REAL >, PixelType > * ptr_cur )
{
  ptr_curve = ptr_cur;
};


template< class PixelType >
double bicotiCurveProperties2DClosed< PixelType > :: GetArea()
{
  return ( 0 );
};


template< class PixelType >
bool bicotiCurveProperties2DClosed< PixelType > :: IsInside( bicotiCoordinate2D< REAL > coord )
{
  return ( true );
};


#endif
