//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _dexel_region_property
#define _dexel_region_property



#include <source/dexel/dexel_region/dexel_region_basic.hpp>



//************************************DECLARATIONS***************************************
//------------------------------------CompareCoord---------------------------------------
template< class CoordType >
class CompareCoord
{
private:
  unsigned dimension;
  
public:
  CompareCoord();
  
  bool operator() ( const CoordType&, const CoordType& ) const;
};



//-----------------------------------CompDistCoord---------------------------------------
template< class CoordType >
class CompDistCoord
{
private:
  CoordType current_coord;
  CoordType current_direc;
  bool initializated;

public:
  void Reset();
  void SetCurrent( CoordType& );
  
  bool operator() ( const CoordType&, const CoordType& );
};



//-----------------------------------bicotiRegionProperty--------------------------------------
template< class CoordType , class PixelType >
class bicotiRegionProperty
{
protected:
  bicotiDexelRegion< CoordType, PixelType > * ptr_region;
  set< CoordType, CompareCoord< CoordType > > sorted_set;
  vector< CoordType > border;
  vector< vector< CoordType > > adjacents;

public:  
  bicotiRegionProperty( bicotiDexelRegion< CoordType, PixelType > * = NULL );

  void SetRegion( bicotiDexelRegion< CoordType, PixelType > * );

  void FixInfo();
  void ComputeBorder();
  void ClassifyAdjacents( double = -1 );

  unsigned GetPixelsInRegionNumber();
  unsigned GetPixelsInBorderNumber();
  unsigned GetDistinctsAdjacentsSetsNumber();
  
  vector< CoordType >& GetBorder();
  vector< CoordType >& GetAdjacents( unsigned );
  
  bicotiCoordinate< REAL > GetBaricenter( bicotiPixelWeight< PixelType > * = NULL );
};




//**********************************IMPLEMENTATIONS**************************************
//------------------------------------CompareCoord---------------------------------------
template< class CoordType >
CompareCoord< CoordType > :: CompareCoord()
{
  CoordType aux_coord;
  dimension = aux_coord.GetDimension() - 1;
};


template< class CoordType >
inline bool CompareCoord< CoordType > :: operator() ( const CoordType& coord_1, const CoordType& coord_2 ) const
{
  CoordType coord1 = coord_1;
  CoordType coord2 = coord_2;
  bool result = false;
  
  for( int index = dimension ; index >= 0; index-- )
    {if ( coord1( index ) < coord2( index ) )
      {result = true;
      break;}
    else
      {if( coord1( index ) > coord2( index ) )
	break;}}
  
  return( result );
};



//-----------------------------------CompDistCoord---------------------------------------
template< class CoordType >
void CompDistCoord< CoordType > :: Reset()
{
  initializated = false;
  
  for( unsigned index = 0; index < current_direc.GetDimension(); index++ )
    current_direc( index ) = 0;
};


template< class CoordType >
void CompDistCoord< CoordType > :: SetCurrent( CoordType& coord )
{
  if( initializated )
    {current_direc = current_coord - coord;
    current_direc = current_direc * ( 1 / current_direc.Norm() );}
  else
    initializated = true;
  
  current_coord = coord;
};


template< class CoordType >
inline bool CompDistCoord< CoordType > :: operator() ( const CoordType& coord_ext_1, const CoordType& coord_ext_2 )
{
  CoordType coord_ext1 = coord_ext_1;
  CoordType coord_ext2 = coord_ext_2;
  coord_ext1 -= current_coord;
  coord_ext2 -= current_coord;
  double norm_1 = coord_ext1.Norm();
  double norm_2 = coord_ext2.Norm();

  if( norm_1 < norm_2 )
    return( true );
  else
    {if( norm_1 > norm_2 )
      return( false );
    else
      {coord_ext1 = coord_ext1 *( 1 / norm_1 );
      coord_ext2 = coord_ext2 * ( 1 / norm_2 );
      return( current_direc.ScalarProduct( coord_ext1 ) >= current_direc.ScalarProduct( coord_ext2 ) );}}
};



//-----------------------------------bicotiRegionProperty--------------------------------------
template< class CoordType, class PixelType >
bicotiRegionProperty< CoordType, PixelType > :: bicotiRegionProperty( bicotiDexelRegion< CoordType, PixelType > * ptr_reg )
{
  ptr_region = ptr_reg;
};


template< class CoordType, class PixelType >
void bicotiRegionProperty< CoordType, PixelType > :: SetRegion( bicotiDexelRegion< CoordType, PixelType > * ptr_reg )
{
  ptr_region = ptr_reg;
};


template< class CoordType, class PixelType >
void bicotiRegionProperty< CoordType, PixelType > :: FixInfo()
{
  //Ordena con un criterio, esto es para estar seguro de cual es el orden para usar ComputeBorder
  if( ptr_region == NULL )
    throw bicotiException( "bicotiRegionProperty", "FixInfo()", 6 );

  CoordType aux_coord;

  sorted_set.erase( sorted_set.begin(), sorted_set.end() );

  for( unsigned index = 0; index < ptr_region->GetPixelsNumber(); index++ )
    {aux_coord = ptr_region->GetCoordinate( index );
    sorted_set.insert( aux_coord );}
};


template< class CoordType, class PixelType >
void bicotiRegionProperty< CoordType, PixelType > :: ComputeBorder()
{
  CoordType aux_coord_1, aux_coord_2;
  unsigned index;
  const unsigned dimension = aux_coord_1.GetDimension();

  if(sorted_set.empty())
    throw bicotiException( "bicotiRegionProperty", "ComputeBorder()", 5 );

  while( !border.empty() )
    border.pop_back();

  set< CoordType, CompareCoord< CoordType > > :: iterator first = sorted_set.begin();
  set< CoordType, CompareCoord< CoordType > > :: iterator last = sorted_set.end();
  set< CoordType, CompareCoord< CoordType > > :: iterator iter;

  while( first != last )
    {aux_coord_1 = *first;
    for( index = 0; index < dimension; index++ )
      {aux_coord_2 = aux_coord_1;
      aux_coord_2( index ) += 1;
      iter = sorted_set.find( aux_coord_2 );
      if( iter == last )
	{border.push_back( aux_coord_1 );
	break;}
      else
	{aux_coord_2( index ) -= 2;
	iter = sorted_set.find( aux_coord_2 );
	if( iter == last )
	  {border.push_back( aux_coord_1 );
	  break;}}}
    first++;}
};


template< class CoordType, class PixelType >
void bicotiRegionProperty< CoordType, PixelType > :: ClassifyAdjacents( double distance_tol )
{
  list< CoordType > border_aux( border.begin(), border.end() );
  list< CoordType > :: iterator current = border_aux.begin();
  list< CoordType > :: iterator last = border_aux.end();

  CompDistCoord< CoordType > dist_comp_obj;
  vector< CoordType > aux_vector;
  unsigned adj_index = 0;
  CoordType aux_coord;
  
  if( distance_tol < 0 )
    distance_tol = sqrt( aux_coord.GetDimension() );
  
  while( !adjacents.empty() )
    adjacents.pop_back();
  
  dist_comp_obj.Reset();
  adjacents.push_back( aux_vector );
  
  while( !border_aux.empty() )
    {aux_coord = *current;
    border_aux.erase( current );
    adjacents[ adj_index ].push_back( aux_coord );
    dist_comp_obj.SetCurrent( aux_coord );
    current = min_element( border_aux.begin(), border_aux.end(), dist_comp_obj );
    if ( current != last )
      {if( (aux_coord - *current ).Norm() > distance_tol )
	{adj_index++;
	adjacents.push_back( aux_vector );
	dist_comp_obj.Reset();}}}
};


template< class CoordType, class PixelType >
unsigned bicotiRegionProperty< CoordType, PixelType > :: GetPixelsInRegionNumber()
{
  if( ptr_region == NULL )
    throw bicotiException( "bicotiRegionProperty", "GetPixelsRegionsNumber()", 6 );

  return( ptr_region->GetPixelsNumber() );
};


template< class CoordType, class PixelType >
unsigned bicotiRegionProperty< CoordType, PixelType > :: GetPixelsInBorderNumber()
{
  return( border.size() );
};


template< class CoordType, class PixelType >
unsigned bicotiRegionProperty< CoordType, PixelType > :: GetDistinctsAdjacentsSetsNumber()
{
  return( adjacents.size() );
};


template< class CoordType, class PixelType >
vector< CoordType >& bicotiRegionProperty< CoordType, PixelType > :: GetBorder()
{
  return( border );
};


template< class CoordType, class PixelType >
vector< CoordType >& bicotiRegionProperty< CoordType, PixelType > :: GetAdjacents( unsigned adj_index )
{
  if( adjacents.size() <= adj_index ) 
    throw bicotiException( "bicotiRegionProperty", "GetAdjacents()", 1 );
  
  return( adjacents[ adj_index ] );
};


template< class CoordType, class PixelType >
bicotiCoordinate< REAL > bicotiRegionProperty< CoordType, PixelType > :: GetBaricenter( bicotiPixelWeight< PixelType > * ptr_pix_weight )
{
  bool delete_on = false;
  CoordType coord_acum;
  double pixel_weight_acum = 0;

  PixelType pixel_aux;
  unsigned index;
  
  if( ptr_pix_weight == NULL )
    {ptr_pix_weight = new bicotiPixelWeightNone< PixelType >;
    delete_on = true;}
  
  coord_acum *= 0;
  
  for( index = 0; index < ptr_region->GetPixelsNumber(); index ++ )
    {pixel_aux = ptr_region->GetPixel( index );
    pixel_weight_acum += ( *ptr_pix_weight )( pixel_aux );
    coord_acum += ( ptr_region->GetCoordinate( index ) * ( *ptr_pix_weight )( pixel_aux ) );}
  
  bicotiCoordinate< REAL > coord_result( coord_acum.GetDimension() );
  for( index = 0; index < coord_acum.GetDimension(); index++ )
    coord_result( index ) = (REAL)coord_acum( index );
  coord_result = coord_result * ( 1 / pixel_weight_acum );
  
  if( delete_on )
    delete ptr_pix_weight;
  
  return ( coord_result );
};              

#endif
