#ifndef _bicoti_homologous_qt_2D_image
#define _bicoti_homologous_qt_2D_image


#include <stdio.h>
#include <qwindow.h>
#include <qcursor.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qpainter.h>

#include <source/actor/actor_graphic_2D_image.hpp>
#include <source/event_loop_manager/qt/homologous_qt_caster.hpp>

//--------------------DECLARATIONS---------------------
//------------------bicotiQT2DImage--------------------
/** Implementacin QT del ImageIterator */
template< class PixelType, class PixelConverter >
class bicotiImageIterator2DGraphicQT;


/** Implementacin QT del bicotiHomologousGraphic2DImage */
template< class PixelType, class PixelConverter >
class bicotiQT2DImage : public QWindow, public bicotiHomologousGraphic2DImage< PixelType >
{
  friend class bicotiImageIterator2DGraphicQT< PixelType, PixelConverter >;

private:
  QPixmap pm;

  vector< int > x_coords;
  vector< int > y_coords;
  vector< bicotiPixelRGBA > pixels;

  bool alpha_channel_enabled;
  
  void ApplyAlpha();

public:
  
  /**
   * Constructor
   *
   *@param parent widget padre
   *@param name nombre del homlogo
   */
  bicotiQT2DImage( QWidget * parent=0, const char * name=0 );
 
  /** Destructor */
  ~bicotiQT2DImage();

  void DirectSetPixel( int, int, bicotiPixelRGBA );
  void ClearSettedPixels();
  void DirectRefresh();

  void RefreshImage();
  void RefreshImage( int, int, int, int );

  void EnableAlphaChannel( bool );
  bool IsAlphaChannelEnabled();

#include <source/event_loop_manager/qt/homologous_qt_graphic.hpp>
#include <source/event_loop_manager/qt/homologous_qt_graphic_2D.hpp>
#include <source/event_loop_manager/qt/homologous_qt_graphic2.hpp>

  void paintEvent( QPaintEvent * );
};



//-------------------IMPLEMENTATION--------------------
//-------------------bicotiQT2DImage-------------------
template< class PixelType, class PixelConverter >
bicotiQT2DImage< PixelType, PixelConverter > :: bicotiQT2DImage( QWidget * parent, const char * name ) : QWindow( parent, name )
{
  ptr_actor = NULL;
  setUpdatesEnabled( true );
  setMouseTracking( true );
}


template< class PixelType, class PixelConverter >
bicotiQT2DImage< PixelType, PixelConverter > :: ~bicotiQT2DImage()
{
  recreate( NULL, 0, QPoint( x(), y() ) , false );
}


template< class PixelType, class PixelConverter >
void bicotiQT2DImage< PixelType, PixelConverter > :: DirectSetPixel( int x_in, int y_in, bicotiPixelRGBA pixel )
{
  x_coords.push_back( x_in );
  y_coords.push_back( y_in );
  pixels.push_back( pixel );
}


template< class PixelType, class PixelConverter >
void bicotiQT2DImage< PixelType, PixelConverter > :: ClearSettedPixels()
{
  x_coords = vector< int >( 0 );
  y_coords = vector< int >( 0 );
  pixels = vector< bicotiPixelRGBA >( 0 );
}

template< class PixelType, class PixelConverter >
void bicotiQT2DImage< PixelType, PixelConverter > :: DirectRefresh()
{
  update();
}


template< class PixelType, class PixelConverter >
void bicotiQT2DImage< PixelType, PixelConverter > :: RefreshImage()
{
  QImage image;
  bicotiActorGraphic2DImage< PixelType > * ptr_act_aux = ((bicotiActorGraphic2DImage< PixelType >*)ptr_actor);
  
  image.create( ptr_act_aux->GetX0Size(), ptr_act_aux->GetX1Size(), 32 );
  image.setAlphaBuffer( true );
  
  int i, j;
  for( i = ptr_act_aux->GetX0Size() - 1; i >= 0; i-- )
    for( j = ptr_act_aux->GetX1Size() - 1; j >= 0; j-- )
      image.setPixel( i, j, PixelConverter :: Convert ( ptr_act_aux->GetPixel( i, j ) ) );
  /*
    if ( ptr_act_aux->IsCreated() )
    {for( i = w - 1; i >= 0; i-- )
    for( j = h - 1; j >= 0; j-- )
    image.setPixel( i, j, (bicotiRGBTools::GetAlpha(ptr_act_aux->GetPixel( i,j )) << 24) | (qRgb( bicotiRGBTools::GetRed(ptr_act_aux->GetPixel( i,j )), bicotiRGBTools::GetGreen(ptr_act_aux->GetPixel( i,j )) ,bicotiRGBTools::GetBlue(ptr_act_aux->GetPixel( i,j )))));}
    else
    {for( i = w - 1; i >= 0; i-- )
    for( j = h - 1; j >= 0; j-- )
    image.setPixel( i, j, 0xFF000000 | qRgb( 0, 0, 0 ) );}
  */

  if( IsAlphaChannelEnabled() )
    ApplyAlpha();
  
  pm.convertFromImage( image, DiffuseAlphaDither ); // Chequear si hay image

  QBitmap bm;
  bm = image.createAlphaMask( DiffuseAlphaDither );
  pm.setMask( bm );
  setBackgroundPixmap( pm );

  x_coords = vector< int >( 0 );
  y_coords = vector< int >( 0 );
  pixels = vector< bicotiPixelRGBA >( 0 );
}


template< class PixelType, class PixelConverter >
void bicotiQT2DImage< PixelType, PixelConverter > :: RefreshImage( int x, int y, int w, int h )
{
  QImage image;
  bicotiActorGraphic2DImage< PixelType > * ptr_act_aux = ((bicotiActorGraphic2DImage< PixelType >*)ptr_actor);
  
  image.create( w, h, 32 );
  image.setAlphaBuffer( true );
  
  int i, j;
  for( i = w - 1; i >= 0; i-- )
    for( j = h - 1; j >= 0; j-- )
      image.setPixel( i, j, PixelConverter :: Convert ( ptr_act_aux->GetPixel( x + i, y + j ) ) );
  /*
    if ( ptr_act_aux->IsCreated() )
    {for( i = w - 1; i >= 0; i-- )
    for( j = h - 1; j >= 0; j-- )
    image.setPixel( i, j, (bicotiRGBTools::GetAlpha(ptr_act_aux->GetPixel( i,j )) << 24) | (qRgb( bicotiRGBTools::GetRed(ptr_act_aux->GetPixel( i,j )), bicotiRGBTools::GetGreen(ptr_act_aux->GetPixel( i,j )) ,bicotiRGBTools::GetBlue(ptr_act_aux->GetPixel( i,j )))));}
    else
    {for( i = w - 1; i >= 0; i-- )
    for( j = h - 1; j >= 0; j-- )
    image.setPixel( i, j, 0xFF000000 | qRgb( 0, 0, 0 ) );}
  */
  
  if( IsAlphaChannelEnabled() )
    ApplyAlpha();
  
  pm.convertFromImage( image, DiffuseAlphaDither ); // Chequear si hay image

  QBitmap bm;
  bm = image.createAlphaMask( DiffuseAlphaDither );
  pm.setMask( bm );
  setBackgroundPixmap( pm );
  
  x_coords = vector< int >( 0 );
  y_coords = vector< int >( 0 );
  pixels = vector< bicotiPixelRGBA >( 0 );
}



template< class PixelType, class PixelConverter >
void bicotiQT2DImage< PixelType, PixelConverter > :: EnableAlphaChannel( bool enable )
{
  alpha_channel_enabled = enable;
}

template< class PixelType, class PixelConverter >
bool bicotiQT2DImage< PixelType, PixelConverter > :: IsAlphaChannelEnabled()
{
  return alpha_channel_enabled;
}

template< class PixelType, class PixelConverter >
void bicotiQT2DImage< PixelType, PixelConverter > :: ApplyAlpha()
{
  /*
  QWidget * parent = parentWidget();
  bicotiActorGraphic2DImage< PixelType > * ptr_act_aux = ((bicotiActorGraphic2DImage< PixelType >*)ptr_actor);
  if( parent != NULL )
    {const QPixmap * parentBackground = parent->backgroundPixmap();
    if (parentBackground != NULL) 
      {int w = width();
      int h = height();
      QImage parent_image = (parentBackground->convertToImage()).copy(x(), y(), width(), height());
      for( int i = w - 1; i >= 0; i-- )
	for( int j = h - 1; j >= 0; j-- ) {
	  float alpha = (bicotiRGBTools::GetAlpha(ptr_act_aux->GetPixel( i,j )))/255;
	  // printf("Alpha %d", bicotiRGBTools::GetAlpha(ptr_act_aux->GetPixel( i,j )));
	  float alpha_c = 1-alpha;
	  QRgb pixel1 = parent_image.pixel(i, j);
	  QRgb pixel2 = image.pixel(i, j);
	  QRgb new_pixel = qRgb(alpha_c*qRed(pixel1)+alpha*qRed(pixel2),alpha_c*qGreen(pixel1)+alpha*qGreen(pixel2),alpha_c*qBlue(pixel1)+alpha*qBlue(pixel2)); 
	  //	    image.setPixel( i, j, new_pixel);
	  image.setPixel( i, j, (bicotiRGBTools::GetAlpha(ptr_act_aux->GetPixel( i,j )) << 24) | new_pixel); 
	}
  */
      /*
      if ( ptr_act_aux->IsCreated() )
	{int w = width();
	int h = height();
	QImage parent_image = (parentBackground->convertToImage()).copy(x(), y(), width(), height());
	for( int i = w - 1; i >= 0; i-- )
	  for( int j = h - 1; j >= 0; j-- ) {
	    float alpha = (bicotiRGBTools::GetAlpha(ptr_act_aux->GetPixel( i,j )))/255;
	    // printf("Alpha %d", bicotiRGBTools::GetAlpha(ptr_act_aux->GetPixel( i,j )));
	    float alpha_c = 1-alpha;
	    QRgb pixel1 = parent_image.pixel(i, j);
	    QRgb pixel2 = image.pixel(i, j);
	    QRgb new_pixel = qRgb(alpha_c*qRed(pixel1)+alpha*qRed(pixel2),alpha_c*qGreen(pixel1)+alpha*qGreen(pixel2),alpha_c*qBlue(pixel1)+alpha*qBlue(pixel2)); 
	    //	    image.setPixel( i, j, new_pixel);
	    image.setPixel( i, j, (bicotiRGBTools::GetAlpha(ptr_act_aux->GetPixel( i,j )) << 24) | new_pixel); 
	  }
	}
      */
  /*
    }
    }
  */
}

#define bicotiLocalTemplate template< class PixelType, class PixelConverter >
#define bicotiLocalClass bicotiQT2DImage< PixelType, PixelConverter >
#include <source/event_loop_manager/qt/homologous_qt_graphic.cpp>
#include <source/event_loop_manager/qt/homologous_qt_graphic_2D.cpp>
#include <source/event_loop_manager/qt/homologous_qt_graphic2.cpp>


template< class PixelType, class PixelConverter >
void bicotiQT2DImage< PixelType, PixelConverter > :: paintEvent( QPaintEvent * pe )
{    
  QPainter p( this );
  QColor color;

  for( int i = x_coords.size() - 1; i >= 0; i-- )
    {color= QColor( bicotiRGBTools :: GetRed( pixels[ i ] ), bicotiRGBTools :: GetGreen( pixels[ i ] ), bicotiRGBTools :: GetBlue( pixels[ i ] ) );
    p.setPen( color );
    p.drawPoint( x_coords[ i ], y_coords[ i ] );}
}


#endif
