#include <source/event_loop_manager/qt/pixel_converter_qt.hpp>


const QRgb bicotiPixelConverterRGBA2QRGB :: Convert( const bicotiPixelRGBA& pix_bic )
{
  return ( qRgb( bicotiRGBTools :: GetRed( pix_bic ), bicotiRGBTools :: GetGreen( pix_bic ), bicotiRGBTools :: GetBlue( pix_bic ) ) );
}


const bicotiPixelRGBA bicotiPixelConverterRGBA2QRGB :: Unconvert( const QRgb& pix_qt )
{
  return ( bicotiRGBTools :: CreateRGBA( qRed( pix_qt ), qGreen( pix_qt ), qBlue( pix_qt ), pix_qt >> 24 ) );
}


const QRgb bicotiPixelConverterChar2QRGB :: Convert( const unsigned char pix_bic )
{
  return ( qRgb( pix_bic, pix_bic, pix_bic ) );
}


const unsigned char bicotiPixelConverterChar2QRGB :: Unconvert( const QRgb pix_qt )
{
  return ( (unsigned char)(0.299 * qRed( pix_qt ) + 0.587 * qGreen( pix_qt ) + 0.114 * qBlue( pix_qt ))  );
}



const QRgb bicotiPixelConverterDouble2QRGB :: Convert( const double pix_bic )
{
  return ( qRgb( pix_bic, pix_bic, pix_bic ) );
}


const double bicotiPixelConverterDouble2QRGB :: Unconvert( const QRgb pix_qt )
{
  return ( (double)(0.299 * qRed( pix_qt ) + 0.587 * qGreen( pix_qt ) + 0.114 * qBlue( pix_qt ))  );
}




const QRgb bicotiPixelConverterRGBChar2QRGB :: Convert( const bicotiRGB< unsigned char, REAL > pix_bic )
{
  return ( qRgb( pix_bic.Red(), pix_bic.Green(), pix_bic.Blue() ) );
}

const bicotiRGB< unsigned char, REAL > bicotiPixelConverterRGBChar2QRGB :: Unconvert( const QRgb pix_qt )
{
  return ( bicotiRGB< unsigned char, REAL >( qRed( pix_qt ), qGreen( pix_qt ), qBlue( pix_qt ) ) );
}



const QRgb bicotiPixelConverterRGBFloat2QRGB :: Convert( const bicotiRGB< float, REAL > pix_bic )
{
  return ( qRgb( pix_bic.Red(), pix_bic.Green(), pix_bic.Blue() ) );
}

const bicotiRGB< float, REAL > bicotiPixelConverterRGBFloat2QRGB :: Unconvert( const QRgb pix_qt )
{
  return ( bicotiRGB< float, REAL >( qRed( pix_qt ), qGreen( pix_qt ), qBlue( pix_qt ) ) );
}


unsigned int bicotiPixelConverterHSVInt2QRGB :: Maximum( unsigned int red, unsigned int green, unsigned int blue )
{
  if ( red > green )
    {if ( red > blue ) 
      return( red );
    else
      return( blue );}
  else
    {if ( green > blue )
      return( green );
    else
      return( blue );}
}


const QRgb bicotiPixelConverterHSVInt2QRGB :: Convert( const bicotiHSV< unsigned int , REAL >  pixel )
{
  unsigned int red, green, blue;

  if( pixel.Saturation() == 0 )
    {red = pixel.Value();
    green = pixel.Value();
    blue = pixel.Value();}
  else
    {REAL phi = (REAL)( pixel.Saturation() * cos( pi * pixel.Hue() / 180 ) );
    REAL psi = (REAL)( pixel.Saturation() * sin( pi * pixel.Hue() / 180 ) / sqrt(3) );
    
    if( ( 60 <= pixel.Hue() ) && ( pixel.Hue() <= 180 ) )
      {red = (unsigned int)(pixel.Value() + phi - psi );
      green = (unsigned int)pixel.Value();
      blue = (unsigned int)( pixel.Value() - 2 * psi );}
    else
      {if( ( 180 < pixel.Hue() ) && ( pixel.Hue() <= 300 ) )
	{red = (unsigned int)( pixel.Value() + phi + psi );
	green = (unsigned int)( pixel.Value() + 2 * psi );
	blue = (unsigned int)pixel.Value();}
      else
	{red = (unsigned int)pixel.Value();
	green = (unsigned int)( pixel.Value() + psi - phi );
	blue = (unsigned int)( pixel.Value() - phi - psi);}}}
  
  return( qRgb( red, green, blue ) );
}


const bicotiHSV< unsigned int, REAL > bicotiPixelConverterHSVInt2QRGB :: Unconvert( const QRgb pixel )
{
  bicotiHSV< unsigned int, REAL > pixel_hsv;
  pixel_hsv.Value() = Maximum( qRed( pixel ), qGreen( pixel ), qBlue( pixel ) );
  
  REAL phi = (REAL)( (REAL)(qGreen( pixel )) - (REAL)(qBlue( pixel )) );
  REAL psi = (REAL)( qGreen( pixel ) + qBlue( pixel ) );
  REAL lambda = (REAL)( (REAL)(qRed( pixel )) - (REAL)(.5 * psi) );

  pixel_hsv.Saturation() = (unsigned int)sqrt( lambda * lambda + .75 * phi * phi );
  
  if ( lambda > 0 )
    {if ( phi >= 0 )
      pixel_hsv.Hue() = (unsigned int)( 180.0 * atan( sqrt(3.0) * phi / ( 2.0 * lambda ) ) / pi );
    else
      pixel_hsv.Hue() = (unsigned int)( 360.0 + 180.0 * atan( sqrt(3.0) * phi / ( 2.0 * lambda ) ) / pi );} //MAL
  else
    {if ( lambda < 0 )
      pixel_hsv.Hue() = 180 + (unsigned int)( 180.0 * atan( sqrt(3.0) * phi / ( 2.0 * lambda ) ) / pi ); // MAL
    else
      {if ( phi > 0 )
	pixel_hsv.Hue() = 90;
      else
	{if ( phi < 0 )
	  pixel_hsv.Hue() = 270;
	else
	  pixel_hsv.Undefined() = true;}}}
  
  return( pixel_hsv );
}
