//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _frame_2D_internal
#define _frame_2D_internal


#include <source/frame/frame_2D.hpp>
 


//*************************DECALARATIONS**************************** 
//--------------------bicotiFrame2DInternal-------------------------
template< class PixelType > 
class bicotiFrame2DInternal : public bicotiFrame2D< PixelType > 
{ 
private: 
  PixelType ** ptr_pixels; 
 
public: 
  bicotiFrame2DInternal( PixelType ,int ); 
  bicotiFrame2DInternal( const bicotiFrame2DInternal< PixelType >& ); 
  ~bicotiFrame2DInternal(); 
  
  void First(); 
  void Next(); 
  
  void operator=( const bicotiFrame2DInternal< PixelType >& ); 
  PixelType& operator()( int, int ); 
}; 


 
 
//************************IMPLEMENTATIONS***************************
//----------------------bicotiFrame2DInternal-----------------------
template< class PixelType > 
bicotiFrame2DInternal< PixelType > :: bicotiFrame2DInternal ( PixelType default_pixel, int size) 
{ 
  if ( size < 0 ) 
    throw bicotiException( "bicotiFrame2DInternal", "Constructor", 4 );
  
  frame_size = size; 
  ptr_pixels = new PixelType * [ frame_size ]; 
  
  for( int i = 0; i < frame_size; i++ ) 
    {ptr_pixels[ i ] = new PixelType[ frame_size ]; 
    for( int j = 0; j < frame_size; j++ ) 
      ptr_pixels[ i ][ j ] = default_pixel;} 
  
  ptr_current = ptr_pixels[ 0 ]; 
  current_x1 = current_x2 = 0; 
  is_done = false; 
}; 
 
 
template< class PixelType > 
bicotiFrame2DInternal< PixelType > :: bicotiFrame2DInternal( const bicotiFrame2DInternal< PixelType >& frame_src ) 
{ 
  frame_size=frame_src.frame_size; 
  ptr_pixels = new PixelType* [ frame_size ]; 
  
  for( int i = 0; i < frame_size; i++ ) 
    {ptr_pixels[ i ] = new PixelType[ frame_size ]; 
    for( int j = 0; j < frame_size; j++ ) 
      ptr_pixels[ i ][ j ] = frame_src.ptr_pixels[ i ][ j ];} 
 
  current_x1 = frame_src.current_x1; 
  current_x2 = frame_src.current_x2; 
  ptr_current = ptr_pixels[ current_x2 ] + current_x1; 
  is_done = frame_src.is_done; 
}; 
 
 
template< class PixelType > 
bicotiFrame2DInternal< PixelType > :: ~bicotiFrame2DInternal() 
{ 
  for( int i = 0; i < frame_size; i++ ) 
    delete[] ptr_pixels[ i ]; 
  delete[] ptr_pixels; 
}; 
 
 
template< class PixelType > 
void bicotiFrame2DInternal< PixelType > :: First() 
{ 
  ptr_current = ptr_pixels[ 0 ]; 
  current_x1 = 0; 
  current_x2 = 0; 
  is_done = false; 
}; 
 
 
template< class PixelType > 
inline void bicotiFrame2DInternal< PixelType > :: Next() 
{ 
  if ( current_x1 < ( frame_size - 1 ) ) 
    {ptr_current++; 
    current_x1++;} 
  else 
    {if ( current_x2 < ( frame_size - 1 ) ) 
      {current_x2++; 
      current_x1 = 0; 
      ptr_current = ptr_pixels[ current_x2 ];} 
    else 
      is_done = true;} 
}; 


template< class PixelType > 
void bicotiFrame2DInternal< PixelType > :: operator=( const bicotiFrame2DInternal< PixelType >& frame_src ) 
{ 
  int i, j; 
  
  for( i = 0; i < frame_size; i++ ) 
    delete[] ptr_pixels[ i ]; 
  delete[] ptr_pixels; 
  
  for( i = 0; i < frame_size; i++ ) 
    {ptr_pixels[ i ] = new PixelType[ frame_size ]; 
    for( j = 0; j < frame_size; j++ ) 
      ptr_pixels[ i ][ j ]=frame_src.ptr_pixels[ i ][ j ];} 
   
  current_x1 = frame_src.current_x1; 
  current_x2 = frame_src.current_x2; 
  ptr_current = ptr_pixels[ current_x2 ] + current_x1; 
  is_done = frame_src.is_done; 
}; 
 
 
template< class PixelType > 
inline PixelType& bicotiFrame2DInternal< PixelType > :: operator()( int x1, int x2 ) 
{ 
#ifdef TOTAL_CHECK
  if ( ( x1 > ( frame_size - 1 ) ) || ( x2 > ( frame_size - 1 ) ) || ( x1 < 0 ) || ( x2 < 0 ) ) 
    throw bicotiException( "bicotiFrame2DInternal", "operator()", 1 );
#endif
  
  return( ptr_pixels[ x2 ][ x1 ] ); 
};          


#endif
