//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _frame_3D_external
#define _frame_3D_external



#include <source/frame/frame_3D.hpp>

 
 
//*************************DECALARATIONS****************************  
//--------------------bicotiFrame3DExternal-------------------------
template< class PixelType > 
class bicotiFrame3DExternal : public bicotiFrame3D< PixelType > 
{ 
private: 
  bicotiMatrix3D< PixelType > * ptr_matrix; 
  int first_x1, first_x2, first_x3; 
 
public: 
  bicotiFrame3DExternal( bicotiMatrix3D< PixelType > *, int ); 
  bicotiFrame3DExternal( const bicotiFrame3DExternal< PixelType >& ); 
   
  void First(); 
  void Next(); 
   
  void Locate( int, int, int ); 
   
  void operator=( const bicotiFrame3DExternal< PixelType >& ); 
  PixelType& operator()( int, int, int ); 
}; 
 
 
 
//************************IMPLEMENTATIONS***************************
//---------------------bicotiFrame3DExternal------------------------
template< class PixelType > 
bicotiFrame3DExternal< PixelType > :: bicotiFrame3DExternal( bicotiMatrix3D< PixelType > * ptr_mat, int size ) 
{ 
  frame_size = size; 
  ptr_matrix = ptr_mat; 
  
  if ( size < 0 ) 
    throw bicotiException( "bicotiFrame3DExternal", "Constructor", 4 );
  
  if ( ( size > ptr_matrix->GetX0Size() ) || ( size > ptr_matrix->GetX1Size() ) || ( size > ptr_matrix->GetX2Size() ) ) 
    throw bicotiException( "bicotiFrame3DExternal", "Constructor", 4 );
  
  ptr_current = (ptr_matrix->ptr_data)[ 0 ][ 0 ]; 
  current_x1 = current_x2 = current_x3 = first_x1 = first_x2 = first_x3 = 0; 
  is_done = false; 
}; 
 
 
template< class PixelType > 
bicotiFrame3DExternal< PixelType > :: bicotiFrame3DExternal( const bicotiFrame3DExternal< PixelType >& frame_src ) 
{ 
  frame_size = frame_src.frame_size; 
  ptr_matrix = frame_src.ptr_matrix; 
  current_x1 = frame_src.current_x1; 
  current_x2 = frame_src.current_x2; 
  current_x3 = frame_src.current_x3; 
  first_x1 = frame_src.first_x1; 
  first_x2 = frame_src.first_x2; 
  first_x3 = frame_src.first_x3; 
  ptr_current = frame_src.ptr_current; 
  is_done = frame_src.is_done; 
}; 
 

template< class PixelType > 
void bicotiFrame3DExternal< PixelType > :: First() 
{ 
  ptr_current = (ptr_matrix->ptr_data)[ first_x3 ][ first_x2 ] + first_x1; 
  current_x1 = 0; 
  current_x2 = 0; 
  current_x3 = 0; 
  is_done = false; 
}; 
 
 
template< class PixelType > 
inline void bicotiFrame3DExternal< PixelType > :: Next() 
{ 
  if ( current_x1 < ( frame_size - 1 ) ) 
    {ptr_current++; 
    current_x1++;} 
  else 
    {if ( current_x2 < ( frame_size - 1 ) ) 
      {current_x2++; 
      current_x1 = 0; 
      ptr_current = (ptr_matrix->ptr_data)[ first_x3 + current_x3 ][ first_x2 + current_x2 ] + first_x1;} 
    else 
      {if ( current_x3 < (frame_size -1) ) 
	{current_x3++; 
	current_x2 = 0; 
	current_x1 = 0; 
	ptr_current = (ptr_matrix->ptr_data)[ first_x3 + current_x3 ][ first_x2 ] + first_x1;} 
      else 
	is_done = true;}} 
}; 
 
 
template< class PixelType > 
inline void bicotiFrame3DExternal< PixelType > :: Locate( int x1, int x2, int x3 ) 
{ 
#ifdef TOTAL_CHECK
  if ( (x1 < 0) || (x2 < 0) || (x3 < 0) || ( ( x1 + frame_size ) > ptr_matrix->GetX0Size() ) || ( ( x2 + frame_size ) > ptr_matrix->GetX1Size() ) || ( ( x3 + frame_size ) > ptr_matrix->GetX2Size() ) ) 
    throw bicotiException( "bicotiFrame3DExternal", "Locate", 1 );
#endif
  
  first_x1 = x1; 
  first_x2 = x2; 
  first_x3 = x3; 
}; 
 
 
template< class PixelType > 
void bicotiFrame3DExternal< PixelType > :: operator=( const bicotiFrame3DExternal< PixelType >& frame_src )
{ 
  frame_size = frame_src.frame_size; 
  ptr_matrix = frame_src.ptr_matrix; 
  current_x1 = frame_src.current_x1; 
  current_x2 = frame_src.current_x2; 
  current_x3 = frame_src.current_x3; 
  first_x1 = frame_src.first_x1; 
  first_x2 = frame_src.first_x2; 
  first_x3 = frame_src.first_x3; 
  ptr_current = frame_src.ptr_current; 
  is_done = frame_src.is_done; 
}; 
 

template< class PixelType > 
inline PixelType& bicotiFrame3DExternal< PixelType > :: operator()( int x1, int x2, int x3 ) 
{ 
#ifdef TOTAL_CHECK
  if ( ( x1 > ( frame_size - 1 ) ) || ( x2 > ( frame_size - 1 ) ) || ( x3 > ( frame_size - 1 ) ) || ( x1 < 0 ) || ( x2 < 0 ) || ( x3 < 0 ) ) 
    throw bicotiException( "bicotiFrame3DExternal", "operator()", 1 );
#endif
  
  return( (ptr_matrix->ptr_data)[ first_x3 + x3 ][ first_x2 + x2 ][ first_x1 + x1 ] ); 
};     


#endif
