//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _frame_3D_internal
#define _frame_3D_internal



#include <source/frame/frame_3D.hpp>
 

 
//*************************DECLARATIONS****************************
//---------------------bicotiFrame3DInternal------------------------
template<class PixelType> 
class bicotiFrame3DInternal : public bicotiFrame3D<PixelType> 
{ 
private: 
  PixelType*** ptr_pixels; 
 
public: 		    
  bicotiFrame3DInternal(PixelType, int); 
  bicotiFrame3DInternal(const bicotiFrame3DInternal<PixelType>&); 
  ~bicotiFrame3DInternal(); 
 
  virtual void First(); 
  virtual void Next(); 
   
  void operator=(const bicotiFrame3DInternal<PixelType>&); 
  PixelType& operator()(int,int,int); 
}; 
 

 

//************************IMPLEMENTATIONS*************************** 
//----------------------bicotiFrame2DInternal-----------------------
template<class PixelType> 
bicotiFrame3DInternal<PixelType> :: bicotiFrame3DInternal(PixelType default_pixel, int size) 
{ 
  if ( size < 0 ) 
    throw bicotiException( "bicotiFrame3DInternal", "Constructor", 1 );
  
  frame_size = size; 
  ptr_pixels = new PixelType** [frame_size]; 
 
  int i,j,k; 
 
  for(i=0; i<frame_size; i++) 
    {ptr_pixels[i] = new PixelType* [frame_size]; 
    for(j=0; j<frame_size; j++) 
      {ptr_pixels[i][j] = new PixelType [frame_size]; 
      for(k=0; k<frame_size; k++) 
	ptr_pixels[i][j][k]=default_pixel;}} 
   
  ptr_current = ptr_pixels[0][0]; 
  current_x1 = current_x2 = current_x3 = 0; 
  is_done = false; 
}; 
 
 
template<class PixelType> 
bicotiFrame3DInternal<PixelType> :: bicotiFrame3DInternal(const bicotiFrame3DInternal<PixelType>& frame_src) 
{ 
  frame_size=frame_src.frame_size; 
  ptr_pixels = new PixelType** [frame_size]; 
 
  int i,j,k; 
 
  for(i=0; i<frame_size; i++) 
    {ptr_pixels[i] = new PixelType* [frame_size]; 
    for(j=0; j<frame_size; j++) 
      {ptr_pixels[i][j] = new PixelType [frame_size]; 
      for(k=0; k<frame_size; k++) 
	ptr_pixels[i][j][k]=frame_src.ptr_pixels[i][j][k];}} 
 
  current_x1 = frame_src.current_x1; 
  current_x2 = frame_src.current_x2; 
  current_x3 = frame_src.current_x3; 
  ptr_current = ptr_pixels[current_x3][current_x2]+current_x1; 
  is_done = frame_src.is_done; 
}; 
 
 
template<class PixelType> 
bicotiFrame3DInternal<PixelType> :: ~bicotiFrame3DInternal() 
{ 
  int i,j; 
   
  for(i=0;i<frame_size;i++) 
    {for(j=0;j<frame_size;j++) 
      delete[] ptr_pixels[i][j]; 
    delete[] ptr_pixels[i];} 
  delete[] ptr_pixels; 
   
}; 
 
 
template<class PixelType> 
void bicotiFrame3DInternal<PixelType> :: First() 
{ 
  ptr_current = ptr_pixels[0][0]; 
  current_x1 = 0; 
  current_x2 = 0; 
  current_x3 = 0; 
  is_done = false; 
}; 
 
 
template<class PixelType> 
inline void bicotiFrame3DInternal<PixelType> :: Next() 
{ 
  if ( current_x1 < (frame_size - 1) ) 
    {ptr_current++; 
    current_x1++;} 
  else 
    {if (current_x2 < (frame_size - 1) ) 
      {current_x2++; 
      current_x1 = 0; 
      ptr_current = ptr_pixels[current_x3][current_x2];} 
    else 
      {if (current_x3 < (frame_size -1) ) 
	{current_x3++; 
	current_x2 = 0; 
	current_x1 = 0; 
	ptr_current = ptr_pixels[current_x3][current_x2];} 
      else 
	is_done = true;}} 
}; 
 
 
template<class PixelType> 
void bicotiFrame3DInternal<PixelType> :: operator=(const bicotiFrame3DInternal<PixelType>& frame_src) 
{ 
  int i,j,k; 
   
  for(i=0;i<frame_size;i++) 
    {for(j=0;j<frame_size;j++) 
      delete[] ptr_pixels[i][j]; 
    delete[] ptr_pixels[i];} 
  delete[] ptr_pixels; 
   
  for(i=0; i<frame_size; i++) 
    for(j=0; j<frame_size; j++) 
      for(k=0; k<frame_size; k++) 
	ptr_pixels[i][j][k] = frame_src.ptr_pixels[i][j][k]; 
   
  current_x1 = frame_src.current_x1; 
  current_x2 = frame_src.current_x2; 
  current_x3 = frame_src.current_x3; 
  ptr_current = ptr_pixels[current_x3][current_x2]+current_x1; 
  is_done = frame_src.is_done; 
}; 
 
 
template<class PixelType> 
inline PixelType& bicotiFrame3DInternal<PixelType> :: operator()(int x1,int x2,int x3) 
{ 
#ifdef TOTAL_CHECK
  if ( (x1 > (frame_size-1)) || (x2 > (frame_size-1)) || (x3 > (frame_size-1))|| (x1 < 0) || (x2 <0) || (x3 <0) ) 
    throw bicotiException( "bicotiFrame3DInternal", "operator()", 1 );
#endif
  
  return(ptr_pixels[x3][x2][x1]); 
};        


#endif
