//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _matrix_2D
#define _matrix_2D



#include <source/frame/matrix_basic.hpp>



//*******************************DECLARATIONS**********************************
//-------------------------------bicotimatrix----------------------------------
template<class PixelType> 
class bicotiFrame2DExternal;


template< class PixelType > 
class bicotiMatrixIterator2D;


template< class PixelType >
class bicotiMatrix2D : public bicotiMatrix< PixelType >
{
  friend bicotiFrame2DExternal< PixelType >;
  friend bicotiMatrixIterator2D< PixelType >;
  
private:
  PixelType ** ptr_data;
  unsigned x0_size , x1_size;
  
public:
  bicotiMatrix2D( PixelType, unsigned = 0, unsigned = 0 );
  bicotiMatrix2D();
  ~bicotiMatrix2D();
  bicotiMatrix2D( const bicotiMatrix2D< PixelType >& );
  
  void operator=( const bicotiMatrix2D< PixelType >& );
  
  bicotiMatrixIterator< PixelType > * CreateIterator();
  
  PixelType& operator() ( unsigned, unsigned );
  bicotiMatrix2D< PixelType > operator*( bicotiMatrix2D< PixelType > );
  
  unsigned GetX0Size();
  unsigned GetX1Size();
  
  PixelType Determinant();
  bicotiMatrix2D< PixelType > Inverse( PixelType );
};



//--------------------------bicotiMatrixIterator2D-----------------------------
template< class PixelType >
class bicotiMatrixIterator2D : public bicotiMatrixIterator< PixelType >
{
private:
  unsigned current_x0, current_x1;
  bicotiMatrix2D< PixelType > * ptr_matrix;
  
public:
  bicotiMatrixIterator2D( bicotiMatrix2D< PixelType >* );
  
  void First();
  void Next();
  PixelType& CurrentPixel();
  bicotiCoordinate< INTEGER > CurrentCoordinate();
};




//*****************************IMPLEMENTATIONS*********************************
//-------------------------------bicotimatrix----------------------------------
template< class PixelType >
bicotiMatrix2D< PixelType > :: bicotiMatrix2D( PixelType def_pix, unsigned x0_s, unsigned x1_s )
{
  unsigned i,j;
  
  x0_size = x0_s;
  x1_size = x1_s;
  
  ptr_data = new PixelType* [ x1_size ];
  for(i = 0; i < x1_size; i++)
    {ptr_data[ i ] = new PixelType [ x0_size ];
    for(j=0; j < x0_size; j++)
      ptr_data[ i ][ j ] = def_pix;}
};


template< class PixelType >
bicotiMatrix2D< PixelType > :: bicotiMatrix2D()
{
  x0_size = 0;
  x1_size = 0;
};


template< class PixelType >
bicotiMatrix2D< PixelType > :: bicotiMatrix2D( const bicotiMatrix2D< PixelType >& matrix_src )
{
  unsigned i,j;

  x0_size = matrix_src.x0_size;
  x1_size = matrix_src.x1_size;

  ptr_data = new PixelType* [ x1_size ];
  for(i = 0; i < x1_size; i++)
    {ptr_data[ i ] = new PixelType [ x0_size ];
    for(j = 0; j < x0_size; j++)
      ptr_data[ i ][ j ] = matrix_src.ptr_data[ i ][ j ];}  
};


template< class PixelType >
bicotiMatrix2D< PixelType > :: ~bicotiMatrix2D()
{
  if ( ( x0_size != 0 ) && ( x1_size != 0) )
    {for(unsigned i = 0; i < x1_size; i++ )
      {delete [] ptr_data[ i ];}
    delete [] ptr_data;}
};


template< class PixelType >
void bicotiMatrix2D< PixelType > :: operator= ( const bicotiMatrix2D< PixelType >& matrix_src )
{
  unsigned i,j;

  if ( ( x0_size != 0 ) && ( x1_size != 0) )
    {for( i = 0; i < x1_size; i++ )
      delete [] ptr_data[ i ];
    delete [] ptr_data;}
  
  x0_size = matrix_src.x0_size;
  x1_size = matrix_src.x1_size;

  ptr_data = new PixelType* [ x1_size ];
  for(i = 0; i < x1_size; i++ )
    {ptr_data[ i ] = new PixelType [ x0_size ];
    for( j = 0; j < x0_size; j++ )
      ptr_data[ i ][ j ] = matrix_src.ptr_data[ i ][ j ];}  
};


template< class PixelType >
bicotiMatrixIterator< PixelType > * bicotiMatrix2D< PixelType > :: CreateIterator()
{
  return( new bicotiMatrixIterator2D< PixelType >( this ) );
};


template< class PixelType >
inline PixelType& bicotiMatrix2D< PixelType > :: operator() ( unsigned x0, unsigned x1 )
{
  return( ptr_data[ x1 ][ x0 ] );
};


template< class PixelType >
bicotiMatrix2D< PixelType > bicotiMatrix2D< PixelType > :: operator* ( bicotiMatrix2D< PixelType > matrix_src )
{
  bicotiMatrix2D<PixelType> output_matrix( matrix_src(0,0), matrix_src.GetX0Size(), x1_size );

  if ( x0_size != matrix_src.GetX1Size() )
    throw bicotiException( "bicotiMatrix2D", "operator*", 2 );
  
  unsigned x0, x1, index;
  
  for( x0 = 0; x0 < matrix_src.GetX0Size(); x0++ )
    for( x1 = 0; x1 < x1_size; x1++ )
      {output_matrix( x0, x1 ) = ptr_data[ x0 ][ 0 ] * matrix_src( x1, 0 );
      for ( index = 1 ; index < x0_size ; index ++ )
	output_matrix( x0 , x1 )+= ptr_data[ x0 ][ index ] * matrix_src( x1 , index );}
  
  return ( output_matrix );
};


template< class PixelType >
unsigned bicotiMatrix2D< PixelType > :: GetX0Size()
{
  return( x0_size );
};


template< class PixelType >
unsigned bicotiMatrix2D< PixelType > :: GetX1Size()
{
  return( x1_size );
};


template< class PixelType >
PixelType bicotiMatrix2D< PixelType > :: Determinant()
{
  return ( Ext_Determinant( this ) );
};


template< class PixelType >
bicotiMatrix2D< PixelType > bicotiMatrix2D< PixelType >  :: Inverse( PixelType zero )
{
  unsigned x0, x1, x0_aux, x1_aux, size;
  PixelType pixel_aux, determinant;
  
  size = x0_size;
  
  bicotiMatrix2D< PixelType > output_matrix( zero, size, size );

  if( x0_size != x1_size )  
    throw bicotiException( "bicotiMatrix2D", "Inverse", 6 );
  
  bicotiMatrix2D< PixelType > aux_matrix( zero, size - 1, size - 1);
  
  determinant = Determinant();
  if ( determinant == zero )
    throw bicotiException( "bicotiMatrix2D", "Inverse", 6 );
  
  for( x0 = 0; x0 < size; x0++ )
    for( x1 = 0; x1 < size; x1++ )
      {for( x0_aux = 0; x0_aux < x0; x0_aux++)
	{for( x1_aux = 0; x1_aux < x1; x1_aux++ )
	  aux_matrix( x0_aux, x1_aux ) = ptr_data[ x1_aux ][ x0_aux ];
	for( x1_aux = x1; x1_aux < size -1; x1_aux++ )
	  aux_matrix( x0_aux, x1_aux ) = ptr_data[ x1_aux +1 ][ x0_aux ];}
      
      for( x0_aux = x0; x0_aux < size - 1; x0_aux++ )
	{for( x1_aux = 0; x1_aux < x1; x1_aux++ )
	  aux_matrix( x0_aux, x1_aux ) = ptr_data[ x1_aux ][ x0_aux + 1];
	for( x1_aux = x1; x1_aux < size - 1; x1_aux++ )
	  aux_matrix( x0_aux, x1_aux ) = ptr_data[ x1_aux + 1 ][ x0_aux + 1 ];}
      
      if ( ( x0 + x1 )%2 == 0 )
	output_matrix( x1, x0 ) = (PixelType)(aux_matrix.Determinant()/determinant);
      else
	output_matrix( x1, x0 ) = (PixelType)((-1)* (aux_matrix.Determinant()/determinant));}
  
  return ( output_matrix );
};



//--------------------------bicotiMatrixIterator2D-----------------------------
template< class PixelType >
bicotiMatrixIterator2D< PixelType > :: bicotiMatrixIterator2D( bicotiMatrix2D< PixelType > * ptr_matrix_src )
{
  ptr_matrix = ptr_matrix_src;
};


template< class PixelType >
void bicotiMatrixIterator2D< PixelType > :: First()
{
  current_x0 = 0;
  current_x1 = 0;
  is_done = false;
};


template< class PixelType >
void bicotiMatrixIterator2D< PixelType > :: Next()
{
  if ( current_x0 < ( ptr_matrix->x0_size - 1 ) )
    current_x0++;
  else
    {if ( current_x1 < ( ptr_matrix->x1_size - 1 ) )
      {current_x0 = 0;
      current_x1++;}
    else
      is_done = true;}
};


template< class PixelType >
PixelType& bicotiMatrixIterator2D< PixelType > :: CurrentPixel()
{
  return( ptr_matrix->ptr_data[ current_x1 ][ current_x0 ] );
};


template< class PixelType >
bicotiCoordinate< INTEGER > bicotiMatrixIterator2D< PixelType > :: CurrentCoordinate()
{
  bicotiCoordinate< INTEGER > coord_aux( 2 );
  coord_aux( 0 ) = (INTEGER)current_x0;
  coord_aux( 1 ) = (INTEGER)current_x1;
  
  return( coord_aux );
};




//-----------------------EXTERNAL FUNCTIONS---------------------------
template< class PixelType , class CoordType >
inline CoordType operator*( bicotiMatrix2D< PixelType >&  matrix, CoordType& coordinate )
{
  CoordType aux_coord;
  unsigned row;
  unsigned column;
  
  if ( ( matrix.GetX0Size() != matrix.GetX1Size() ) || ( matrix.GetX0Size() != coordinate.GetDimension() ) )
    throw bicotiException( "bicotiMatrix2D", "operator*", 2 );
  
  for( row = 0; row < coordinate.GetDimension(); row++ )
    {for( column = 0; column < coordinate.GetDimension(); column++ )
      aux_coord( row ) += coordinate( column ) * matrix( column,row );}
  
  return ( aux_coord );
};


template< class PixelType >
PixelType Ext_Determinant( bicotiMatrix2D< PixelType >* ptr_matrix )
{
  unsigned column;
  unsigned size;
  unsigned index_1, index_2;
  PixelType aux_pixel;
  bicotiMatrix2D< PixelType > * ptr_sub_mat;
  
  if ( ptr_matrix->GetX0Size() != ptr_matrix->GetX1Size() )
    throw bicotiException( "bicotiMatrix2D", "Determinant", 2 );
  
  size = ptr_matrix->GetX0Size();
  
  if ( size == 1 )
    aux_pixel = ( *ptr_matrix )( 0, 0 );
  else
    {ptr_sub_mat = new bicotiMatrix2D< PixelType >( ( *ptr_matrix )( 0, 0 ), size - 1 , size - 1 );
    
    for( index_1 = 0; index_1 < size - 1; index_1++ )
      for( index_2 = 0; index_2 < size - 1; index_2++ )
	( *ptr_sub_mat )( index_1, index_2 ) = ( *ptr_matrix )( index_1 + 1, index_2 + 1 );
    
    aux_pixel = ( ( *ptr_matrix )( 0, 0 ) )*Ext_Determinant( ptr_sub_mat );
    
    for ( column = 1; column < size; column++ )
      {for( index_1 = 0; index_1 < column; index_1++ )
	for( index_2 = 0; index_2 < size - 1; index_2++)
	  ( *ptr_sub_mat )( index_1, index_2 ) = ( *ptr_matrix )( index_1, index_2 + 1 );
      
      for( index_1 = column; index_1 < size - 1; index_1++)
	for( index_2 = 0; index_2 < size - 1; index_2++ )
	  ( *ptr_sub_mat )( index_1, index_2 ) = ( *ptr_matrix )( index_1 + 1, index_2 + 1 );
      
      if ( column%2 == 0)
	aux_pixel += ( ( *ptr_matrix )( column, 0 ) )*Ext_Determinant( ptr_sub_mat );
      else
	aux_pixel -= ( ( *ptr_matrix )( column, 0 ) )*Ext_Determinant( ptr_sub_mat );}
    
    delete ptr_sub_mat;}
  
  return ( aux_pixel );
};               



#endif
