//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _matrix_3D
#define _matrix_3D



#include <source/frame/matrix_basic.hpp>



//*******************************DECLARATIONS**********************************
//-------------------------------bicotimatrix----------------------------------
template<class Pixel> 
class bicotiFrame3DExternal;


template< class PixelType > 
class bicotiMatrixIterator3D;


template< class PixelType >
class bicotiMatrix3D : public bicotiMatrix< PixelType >
{
  friend bicotiFrame3DExternal< PixelType >;
  friend bicotiMatrixIterator3D< PixelType >;

private:
  PixelType *** ptr_data;
  unsigned x0_size, x1_size, x2_size;
  
public:
  bicotiMatrix3D( PixelType , unsigned = 0, unsigned = 0, unsigned = 0 );
  bicotiMatrix3D( const bicotiMatrix3D< PixelType >& );
  ~bicotiMatrix3D();

  void operator=( const bicotiMatrix3D< PixelType >& );

  bicotiMatrixIterator< PixelType > * CreateIterator();

  PixelType& operator() ( unsigned, unsigned, unsigned );
  
  unsigned GetX0Size();
  unsigned GetX1Size();
  unsigned GetX2Size();
};



//--------------------------bicotiMatrixIterator2D-----------------------------
template< class PixelType >
class bicotiMatrixIterator3D : public bicotiMatrixIterator< PixelType >
{
private:
  unsigned current_x0, current_x1, current_x2;
  bicotiMatrix3D< PixelType > * ptr_matrix;
  
public:
  bicotiMatrixIterator3D( bicotiMatrix3D< PixelType >* );
  
  void First();
  void Next();
  PixelType& CurrentPixel();
  bicotiCoordinate< INTEGER > CurrentCoordinate();
};




//*****************************IMPLEMENTATIONS*********************************
//-------------------------------bicotimatrix----------------------------------
template< class PixelType >
bicotiMatrix3D< PixelType > :: bicotiMatrix3D(PixelType def_pix, unsigned x0_s, unsigned x1_s, unsigned x2_s)
{
  unsigned i,j,k;

  x0_size = x0_s;
  x1_size = x1_s;
  x2_size = x2_s;
  
  ptr_data = new PixelType** [x2_size];
  for(i=0; i<x2_size; i++)
    {ptr_data[i] = new PixelType* [x1_size];
    for(j=0; j<x1_size; j++)
      {ptr_data[i][j] = new PixelType[x0_size];
      for(k=0; k<x0_size; k++)
	ptr_data[i][j][k] = def_pix;}}
};


template< class PixelType >
bicotiMatrix3D< PixelType > :: bicotiMatrix3D(const bicotiMatrix3D< PixelType >& matrix_src)
{
  unsigned i,j;

  x0_size = matrix_src.x0_size;
  x1_size = matrix_src.x1_size;
  x2_size = matrix_src.x2_size;

  ptr_data = new PixelType** [x2_size];
  for(i=0; i<x2_size; i++)
    {ptr_data[i] = new PixelType* [x1_size];
    for(j=0; j<x1_size; j++)
      {ptr_data[i][j] = new PixelType[x0_size];
      for(k=0; k<x0_size; k++)
	ptr_data[i][j][k] = matrix_src.ptr_data[i][j][k];}}
};


template< class PixelType >
bicotiMatrix3D< PixelType > :: ~bicotiMatrix3D()
{
  unsigned i,j;

  for(i=0; i<x2_size; i++)
    {for(j=0; j<x1_size; j++)
      delete [] ptr_data[i][j];
    delete [] ptr_data[i];}
  delete [] ptr_data;
};


template< class PixelType >
void bicotiMatrix3D< PixelType > :: operator= (const bicotiMatrix3D< PixelType >& matrix_src)
{
  unsigned i,j;

  for(i=0; i<x2_size; i++)
    {for(j=0; j<x1_size; j++)
      delete [] ptr_data[i][j];
    delete [] ptr_data[i];}
  delete [] ptr_data;

  x0_size = matrix_src.x0_size;
  x1_size = matrix_src.x1_size;
  x2_size = matrix_src.x2_size;

  ptr_data = new PixelType** [x2_size];
  for(i=0; i<x0_size; i++)
    {ptr_data[i] = new PixelType* [x1_size];
    for(j=0; j<x1_size; j++)
      {ptr_data[i][j] = new PixelType[x0_size];
      for(k=0; k<x2_size; k++)
	ptr_data[i][j][k] = matrix_src.ptr_data[i][j][k];}}
};


template< class PixelType >
bicotiMatrixIterator< PixelType > * bicotiMatrix3D< PixelType > :: CreateIterator()
{
  return( new bicotiMatrixIterator3D< PixelType >( this ) );
};


template< class PixelType >
PixelType& bicotiMatrix3D< PixelType > :: operator() (unsigned x0, unsigned x1, unsigned x2)
{
  return(ptr_data[x2][x1][x0]);
};


template< class PixelType >
unsigned bicotiMatrix3D< PixelType > :: GetX0Size()
{
  return(x0_size);
};


template< class PixelType >
unsigned bicotiMatrix3D< PixelType > :: GetX1Size()
{
  return(x1_size);
};


template< class PixelType >
unsigned bicotiMatrix3D< PixelType > :: GetX2Size()
{
  return(x2_size);
};



//--------------------------bicotiMatrixIterator3D-----------------------------
template< class PixelType >
bicotiMatrixIterator3D< PixelType > :: bicotiMatrixIterator3D( bicotiMatrix3D< PixelType > * ptr_matrix_src )
{
  ptr_matrix = ptr_matrix_src;
};


template< class PixelType >
void bicotiMatrixIterator3D< PixelType > :: First()
{
  current_x0 = 0;
  current_x1 = 0;
  current_x2 = 0;
  is_done = false;
};


template< class PixelType >
void bicotiMatrixIterator3D< PixelType > :: Next()
{
  if ( current_x0 < ( ptr_matrix->x0_size - 1 ) )
    current_x0++;
  else
    {if ( current_x1 < ( ptr_matrix->x1_size - 1 ) )
      {current_x0 = 0;
      current_x1++;}
    else
      {if ( current_x2 < ( ptr_matrix->x2_size - 1 ) )	
	{current_x0 = 0;
	current_x1 = 0;
	current_x2++;}
      else
	is_done = true;}}
};


template< class PixelType >
PixelType& bicotiMatrixIterator3D< PixelType > :: CurrentPixel()
{
  return( ptr_matrix->ptr_data[current_x2][current_x1][current_x0] );
};


template< class PixelType >
bicotiCoordinate< INTEGER > bicotiMatrixIterator3D< PixelType > :: CurrentCoordinate()
{
  bicotiCoordinate< INTEGER > coord_aux( 3 );
  coord_aux( 0 ) = (INTEGER)current_x0;
  coord_aux( 1 ) = (INTEGER)current_x1;
  coord_aux( 2 ) = (INTEGER)current_x2;
  
  return( coord_aux );
};       


#endif
