//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _cutter
#define _cutter


#include <source/geometrical/image_subspace.hpp>


//--------------------------DECLARACIONES------------------------- 
//*******************DECLARACIONES DE bicotiCutter********************** 
template < class PixelType , class CoordType > 
class bicotiCutter 
{ 
private:
  bicotiImageImplementation< PixelType >* ptr_image_source; 
  bicotiImageImplementation< PixelType >* ptr_image_dest; 
  bicotiImageSubspace< CoordType >* ptr_subspace; 
  void CheckParameters();

public: 
  bicotiCutter(bicotiImageImplementation< PixelType >*,bicotiImageImplementation< PixelType >*,bicotiImageSubspace< CoordType >*);
  bicotiCutter();
  
  void SetImagesImplementations( bicotiImageImplementation< PixelType > *, bicotiImageImplementation< PixelType > * );
  void SetImageSubspace( bicotiImageSubspace< CoordType >* );
  void operator() ( bicotiImagePixelInterpolator< PixelType > *); 
}; 

//------------------------IMPLEMENTACIONES------------------------- 
//*******************IMPLEMENTACION DE CUTTER********************** 

template< class PixelType , class CoordType > 
bicotiCutter< PixelType , CoordType > :: bicotiCutter(bicotiImageImplementation< PixelType >* ptr_dest,bicotiImageImplementation< PixelType >* ptr_source , bicotiImageSubspace< CoordType >* ptr_subs) 
{ 
  ptr_image_source = ptr_source; 
  ptr_image_dest = ptr_dest; 
  ptr_subspace = ptr_subs; 
} 


template< class PixelType , class CoordType >
bicotiCutter< PixelType , CoordType > :: bicotiCutter()
{
  ptr_image_source = NULL; 
  ptr_image_dest = NULL; 
  ptr_subspace = NULL; 
}


template< class PixelType , class CoordType > 
void bicotiCutter< PixelType , CoordType > :: operator()( bicotiImagePixelInterpolator< PixelType > * ptr_interpolator ) 
{ 
  CheckParameters();//Cheaquea los parametros y redimensiona la imagen salida.
  bicotiImageIterator< PixelType >* ptr_iterator_dest; 
  ptr_iterator_dest = ptr_image_dest->CreateIterator();
  while( !ptr_subspace->IsDone() ) 
    {**ptr_iterator_dest = (*ptr_interpolator)( ptr_subspace->GetCurrentCoordinate() );
    (*ptr_iterator_dest)++; 
    ptr_subspace->Next();} 
  delete ptr_iterator_dest; 
}                          


template< class PixelType , class CoordType >
void  bicotiCutter< PixelType , CoordType > :: CheckParameters()
{
   if( ( ptr_image_source != NULL ) && ( ptr_image_dest != NULL ) && ( ptr_subspace != NULL ) ) 
     {bicotiImageDescriptor *ptr_descript_image_source, *ptr_descript_image_dest;
     ptr_descript_image_source = ptr_image_source->CreateImageDescriptor();
     ptr_descript_image_dest = ptr_image_dest->CreateImageDescriptor();
     CoordType dummy;

     if ( ( dummy.GetDimension() != ptr_descript_image_source->GetDimension() ) || 
	  ( ptr_subspace->GetDimension() != ptr_descript_image_dest->GetDimension() ) )
       {delete ptr_descript_image_dest;
       delete ptr_descript_image_source;
       throw bicotiException( "bicotiCutter", "CheckParameters()", 2 );}
     else
       {unsigned i;
       for( i = 0; i < ptr_subspace->GetDimension() ; i++ )
	 ptr_descript_image_dest->Size( i ) = ptr_subspace->GetSize( i );
       ptr_image_dest->Resize( ptr_descript_image_dest );
       delete ptr_descript_image_source;
       delete ptr_descript_image_dest;}}
   else
     throw bicotiException( "bicotiCutter", "CheckParameters()", 6 );
}


template< class PixelType , class CoordType >
void bicotiCutter< PixelType , CoordType > :: SetImagesImplementations( bicotiImageImplementation< PixelType >* ptr_dest , bicotiImageImplementation< PixelType >* ptr_source )
{
  ptr_image_source = ptr_source; 
  ptr_image_dest = ptr_dest; 
}


template< class PixelType , class CoordType >
void bicotiCutter< PixelType , CoordType > :: SetImageSubspace( bicotiImageSubspace< CoordType >* ptr_subs )
{
  ptr_subspace = ptr_subs; 
}


#endif
