//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _image_2D
#define _image_2D



#include <source/image/image_basic.hpp>
#include <source/mapper/mapper.hpp>





//**********************DECLARATIONS**************************
template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType = ColorType, class PrecisionType = PixelType  >
class bicotiImage2D : public bicotiImage< ImageType, PixelType >
{
protected:
  bicotiPropertyFacade< bicotiCoordinate2D< INTEGER >, ColorType, PixelType, PrecisionType > * ptr_property;
  bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > * ptr_neighbour_filter;
  bicotiGeometricalFacade2D< PixelType, PrecisionType > * ptr_geometrical;
  bicotiDexelFacade2D< PixelType > * ptr_dexel; 

  bicotiMapperFacade2D< PixelType , PixelConverterMapper > * ptr_mapper;
    
  void DeleteMe();
  void ResetPointers();

public:
  typedef bicotiCoordinate2D< INTEGER > IntegerCoordinate;
  typedef bicotiCoordinate2D< REAL > RealCoordinate;
  typedef bicotiPropertyFacade< bicotiCoordinate2D< INTEGER >, ColorType, PixelType, PrecisionType > Property;
  typedef bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > NeighbourFilter; 
  typedef bicotiGeometricalFacade2D< PixelType, PrecisionType > Geometrical;
  typedef bicotiDexelFacade2D< PixelType > Dexel; 

  typedef bicotiMapperFacade2D< PixelType, PixelConverterMapper > Mapper;

  bicotiPropertyFacade< bicotiCoordinate2D< INTEGER >, ColorType, PixelType, PrecisionType > * GetProperty();
  void DestroyProperty();
  
  void BuildOperatorUnary();
  bicotiOperatorUnaryFacade< PixelType > * CreateOperatorUnary();
  
  void BuildOperatorMultiple();
  bicotiOperatorMultipleFacade< PixelType > * CreateOperatorMultiple();

  void BuildGeometrical();
  void DestroyGeometrical();
  bicotiGeometricalFacade2D< PixelType, PrecisionType > * GetGeometrical();
  ImageType * GetGeometricalResult();
  void LoadGeometricalResult();
  
  bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > * GetNeighbourFilter();
  void DestroyNeighbourFilter();
  void LoadNeighbourFilterResult();
  ImageType * GetNeighbourFilterResult();

  bicotiDexelFacade2D< PixelType > * GetDexel();
  bool IsDexelCreated();
  void DestroyDexel();
  
  virtual void BuildImplementation( unsigned, unsigned )=0;

  virtual void BuildMapper() = 0;
  bicotiMapperFacade2D< PixelType , PixelConverterMapper > * GetMapper();
  void DestroyMapper();
};


//********************IMPLEMENTATIONS*************************
template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: DeleteMe()
{
  if ( ptr_implementation != NULL)
    delete ptr_implementation;
  
  if( ptr_property != NULL)
    delete ptr_property;

  if( ptr_operator_unary != NULL)
    delete ptr_operator_unary;

  if( ptr_operator_multiple != NULL)
    delete ptr_operator_multiple;
  
  if ( ptr_neighbour_filter != NULL)
    delete ptr_neighbour_filter;
  
  if ( ptr_geometrical != NULL )
    delete ptr_geometrical;
  
  if ( ptr_dexel != NULL )
    delete ptr_dexel;

  if ( ptr_mapper != NULL )
    delete ptr_mapper;

  ResetPointers();
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: ResetPointers()
{
  ptr_implementation = NULL;
  ptr_property = NULL;
  ptr_operator_unary = NULL;
  ptr_operator_multiple = NULL;
  ptr_neighbour_filter = NULL;
  ptr_geometrical = NULL;
  ptr_dexel = NULL;
  ptr_mapper = NULL;
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
bicotiPropertyFacade< bicotiCoordinate2D< INTEGER >, ColorType, PixelType, PrecisionType > * bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: GetProperty()
{
  return( ptr_property );
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: DestroyProperty()
{
  if(ptr_property != NULL)
    {delete ptr_property;
    ptr_property = NULL;}
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: BuildOperatorUnary()
{
  if(ptr_operator_unary != NULL)
    delete ptr_operator_unary;
  
  ptr_operator_unary = new bicotiOperatorUnaryFacade< PixelType >( ptr_implementation );
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
bicotiOperatorUnaryFacade< PixelType >* bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: CreateOperatorUnary()
{
  return( new bicotiOperatorUnaryFacade< PixelType >( ptr_implementation ) );
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: BuildOperatorMultiple()
{
  if(ptr_operator_multiple != NULL)
    delete ptr_operator_multiple;
  
  ptr_operator_multiple = new bicotiOperatorMultipleFacade< PixelType >( ptr_implementation );
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
bicotiOperatorMultipleFacade< PixelType >* bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: CreateOperatorMultiple()
{
  return( new bicotiOperatorMultipleFacade< PixelType >( ptr_implementation ) );
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: BuildGeometrical()
{
  if( ptr_geometrical != NULL )
    delete ptr_geometrical;
  
  ptr_geometrical = new bicotiGeometricalFacade2D< PixelType, PrecisionType > ( ptr_implementation );
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: DestroyGeometrical()
{
  if( ptr_geometrical != NULL )
    {delete ptr_geometrical;
    ptr_geometrical = NULL;}
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
bicotiGeometricalFacade2D< PixelType, PrecisionType > * bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: GetGeometrical()
{
 if( ptr_geometrical == NULL )
    throw bicotiException( "bicotiImage2D", "GetGeometrical()", 5 );

  return( ptr_geometrical );
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
ImageType * bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: GetGeometricalResult()
{ 
  if( ptr_geometrical == NULL )
    throw bicotiException( "bicotiImage2D", "GetGeometricalResult()", 5 );

  bicotiImageImplementation< PixelType > * ptr_aux = ptr_geometrical->GetResult();

  return( ( ImageType * ) ptr_aux );
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: LoadGeometricalResult()
{
  if( ptr_geometrical != NULL )
    {delete ptr_implementation;
    ptr_implementation = (ImageType*)ptr_geometrical->LoadResult();}
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > * bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: GetNeighbourFilter()
{
 if( ptr_neighbour_filter == NULL )
    throw bicotiException( "bicotiImage2D", "GetNeighbourFilter()", 5 );

  return( ptr_neighbour_filter );
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: DestroyNeighbourFilter()
{
  if( ptr_neighbour_filter != NULL )
    {delete ptr_neighbour_filter;
    ptr_neighbour_filter = NULL;}
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: LoadNeighbourFilterResult()
{
  if( ptr_neighbour_filter != NULL )
    {delete ptr_implementation;
    ptr_implementation = (ImageType*)ptr_neighbour_filter->LoadResult();}
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
ImageType * bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: GetNeighbourFilterResult()
{
  if(ptr_neighbour_filter == NULL)
    throw bicotiException( "bicotiImage2D", "GetNeighbourFilterResult()", 5 );
  
  bicotiImageImplementation< PixelType > * ptr_aux = ptr_neighbour_filter->GetResult();
  
  return( ( ImageType * ) ptr_aux );
}
   

template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
bicotiDexelFacade2D< PixelType > * bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: GetDexel()
{
  if(ptr_dexel == NULL)
     throw bicotiException( "bicotiImage2D", "GetDexel()", 5 );

  return( ptr_dexel );
}

template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
bool bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: IsDexelCreated()
{
  return( ptr_dexel != NULL );
}

template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: DestroyDexel()
{
  if ( ptr_dexel != NULL )
  {delete ptr_dexel;
  ptr_dexel = NULL;}
}


template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
bicotiMapperFacade2D< PixelType , PixelConverterMapper > * bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: GetMapper()
{
  if( ptr_mapper == NULL )
    throw bicotiException( "bicotiImage2D", "GetMapper()", 5 );

  return ptr_mapper;
}

template< class ImageType, class ColorType, class PixelConverterMapper, class PixelType, class PrecisionType >
void bicotiImage2D< ImageType, ColorType, PixelConverterMapper, PixelType, PrecisionType > :: DestroyMapper()
{
  if( ptr_mapper != NULL )
    {delete ptr_mapper;
    ptr_mapper = NULL;}
}




#endif           
