//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _image_3D
#define _image_3D



#include <source/image/image_basic.hpp>



//**********************DECLARATIONS**************************
template< class ImageType, class ColorType, class PixelType = ColorType, class PrecisionType = PixelType  >
class bicotiImage3D : public bicotiImage< ImageType, PixelType >
{
protected:
  bicotiPropertyFacade< bicotiCoordinate3D< INTEGER >, ColorType, PixelType, PrecisionType > * ptr_property;
  bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > * ptr_neighbour_filter;

  bicotiGeometricalFacade3D< PixelType, PrecisionType > * ptr_geometrical;
  bicotiDexelFacade3D< ColorType > * ptr_dexel; 
    
  void DeleteMe();
  void ResetPointers();

public:
  typedef bicotiCoordinate3D< INTEGER > IntegerCoordinate;
  typedef bicotiCoordinate3D< REAL > RealCoordinate;
  typedef bicotiPropertyFacade< bicotiCoordinate3D< INTEGER >, ColorType, PixelType, PrecisionType > Property;
  typedef bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > NeighbourFilter; 
  typedef bicotiGeometricalFacade3D< PixelType, PrecisionType > Geometrical;
  typedef bicotiDexelFacade3D< ColorType > Dexel; 

  bicotiPropertyFacade< bicotiCoordinate3D< INTEGER >, ColorType, PixelType, PrecisionType > * GetProperty();
  void DestroyProperty();
  
  void BuildOperatorUnary();
  bicotiOperatorUnaryFacade< PixelType > * CreateOperatorUnary();
  
  void BuildOperatorMultiple();
  bicotiOperatorMultipleFacade< PixelType > * CreateOperatorMultiple();

  void BuildGeometrical();
  void DestroyGeometrical();
  bicotiGeometricalFacade3D< PixelType, PrecisionType > * GetGeometrical();
  ImageType * GetGeometricalResult();
  void LoadGeometricalResult();
  
  bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > * GetNeighbourFilter();
  void DestroyNeighbourFilter();
  void LoadNeighbourFilterResult();
  ImageType * GetNeighbourFilterResult();

  bicotiDexelFacade3D< PixelType > * GetDexel();
  void DestroyDexel();
};




//********************IMPLEMENTATIONS*************************
template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: DeleteMe()
{
  if ( ptr_implementation != NULL)
    delete ptr_implementation;
  
  if( ptr_property != NULL)
    delete ptr_property;

  if( ptr_operator_unary != NULL)
    delete ptr_operator_unary;

  if( ptr_operator_multiple != NULL)
    delete ptr_operator_multiple;
  
  if ( ptr_neighbour_filter != NULL)
    delete ptr_neighbour_filter;
  
  if ( ptr_geometrical != NULL )
    delete ptr_geometrical;
  
  if ( ptr_dexel != NULL )
    delete ptr_dexel;

  ResetPointers();
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: ResetPointers()
{
  ptr_implementation = NULL;
  ptr_property = NULL;
  ptr_operator_unary = NULL;
  ptr_operator_multiple = NULL;
  ptr_neighbour_filter = NULL;
  ptr_geometrical = NULL;
  ptr_dexel = NULL;
};

template< class ImageType, class ColorType, class PixelType, class PrecisionType >
bicotiPropertyFacade< bicotiCoordinate3D< INTEGER >, ColorType, PixelType, PrecisionType > * bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: GetProperty()
{
  return( ptr_property );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: DestroyProperty()
{
  if(ptr_property != NULL)
    {delete ptr_property;
    ptr_property = NULL;}
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: BuildOperatorUnary()
{
  if(ptr_operator_unary != NULL)
    delete ptr_operator_unary;
  
  ptr_operator_unary = new bicotiOperatorUnaryFacade< PixelType >( ptr_implementation );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
bicotiOperatorUnaryFacade< PixelType >* bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: CreateOperatorUnary()
{
  return( new bicotiOperatorUnaryFacade< PixelType >( ptr_implementation ) );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: BuildOperatorMultiple()
{
  if(ptr_operator_multiple != NULL)
    delete ptr_operator_multiple;
  
  ptr_operator_multiple = new bicotiOperatorMultipleFacade< PixelType >( ptr_implementation );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
bicotiOperatorMultipleFacade< PixelType >* bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: CreateOperatorMultiple()
{
  return( new bicotiOperatorMultipleFacade< PixelType >( ptr_implementation ) );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: BuildGeometrical()
{
  if( ptr_geometrical != NULL )
    delete ptr_geometrical;
  
  ptr_geometrical = new bicotiGeometricalFacade3D< PixelType, PrecisionType > ( ptr_implementation );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: DestroyGeometrical()
{
  if( ptr_geometrical != NULL )
    {delete ptr_geometrical;
    ptr_geometrical = NULL;}
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
bicotiGeometricalFacade3D< PixelType, PrecisionType > * bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: GetGeometrical()
{
  if( ptr_geometrical == NULL )
    throw bicotiException( "bicotiImage3D", "GetGeometrical()", 5 );

  return( ptr_geometrical );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
ImageType * bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: GetGeometricalResult()
{
  
  if( ptr_geometrical == NULL )
    throw bicotiException( "bicotiImage3D", "GetGeometricalResult()", 5 );
  
  bicotiImageImplementation< PixelType > * ptr_aux = ptr_geometrical->GetResult();
  
  return( ( ImageType * ) ptr_aux );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: LoadGeometricalResult()
{
  if( ptr_geometrical != NULL )
    {delete ptr_implementation;
    ptr_implementation = (ImageType*)ptr_geometrical->LoadResult();}
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > * bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: GetNeighbourFilter()
{
  if( ptr_neighbour_filter == NULL )
    throw bicotiException( "bicotiImage3D", "GetNeighbourFilter()", 5 );

  return( ptr_neighbour_filter );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: DestroyNeighbourFilter()
{
  if( ptr_neighbour_filter != NULL )
    {delete ptr_neighbour_filter;
    ptr_neighbour_filter = NULL;}
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: LoadNeighbourFilterResult()
{
  if( ptr_neighbour_filter != NULL )
    {delete ptr_implementation;
    ptr_implementation = (ImageType*)ptr_neighbour_filter->LoadResult();}
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
ImageType * bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: GetNeighbourFilterResult()
{
  if(ptr_neighbour_filter == NULL)
    throw bicotiException( "bicotiImage3D", "GetNeighbourFilterResult()", 5 );
  
  bicotiImageImplementation< PixelType > * ptr_aux = ptr_neighbour_filter->GetResult();
  
  return( ( ImageType * ) ptr_aux );
};
   

template< class ImageType, class ColorType, class PixelType, class PrecisionType >
bicotiDexelFacade3D< PixelType > * bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: GetDexel()
{
  if(ptr_dexel == NULL)
     throw bicotiException( "bicotiImage3D", "GetDexel()", 5 );

  return( ptr_dexel );
};


template< class ImageType, class ColorType, class PixelType, class PrecisionType >
void bicotiImage3D< ImageType, ColorType, PixelType, PrecisionType > :: DestroyDexel()
{
  if ( ptr_dexel != NULL )
  {delete ptr_dexel;
  ptr_dexel = NULL;}
};



#endif                  
