//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _image_3D_single 
#define _image_3D_single 

 
 
#include <source/image/image_3D.hpp>
 
 
 
//**********************DECLARATIONS************************** 
template< class ImageType, class ColorType > 
class bicotiImage3DSingle : public bicotiImage3D< ImageType, ColorType, ColorType, REAL > 
{ 
public: 
  void BuildImplementation( unsigned, unsigned ); 
 
  void BuildProperty(); 
  bicotiPropertyFacade< bicotiCoordinate3D< INTEGER >, ColorType, ColorType, REAL > * CreateProperty(); 
 
  void BuildNeighbourFilter(); 
  bicotiNeighbourFilterFacade3D< ColorType, REAL > * CreateNeighbourFilter(); 
 
  void BuildDexel(); 
  bicotiDexelFacade3DSingle< ColorType > * CreateDexel(); 
}; 
 

 
 
//********************IMPLEMENTATIONS************************* 
template< class ImageType, class ColorType > 
void bicotiImage3DSingle< ImageType, ColorType > :: BuildImplementation( unsigned x0_size, unsigned x1_size ) 
{ 
  if(ptr_implementation != NULL) 
    delete ptr_implementation; 
   
  ptr_implementation = new ImageType( 0, x0_size, x1_size, 1, single ); 
}; 
 
 
template< class ImageType, class ColorType > 
void bicotiImage3DSingle< ImageType, ColorType > :: BuildProperty() 
{ 
  if( ptr_property != NULL ) 
    delete ptr_property; 
   
  ptr_property = new bicotiPropertyFacadeSingle< bicotiCoordinate3D< INTEGER >, ColorType >( ptr_implementation ); 
}; 
 
 
template< class ImageType, class ColorType > 
bicotiPropertyFacade< bicotiCoordinate3D< INTEGER >, ColorType, ColorType, REAL >* bicotiImage3DSingle< ImageType, ColorType > :: CreateProperty() 
{ 
  return ( new bicotiPropertyFacadeSingle< bicotiCoordinate3D< INTEGER >, ColorType >( ptr_implementation )); 
}; 
 
 
template< class ImageType, class ColorType > 
void bicotiImage3DSingle< ImageType, ColorType > :: BuildNeighbourFilter() 
{ 
  if( ptr_neighbour_filter != NULL ) 
    delete ptr_neighbour_filter; 
 
  ptr_neighbour_filter = new bicotiNeighbourFilterFacade3DSingle< ColorType, REAL > ( ptr_implementation ); 
}; 
 
 
template< class ImageType, class ColorType > 
bicotiNeighbourFilterFacade3D< ColorType, REAL >* bicotiImage3DSingle< ImageType, ColorType > ::  CreateNeighbourFilter() 
{ 
  return( new bicotiNeighbourFilterFacade3DSingle< ColorType, REAL > ( ptr_implementation ) ); 
}; 
 
 
template< class ImageType, class ColorType > 
void bicotiImage3DSingle< ImageType, ColorType > :: BuildDexel() 
{ 
  if( ptr_dexel != NULL ) 
    delete ptr_dexel; 
 
  ptr_dexel = new bicotiDexelFacade3DSingle< ColorType >( ptr_implementation ); 
 
}; 
 

template< class ImageType, class ColorType > 
bicotiDexelFacade3DSingle< ColorType > * bicotiImage3DSingle< ImageType, ColorType > :: CreateDexel() 
{ 
  return( new bicotiDexelFacade3DSingle< ColorType >( ptr_implementation )); 
}; 
 


#endif     
