//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _image_basic
#define _image_basic



#include <source/operator_unary/operator_unary.hpp>
#include <source/operator_multiple/operator_multiple.hpp>
#include <source/neighbour_filter/neighbour_filter.hpp>
#include <source/geometrical/geometrical.hpp>



//*************************DECLARATIONS*****************************
//--------------------------bicotiImage-----------------------------
template< class ImageType, class PixelType >
class bicotiImage
{
protected:
  ImageType * ptr_implementation;
  bicotiOperatorUnaryFacade< PixelType > * ptr_operator_unary;
  bicotiOperatorMultipleFacade< PixelType > * ptr_operator_multiple;
 
  virtual void DeleteMe() = 0; 
  virtual void ResetPointers() = 0; 
 
public: 
  typedef PixelType Pixel; 
  typedef ImageType Implementation; 
  typedef bicotiImageIterator< PixelType > Iterator; 
  typedef bicotiOperatorUnaryFacade< PixelType > OperatorUnary; 
  typedef bicotiOperatorMultipleFacade< PixelType > OperatorMultiple; 
 
 
  ImageType * GetImplementation(); 
  void SetImplementation( ImageType * ); 
 
  void DestroyImplementation(); 
 
  bicotiOperatorUnaryFacade< PixelType > * GetOperatorUnary(); 
  void DestroyOperatorUnary(); 
 
  bicotiOperatorMultipleFacade< PixelType > * GetOperatorMultiple(); 
  void DestroyOperatorMultiple(); 
 
  bicotiImageIterator< PixelType > * CreateIterator(); 
 
  virtual void DestroyDexel() = 0; 
}; 
 
 

 
//********************IMPLEMENTATIONS************************* 
//--------------------------Image----------------------------- 
template< class ImageType, class PixelType > 
ImageType * bicotiImage< ImageType, PixelType > :: GetImplementation() 
{ 
  return( ptr_implementation ); 
}; 
 
 
template< class ImageType, class PixelType > 
void bicotiImage< ImageType, PixelType > :: SetImplementation( ImageType * ptr_imp ) 
{ 
  if ( ptr_implementation != NULL ) 
    delete ptr_implementation; 
 
  ptr_implementation = ptr_imp; 
}; 
 
 
template< class ImageType, class PixelType > 
bicotiOperatorUnaryFacade< PixelType > * bicotiImage< ImageType, PixelType > :: GetOperatorUnary() 
{ 
  return( ptr_operator_unary ); 
}; 
 
 
template< class ImageType, class PixelType > 
bicotiOperatorMultipleFacade< PixelType > * bicotiImage< ImageType, PixelType > :: GetOperatorMultiple() 
{ 
  return( ptr_operator_multiple ); 
}; 
 
 
template< class ImageType, class PixelType > 
void bicotiImage< ImageType, PixelType > :: DestroyImplementation() 
{ 
  if ( ptr_implementation != NULL ) 
    {delete ptr_implementation; 
    ptr_implementation = NULL;} 
}; 
 
 
template< class ImageType, class PixelType > 
void bicotiImage< ImageType, PixelType > :: DestroyOperatorUnary() 
{ 
  if(ptr_operator_unary != NULL) 
    {delete ptr_operator_unary; 
    ptr_operator_unary = NULL;} 
}; 
 
 
template< class ImageType, class PixelType > 
void bicotiImage< ImageType, PixelType > :: DestroyOperatorMultiple() 
{ 
  if(ptr_operator_multiple != NULL) 
    {delete ptr_operator_multiple; 
    ptr_operator_multiple = NULL;} 
}; 
 
 
template< class ImageType, class PixelType > 
bicotiImageIterator< PixelType > * bicotiImage< ImageType, PixelType > :: CreateIterator() 
{ 
  return( ptr_implementation->CreateIterator() ); 
}; 
 
 
#endif  
