//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _image_implementation_2D 
#define _image_implementation_2D 
 
 
#include <source/image_implementation/image_implementation_basic.hpp>
 

//**************************DECLARATIONS************************** 
//-----------------------Image Declarations-----------------------
//------------------------Implementation2D------------------------
template< class PixelType >  
class bicotiImageIterator2D; 
 
template< class PixelType >  
class bicotiImageFrameIterator2D; 
 
template< class PixelType >  
class bicotiImageExtrapolationStrategy2D; 
 
 
template< class PixelType >  
class bicotiImageImplementation2D : public bicotiImageImplementation< PixelType > 
{ 
protected: 
  int x0_size, x1_size; 
 
  virtual void DeleteMe() = 0; 
  virtual void CreateMe( PixelType, int, int, double, PixelCases ) = 0; 
 
public: 
  virtual ~bicotiImageImplementation2D(){}; 
 
  bicotiImageDescriptor * CreateImageDescriptor();

  int GetX0Size(); 
  int GetX1Size(); 

  virtual PixelType GetPixel( bicotiCoordinate< INTEGER >& ) = 0;
  virtual void SetPixel( bicotiCoordinate< INTEGER >&, const PixelType& ) = 0;
  virtual bicotiImageIterator< PixelType > * CreateIterator() = 0; 
  virtual PixelType GetPixel( int, int ) = 0; 

  void CopyMeFrom( bicotiImageImplementation2D< PixelType > * ); 
   
  bicotiImageFrameIterator< PixelType > * CreateFrameIterator( int, bicotiImageExtrapolationStrategy< PixelType > * ); 

  virtual bicotiImageImplementation< PixelType > * CreateImageImplementation() = 0;
  virtual bicotiImageImplementation< PixelType > * CreateImageImplementation( bicotiImageDescriptor * ) = 0;

  void Resize( bicotiImageDescriptor * );
  void Resize( int, int );
}; 
 
 
template< class PixelType >  
class bicotiImageImplementation2DSimple : public bicotiImageImplementation2D< PixelType >  
{ 
public: 
  virtual ~bicotiImageImplementation2DSimple(){}; 

  virtual void SetPixel( int, int, const PixelType& ) = 0; 
  virtual void SetPixel( bicotiCoordinate< INTEGER >&, const PixelType& ) = 0;   
  virtual bicotiImageIterator< PixelType > * CreateIterator() = 0; 
  virtual PixelType GetPixel( int, int ) = 0; 
  virtual PixelType GetPixel( bicotiCoordinate< INTEGER >& ) = 0;
  virtual bicotiImageImplementation< PixelType > * CreateImageImplementation() = 0;
  virtual bicotiImageImplementation< PixelType > * CreateImageImplementation( bicotiImageDescriptor * ) = 0;
}; 
 
 
 
//-----------------bicotiImageIterator2D--------------------------
template< class PixelType >  
class bicotiImageBorderStrategy2D; 
 
template< class PixelType >  
class bicotiImageBorderStrategy2DX0; 
 
template< class PixelType >  
class bicotiImageBorderStrategy2DX1; 
  
template< class PixelType >  
class bicotiImageIterator2D : public bicotiImageIterator< PixelType > 
{  
  friend class bicotiImageBorderStrategy2DX0< PixelType >;       
  friend class bicotiImageBorderStrategy2DX1< PixelType >;       
   
protected: 
  int current_x0,current_x1; 
  int first_x0,first_x1,last_x0,last_x1; 
   
  bicotiImageBorderStrategy2DX0< PixelType > x0_strategy; 
  bicotiImageBorderStrategy2DX1< PixelType > x1_strategy; 
   
  virtual void Refresh() = 0; 
  virtual void UpdateX0() = 0; 
   
public: 
  virtual ~bicotiImageIterator2D(){}; 
   
   
  void operator++( int ); 
  void operator--( int ); 
  void operator+=( unsigned ); //OJO Habra que hacerlo de INT 
  void operator-=( unsigned ); 
 
  void First(); 
  void Last(); 
  void Locate( int,int ); 
   
  void X0Increment( unsigned =1 ); 
  void X0Decrement( unsigned =1 ); 
  void X1Increment( unsigned =1 ); 
  void X1Decrement( unsigned =1 ); 
   
  virtual void SetCurrentPixel( PixelType ) = 0; 
   
  int GetCurrentX0(); 
  int GetCurrentX1(); 

  bicotiCoordinate< INTEGER > GetCurrentCoordinateInteger();
  bicotiCoordinate< REAL > GetCurrentCoordinateReal();

  virtual void SetExtremeCoordinates( int, int, int, int ) = 0; 
  void UseBorderStrategyX0( bicotiImageBorderStrategy2DX0< PixelType > ); 
  void UseBorderStrategyX1( bicotiImageBorderStrategy2DX1< PixelType > ); 
}; 
  
 

//--------------bicotiImageBorderStrategy2D-----------------------
template< class PixelType >  
class bicotiImageBorderStrategy2D : public bicotiImageBorderStrategy< PixelType > 
{ 
protected: 
  bicotiImageIterator2D< PixelType > * ptr_iterator; 
  int step;
 
public: 
  void UseIterator( bicotiImageIterator2D< PixelType > * ); 
  void UseStep( int );
}; 
 
 
template< class PixelType >  
class bicotiImageBorderStrategy2DX0 : public bicotiImageBorderStrategy2D< PixelType > 
{   
public: 
  bicotiImageBorderStrategy2DX0( int = 1 ); 
  bicotiImageBorderStrategy2DX0( bicotiImageIterator2D< PixelType > *, int = 1 ); 
    
  void SetNextCoordinates(); 
}; 
 
 
template< class PixelType >  
class bicotiImageBorderStrategy2DX1 : public bicotiImageBorderStrategy2D< PixelType > 
{  
public: 
  bicotiImageBorderStrategy2DX1( int = 1 ); 
  bicotiImageBorderStrategy2DX1( bicotiImageIterator2D< PixelType > *, int = 1 ); 
    
  void SetNextCoordinates(); 
}; 
  
 
 
//--------------bicotiImageBorderStrategy2D-----------------------
template< class PixelType >  
class bicotiImageFrameIterator2D : public bicotiImageFrameIterator< PixelType > 
{ 
private: 
  unsigned current_x0,current_x1; 
           
  bicotiImageImplementation2D< PixelType > * ptr_image; 
  bicotiMatrix2D< PixelType > * ptr_matrix; 
  bicotiImageExtrapolationStrategy2D< PixelType > * ptr_bord_strat; 
  bicotiFrame2DExternal< PixelType > * ptr_frame; 
 
public: 
  bicotiImageFrameIterator2D( bicotiImageImplementation2D< PixelType > *, int, bicotiImageExtrapolationStrategy2D< PixelType > * ); 

  ~bicotiImageFrameIterator2D(); 
 
  bicotiFrame< PixelType > * GetCurrentFrame(); 
 
  void First(); 
  void Last(); 
  void Locate( unsigned, unsigned ); 
 
  unsigned GetCurrentX0(); 
  unsigned GetCurrentX1(); 
 
  void operator++( int ); 
  void operator--( int ); 
  void operator+=( unsigned ); 
  void operator-=( unsigned ); 
}; 
 
 
 
 
//**********************IMPLEMENTATIONS***************************
//----------------------Implementation2D--------------------------
template< class PixelType >
bicotiImageDescriptor * bicotiImageImplementation2D< PixelType > :: CreateImageDescriptor()
{
  bicotiImageDescriptor * ptr_ima_desc;
  ptr_ima_desc = new bicotiImageDescriptor( 2, scale, pixel_case );

  ptr_ima_desc->Size(0) = x0_size;
  ptr_ima_desc->Size(1) = x1_size;
  
  return ( ptr_ima_desc );
};


template< class PixelType >  
inline int bicotiImageImplementation2D< PixelType > :: GetX0Size()  
{ 
  return( x0_size ); 
}; 
 
 
template< class PixelType >  
inline int bicotiImageImplementation2D< PixelType > :: GetX1Size()  
{ 
  return( x1_size ); 
}; 
 
 
template< class PixelType > 
void bicotiImageImplementation2D< PixelType > :: CopyMeFrom( bicotiImageImplementation2D< PixelType > * ptr_ima_src ) 
{ 
  bicotiImageIterator< PixelType > * ptr_iter_src, * ptr_iter; 
 
  if ( (x0_size != ptr_ima_src->x0_size) || (x1_size != ptr_ima_src->x1_size) )
    {DeleteMe(); 
    CreateMe( ptr_ima_src->default_pixel, ptr_ima_src->x0_size, ptr_ima_src->x1_size, ptr_ima_src->scale,ptr_ima_src->pixel_case );} 
  else
    {pixel_case = ptr_ima_src->pixel_case;
    default_pixel = ptr_ima_src->default_pixel;
    scale = ptr_ima_src->scale;}
  
  ptr_iter_src = ptr_ima_src->CreateIterator();
  ptr_iter = CreateIterator();
 
  ptr_iter_src->First(); 
  ptr_iter->First(); 
   
  do 
    {( **ptr_iter ) = ( **ptr_iter_src ); 
    ( *ptr_iter )++; 
    ( *ptr_iter_src )++;} while ( !ptr_iter->IsDone() ); 
   
  delete ptr_iter; 
  delete ptr_iter_src; 
}; 
 
 
template< class PixelType > 
bicotiImageFrameIterator< PixelType > * bicotiImageImplementation2D< PixelType > :: CreateFrameIterator( int frame_size, bicotiImageExtrapolationStrategy< PixelType >* ptr_extrapol_strat ) 
{ 
  return ( new bicotiImageFrameIterator2D< PixelType >( this, frame_size, ( bicotiImageExtrapolationStrategy2D< PixelType > * )ptr_extrapol_strat ) ); 
}; 


template< class PixelType > 
void bicotiImageImplementation2D< PixelType > :: Resize( bicotiImageDescriptor *  ptr_desc ) 
{ 
  if( ptr_desc->GetDimension() != 2 )
    throw bicotiException( "bicotiImageImplementation2D", "Resize()", 2 );

  DeleteMe();
  CreateMe( default_pixel, ptr_desc->Size( 0 ), ptr_desc->Size( 1 ), scale, pixel_case );
}; 


template< class PixelType > 
void bicotiImageImplementation2D< PixelType > :: Resize( int x0_size, int x1_size ) 
{ 
  DeleteMe();
  CreateMe( default_pixel, x0_size, x1_size, scale, pixel_case );
}; 

 
 
//--------------------bicotiImageIterator2D-----------------------
template< class PixelType >  
inline void bicotiImageIterator2D< PixelType > :: operator++( int ) 
{  
  if ( current_x0 < last_x0 ) 
    {current_x0++;  
    ptr_current++;} 
  else 
    {UpdateX0(); 
    x0_strategy.SetNextCoordinates(); 
    Refresh();}
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: operator--( int ) 
{  
  if ( current_x0 > first_x0 ) 
    {current_x0--;  
    ptr_current--;} 
  else 
    {UpdateX0(); 
    x0_strategy.SetNextCoordinates(); 
    Refresh();}
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: operator+=( unsigned step ) 
{  
  X0Increment( step );  
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: operator-=( unsigned step ) 
{  
  X0Decrement( step );  
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: X0Increment( unsigned step ) 
{   
  unsigned stp = step; 
   
  while ( stp > ( last_x0-current_x0 ) ) 
    {stp = stp - ( last_x0 - current_x0 ) - 1; 
    current_x0 = last_x0; 
    UpdateX0(); 
    x0_strategy.SetNextCoordinates();}
  current_x0 = current_x0 + stp; 
  Refresh(); 
}; 
 
     
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: X0Decrement( unsigned step ) 
{   
  unsigned stp = step; 
   
  while ( stp > ( current_x0 - first_x0 ) ) 
    {stp = stp - ( current_x0 - first_x0 ) - 1; 
    current_x0 = first_x0; 
    UpdateX0(); 
    x0_strategy.SetNextCoordinates();}
   
  current_x0 = current_x0 - stp; 
  Refresh();  
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: X1Decrement( unsigned step ) 
{   
  unsigned stp = step; 
   
  while ( stp > ( current_x1 - first_x1 ) ) 
    {stp = stp - ( current_x1 - first_x1 ) - 1; 
    current_x1 = first_x1; 
    UpdateX0(); 
    x1_strategy.SetNextCoordinates();}
   
  current_x1 = current_x1 - stp; 
  Refresh();  
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: X1Increment( unsigned step ) 
{   
  unsigned stp = step; 
	 
  while ( stp > ( last_x1 - current_x1 ) ) 
    {stp = stp - ( last_x1 - current_x1 ) - 1; 
    current_x1 = last_x1; 
    UpdateX0(); 
    x1_strategy.SetNextCoordinates();}
   
  current_x1 = current_x1 + stp; 
  Refresh(); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: First() 
{ 
  UpdateX0(); 
  is_done = false; 
  current_x0 = first_x0; 
  current_x1 = first_x1; 
  Refresh();      
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: Last() 
{ 
  UpdateX0(); 
  is_done = false; 
  current_x0 = last_x0; 
  current_x1 = last_x1; 
  Refresh();     
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: Locate( int x_1, int x_2 ) 
{ 
  UpdateX0(); 
  is_done = false; 
  if( ( x_2 >= first_x1 ) && ( x_2 <= last_x1 ) ) 
    current_x1 = x_2; 
  if( ( x_1 >= first_x0 ) && ( x_1 <= last_x0 ) ) 
    current_x0 = x_1; 
  Refresh(); 
}; 
 
 
template< class PixelType >  
int bicotiImageIterator2D< PixelType > :: GetCurrentX0() 
{ 
  return( current_x0 ); 
}; 
 
 
template< class PixelType >  
int bicotiImageIterator2D< PixelType > :: GetCurrentX1() 
{ 
  return( current_x1 ); 
}; 
 
 
template< class PixelType >
inline bicotiCoordinate< INTEGER > bicotiImageIterator2D< PixelType > :: GetCurrentCoordinateInteger()
{
  bicotiCoordinate< INTEGER > aux_coord( 2 );

  aux_coord( 0 ) = ( INTEGER )current_x0;
  aux_coord( 1 ) = ( INTEGER )current_x1;

  return( aux_coord );
};


template< class PixelType >
inline bicotiCoordinate< REAL > bicotiImageIterator2D< PixelType > :: GetCurrentCoordinateReal()
{
  bicotiCoordinate< REAL > aux_coord( 2 );

  aux_coord( 0 ) = ( REAL )current_x0;
  aux_coord( 1 ) = ( REAL )current_x1;

  return( aux_coord );
};


template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: UseBorderStrategyX0( bicotiImageBorderStrategy2DX0< PixelType > x_0_strategy ) 
{ 
  x0_strategy = x_0_strategy; 
  x0_strategy.UseIterator( this ); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2D< PixelType > :: UseBorderStrategyX1( bicotiImageBorderStrategy2DX1< PixelType > x_1_strategy ) 
{ 
  x1_strategy = x_1_strategy; 
  x1_strategy.UseIterator( this ); 
}; 
 

 
//-----------------bicotiImageBorderStrategy2D--------------------
template< class PixelType > 
void bicotiImageBorderStrategy2D< PixelType > :: UseIterator( bicotiImageIterator2D< PixelType > * ptr_it ) 
{ 
  ptr_iterator = ptr_it; 
}; 


template< class PixelType > 
void bicotiImageBorderStrategy2D< PixelType > :: UseStep( int stp ) 
{ 
  step = stp; 
}; 


 
//----------------bicotiImageBorderStrategy2DX0-------------------
template< class PixelType >  
bicotiImageBorderStrategy2DX0< PixelType > :: bicotiImageBorderStrategy2DX0( int stp ) 
{ 
  step = stp;	 
  ptr_iterator = NULL;     
}; 
 
 
template< class PixelType >  
bicotiImageBorderStrategy2DX0< PixelType > :: bicotiImageBorderStrategy2DX0( bicotiImageIterator2D< PixelType > * ptr_it, int stp ) 
{ 
  step = stp;	 
  ptr_iterator = ptr_it;     
}; 
 
 
template< class PixelType >  
void bicotiImageBorderStrategy2DX0< PixelType > :: SetNextCoordinates() 
{ 
  if ( ptr_iterator->current_x0 == ptr_iterator->last_x0 ) 
    {if ( ( step == 0 ) || ( ( step > 0 ) && ( ( ptr_iterator->current_x1 + step ) > ptr_iterator->last_x1 ) ) || ( ( step < 0 ) && ( ( ( int )ptr_iterator->current_x1 + step ) < ( int )ptr_iterator->first_x1 ) ) ) 
      ptr_iterator->is_done = true;
    else 
      {ptr_iterator->current_x0 = ptr_iterator->first_x0; 
      ptr_iterator->current_x1 += step;}} 
  else 
    if ( ptr_iterator->current_x0 == ptr_iterator->first_x0 ) 
      {if ( ( step == 0 ) || ( ( step > 0 ) && ( ( ( int )ptr_iterator->current_x1 - step ) < ( int )ptr_iterator->first_x1 ) ) || ( ( step < 0 ) && ( ( ptr_iterator->current_x1 - step ) > ptr_iterator->last_x1 ) ) ) 
	ptr_iterator-> is_done=true;
      else 
	{ptr_iterator->current_x0 = ptr_iterator->last_x0; 
	ptr_iterator->current_x1 -= step;}} 
    else 
      throw bicotiException( "bicotiImageBorderStrategy2DX0", "SetNextCoordinates", 4 );
}; 

 
 
//----------------bicotiImageBorderStrategy2DX1-------------------
template< class PixelType >  
bicotiImageBorderStrategy2DX1< PixelType > :: bicotiImageBorderStrategy2DX1( int stp ) 
{ 
  step = stp; 
  ptr_iterator = NULL;     
}; 
 
 
template< class PixelType >  
bicotiImageBorderStrategy2DX1< PixelType > :: bicotiImageBorderStrategy2DX1( bicotiImageIterator2D< PixelType >* ptr_it, int stp ) 
{ 
  step = stp;     
  ptr_iterator = ptr_it;     
}; 
 

template< class PixelType >  
void bicotiImageBorderStrategy2DX1< PixelType > :: SetNextCoordinates() 
{ 
  if ( ptr_iterator->current_x1 == ptr_iterator->last_x1 ) 
    {if ( ( step == 0 ) || ( ( step > 0 ) && ( ( ptr_iterator->current_x0 + step ) > ptr_iterator->last_x0 ) ) || ( ( step < 0 ) && ( ( ( int )ptr_iterator->current_x0 + step ) < ( int )ptr_iterator->first_x0 ) ) ) 
      ptr_iterator-> is_done=true;
    else 
      {ptr_iterator->current_x1 = ptr_iterator->first_x1; 
      ptr_iterator->current_x0 += step;}} 
  else 
    if ( ptr_iterator->current_x1 == ptr_iterator->first_x1 ) 
      {if ( ( step == 0 ) || ( ( step > 0 ) && ( ( ( int )ptr_iterator->current_x0 - step ) < ( int )ptr_iterator->first_x0 ) ) || ( ( step < 0 ) && ( ( ptr_iterator->current_x0 - step ) > ptr_iterator->last_x0 ) ) ) 
	ptr_iterator->is_done = true;
      else 
	{ptr_iterator->current_x1 = ptr_iterator->last_x1; 
	ptr_iterator->current_x0 -= step;}} 
    else 
      throw bicotiException( "bicotiImageBorderStrategy2DX1", "SetNextCoordinates", 4 );
}; 
 

 
//----------------bicotiImageFrameIterator2D----------------------
template< class PixelType >  
bicotiImageFrameIterator2D< PixelType > :: bicotiImageFrameIterator2D( bicotiImageImplementation2D< PixelType > * ptr_ima, int fs , bicotiImageExtrapolationStrategy2D< PixelType >* ptr_extrapol_strat ) 
{ 
  int fs_local, i, j; 
 
  if ( fs%2 == 0 ) 
    throw bicotiException( "bicotiImageFrameIterator2D", "Constructor", 4 );

  if ( fs < 0 ) 
    throw bicotiException( "bicotiImageFrameIterator2D", "Constructor", 4 );
 
  ptr_image = ptr_ima; 
  fs_local = fs; 
 
  ptr_matrix = new bicotiMatrix2D< PixelType >( ptr_image->GetDefaultPixel(), ( ptr_image->GetX0Size() + fs_local - 1 ), ( ptr_image->GetX1Size() + fs_local - 1 ) ); 
 
  ptr_bord_strat = ptr_extrapol_strat; 
  ptr_bord_strat->SetImplementation( ptr_image ); 
 
  for( i = 0; i < ( ptr_image->GetX0Size() + fs_local - 1 ); i++ ) 
    {for( j = 0; j < ( ptr_image->GetX1Size() + fs_local - 1 ); j++ ) 
      ( *ptr_matrix )( i, j ) = ( *ptr_bord_strat )( i - fs_local/2, j - fs_local/2 );} 
 
  ptr_frame = new bicotiFrame2DExternal< PixelType > ( ptr_matrix, fs_local ); 
  current_x0 = current_x1 = 0; 
}; 
 
 
template< class PixelType >  
bicotiImageFrameIterator2D< PixelType > :: ~bicotiImageFrameIterator2D() 
{  
  delete ptr_frame; 
  delete ptr_matrix; 
}; 
 
 
template< class PixelType > 
inline bicotiFrame< PixelType > * bicotiImageFrameIterator2D< PixelType > :: GetCurrentFrame() 
{ 
  return ( ptr_frame ); 
}; 
 
 
template< class PixelType >  
void bicotiImageFrameIterator2D< PixelType > :: First() 
{ 
  is_done = false; 
 
  ptr_frame->Locate(0,0); 
  current_x0 = current_x1 = 0; 
}; 
 
 
template< class PixelType >  
void bicotiImageFrameIterator2D< PixelType > :: Last() 
{ 
  is_done = false; 
 
  current_x0 = ptr_image->GetX0Size() - 1; 
  current_x1 = ptr_image->GetX1Size() - 1; 
 
  ptr_frame->Locate( current_x0, current_x1 ); 
}; 
 
 
template< class PixelType >  
void bicotiImageFrameIterator2D< PixelType > :: Locate( unsigned x_1, unsigned x_2 ) 
{ 
  current_x0 = x_1; 
  current_x1 = x_2; 
   
  ptr_frame->Locate( current_x0, current_x1 ); 
}; 
 
 
template< class PixelType >  
unsigned bicotiImageFrameIterator2D< PixelType > :: GetCurrentX0() 
{ 
  return( current_x0 ); 
}; 
 
 
template< class PixelType >  
unsigned bicotiImageFrameIterator2D< PixelType > :: GetCurrentX1() 
{ 
  return( current_x1 ); 
}; 
 
 
template< class PixelType >  
inline void bicotiImageFrameIterator2D< PixelType > :: operator++( int ) 
{ 
  if ( current_x0 < ( ptr_image->GetX0Size() - 1 ) ) 
    {current_x0++; 
    ptr_frame->Locate( current_x0, current_x1 );} 
  else 
    {if ( current_x1 < ( ptr_image->GetX1Size() - 1 ) ) 
      {current_x1++; 
      current_x0 = 0; 
      ptr_frame->Locate( current_x0, current_x1 );} 
    else 
      is_done = true;} 
}; 
 
 
template< class PixelType >  
void bicotiImageFrameIterator2D< PixelType > :: operator--( int ) 
{ 
  if ( current_x0 > 0 ) 
    {current_x0--; 
    ptr_frame->Locate( current_x0, current_x1 );} 
  else 
    {if ( current_x1 > 0 ) 
      {current_x1--; 
      current_x0 = ptr_image->GetX0Size() - 1; 
      ptr_frame->Locate( current_x0, current_x1 );} 
    else 
      is_done = true;} 
}; 
 
 
template< class PixelType >  
void bicotiImageFrameIterator2D< PixelType > :: operator+=( unsigned step ) // OJO Arreglar despues 
{  
  for( unsigned i = 0; i < step; i++ ) 
    ( *this )++; 
}; 
 
 
template< class PixelType >  
void bicotiImageFrameIterator2D< PixelType > :: operator-=( unsigned step ) // OJO Arreglar despues 
{  
  for( unsigned i = 0; i < step; i++ ) 
    ( *this )--; 
}; 
 
 
 
#endif 
