//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _image_extrapolation_strategy_2D_array
#define _image_extrapolation_strategy_2D_array



#include <source/image_implementation/image_implementation_2D.hpp>
 


//**************************DELCARATIONS************************** 
//------------------------------Array-----------------------------
template< class PixelType >  
class bicotiImageIterator2DArray; 
 
 
template< class PixelType >  
class bicotiImageImplementation2DArray : public bicotiImageImplementation2DSimple< PixelType > 
{ 
  friend class bicotiImageIterator2DArray< PixelType >; 
   
private: 
  PixelType ** ptr_pixels; 
   
  void DeleteMe(); 
  void CreateMe( PixelType, int, int, double, PixelCases ); 
 
public: 
  bicotiImageImplementation2DArray( PixelType, int = 0, int = 0, double = 1, PixelCases = unknown ); 
  bicotiImageImplementation2DArray( const bicotiImageImplementation2DArray< PixelType >& ); 
  ~bicotiImageImplementation2DArray(); 

  void SetPixel( bicotiCoordinate< INTEGER >&, const PixelType& );
  PixelType GetPixel( bicotiCoordinate< INTEGER >& );

  void SetPixel( int, int, const PixelType& ); 
  PixelType GetPixel( int, int ); 
   
  bicotiImageIterator< PixelType > * CreateIterator(); 
  bicotiImageImplementation< PixelType > * CreateImageImplementation();
  bicotiImageImplementation< PixelType > * CreateImageImplementation( bicotiImageDescriptor * );
}; 
 
 
 
//---------------------bicotiImageIterator2DArray-----------------
template< class PixelType >  
class bicotiImageIterator2DArray : public bicotiImageIterator2D< PixelType > 
{ 
private: 
  bicotiImageImplementation2DArray< PixelType > * ptr_image; 
   
  void CreateMe( bicotiImageImplementation2DArray< PixelType > * ); 
  void UpdateX0(){}; 
  void Refresh(); 

public: 
  bicotiImageIterator2DArray( bicotiImageImplementation2DArray< PixelType >* ); 
  bicotiImageIterator2DArray( bicotiImageImplementation2DArray< PixelType >*, bicotiImageBorderStrategy2DX0< PixelType >, bicotiImageBorderStrategy2DX0< PixelType > ); 
  ~bicotiImageIterator2DArray(){}; 
   
  void SetCurrentPixel( PixelType );
  void SetExtremeCoordinates( int, int, int, int );     
}; 
 
 

//************************IMPLEMENTATIONS*************************
//-----------------------------Array------------------------------
template< class PixelType >  
bicotiImageImplementation2DArray< PixelType > :: bicotiImageImplementation2DArray( PixelType def_pixel, int x_0, int x_1, double scale_value, PixelCases pix_case ) 
{  
  CreateMe( def_pixel, x_0, x_1, scale_value, pix_case ); 
}; 
 
 
template< class PixelType >  
bicotiImageImplementation2DArray< PixelType > :: ~bicotiImageImplementation2DArray() 
{ 
  DeleteMe(); 
}; 
 
 
template< class PixelType > 
void bicotiImageImplementation2DArray< PixelType > :: CreateMe( PixelType def_pixel, int x_0, int x_1, double scale_value, PixelCases pix_case ) 
{ 
  if ( ( x_1 >= 0 ) || ( x_0 >= 0 ) )  
  {int i, j; 
   
  default_pixel = def_pixel; 
  x0_size = x_0; 
  x1_size = x_1; 
  scale = scale_value; 
  pixel_case = pix_case;
  
  ptr_pixels = new PixelType* [ x1_size ]; 
  for ( i = 0; i < x1_size; i++ ) 
    ptr_pixels[ i ] = new PixelType[ x0_size ]; 
  for ( i = 0; i < x1_size; i++ ) 
    {for ( j = 0; j < x0_size; j++ ) 
      ptr_pixels[ i ][ j ] = default_pixel;}}  
  else  
    throw bicotiException( "bicotiImageImplementation2DArray", "Constructor", 2 );
}; 

 
template< class PixelType >  
bicotiImageImplementation2DArray< PixelType > :: bicotiImageImplementation2DArray( const bicotiImageImplementation2DArray< PixelType >& imag_src ) 
{ 
  default_pixel = imag_src.default_pixel; 
  x0_size = imag_src.x0_size; 
  x1_size = imag_src.x1_size; 
  scale = imag_src.scale; 
  pixel_case = imag_src.pixel_case;
  
  int i, j; 
   
  ptr_pixels = new PixelType* [ x1_size ]; 
  for ( i = 0; i < x1_size; i++ ) 
    ptr_pixels[ i ] = new PixelType[ x0_size ]; 
  for ( i =0; i < x1_size; i++ ) 
    {for ( j = 0; j < x0_size; j++ ) 
      ptr_pixels[ i ][ j ] = imag_src.ptr_pixels[ i ][ j ];} 
}; 
 
 
template< class PixelType > 
void bicotiImageImplementation2DArray< PixelType > :: DeleteMe() 
{ 
  int i; 
 
  for ( i = 0; i < x1_size; i++ ) 
    if ( x0_size > 0 ) 
      delete [] ptr_pixels[ i ]; 
  delete [] ptr_pixels; 
}; 
 
 
template< class PixelType >  
void bicotiImageImplementation2DArray< PixelType > :: SetPixel( bicotiCoordinate< INTEGER >& coord, const PixelType& value ) 
{ 
  unsigned x_0, x_1;
  bicotiCoordinate2D< INTEGER > aux_coord = (bicotiCoordinate2D< INTEGER >) coord;

  x_0 = aux_coord.X0();
  x_1 = aux_coord.X1();
  
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( (  x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiImageImplementation2DArray", "SetPixel()", 1 );
#endif
  
  ptr_pixels[ x_1 ][ x_0 ] = value; 
}; 


template< class PixelType >  
PixelType bicotiImageImplementation2DArray< PixelType > :: GetPixel( bicotiCoordinate< INTEGER >& coord ) 
{ 
  unsigned x_0, x_1;
  bicotiCoordinate2D< INTEGER > aux_coord = (bicotiCoordinate2D< INTEGER >) coord;

  x_0 = aux_coord.X0();
  x_1 = aux_coord.X1();
  
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( (  x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiImageImplementation2DArray", "GetPixel()", 1 );
#endif
  
  return ptr_pixels[ x_1 ][ x_0 ]; 
}; 

 
template< class PixelType >  
void bicotiImageImplementation2DArray< PixelType > :: SetPixel( int x_0, int x_1, const PixelType& value ) 
{ 
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( (  x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiImageImplementation2DArray", "SetPixel()", 1 );
#endif

  ptr_pixels[ x_1 ][ x_0 ] = value; 
}; 


template< class PixelType >  
PixelType bicotiImageImplementation2DArray< PixelType > :: GetPixel( int x_0, int x_1 ) 
{ 
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( (  x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiImageImplementation2DArray", "GetPixel()", 1 );
#endif

  return ptr_pixels[ x_1 ][ x_0 ]; 
}; 
 
 
template< class PixelType >  
bicotiImageIterator< PixelType > * bicotiImageImplementation2DArray< PixelType > :: CreateIterator() 
{ 
  return( new bicotiImageIterator2DArray< PixelType >( this ) ); 
}; 


template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiImageImplementation2DArray< PixelType > :: CreateImageImplementation()
{
  return( new bicotiImageImplementation2DArray< PixelType >( default_pixel, x0_size, x1_size, scale, pixel_case ) );
};


template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiImageImplementation2DArray< PixelType > :: CreateImageImplementation( bicotiImageDescriptor * ptr_desc )
{
  double scale_aux = ptr_desc->GetScale();
  PixelCases pix_case_aux = ptr_desc->GetPixelCase();
  
  if ( ( ptr_desc->GetDimension() ) != 2 )
    throw bicotiException( "bicotiImageImplementation2DArray", "CreateImageImplementation()", 2 );
  
  unsigned x0_size_aux = ptr_desc->Size( 0 );
  unsigned x1_size_aux = ptr_desc->Size( 1 );
  
  return( new bicotiImageImplementation2DArray< PixelType >( default_pixel, x0_size_aux, x1_size_aux, scale_aux, pix_case_aux ) );
};



//------------------bicotiImageIterator2DArray--------------------
template< class PixelType >  
void bicotiImageIterator2DArray< PixelType > :: CreateMe( bicotiImageImplementation2DArray< PixelType > * ptr_I ) 
{ 
  if ( (ptr_I->GetX0Size() != 0 ) && ( ptr_I->GetX1Size() != 0 ) ) 
    {is_done = false; 
    ptr_image = ptr_I; 
    first_x1 = 0; 
    first_x0 = 0; 
    last_x1 = ptr_image->x1_size - 1; 
    last_x0 = ptr_image->x0_size - 1; 
    current_x0 = first_x0; 
    current_x1 = first_x1; 
    Refresh();} 
  else 
    throw bicotiException( "bicotiImageImplementation2DArray", "CreateMe()", 6 );
}; 
   
     
template< class PixelType >  
bicotiImageIterator2DArray< PixelType > :: bicotiImageIterator2DArray( bicotiImageImplementation2DArray< PixelType > * ptr_I ) 
{   
  CreateMe( ptr_I ); 
  x0_strategy = bicotiImageBorderStrategy2DX0< PixelType >( this ); 
  x1_strategy = bicotiImageBorderStrategy2DX1< PixelType >( this );  
}; 

 
template< class PixelType >  
bicotiImageIterator2DArray< PixelType > :: bicotiImageIterator2DArray( bicotiImageImplementation2DArray< PixelType >* ptr_I, bicotiImageBorderStrategy2DX0< PixelType > hor, bicotiImageBorderStrategy2DX0< PixelType > ver ) 
{   
  CreateMe( ptr_I ); 
  x0_strategy = hor; 
  x1_strategy = ver; 
  x0_strategy.UseIterator( this ); 
  x1_strategy.UseIterator( this ); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DArray< PixelType > :: SetCurrentPixel( PixelType val ) 
{  
  *ptr_current = val;  
}; 


template< class PixelType >  
inline void bicotiImageIterator2DArray< PixelType > :: Refresh() 
{   
  ptr_current = ptr_image->ptr_pixels[ current_x1 ] + current_x0;  
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DArray< PixelType > :: SetExtremeCoordinates( int first_x_0, int first_x_1, int last_x_0, int last_x_1 ) 
{ 
  if( ( last_x_1 < first_x_1 ) || ( last_x_0 < first_x_0 ) ) 
    throw bicotiException( "bicotiImageIterator2DArray", "SetExtremeCoordinates()", 1 );
  else 
    {if( ( last_x_1 >= 0 ) && ( last_x_1 < ptr_image->GetX1Size() ) ) 
      last_x1 = last_x_1; 
    else 
      throw bicotiException( "bicotiImageIterator2DArray", "SetExtremeCoordinates()", 1 );};
  
  if( ( last_x_0 >= 0 ) && ( last_x_0 < ptr_image->GetX0Size() ) ) 
    last_x0 = last_x_0; 
  else 
    throw bicotiException( "bicotiImageIterator2DArray", "SetExtremeCoordinates()", 1 );
  
  first_x1 = first_x_1; 
  first_x0 = first_x_0; 
  First();
};                  


#endif
