//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _image_implementation_2D_sparse
#define _image_implementation_2D_sparse



#include <source/image_implementation/image_implementation_2D.hpp>
 

 
//**************************DECLARATIONS************************** 
//----------------------------Sparse------------------------------
template< class PixelType >  
class bicotiImageIterator2DSparse; 
 
 
template< class PixelType >  
class bicotiImageImplementation2DSparse : public bicotiImageImplementation2DSimple< PixelType > 
{ 
  friend class bicotiImageIterator2DSparse< PixelType >; 
   
private: 
  unsigned * ptr_x0_index;  // Columnas con valor de pixel !=0.	 
  unsigned * ptr_x1_index;  //Dada una fila, indica indice en ptr_pixels 
  PixelType * ptr_pixels;	      // Valor de cada pixel !=0. 
   
  void DeleteMe(); 
  void CreateMe( PixelType, int, int, double, PixelCases = unknown ); 
  
public: 
  bicotiImageImplementation2DSparse( PixelType, int = 0, int = 0, double = 1, PixelCases = unknown ); 
  ~bicotiImageImplementation2DSparse(); 
  bicotiImageImplementation2DSparse( const bicotiImageImplementation2DSparse< PixelType >& ); 
  
  void SetPixel( bicotiCoordinate< INTEGER >&, const PixelType& );
  PixelType GetPixel( bicotiCoordinate< INTEGER >& );

  void SetPixel( int, int, const PixelType& ); 
  PixelType GetPixel( int, int ); 
   
  bicotiImageIterator< PixelType > * CreateIterator(); 
  
  bicotiImageImplementation< PixelType > * CreateImageImplementation();
  bicotiImageImplementation< PixelType > * CreateImageImplementation( bicotiImageDescriptor * );
}; 
 

 
//-------------------bicotiImageIterator2DSparse------------------
template< class PixelType >  
class bicotiImageIterator2DSparse : public bicotiImageIterator2D< PixelType > 
{ 
private: 
  bicotiImageImplementation2DSparse< PixelType > * ptr_image; 
  PixelType * ptr_x0;
  
  void CreateMe( bicotiImageImplementation2DSparse< PixelType > * ); 
  void LoadX0( int );   
  void Refresh(); 
  void UpdatePix(); 
  void UpdateX0(); 
 
public: 
  bicotiImageIterator2DSparse( bicotiImageImplementation2DSparse< PixelType > * ); 
  bicotiImageIterator2DSparse( bicotiImageImplementation2DSparse< PixelType > *, bicotiImageBorderStrategy2D< PixelType > *, bicotiImageBorderStrategy2D< PixelType >* ); 
  ~bicotiImageIterator2DSparse(){}; 
   
  PixelType& operator*(); 
  void SetCurrentPixel( PixelType ); 
  void SetExtremeCoordinates( int, int, int, int ); 
}; 
 

 
 
//************************IMPLEMENTATIONS*************************
//----------------------------Sparse------------------------------
template< class PixelType >  
bicotiImageImplementation2DSparse< PixelType > :: bicotiImageImplementation2DSparse( PixelType def_pixel, int x_0, int x_1, double scale_value, PixelCases pix_case ) 
{ 
  CreateMe( def_pixel, x_0, x_1, scale_value, pix_case ); 
}; 

 
template< class PixelType >  
bicotiImageImplementation2DSparse< PixelType > :: ~bicotiImageImplementation2DSparse() 
{ 
  DeleteMe(); 
}; 
 
 
template< class PixelType >  
void bicotiImageImplementation2DSparse< PixelType > :: CreateMe( PixelType def_pixel, int x_0, int x_1, double scale_value, PixelCases pix_case ) 
{ 
  if ( ( x_0 >= 0 ) && ( x_1 >= 0 ) ) 
    {default_pixel = def_pixel; 
    x0_size = x_0; 
    x1_size = x_1; 
    scale = scale_value; 
    pixel_case = pix_case;
     
    ptr_x0_index = new unsigned[ x1_size + 1 ]; 
    for ( int i = 0; i <= x1_size; i++ ) 
      ptr_x0_index[ i ] = 0;} 
  else 
    throw bicotiException( "bicotiImageImplementation2DSparse", "Constructor", 2 );
}; 


template< class PixelType >  
void bicotiImageImplementation2DSparse< PixelType > :: DeleteMe() 
{ 
  if ( ptr_x0_index[ x1_size ] > 0 ) 
    {delete [] ptr_pixels; 
    delete [] ptr_x0_index;} 
  delete [] ptr_x0_index; 
}; 
 
 
template< class PixelType >  
bicotiImageImplementation2DSparse< PixelType > :: bicotiImageImplementation2DSparse( const bicotiImageImplementation2DSparse< PixelType >& imag_src ) 
{ 
  int i; 
  default_pixel = imag_src.default_pixel; 
  x0_size = imag_src.x0_size; 
  x1_size = imag_src.x1_size; 
  scale = imag_src.scale; 
  pixel_case = imag_src.pixel_case;
   
  ptr_x0_index = new unsigned[ x1_size + 1 ]; 
  for ( i = 0; i <= x1_size; i++ ) 
    ptr_x0_index[ i ] = imag_src.ptr_x0_index[ i ]; 
  
  ptr_x0_index = new unsigned[ ptr_x0_index[ x1_size ] ]; 
  ptr_pixels = new PixelType [ ptr_x0_index[ x1_size ] ]; 
  for( i = 0; i < ptr_x0_index[ x1_size ]; i++ ) 
    {ptr_x0_index[ i ] = imag_src.ptr_x0_index[ i ]; 
    ptr_pixels[ i ] = imag_src.ptr_pixels[ i ];} 
}; 
 
 
template< class PixelType >  
void bicotiImageImplementation2DSparse< PixelType > :: SetPixel( bicotiCoordinate< INTEGER >& coord, const PixelType& val )  
{ 
  unsigned pos; 
  unsigned * ptr_x0_aux; 
  PixelType * ptr_pixels_aux; 
  int i; 
   
  unsigned x_0, x_1;
  bicotiCoordinate2D< INTEGER > aux_coord = (bicotiCoordinate2D< INTEGER >) coord;

  x_0 = aux_coord.X0();
  x_1 = aux_coord.X1();

#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( x_1 + 1 > x1_size ) || ( x_0 < 0 ) || ( x_0 + 1 > x0_size ) )     throw bicotiException( "bicotiImageImplementation2DSparse", "SetPixel()", 1 );
#endif
  
  if ( ptr_x0_index[ x1_size ] == 0 ) 
    {if ( val == default_pixel ) 
      return; 
    else 
      { ptr_pixels = new PixelType[ 1 ]; 
      ptr_x0_index = new unsigned[ 1 ]; 
      pos = 0; 
      ptr_pixels[ pos ] = val; 
      ptr_x0_index[ pos ] = x_0; 
      for ( i = x_1 + 1; i <= x1_size; i++ ) 
	ptr_x0_index[ i ]++;}} 
  else 
    {pos = ptr_x0_index[x_1]; 
    if ( ptr_x0_index[ x_1 ] < ptr_x0_index[ x_1 + 1 ] ) 
      {while ( ( pos <= ptr_x0_index[ x_1 + 1 ] - 1 ) && ( ptr_x0_index[ pos ] < x_0 ) ) 
	pos++; //OJO QUE PODRIA HABER PROBLEMAS CON LA COMPARACION ANTERIOR 
      
      if ( ( pos < ptr_x0_index[ x1_size ] ) && ( ptr_x0_index[ pos ] == x_0 ) && ( pos < ptr_x0_index[ x_1 + 1 ] ) ) // ACA 
	{if ( val == default_pixel ) 
	  {ptr_pixels_aux = new PixelType [ ptr_x0_index[ x1_size ] - 1 ]; 
	  ptr_x0_aux= new unsigned [ ptr_x0_index[ x1_size ] - 1 ]; 
	  for ( i = 0; i < pos; i++ ) 
	    {ptr_pixels_aux[ i ] = ptr_pixels[ i ]; 
	    ptr_x0_aux[ i ] = ptr_x0_index[ i ];} 
	  for ( i = pos + 1; i < ptr_x0_index[ x1_size ]; i++ ) 
	    {ptr_pixels_aux[ i - 1 ] = ptr_pixels[ i ]; 
	    ptr_x0_aux[ i - 1 ] = ptr_x0_index[ i ];} 
	  delete [] ptr_pixels; 
	  delete [] ptr_x0_index; 
	  ptr_pixels = ptr_pixels_aux; 
	  ptr_x0_index = ptr_x0_aux; 
	  for ( i = x_1 + 1; i <= x1_size; i++ ) 
	    ptr_x0_index[ i ]--;} 
	else 
	  ptr_pixels[ pos ] = val; 
	return;} 
      else 
	{if ( val == default_pixel ) 
	  return; 
	else 
	  {ptr_pixels_aux = new PixelType [ ptr_x0_index[ x1_size ] + 1 ]; 
	  ptr_x0_aux = new unsigned [ ptr_x0_index[ x1_size ] + 1 ]; 
	  for ( i = 0; i < pos; i++ ) 
	    {ptr_pixels_aux[ i ] = ptr_pixels[ i ]; 
	    ptr_x0_aux[ i ] = ptr_x0_index[ i ];} 
	  ptr_pixels_aux[ pos ] = val; 
	  ptr_x0_aux[ pos ] = x_0; 
	  for ( i = pos + 1; i <= ptr_x0_index[ x1_size ]; i++ ) 
	    {ptr_pixels_aux[ i ] = ptr_pixels[ i - 1 ]; 
	    ptr_x0_aux[ i ] = ptr_x0_index[ i - 1 ];} 
	  delete [] ptr_pixels; 
	  delete [] ptr_x0_index; 
	  ptr_pixels = ptr_pixels_aux; 
	  ptr_x0_index = ptr_x0_aux; 
	  for  ( i = x_1 + 1; i <= x1_size; i++ ) 
	    ptr_x0_index[ i ]++; 
	  return;}}} 
    else  
      {ptr_pixels_aux = new PixelType [ ptr_x0_index[ x1_size ] + 1 ]; 
      ptr_x0_aux = new unsigned [ ptr_x0_index[ x1_size ] + 1 ]; 
      for ( i = 0; i < pos; i++ ) 
	{ptr_pixels_aux[ i ] = ptr_pixels[ i ]; 
	ptr_x0_aux[ i ] = ptr_x0_index[ i ];} 
      ptr_pixels_aux[ pos ] = val; 
      ptr_x0_aux[ pos ] = x_0; 
      for ( i = pos + 1; i <= ptr_x0_index[ x1_size ]; i++ ) 
	{ptr_pixels_aux[ i ] = ptr_pixels[ i - 1 ]; 
	ptr_x0_aux[ i ] = ptr_x0_index[ i - 1 ];} 
      delete [] ptr_pixels; 
      delete [] ptr_x0_index; 
      ptr_pixels = ptr_pixels_aux; 
      ptr_x0_index = ptr_x0_aux; 
      for ( i = x_1 + 1; i <= x1_size; i++ ) 
	ptr_x0_index[ i ]++; 
      return;}}
}; 


template< class PixelType >  
PixelType bicotiImageImplementation2DSparse< PixelType > :: GetPixel( bicotiCoordinate< INTEGER >& coord ) 
{ 
  unsigned pos; 
  
  unsigned x_0, x_1;
  bicotiCoordinate2D< INTEGER > aux_coord = (bicotiCoordinate2D< INTEGER >) coord;
  
  x_0 = aux_coord.X0();
  x_1 = aux_coord.X1();
  
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( ( x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiImageImplementation2DSparse", "GetPixel()", 1 );
#endif

  pos = ptr_x0_index[ x_1 ]; 
  while ( ( pos < ptr_x0_index[ x_1 + 1 ] ) && ( ptr_x0_index[ pos ] < x_0 ) ) 
    pos++; 
  if ( ( pos < ptr_x0_index[ x_1 + 1 ] ) && ( ptr_x0_index[ pos ] == x_0 ) ) 
    return ptr_pixels[ pos ]; 
  else 
    return default_pixel; 
}; 
 

template< class PixelType >  
void bicotiImageImplementation2DSparse< PixelType > :: SetPixel( int x_0, int x_1, const PixelType& val )  
{ 
  unsigned pos; 
  unsigned * ptr_x0_aux; 
  PixelType * ptr_pixels_aux; 
  int i; 
  
#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( x_1 + 1 > x1_size ) || ( x_0 < 0 ) || ( x_0 + 1 > x0_size ) )     throw bicotiException( "bicotiImageImplementation2DSparse", "SetPixel()", 1 );
#endif
  
  if ( ptr_x0_index[ x1_size ] == 0 ) 
    {if ( val == default_pixel ) 
      return; 
    else 
      {ptr_pixels = new PixelType[ 1 ]; 
      ptr_x0_index = new unsigned[ 1 ]; 
      pos = 0; 
      ptr_pixels[ pos ] = val; 
      ptr_x0_index[ pos ] = x_0; 
      for ( i = x_1 + 1; i <= x1_size; i++ ) 
	ptr_x0_index[ i ]++;}} 
  else 
    {pos = ptr_x0_index[ x_1 ]; 
    if ( ptr_x0_index[ x_1 ] < ptr_x0_index[ x_1 + 1 ] ) 
      {while ( ( pos <= ptr_x0_index[ x_1 + 1 ] - 1 ) && ( ptr_x0_index[ pos ] < x_0 ) ) 
	pos++; //OJO QUE PODRIA HABER PROBLEMAS CON LA COMPARACION ANTERIOR 
      
	if ( ( pos < ptr_x0_index[ x1_size ] ) && ( ptr_x0_index[ pos ] == x_0 ) && ( pos < ptr_x0_index[ x_1 + 1 ] ) ) // ACA 
	  {if ( val == default_pixel ) 
	    {ptr_pixels_aux = new PixelType [ ptr_x0_index[ x1_size ] - 1 ]; 
	    ptr_x0_aux = new unsigned [ ptr_x0_index[ x1_size ] - 1 ]; 
	    for ( i = 0; i < pos; i++ ) 
	      {ptr_pixels_aux[ i ] = ptr_pixels[ i ]; 
	      ptr_x0_aux[ i ] = ptr_x0_index[ i ];} 
	    for ( i = pos + 1; i < ptr_x0_index[ x1_size ]; i++ ) 
	      {ptr_pixels_aux[ i - 1 ] = ptr_pixels[ i ]; 
	      ptr_x0_aux[ i - 1] = ptr_x0_index[ i ];} 
	    delete [] ptr_pixels; 
	    delete [] ptr_x0_index; 
	    ptr_pixels = ptr_pixels_aux; 
	    ptr_x0_index = ptr_x0_aux; 
	    for ( i = x_1 + 1; i <= x1_size; i++ ) 
	      ptr_x0_index[ i ]--;} 
	  else 
	    ptr_pixels[ pos ] = val; 
	  return;} 
	else 
	  {if ( val == default_pixel ) 
	    return; 
	  else 
	    { ptr_pixels_aux = new PixelType [ ptr_x0_index[ x1_size ] + 1 ]; 
	    ptr_x0_aux = new unsigned [ ptr_x0_index[ x1_size ] + 1 ]; 
	    for ( i = 0; i < pos; i++ ) 
	      {ptr_pixels_aux[ i ] = ptr_pixels[ i ]; 
	      ptr_x0_aux[ i ] = ptr_x0_index[ i ];} 
	    ptr_pixels_aux[ pos ] = val; 
	    ptr_x0_aux[ pos ] = x_0; 
	    for ( i = pos + 1; i <= ptr_x0_index[ x1_size ]; i++ ) 
	      {ptr_pixels_aux[ i ] = ptr_pixels[ i - 1 ]; 
	      ptr_x0_aux[ i ] = ptr_x0_index[ i - 1 ];} 
	    delete [] ptr_pixels; 
	    delete [] ptr_x0_index; 
	    ptr_pixels = ptr_pixels_aux; 
	    ptr_x0_index = ptr_x0_aux; 
	    for ( i = x_1 + 1; i <= x1_size; i++ ) 
	      ptr_x0_index[ i ]++; 
	    return;}}} 
    else  
      {ptr_pixels_aux = new PixelType [ ptr_x0_index[ x1_size ] + 1 ]; 
      ptr_x0_aux = new unsigned [ ptr_x0_index[ x1_size ] + 1 ]; 
      for ( i = 0; i < pos; i++ )  
	{ptr_pixels_aux[ i ] = ptr_pixels[ i ]; 
	ptr_x0_aux[ i ] = ptr_x0_index[ i ];} 
      ptr_pixels_aux[ pos ] = val; 
      ptr_x0_aux[ pos ] = x_0; 
      for ( i = pos + 1; i <= ptr_x0_index[ x1_size ]; i++ ) 
	{ptr_pixels_aux[ i ] = ptr_pixels[ i - 1 ]; 
	ptr_x0_aux[ i ] = ptr_x0_index[ i - 1 ];} 
      delete [] ptr_pixels; 
      delete [] ptr_x0_index; 
      ptr_pixels = ptr_pixels_aux; 
      ptr_x0_index = ptr_x0_aux; 
      for ( i = x_1 + 1; i <= x1_size; i++ ) 
	ptr_x0_index[ i ]++; 
      return;}}
}; 


template< class PixelType >  
PixelType bicotiImageImplementation2DSparse< PixelType > :: GetPixel( int x_0, int x_1 ) 
{ 
  unsigned pos; 

#ifdef TOTAL_CHECK
  if ( ( x_1 < 0 ) || ( ( x_1 + 1 ) > x1_size ) || ( x_0 < 0 ) || ( ( x_0 + 1 ) > x0_size ) ) 
    throw bicotiException( "bicotiImageImplementation2DSparse", "GetPixel()", 1 );
#endif
   
  pos = ptr_x0_index[ x_1 ]; 
  while( ( pos < ptr_x0_index[ x_1 + 1 ] ) && ( ptr_x0_index[ pos ] < x_0 ) ) 
    pos++; 
  if ( ( pos < ptr_x0_index[ x_1 + 1 ] ) && ( ptr_x0_index[ pos ] == x_0 ) ) 
    return( ptr_pixels[ pos ] ); 
  else 
    return( default_pixel );
}; 

 
template< class PixelType >  
bicotiImageIterator< PixelType > * bicotiImageImplementation2DSparse< PixelType > :: CreateIterator() 
{ 
  return( new bicotiImageIterator2DSparse< PixelType >( this ) ); 
}; 
 
 
template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiImageImplementation2DSparse< PixelType > :: CreateImageImplementation()
{
  return( new bicotiImageImplementation2DSparse< PixelType >( default_pixel, x0_size, x1_size, scale, pixel_case ) );
};


template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiImageImplementation2DSparse< PixelType > :: CreateImageImplementation( bicotiImageDescriptor * ptr_desc )
{
  double scale_aux = ptr_desc->GetScale();
  PixelCases pix_case_aux = ptr_desc->GetPixelCase();
  
  if ( ( ptr_desc->GetDimension() ) != 2 )
    throw bicotiException( "bicotiImageImplementation2DSparse", "CreateImageImplementation()", 2 );
  
  unsigned x0_size_aux = ptr_desc->Size( 0 );
  unsigned x1_size_aux = ptr_desc->Size( 1 );
  
  return( new bicotiImageImplementation2DSparse< PixelType >( default_pixel, x0_size_aux, x1_size_aux, scale_aux, pix_case_aux ) );
};
 

 
//************************IMPLEMENTATIONS*************************
//--------------------bicotiImageIterator2DSparse-----------------
template< class PixelType >  
void bicotiImageIterator2DSparse< PixelType > :: CreateMe( bicotiImageImplementation2DSparse< PixelType > * ptr_I ) 
{   
  if ( ( ptr_I->GetX0Size() != 0 ) && ( ptr_I->GetX1Size() != 0 ) ) 
    {is_done = false; 
    ptr_image = ptr_I; 
    first_x0 = 0; 
    first_x1 = 0; 
    last_x0 = ptr_image->x0_size - 1; 
    last_x1 = ptr_image->x1_size - 1; 
    current_x0 = first_x0; 
    current_x1 = first_x1; 
    ptr_x0 = new PixelType[ ptr_image->x0_size ];
    Refresh();} 
  else 
    throw bicotiException( "bicotiImageImplementation2DSparse", "CreateMe()", 6 );
}; 


template< class PixelType >  
bicotiImageIterator2DSparse< PixelType > :: bicotiImageIterator2DSparse( bicotiImageImplementation2DSparse< PixelType > * ptr_I ) 
{   
  CreateMe( ptr_I ); 
  x0_strategy = bicotiImageBorderStrategy2DX0< PixelType >( this ); 
  x1_strategy = bicotiImageBorderStrategy2DX1< PixelType >( this );  
}; 
 
 
template< class PixelType >  
bicotiImageIterator2DSparse< PixelType > :: bicotiImageIterator2DSparse( bicotiImageImplementation2DSparse< PixelType > * ptr_I, bicotiImageBorderStrategy2D< PixelType > * ptr_hor, bicotiImageBorderStrategy2D< PixelType > * ptr_ver ) 
{   
//---Esto me parece que ta mal ---
/*
  CreateMe( ptr_I ); 
  ptr_x0_strat = ptr_hor;  
  ptr_x1_strat = ptr_ver;  
  ptr_x0_strat->UseIterator( this ); 
  ptr_x1_strat->UseIterator( this ); 
*/
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DSparse< PixelType > :: SetCurrentPixel( PixelType value ) 
{   
  ptr_image->SetPixel( current_x0, current_x1, value );
  ptr_x0[ current_x0 ] = value;
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DSparse< PixelType > :: LoadX0( int x_1 ) 
{   
  unsigned pos = ptr_image->ptr_x0_index[ x_1 ], dest = 0; 
  while( pos < ptr_image->ptr_x0_index[ x_1 + 1 ] ) 
    {while( dest < ptr_image->ptr_x0_index[ pos ] ) 
      ptr_x0[ dest++ ] = ptr_image->default_pixel; 
    ptr_x0[ dest++ ] = ptr_image->ptr_pixels[ pos++ ];};
  while( dest < ptr_image->x0_size ) 
    ptr_x0[ dest++ ] = ptr_image->default_pixel; 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DSparse< PixelType > :: Refresh() 
{  
  LoadX0( current_x1 ); 
  ptr_current = ptr_x0 + current_x0;  
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DSparse< PixelType > :: SetExtremeCoordinates( int first_x_0, int first_x_1, int last_x_0, int last_x_1 ) 
{ 
  if( ( last_x_1 < first_x_1 ) || ( last_x_0 < first_x_0 ) ) 
    throw bicotiException( "bicotiImageIterator2DSparse", "SetExtremeCoordinates()", 1 );
  else 
    {if( ( last_x_1 >= 0 ) && ( last_x_1 < ptr_image->GetX1Size() ) ) 
      last_x1 = last_x_1; 
    else 
      throw bicotiException( "bicotiImageIterator2DSparse", "SetExtremeCoordinates()", 1 );};
  
  if( ( last_x_0 >= 0 ) && ( last_x_0 < ptr_image->GetX0Size() ) ) 
    last_x0 = last_x_0; 
  else 
    throw bicotiException( "bicotiImageIterator2DSparse", "SetExtremeCoordinates()", 1 );
  
  first_x1 = first_x_1; 
  first_x0 = first_x_0; 
  First(); 
}; 


template< class PixelType >  
PixelType& bicotiImageIterator2DSparse< PixelType > :: operator*() 
{ 
UpdatePix(); 
  return( *ptr_current ); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DSparse< PixelType > :: UpdatePix() 
{   
  ptr_image->SetPixel( current_x0, current_x1, *ptr_current );  
}; 
 
 
template< class PixelType >  
void bicotiImageIterator2DSparse< PixelType > :: UpdateX0() 
{ 
  unsigned i;  
  for( i = 0; i < ptr_image->GetX0Size(); i++ ) 
    {ptr_image->SetPixel( i, current_x1, ptr_x0[ i ] );} 
};



#endif
