//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _image_implementation_3D 
#define _image_implementation_3D 


 
#include <source/image_implementation/image_implementation_basic.hpp>


 
//*************************DECLARATIONS***************************
//-------------------bicotiImageImplementation3D------------------
template< class PixelType >  
class bicotiImageIterator3D; 

template< class PixelType >  
class bicotiImageFrameIterator3D; 

template< class PixelType >  
class bicotiImageExtrapolationStrategy3D; 

 
template< class PixelType >  
class bicotiImageImplementation3D : public bicotiImageImplementation< PixelType > 
{ 
protected:	  
  int x0_size, x1_size, x2_size; 
  
  virtual void DeleteMe() = 0;  
  virtual void CreateMe( PixelType, int, int, int, double, PixelCases ) = 0;  
 
public: 
  virtual ~bicotiImageImplementation3D(){}; 
 
  bicotiImageDescriptor * CreateImageDescriptor();

  int GetX0Size(); 
  int GetX1Size(); 
  int GetX2Size(); 
   
  virtual void SetPixel( int, int, int, const PixelType& ) = 0; 
  virtual PixelType GetPixel( int, int, int ) = 0; 
  virtual PixelType GetPixel( bicotiCoordinate< INTEGER >& ) = 0;
  virtual void SetPixel( bicotiCoordinate< INTEGER >&, const PixelType& ) = 0;
  virtual bicotiImageIterator< PixelType > * CreateIterator() = 0; 
 
  void CopyMeFrom( bicotiImageImplementation3D< PixelType > * ); 
 
  virtual bicotiImageImplementation< PixelType > * CreateImageImplementation() = 0;
  virtual bicotiImageImplementation< PixelType > * CreateImageImplementation( bicotiImageDescriptor * ) = 0;

  bicotiImageFrameIterator< PixelType > * CreateFrameIterator( int, bicotiImageExtrapolationStrategy< PixelType > * ); 

  void Resize( bicotiImageDescriptor * );
  void Resize( int, int, int );
}; 
 
 
 
//----------------------bicotiImageIterator3D---------------------
template< class PixelType > class bicotiImageBorderStrategy3DX0; 
template< class PixelType > class bicotiImageBorderStrategy3DX1; 
template< class PixelType > class bicotiImageBorderStrategy3DX2; 
 
template< class PixelType >  
class bicotiImageIterator3D : public bicotiImageIterator< PixelType > 
{ 
  friend class bicotiImageBorderStrategy3DX0< PixelType >; 
  friend class bicotiImageBorderStrategy3DX1< PixelType >; 
  friend class bicotiImageBorderStrategy3DX2< PixelType >; 
   
protected: 
  int current_x0, current_x1, current_x2; 
  int first_x0, first_x1, first_x2, last_x0, last_x1, last_x2; 
   
  bicotiImageBorderStrategy3DX0< PixelType > x0_strategy; 
  bicotiImageBorderStrategy3DX1< PixelType > x1_strategy; 
  bicotiImageBorderStrategy3DX2< PixelType > x2_strategy; 
 
  virtual void Refresh() = 0; 
   
public: 
  virtual ~bicotiImageIterator3D(){}; 
 
  void operator++( int ); 
  void operator--( int ); 
  void operator+=( unsigned ); 
  void operator-=( unsigned ); 
   
  void X0Decrement( unsigned = 1 ); 
  void X0Increment( unsigned = 1 ); 
  void X1Decrement( unsigned = 1 ); 
  void X1Increment( unsigned = 1 ); 
  void X2Increment( unsigned = 1 ); 
  void X2Decrement( unsigned = 1 ); 
   
  void Locate( int, int, int ); 
  void First(); 
  void Last(); 
  int GetCurrentX0(); 
  int GetCurrentX1(); 
  int GetCurrentX2(); 

  bicotiCoordinate< INTEGER > GetCurrentCoordinateInteger();
  bicotiCoordinate< REAL > GetCurrentCoordinateReal();
   
  void UseBorderStrategyX0( bicotiImageBorderStrategy3DX0< PixelType > ); 
  void UseBorderStrategyX1( bicotiImageBorderStrategy3DX1< PixelType > ); 
  void UseBorderStrategyX2( bicotiImageBorderStrategy3DX2< PixelType > ); 
   
  virtual void SetCurrentPixel( PixelType ); 
  virtual void SetExtremeCoordinates( int, int, int, int, int, int ) = 0; 
}; 
 
 

//----------------------BorderStrategy3D--------------------------
template< class PixelType >  
class bicotiImageBorderStrategy3D : public bicotiImageBorderStrategy< PixelType > 
{ 
protected: 
  bicotiImageIterator3D< PixelType > * ptr_iterator; 
  int step, level;
 
public: 
  void UseIterator( bicotiImageIterator3D< PixelType > * );
  void UseStep( int );
  void UseLevel( int ); 
}; 
 
 
template< class PixelType >  
class bicotiImageBorderStrategy3DX0 : public bicotiImageBorderStrategy3D< PixelType > 
{  
public: 
  bicotiImageBorderStrategy3DX0( int = 1, int = 0 ); 
  bicotiImageBorderStrategy3DX0( bicotiImageIterator3D< PixelType > *, int = 1 , int = 0 ); 
  
  void SetNextCoordinates(); 
}; 
 
 
template< class PixelType >  
class bicotiImageBorderStrategy3DX1 : public bicotiImageBorderStrategy3D< PixelType > 
{  
public: 
  bicotiImageBorderStrategy3DX1( int = 0, int = 1 ); 
  bicotiImageBorderStrategy3DX1( bicotiImageIterator3D< PixelType > *, int = 0, int = 1 ); 
   
  void SetNextCoordinates(); 
}; 
 
 
template< class PixelType >  
class bicotiImageBorderStrategy3DX2 : public bicotiImageBorderStrategy3D< PixelType > 
{ 
public: 
  bicotiImageBorderStrategy3DX2( int = 0, int = 0 ); 
  bicotiImageBorderStrategy3DX2( bicotiImageIterator3D< PixelType > *, int = 0, int = 0 ); 
   
  void SetNextCoordinates(); 
}; 
 


//----------------------bicotiFrameIterator-----------------------
template< class PixelType >   
class bicotiImageFrameIterator3D : public bicotiImageFrameIterator< PixelType >  
{  
private:  
  unsigned current_x0, current_x1, current_x2;  
            
  bicotiMatrix3D< PixelType > * ptr_matrix;  
  bicotiImageExtrapolationStrategy3D< PixelType > * ptr_bord_strat;  
  bicotiFrame3DExternal< PixelType > * ptr_frame;  
  bicotiImageImplementation3D< PixelType >* ptr_image;  
  void Reload( unsigned, unsigned, unsigned );  
  
public:  
  bicotiImageFrameIterator3D( bicotiImageImplementation3D< PixelType > *, int, bicotiImageExtrapolationStrategy3D< PixelType > * );  
  ~bicotiImageFrameIterator3D();  
    
  bicotiFrame< PixelType > * GetCurrentFrame();  
  
  void First();  
  void Last();  
  void Locate( unsigned, unsigned, unsigned );  
  
  unsigned GetCurrentX0();  
  unsigned GetCurrentX1();  
  unsigned GetCurrentX2();  
  
  void operator++( int );  
  void operator--( int );  
 
  void operator+=( unsigned ); // OJO Habra que sacarle el unsigned  
  void operator-=( unsigned );  
}; 




//***********************IMPLEMENTATIONS**************************
//----------------------Implementation3D--------------------------
template< class PixelType >
bicotiImageDescriptor * bicotiImageImplementation3D< PixelType > :: CreateImageDescriptor()
{
  bicotiImageDescriptor * ptr_ima_desc;
  ptr_ima_desc = new bicotiImageDescriptor( 3, scale, pixel_case );

  ptr_ima_desc->Size(0) = x0_size;
  ptr_ima_desc->Size(1) = x1_size;
  ptr_ima_desc->Size(2) = x2_size;

  return ( ptr_ima_desc );
};


template< class PixelType > 
inline int bicotiImageImplementation3D< PixelType > :: GetX0Size() 
{ 
  return( x0_size ); 
}; 
 
 
template< class PixelType > 
inline int bicotiImageImplementation3D< PixelType > :: GetX1Size() 
{ 
  return( x1_size ); 
}; 
 
 
template< class PixelType > 
inline int bicotiImageImplementation3D< PixelType > :: GetX2Size() 
{ 
  return( x2_size ); 
}; 
 
 
template< class PixelType >  
void bicotiImageImplementation3D< PixelType > :: CopyMeFrom( bicotiImageImplementation3D< PixelType > * ptr_ima_src )  
{  
  bicotiImageIterator< PixelType > * ptr_iter_src, * ptr_iter;  
  
  if ( ( x0_size != ptr_ima_src->x0_size ) || ( x1_size != ptr_ima_src->x1_size ) || ( x2_size != ptr_ima_src->x2_size ) )  
    {DeleteMe();  
    CreateMe( ptr_ima_src->default_pixel, ptr_ima_src->x0_size, ptr_ima_src->x1_size, ptr_ima_src->x2_size, ptr_ima_src->scale, ptr_ima_src->pixel_case );}  
  else
    {pixel_case = ptr_ima_src->pixel_case;
    default_pixel = ptr_ima_src->default_pixel;
    scale = ptr_ima_src->scale;}
  
  ptr_iter_src = ptr_ima_src->CreateIterator();  
  ptr_iter = CreateIterator();  
  
  ptr_iter_src->First();  
  ptr_iter->First();  
  
  do  
    {( **ptr_iter ) = ( **ptr_iter_src );  
    ( *ptr_iter )++;  
    ( *ptr_iter_src) ++;} while ( !ptr_iter->IsDone() );  

  delete ptr_iter;  
  delete ptr_iter_src;  
}; 
 
 
template< class PixelType >  
bicotiImageFrameIterator< PixelType >* bicotiImageImplementation3D< PixelType > :: CreateFrameIterator( int frm_size, bicotiImageExtrapolationStrategy< PixelType > * ptr_fbs ) 
{ 
  return( new bicotiImageFrameIterator3D< PixelType >( this, frm_size, ( bicotiImageExtrapolationStrategy3D< PixelType > * )ptr_fbs ) );  
}; 
 
  
template< class PixelType > 
void bicotiImageImplementation3D< PixelType > :: Resize( bicotiImageDescriptor *  ptr_desc ) 
{ 
  if( ptr_desc->GetDimension() != 3 )
    throw bicotiException( "bicotiImageImplementation3D", "Resize()", 2 );
  
  DeleteMe();
  CreateMe( default_pixel, ptr_desc->Size( 0 ), ptr_desc->Size( 1 ), ptr_desc->Size( 2 ), scale, pixel_case );
}; 


template< class PixelType > 
void bicotiImageImplementation3D< PixelType > :: Resize( int x0_size, int x1_size, int x2_size ) 
{ 
  DeleteMe();
  CreateMe( default_pixel, x0_size, x1_size, x2_size, scale, pixel_case );
}; 



//--------------------bicotiImageIterator3D-----------------------
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: First() 
{ 
  is_done = false; 
  current_x0 = first_x0; 
  current_x1 = first_x1; 
  current_x2 = first_x2; 
  Refresh(); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: Last() 
{ 
  is_done = false; 
  current_x0 = last_x0; 
  current_x1 = last_x1; 
  current_x2 = last_x2; 
  Refresh(); 
}; 
 
 
template< class PixelType >  
int bicotiImageIterator3D< PixelType > :: GetCurrentX0() 
{	 
  return( current_x0 );	 
}; 
 
 
template< class PixelType >  
int bicotiImageIterator3D< PixelType > :: GetCurrentX1() 
{	 
  return( current_x1 );	 
}; 
 
 
template< class PixelType >  
int bicotiImageIterator3D< PixelType > :: GetCurrentX2() 
{	 
  return( current_x2 );	 
}; 
 
 
template< class PixelType >
inline bicotiCoordinate< INTEGER > bicotiImageIterator3D< PixelType > :: GetCurrentCoordinateInteger()
{
  bicotiCoordinate< INTEGER > aux_coord( 3 );

  aux_coord( 0 ) = ( INTEGER )current_x0;
  aux_coord( 1 ) = ( INTEGER )current_x1;
  aux_coord( 2 ) = ( INTEGER )current_x2;

  return( aux_coord );
};


template< class PixelType >
inline bicotiCoordinate< REAL > bicotiImageIterator3D< PixelType > :: GetCurrentCoordinateReal()
{
  bicotiCoordinate< REAL > aux_coord( 3 );

  aux_coord( 0 ) = ( REAL )current_x0;
  aux_coord( 1 ) = ( REAL )current_x1;
  aux_coord( 2 ) = ( REAL )current_x2;

  return( aux_coord );
};


template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: UseBorderStrategyX0( bicotiImageBorderStrategy3DX0< PixelType > xstrat )
{	 
  x0_strategy = xstrat;	 
  x0_strategy.UseIterator( this ); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: UseBorderStrategyX1( bicotiImageBorderStrategy3DX1< PixelType > ystrat ) 
{	 
  x1_strategy = ystrat;	 
  x1_strategy.UseIterator( this ); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: UseBorderStrategyX2( bicotiImageBorderStrategy3DX2< PixelType > zstrat ) 
{	 
  x2_strategy = zstrat;	 
  x2_strategy.UseIterator( this ); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: SetCurrentPixel( PixelType value ) 
{	 
  *ptr_current = value;	 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: operator+=( unsigned stp ) 
{	 
  X0Increment( stp );	   
}; 
 
 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: operator-=( unsigned stp ) 
{	 
  X0Decrement( stp );	   
}; 
 
 
template< class PixelType >  
inline void bicotiImageIterator3D< PixelType > :: operator++( int ) 
{ 	 
  current_x0++; 
  if( current_x0 > last_x0 ) 
    {x0_strategy.SetNextCoordinates(); 

    if( ( ( last_x1 < current_x1 ) || ( first_x1 > current_x1 ) ) && ( !is_done ) ) 
       x1_strategy.SetNextCoordinates();
     
    if( ( ( current_x2 > last_x2 ) || ( first_x2 > current_x2 ) ) && ( !is_done ) ) 
      is_done = true;}
   
  if ( !is_done )
    Refresh();		 
}; 
 
 
 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: Locate( int x_0, int x_1, int x_2 ) 
{ 
  is_done = false; 
  if ( ( x_1 >= first_x1 ) && ( x_1 <= last_x1 ) ) 
    current_x1 = x_1; 
  if ( ( x_0 >= first_x0 ) && ( x_0 <= last_x0 ) ) 
    current_x0 = x_0; 
  if ( ( x_2 >= first_x2 ) && ( x_2 <= last_x2 ) ) 
    current_x2 = x_2; 
  Refresh(); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: operator--( int ) 
{ 	 
  current_x0--; 
  if ( first_x0 > current_x0 ) 
    {x0_strategy.SetNextCoordinates();					 
     
    if( ( ( current_x1 > last_x1 ) || ( first_x1 > current_x1 ) ) && ( !is_done ) )	 
      x1_strategy.SetNextCoordinates(); 
     
    if( ( ( current_x2 > last_x2 ) || ( first_x2 > current_x2 ) ) && ( !is_done ) )			 
       is_done = true;}//x2_strategy.SetNextCoordinates();} 
   
  if ( !is_done )//porque el Refresh carga un puntero 
    Refresh(); 
}; 

 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: X0Decrement( unsigned step ) 
{  
  int stp = step; 
  while( stp > ( current_x0 - first_x0 ) ) 
    {stp = stp - ( current_x0 - first_x0 ) - 1; 
    current_x0 = first_x0; 
    ( *this )--;} 
  current_x0 -= stp; 
   
  if ( !is_done ) 
    Refresh(); 
}; 

 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: X0Increment( unsigned step ) 
{ 
  int stp = step; 
  while( stp > ( last_x0 - current_x0 ) ) 
    {stp = stp - ( last_x0 - current_x0 ) - 1; 
    current_x0 = last_x0; 
    ( *this )++;} 
  current_x0 += stp; 

  if ( !is_done ) 
    Refresh(); 
}; 
 
 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: X1Decrement( unsigned step ) 
{  
  int stp = step; 
  while( stp > ( current_x1 - first_x1 ) ) 
    {stp = stp - ( current_x1 - first_x1 ) - 1; 
    current_x1 = first_x1; 
    current_x1--;
    if ( first_x1 > current_x1 )
      {x1_strategy.SetNextCoordinate();		
      
      if( ( ( current_x0 > last_x0 ) || ( first_x0 > current_x0 ) ) && ( !is_done ) )
	x0_strategy.SetNextCoordinates();
      
      if( ( ( current_x2 > last_x2 ) || ( first_x2 > current_x2 ) ) && ( !is_done ) )
	is_done = true;}
    
    if (!is_done)//porque el Refresh carga un puntero
      Refresh();} 
  
  current_x1 -= stp; 
  if ( !is_done ) 
    Refresh(); 
}; 

 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: X1Increment( unsigned step ) 
{  
  int stp = step; 
  while( stp > ( last_x1 - current_x1 ) ) 
    {stp = stp - ( last_x1 - current_x1 ) - 1; 
    current_x1 = last_x1; 
    current_x1++;
    if ( current_x1 > last_x1 )
      {x1_strategy.SetNextCoordinates();
      
      if( ( ( current_x0 > last_x0 ) || ( first_x0 > current_x0 ) ) && ( !is_done ) )
	x0_strategy.SetNextCoordinates();
      
      if( ( ( current_x2 > last_x2 ) || ( first_x2 > current_x2 ) )	&& ( !is_done ) )		
	is_done = true;}
    
    if (!is_done)
      Refresh();} 
  
  current_x1 += stp;  
  if ( !is_done ) 
    Refresh(); 
}; 

 
template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: X2Increment( unsigned step ) 
{  
  int stp = step; 
  while( stp > ( last_x2 - current_x2 ) ) 
    {stp = stp - ( last_x2 - current_x2 ) - 1; 
    current_x2 = last_x2; 
    
    current_x2++; 
    if ( current_x2 > last_x2 ) 
      {x2_strategy.SetNextCoordiante();			 
      
      if( ( ( current_x0 > last_x0 ) || ( first_x0 > current_x0 ) ) && ( !is_done ) ) 
	x0_strategy.SetNextCoordiante(); 
      
      if( ( ( current_x1 > last_x1 ) || ( first_x1 > current_x1 ) ) && ( !is_done ) )
	is_done = true;}} 
  current_x2 += stp; 
  
  if ( !is_done ) 
    Refresh(); 
}; 


template< class PixelType >  
void bicotiImageIterator3D< PixelType > :: X2Decrement( unsigned step ) 
{ 	 
  int stp = step; 
  while( stp > ( current_x2 - first_x2 ) ) 
    {stp = stp - ( current_x2 - first_x2 ) - 1; 
    current_x2 = first_x2; 
    
    current_x2--; 
    if ( first_x2 > current_x2 ) 
      {x2_strategy.SetNextCoordiante();				 
      
      if( ( ( current_x0 > last_x0 ) || ( first_x0 > current_x0 ) ) && ( !is_done ) ) 
	x0_strategy.SetNextCoordiante(); 
      
      if( ( ( current_x1 > last_x1 ) || ( first_x1 > current_x1 ) ) && ( !is_done ) )
	is_done = true;}}
  
  current_x2 -= stp; 
  
  if ( !is_done ) 
    Refresh(); 
}; 



//------------------bicotiImageBorderStrategy3D-------------------
template< class PixelType >  
void bicotiImageBorderStrategy3D< PixelType > :: UseIterator( bicotiImageIterator3D< PixelType > * ptr_it ) 
{ 
  ptr_iterator = ptr_it; 
}; 


template< class PixelType >  
void bicotiImageBorderStrategy3D< PixelType > :: UseStep( int st ) 
{ 
  step = st; 
}; 


template< class PixelType >  
void bicotiImageBorderStrategy3D< PixelType > :: UseLevel( int st_level ) 
{ 
  level = st_level; 
}; 



//-----------------bicotiImageBorderStrategy3DX0------------------
template< class PixelType >  
bicotiImageBorderStrategy3DX0< PixelType > :: bicotiImageBorderStrategy3DX0( int st, int st_level ) 
{ 
  step = st; 
  level = st_level; 
  ptr_iterator = NULL; 
}; 
 
 
template< class PixelType >  
bicotiImageBorderStrategy3DX0< PixelType > :: bicotiImageBorderStrategy3DX0( bicotiImageIterator3D< PixelType > * ptr_it, int st, int st_level ) 
{ 
  step = st; 
  level = st_level; 
  ptr_iterator = ptr_it; 
}; 
 
 
template< class PixelType >  
void bicotiImageBorderStrategy3DX0< PixelType > :: SetNextCoordinates() 
{	 
  if ( ( step == 0 ) && ( level == 0 ) ) 
    ptr_iterator->is_done = true; 
  else 
    {if( ( ptr_iterator->current_x0 > ptr_iterator->last_x0 ) ) 
      {ptr_iterator->current_x2 += level; 
      ptr_iterator->current_x0 = ptr_iterator->first_x0; 
      ptr_iterator->current_x1 += step;} 
    else 
      {if ( ( ptr_iterator->first_x0 > ptr_iterator->current_x0 ) ) 
	{ptr_iterator->current_x2 -= level; 
	ptr_iterator->current_x0 = ptr_iterator->last_x0; 
	ptr_iterator->current_x1 -= step;}}} 
}; 
 
 
 
//-----------------bicotiImageBorderStrategy3DX1------------------
template< class PixelType >  
bicotiImageBorderStrategy3DX1< PixelType > :: bicotiImageBorderStrategy3DX1( int st, int st_level ) 
{ 
  step = st; 
  level = st_level; 
  ptr_iterator = NULL; 
}; 
 
 
template< class PixelType >  
bicotiImageBorderStrategy3DX1< PixelType > :: bicotiImageBorderStrategy3DX1( bicotiImageIterator3D< PixelType > * ptr_it, int st, int st_level ) 
{ 
  step = st; 
  level = st_level; 
  ptr_iterator = ptr_it; 
}; 
 
 
template< class PixelType >  
void bicotiImageBorderStrategy3DX1< PixelType > :: SetNextCoordinates() 
{	 
  if( ( step == 0 ) && ( level == 0 ) ) 
    ptr_iterator->is_done = true; 
  else 
    {if( ( ptr_iterator->current_x1 > ptr_iterator->last_x1 )) 
      {ptr_iterator->current_x2 += level; 
      ptr_iterator->current_x1 = ptr_iterator->first_x1; 
      ptr_iterator->current_x0 += step;} 
    else 
      {if( ( ptr_iterator->first_x1 > ptr_iterator->current_x1 ) ) 
	{ptr_iterator->current_x2 -= level; 
	ptr_iterator->current_x1 = ptr_iterator->last_x1; 
	ptr_iterator->current_x0 -= step;}}} 
}; 
 
 
 
//-----------------bicotiImageBorderStrategy3DX2------------------
template< class PixelType >  
bicotiImageBorderStrategy3DX2< PixelType > :: bicotiImageBorderStrategy3DX2( int st, int st_level ) 
{ 
  step = st; 
  level = st_level; 
  ptr_iterator = NULL; 
}; 
 
 
template< class PixelType >  
bicotiImageBorderStrategy3DX2< PixelType > :: bicotiImageBorderStrategy3DX2( bicotiImageIterator3D< PixelType > * ptr_it, int st, int st_level ) 
{ 
  step = st; 
  level = st_level; 
  ptr_iterator = ptr_it; 
}; 
 
 
template< class PixelType >  
void bicotiImageBorderStrategy3DX2< PixelType > :: SetNextCoordinates() 
{ 
  if( ( step == 0 ) && ( level == 0 ) ) 
    ptr_iterator->is_done = true; 
  else 
    {if( ( ptr_iterator->current_x2 > ptr_iterator->last_x2 ) ) 
      {ptr_iterator->current_x1 += level; 
      ptr_iterator->current_x2 = ptr_iterator->first_x2; 
      ptr_iterator->current_x0 += step;} 
    else 
      {if( ( ptr_iterator->first_x2 > ptr_iterator->current_x2 ) ) 
	{ptr_iterator->current_x1 -= level; 
	ptr_iterator->current_x2 = ptr_iterator->last_x2; 
	ptr_iterator->current_x0 -= step;}}} 
}; 
 
 

//------------------bicotiImageFrameIterator3D--------------------
template< class PixelType >   
bicotiImageFrameIterator3D< PixelType > :: bicotiImageFrameIterator3D( bicotiImageImplementation3D< PixelType >* ptr_ima, int fs, bicotiImageExtrapolationStrategy3D< PixelType > * ptr_fbs ) 
{ 
  if ( fs%2 == 0 ) 
    throw bicotiException( "bicotiImageFrameIterator3D", "Constructor", 4 );

  if ( fs < 0 ) 
    throw bicotiException( "bicotiImageFrameIterator3D", "Constructor", 4 );

  ptr_image = ptr_ima;   
  ptr_matrix = new bicotiMatrix3D< PixelType >( ptr_image->GetDefaultPixel(), ( ptr_image->GetX0Size() + fs - 1 ), fs, fs);    
  ptr_bord_strat = ptr_fbs;  
  ptr_bord_strat->SetImplementation( ptr_image );  
  ptr_frame = new bicotiFrame3DExternal< PixelType >( ptr_matrix, fs );  
  current_x0 = current_x1 = current_x2 = 0;  
  Reload( current_x0, current_x1, current_x2 ); 
};  
  
  
template< class PixelType >  
void bicotiImageFrameIterator3D< PixelType > :: Reload( unsigned current_x_1, unsigned current_x_2, unsigned current_x_3)  
{  
  unsigned i, j, k;  
  int fs_local = ptr_frame->GetFrameSize();  
  
  for( i = 0; i < ( ptr_image->GetX0Size() + fs_local - 1 ); i++ )  
    for( j = 0; j < fs_local; j++ )  
      for( k = 0; k < fs_local; k++ )  
	( *ptr_matrix )( i, j, k ) = ( *ptr_bord_strat )( i - fs_local/2, current_x_2 + j - fs_local/2, current_x_3 + k - fs_local/2 );
};
 

template< class PixelType >   
inline void bicotiImageFrameIterator3D< PixelType > :: operator++( int step ) 
{  
  if( current_x0 < ( ptr_image->GetX0Size() - 1 ) )  
    {current_x0++;  
    ptr_frame->Locate( current_x0, 0, 0 );}  
  else  
    {if( current_x1 < ( ptr_image->GetX1Size() - 1 ) )  
      {current_x1++;  
      current_x0 = 0;  
      Reload( current_x0, current_x1, current_x2 ); 
      ptr_frame->Locate( 0, 0, 0 );} 
    else  
      {if( current_x2 < ( ptr_image->GetX2Size() - 1 ) )  
	{current_x2++;  
	current_x0 = 0;  
	current_x1 = 0;  
	Reload( current_x0, current_x1, current_x2 );  
	ptr_frame->Locate( 0, 0, 0 );} 
      else  
	is_done = true;}}    
}; 


template< class PixelType >   
void bicotiImageFrameIterator3D< PixelType > :: operator--( int step ) 
{  
  
  if ( current_x0 > 0 )  
    {current_x0--;  
    ptr_frame->Locate( current_x0, 0, 0 );}  
  else  
    {if ( current_x1 > 0 )  
      {current_x1--;  
      current_x0 = ptr_image->GetX0Size() - 1;  
      Reload( current_x0, current_x1, current_x2 );  
      ptr_frame->Locate( 0, 0, 0 );}  
    else  
      {if ( current_x2 > 0 )  
	{current_x2--;  
	current_x0 = ptr_image->GetX0Size() - 1;  
	current_x1 = ptr_image->GetX1Size() - 1;  
	Reload( current_x0, current_x1, current_x2 );  
	ptr_frame->Locate( 0, 0, 0 );} 
      else  
	is_done = true;}}  
};

 
 
template< class PixelType >   
void bicotiImageFrameIterator3D< PixelType > :: operator+=( unsigned step ) 
{  
  for( unsigned i = 0; i < step; i++ )  
    ( *this )++; 
}; 


template< class PixelType >   
void bicotiImageFrameIterator3D< PixelType > :: operator-=( unsigned step ) 
{  
  for( unsigned i = 0; i < step; i++ )  
    ( *this )--; 
};  


template< class PixelType >   
unsigned bicotiImageFrameIterator3D< PixelType > :: GetCurrentX0()  
{  
  return( current_x0 );  
};  

  
template< class PixelType >   
unsigned bicotiImageFrameIterator3D< PixelType > :: GetCurrentX1()  
{  
  return( current_x1 );  
};  
  
  
template< class PixelType >   
unsigned bicotiImageFrameIterator3D< PixelType > :: GetCurrentX2()  
{  
  return( current_x2 );  
};  
  
  
template< class PixelType >   
void bicotiImageFrameIterator3D< PixelType > :: First()  
{  
  is_done = false;  
  
  ptr_frame->Locate( 0, 0, 0 );  
  current_x0 = current_x1 = current_x2 = 0;  
};  
  
  
template< class PixelType >   
void bicotiImageFrameIterator3D< PixelType > :: Last()  
{  
  is_done = false;  
  
  current_x0 = ptr_image->GetX0Size() - 1;  
  current_x1 = ptr_image->GetX1Size() - 1;  
  current_x2 = ptr_image->GetX2Size() - 1;  
  
  ptr_frame->Locate( current_x0, current_x1, current_x2 );  
};  
  
  
template< class PixelType >   
void bicotiImageFrameIterator3D< PixelType > :: Locate( unsigned x_1, unsigned x_2, unsigned x_3 )  
{  
  current_x0 = x_1;  
  current_x1 = x_2;  
  current_x2 = x_3;  
  
  ptr_frame->Locate( current_x0, current_x1, current_x2 );  
};  
  
  
template< class PixelType >   
bicotiImageFrameIterator3D< PixelType > :: ~bicotiImageFrameIterator3D()  
{   
  delete ptr_frame;  
  delete ptr_matrix;  
};  
  
  
template< class PixelType >  
bicotiFrame< PixelType > * bicotiImageFrameIterator3D< PixelType > :: GetCurrentFrame()  
{  
  return ( ptr_frame );  
}; 



#endif        
