//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _image_implementation_3D_linear
#define _image_implementation_3D_linear


#include <source/image_implementation/image_implementation_3D.hpp>

 
 
//***********************DECLARATIONS*****************************
//--------------------------Linear--------------------------------
template< class PixelType >  
class bicotiImageIterator3DLinear; 
 
 
template< class PixelType >  
class bicotiImageImplementation3DLinear : public bicotiImageImplementation3D< PixelType > 
{    
  friend class bicotiImageIterator3DLinear< PixelType >; 
 
private: 
  PixelType * ptr_pixels;  
  void DeleteMe();  
  void CreateMe( PixelType, int, int, int, double, PixelCases ); 
   
public: 
  bicotiImageImplementation3DLinear( PixelType, int = 0, int = 0, int = 0, double = 1, PixelCases = unknown ); 
  ~bicotiImageImplementation3DLinear(); 
  bicotiImageImplementation3DLinear( const bicotiImageImplementation3DLinear< PixelType >& ); 
    
  void SetPixel( bicotiCoordinate< unsigned >&, const PixelType& );
  PixelType GetPixel( bicotiCoordinate< unsigned >& );

  void SetPixel( int, int, int, const PixelType& ); 
  PixelType GetPixel( int, int, int ); 
   
  bicotiImageIterator< PixelType > * CreateIterator(); 

  bicotiImageImplementation< PixelType > * CreateImageImplementation();
  bicotiImageImplementation< PixelType > * CreateImageImplementation( bicotiImageDescriptor * );
}; 
 
 

//----------------bicotiImageIterator3DLinear---------------------
template< class PixelType >  
class bicotiImageIterator3DLinear : public bicotiImageIterator3D< PixelType > 
{
private: 
  bicotiImageImplementation3DLinear< PixelType > * ptr_image; 
  void CreateMe( bicotiImageImplementation3DLinear< PixelType > * ); 
  void Refresh(); 
 
public: 
  bicotiImageIterator3DLinear( bicotiImageImplementation3DLinear< PixelType > * ); 
  bicotiImageIterator3DLinear( bicotiImageImplementation3DLinear< PixelType > *, bicotiImageBorderStrategy3DX0< PixelType >, bicotiImageBorderStrategy3DX1< PixelType >, bicotiImageBorderStrategy3DX2< PixelType > ); 
  ~bicotiImageIterator3DLinear(); 
   
  void SetExtremeCoordinates( int, int, int, int, int, int ); 
};   
 
 
 
//************************IMPLEMENTATIONS*************************
//-----------------------------Linear-----------------------------
template< class PixelType >  
bicotiImageImplementation3DLinear< PixelType > :: bicotiImageImplementation3DLinear( PixelType def_pixel, int x_0, int x_1, int x_2, double scale_value, PixelCases pix_case ) 
{ 
  CreateMe( def_pixel, x_0, x_1, x_2, scale_value, pix_case ); 
}; 
  
 
template< class PixelType >  
bicotiImageImplementation3DLinear< PixelType > :: ~bicotiImageImplementation3DLinear() 
{  
  DeleteMe(); 
}; 
 
 
template< class PixelType >  
void bicotiImageImplementation3DLinear< PixelType > :: CreateMe( PixelType def_pixel, int x_0, int x_1, int x_2, double scale_value, PixelCases pix_case )  
{ 
  if ( ( x_1 >= 0 ) || ( x_0 >= 0 ) || ( x_2 >= 0 ) )   
    {int i;  
    
    default_pixel = def_pixel;  
    x0_size = x_0;  
    x1_size = x_1;  
    x2_size = x_2;  
    scale = scale_value; 
    pixel_case = pix_case;
    
    ptr_pixels = new PixelType[ x1_size * x2_size * x0_size ];  
    
    for ( i = 0; i < ( x1_size * x2_size * x0_size ); i++ )  
      ptr_pixels[ i ] = default_pixel;}
  else   
    throw bicotiException( "bicotiImageImplementation3DLinear", "CreateMe()", 2 );
};


template< class PixelType >   
void bicotiImageImplementation3DLinear< PixelType > :: DeleteMe()  
{ 
  delete[] ptr_pixels;  
} 
 
 
template< class PixelType >  
bicotiImageImplementation3DLinear< PixelType > :: bicotiImageImplementation3DLinear( const bicotiImageImplementation3DLinear< PixelType >& image_src ) 
{ 
  default_pixel = image_src.default_pixel; 
  x0_size = image_src.x0_size; 
  x1_size = image_src.x1_size; 
  x2_size = image_src.x2_size; 
  scale = image_src.scale; 
  pixel_case = image_src.pixel_case;
   
  int i; 
   
  ptr_pixels = new PixelType[ x1_size * x2_size * x0_size ]; 
  
  for ( i = 0; i < ( x1_size * x2_size * x0_size ); i++ ) 
    ptr_pixels[ i ] = image_src.ptr_pixels[ i ]; 
}; 

 
template< class PixelType >  
void bicotiImageImplementation3DLinear< PixelType > :: SetPixel( bicotiCoordinate< INTEGER >& coord, const PixelType& value ) 
{ 
  unsigned x_0, x_1, x_2;
  bicotiCoordinate3D< INTEGER > aux_coord = (bicotiCoordinate3D< INTEGER >) coord;
  
  x_0 = aux_coord.X0();
  x_1 = aux_coord.X1();
  x_2 = aux_coord.X2();
  
  int temp = x1_size * x0_size; 

#ifdef TOTAL_CHECK
  if( ( ( x_1 + 1 ) > x1_size ) || ( ( x_0 + 1 ) > x0_size ) || ( ( x_2 + 1 ) > x2_size ) ) 
    throw bicotiException( "bicotiImageImplementation3DLinear", "SetPixel()", 1 );
#endif

  ptr_pixels[ x_1 * x0_size + x_0 + x_2 * temp ] = value; 
}; 


template< class PixelType >  
PixelType bicotiImageImplementation3DLinear< PixelType > :: GetPixel( bicotiCoordinate< INTEGER >& coord ) 
{ 
  unsigned x_0, x_1, x_2;
  
  bicotiCoordinate3D< INTEGER > aux_coord = (bicotiCoordinate3D< INTEGER >) coord;
  
  x_0 = aux_coord.X0();
  x_1 = aux_coord.X1();
  x_2 = aux_coord.X2();
  
  int temp = x1_size * x0_size; 

#ifdef TOTAL_CHECK
  if( ( ( x_1 + 1 ) > x1_size ) || ( ( x_0 + 1 ) > x0_size ) || ( ( x_2 + 1 ) > x2_size ) ) 
    throw bicotiException( "bicotiImageImplementation3DLinear", "GetPixel()", 1 );
#endif

  return( ptr_pixels[ x_1 * x0_size + x_0 + x_2 * temp ] ); 
}; 
 
 
template< class PixelType >  
void bicotiImageImplementation3DLinear< PixelType > :: SetPixel( int x_0, int x_1, int x_2, const PixelType& value ) 
{ 
  int temp = x1_size * x0_size; 
  
#ifdef TOTAL_CHECK
  if( ( ( x_1 + 1 ) > x1_size ) || ( ( x_0 + 1 ) > x0_size ) || ( ( x_2 + 1 ) > x2_size ) ) 
    throw bicotiException( "bicotiImageImplementation3DLinear", "SetPixel()", 1 );
#endif
  
  ptr_pixels[ x_1 * x0_size + x_0 + x_2 * temp ] = value; 
}; 
 

template< class PixelType >  
PixelType bicotiImageImplementation3DLinear< PixelType > :: GetPixel( int x_0, int x_1, int x_2 ) 
{ 
  int temp = x1_size * x0_size; 

#ifdef TOTAL_CHECK
  if( ( ( x_1 + 1 ) > x1_size ) || ( ( x_0 + 1 ) > x0_size ) || ( ( x_2 + 1 ) > x2_size ) ) 
    throw bicotiException( "bicotiImageImplementation3DLinear", "GetPixel()", 1 );
#endif
  
  return( ptr_pixels[ x_1 * x0_size + x_0 + x_2 * temp ] ); 
}; 

 
template< class PixelType >  
bicotiImageIterator< PixelType >* bicotiImageImplementation3DLinear< PixelType > :: CreateIterator() 
{ 
  return( new bicotiImageIterator3DLinear< PixelType >( this ) ); 
}; 
 
 
template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiImageImplementation3DLinear< PixelType > :: CreateImageImplementation()
{
  return( new bicotiImageImplementation3DLinear< PixelType >( default_pixel, x0_size, x1_size, x2_size, scale, pixel_case ) );
};

 
template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiImageImplementation3DLinear< PixelType > :: CreateImageImplementation( bicotiImageDescriptor * ptr_desc )
{
  double scale_aux = ptr_desc->GetScale();
  PixelCases pix_case_aux = ptr_desc->GetPixelCase();
  
  if ( ( ptr_desc->GetDimension() ) != 3 )
    throw bicotiException( "bicotiImageImplementation3DLinear", "CreateImageImplementation()", 2 );
  
  unsigned x0_size_aux = ptr_desc->Size( 0 );
  unsigned x1_size_aux = ptr_desc->Size( 1 );
  unsigned x2_size_aux = ptr_desc->Size( 2 );
  
  return( new bicotiImageImplementation3DLinear< PixelType >( default_pixel, x0_size_aux, x1_size_aux ,x2_size_aux, scale_aux, pix_case_aux ) );
};
 

 
//-------------------bicotiImageIterator3DLinear------------------
template< class PixelType >  
void bicotiImageIterator3DLinear< PixelType > :: CreateMe( bicotiImageImplementation3DLinear< PixelType > * ptr_I ) 
{    
  ptr_image = ptr_I; 
  if( ( ptr_image->x0_size != 0 ) && ( ptr_image->x1_size != 0 ) && ( ptr_image->x2_size != 0 ) )  
    {is_done = false; 
    first_x1 = 0; 
    first_x0 = 0; 
    first_x2 = 0;  
    last_x1 = ptr_image->x1_size - 1; 
    last_x0 = ptr_image->x0_size - 1; 
    last_x2 = ptr_image->x2_size - 1; 
    current_x0 = first_x0; 
    current_x1 = first_x1; 
    current_x2 = first_x2; 
    Refresh();}  
  else  
    throw bicotiException( "bicotiImageImplementation3DLinear", "CreateMe()", 6 );
}; 

 
template< class PixelType >  
void bicotiImageIterator3DLinear< PixelType > :: SetExtremeCoordinates( int first_x_0, int first_x_1, int first_x_2, int last_x_0, int last_x_1, int last_x_2 ) 
{ 
  if( ( last_x_0 < first_x_0 ) || ( last_x_1 < first_x_1 ) || ( last_x_2 < first_x_2 ) )
    throw bicotiException( "bicotiImageIterator3DLinear", "SetExtremeCoordinates()", 1 );
  else 
    {if( ( last_x_0 >= 0 ) && ( last_x_0 < ptr_image->GetX0Size() ) )
      last_x0 = last_x_0;
    else
      throw bicotiException( "bicotiImageIterator3DLinear", "SetExtremeCoordinates()", 1 );};
  
  if( ( last_x_1 >= 0 ) && ( last_x_1 < ptr_image->GetX1Size() ) )
    last_x1 = last_x_1;
  else
    throw bicotiException( "bicotiImageIterator3DLinear", "SetExtremeCoordinates()", 1 );
  
  if( ( last_x_2 >= 0 ) && ( last_x_2 < ptr_image->GetX2Size() ) )
    last_x2 = last_x_2;
  else
    throw bicotiException( "bicotiImageIterator3DLinear", "SetExtremeCoordinates()", 1 );
  
  first_x1 = first_x_1; 
  first_x0 = first_x_0;
  first_x2 = first_x_2;
  First(); 
}; 


template< class PixelType >  
bicotiImageIterator3DLinear< PixelType > :: bicotiImageIterator3DLinear( bicotiImageImplementation3DLinear< PixelType > * ptr_I ) 
{ 
  CreateMe( ptr_I ); 
  x0_strategy = bicotiImageBorderStrategy3DX0< PixelType >( this ); 
  x1_strategy = bicotiImageBorderStrategy3DX1< PixelType >( this ); 
  x2_strategy = bicotiImageBorderStrategy3DX2< PixelType >( this ); 
}; 


template< class PixelType >  
bicotiImageIterator3DLinear< PixelType > :: bicotiImageIterator3DLinear( bicotiImageImplementation3DLinear< PixelType > * ptr_I, bicotiImageBorderStrategy3DX0< PixelType > xstrat, bicotiImageBorderStrategy3DX1< PixelType > ystrat, bicotiImageBorderStrategy3DX2< PixelType > zstrat ) 
{ 
  CreateMe( ptr_I ); 
  x0_strategy = xstrat; 
  x1_strategy = ystrat; 
  x2_strategy = zstrat; 
}; 
 
 
 
template< class PixelType >  
bicotiImageIterator3DLinear< PixelType > :: ~bicotiImageIterator3DLinear() 
{	 
}; 
 
 
template< class PixelType >  
inline void bicotiImageIterator3DLinear< PixelType > :: Refresh() 
{ 
  ptr_current = ptr_image->ptr_pixels + ( current_x1 * ( ptr_image->x0_size ) ) + ( current_x2 * ( ptr_image->x1_size ) * ( ptr_image->x0_size ) ) + current_x0; 
};         


#endif
