//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _image_implementation_basic 
#define _image_implementation_basic 


 
#include <source/frame/frame.hpp>


 
//***********************DECLARATIONS****************************
//-------------------bicotiImageDescriptor-----------------------
class bicotiImageDescriptor
{
private:
  vector< unsigned > dimensions_list;
  PixelCases pixel_case;
  double scale;

public:
  bicotiImageDescriptor( unsigned = 0, double = 1, PixelCases = unknown );
  
  void operator=( const bicotiImageDescriptor& );

  unsigned GetDimension();
  unsigned& Size( int );
  double GetScale();

  PixelCases GetPixelCase();
};



//--------------------ImageImplementations-----------------------
template < class PixelType > 
class bicotiImageIterator; 

 
template < class PixelType > 
class bicotiImageFrameIterator; 

 
template < class PixelType > 
class bicotiImageExtrapolationStrategy; 
 

template < class PixelType > 
class bicotiImageImplementation		 
{ 
protected: 
  double scale; 
  PixelType default_pixel; 
  PixelCases pixel_case;
 
public:  
  virtual ~bicotiImageImplementation(){}; 
 
  virtual bicotiImageDescriptor * CreateImageDescriptor() = 0;

  double GetScale(); 
  PixelType GetDefaultPixel(); 
  PixelCases PixelCase();

  virtual PixelType GetPixel( bicotiCoordinate< INTEGER >& ) = 0;
  virtual void SetPixel( bicotiCoordinate< INTEGER >&, const PixelType& ) = 0;
  
  virtual bicotiImageIterator< PixelType > * CreateIterator() = 0; 
  virtual bicotiImageFrameIterator< PixelType > * CreateFrameIterator( int, bicotiImageExtrapolationStrategy< PixelType > * ) = 0; 
  virtual bicotiImageImplementation< PixelType > * CreateImageImplementation() = 0;
  virtual bicotiImageImplementation< PixelType > * CreateImageImplementation( bicotiImageDescriptor * ) = 0;
  virtual void Resize( bicotiImageDescriptor * ) = 0;
}; 


 
//---------------------bicotiImageIterator-----------------------
template< class PixelType >  
class bicotiImageIterator 
{ 
protected: 
  bool is_done; 
  PixelType * ptr_current; 
   
public: 
  PixelType& operator*(); 
  virtual void operator++( int ) = 0; 
  virtual void operator--( int ) = 0; 
  virtual void operator+=( unsigned ) = 0; 
  virtual void operator-=( unsigned ) = 0; // OJO Habra que eliminar unsigned en ambos casos 
 
  virtual void First() = 0; 
  virtual void Last() = 0; 
   
  PixelType * GetCurrentPixel(); 
  virtual void SetCurrentPixel( PixelType ) = 0; 
  
  virtual bicotiCoordinate< INTEGER > GetCurrentCoordinateInteger() = 0;
  virtual bicotiCoordinate< REAL > GetCurrentCoordinateReal() = 0;

  bool IsDone(); 
}; 
 


//---------------------ImageBorderStrategy-----------------------
template< class PixelType >  
class bicotiImageBorderStrategy 
{   
public: 
  virtual void SetNextCoordinates() = 0; 
}; 
 

 
//------------------bicotiImageFrameIterator---------------------
template< class PixelType >  
class bicotiImageFrameIterator 
{ 
protected: 
  bool is_done;   
 
public: 
  virtual ~bicotiImageFrameIterator(){}; 
   
  bool IsDone(); 

  virtual bicotiFrame< PixelType > * GetCurrentFrame() = 0; 
 
  virtual void First() = 0; 
  virtual void Last() = 0; 
 
  virtual void operator++( int ) = 0; 
  virtual void operator--( int ) = 0; 
  virtual void operator+=( unsigned ) = 0; // OJO Habra que sacarle el unsigned 
  virtual void operator-=( unsigned ) = 0; 
}; 
 
 

 
//***********************IMPLEMENTATIONS*************************
//------------------bicotiImageImplementation--------------------
template< class PixelType >  
PixelType bicotiImageImplementation< PixelType > :: GetDefaultPixel() 
{ 
  return( default_pixel ); 
}; 
 
 
template< class PixelType >  
PixelCases bicotiImageImplementation< PixelType > :: PixelCase() 
{ 
  return( pixel_case ); 
}; 
 
 
template< class PixelType >  
double bicotiImageImplementation< PixelType > :: GetScale() 
{ 
  return( scale ); 
}; 
 
 

//-----------------------bicotiImageIterator----------------------
template< class PixelType >  
inline PixelType& bicotiImageIterator< PixelType > :: operator*() 
{ 
  return( *ptr_current ); 
}; 
 
 
template< class PixelType >  
inline bool bicotiImageIterator< PixelType > :: IsDone() 
{  
  return( is_done ); 
}; 
 
 
template< class PixelType >  
inline PixelType * bicotiImageIterator< PixelType > :: GetCurrentPixel() 
{ 
  return( ptr_current ); 
}; 
 


//---------------------bicotiImageFrameIterator-------------------
template< class PixelType >  
inline bool bicotiImageFrameIterator< PixelType > :: IsDone() 
{ 
   return ( is_done ); 
}; 



//---------------------------Copy---------------------------------
template< class PixelType1, class PixelType2 > 
void Copy( bicotiImageImplementation< PixelType1 > ** ptr_ima_dest, bicotiImageImplementation< PixelType2 > * ptr_ima_src ) 
{ 
  bicotiImageImplementation< PixelType1 > * ptr_ima_aux = *ptr_ima_dest;
  
  bicotiImageDescriptor * ptr_desc;
  ptr_desc = ptr_ima_src->CreateImageDescriptor();
  *ptr_ima_dest = ptr_ima_aux->CreateImageImplementation( ptr_desc );
  delete ptr_ima_aux;
  
  bicotiImageIterator< PixelType1 > * ptr_iter_dest; 
  bicotiImageIterator< PixelType2 > * ptr_iter_src; 
  
  ptr_iter_dest = ( *ptr_ima_dest )->CreateIterator(); 
  ptr_iter_src = ptr_ima_src->CreateIterator(); 
  
  for( ptr_iter_dest->First(); !ptr_iter_dest->IsDone(); ( *ptr_iter_dest )++ , ( *ptr_iter_src )++ ) 
    ( **ptr_iter_dest ) = ( PixelType1 ) ( **ptr_iter_src ); 

  delete ptr_iter_dest;
  delete ptr_iter_src;
};  



#endif                     
