//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _image_pixel_interpolator_3D_lagrange
#define _image_pixel_interpolator_3D_lagrange



#include <source/image_pixel_interpolator/image_pixel_interpolator_3D.hpp>



//************************DECLARATIONS****************************
template< class PixelType, class PrecisionType = PixelType >  
class bicotiImagePixelInterpolator3DLagrange : public bicotiImagePixelInterpolator3D< PixelType > 
{  
public:   
  bicotiImagePixelInterpolator3DLagrange( bicotiImageImplementation3D< PixelType > * = NULL );  
  
  PixelType operator() ( bicotiCoordinate< REAL >& );
};  




//***********************IMPLEMENTATIONS**************************
//-------------bicotiImagePixelInterpolator2DLagrange-------------
template< class PixelType, class PrecisionType >  
bicotiImagePixelInterpolator3DLagrange< PixelType, PrecisionType > :: bicotiImagePixelInterpolator3DLagrange(bicotiImageImplementation3D< PixelType >* ptr_ima_imp)  
{  
  ptr_image_imp = ptr_ima_imp;
};


template< class PixelType, class PrecisionType >
inline PixelType bicotiImagePixelInterpolator3DLagrange< PixelType, PrecisionType > :: operator() ( bicotiCoordinate< REAL >& coord )     
{  
  PrecisionType pixel_inter;
  bicotiCoordinate3D< REAL > aux_coord;
  aux_coord = (bicotiCoordinate3D< REAL >)coord;
  
#ifdef TOTAL_CHECK
  if ( ( aux_coord.GetDimension() != 3 ) || ( ptr_image_imp == NULL ) )
    throw bicotiException( "bicotiImagePixelInterpolator3DLagrange", "operator()", 3 );
#endif
  
  if ( InImage( aux_coord ) && ( !InBorder( aux_coord ) ) )
    {bicotiCoordinate3D< REAL > coord_0, coord_1, coord_2, coord_3, coord_4, coord_5, coord_6, coord_7;
    
    REAL aux_inter_x0, aux_inter_x1, aux_inter_x2;
    
    aux_inter_x0 = ( REAL )floor( aux_coord.X0() );
    aux_inter_x1 = ( REAL )floor( aux_coord.X1() );
    aux_inter_x2 = ( REAL )floor( aux_coord.X2() );
    
    coord_0.X0() = aux_inter_x0;
    coord_0.X1() = aux_inter_x1 + 1;
    coord_0.X2() = aux_inter_x2;
    
    coord_1.X0() = aux_inter_x0 + 1;
    coord_1.X1() = aux_inter_x1 + 1;
    coord_1.X2() = aux_inter_x2;
    
    coord_2.X0() = aux_inter_x0;
    coord_2.X1() = aux_inter_x1 + 1;
    coord_2.X2() = aux_inter_x2 + 1;
    
    coord_3.X0() = aux_inter_x0 + 1;
    coord_3.X1() = aux_inter_x1 + 1;
    coord_3.X2() = aux_inter_x2 + 1;
    
    coord_4.X0() = aux_inter_x0;
    coord_4.X1() = aux_inter_x1;
    coord_4.X2() = aux_inter_x2;
    
    coord_5.X0() = aux_inter_x0 + 1;
    coord_5.X1() = aux_inter_x1;
    coord_5.X2() = aux_inter_x2;
    
    coord_6.X0() = aux_inter_x0;
    coord_6.X1() = aux_inter_x1;
    coord_6.X2() = aux_inter_x2 + 1;
    
    coord_7.X0() = aux_inter_x0 + 1;
    coord_7.X1() = aux_inter_x1;
    coord_7.X2() = aux_inter_x2 + 1;
    

    REAL aux_r, aux_s, aux_t, w0, w1, w2, w3, w4, w5, w6, w7;
    aux_r = ( aux_coord.X0() - coord_0.X0() );
    aux_s = ( aux_coord.X2() - coord_0.X2() );
    aux_t = ( coord_0.X1() - aux_coord.X1() );
    
    w0 = ( 1 - aux_r ) * ( 1 - aux_s ) * ( 1 - aux_t );
    w1 = aux_r * ( 1 - aux_s ) * ( 1 - aux_t );
    w2 = ( 1 - aux_r ) * aux_s * ( 1 - aux_t );
    w3 = aux_r * aux_s * ( 1 - aux_t ) ;   
    w4 = ( 1 - aux_r ) * ( 1 - aux_s ) * aux_t;
    w5 = aux_r * ( 1 - aux_s ) * aux_t;
    w6 = ( 1 -aux_r ) * aux_s * aux_t;
    w7 = aux_r * aux_s * aux_t;
    
    PrecisionType pixel_aux_0, pixel_aux_1, pixel_aux_2, pixel_aux_3, pixel_aux_4, pixel_aux_5, pixel_aux_6, pixel_aux_7;
    
    int temp_x0, temp_x1, temp_x2;
    
    temp_x0 = (int)coord_0.X0(); 
    temp_x1 = (int)coord_0.X1();
    temp_x2 = (int)coord_0.X2();
    pixel_aux_0 = (PrecisionType)( ( ( bicotiImageImplementation3D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1, temp_x2 ) );
    
    temp_x0 = (int)coord_1.X0(); 
    temp_x1 = (int)coord_1.X1();
    temp_x2 = (int)coord_1.X2();
    pixel_aux_1 = (PrecisionType)( ( ( bicotiImageImplementation3D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1, temp_x2 ) );
    
    temp_x0 = (int)coord_2.X0(); 
    temp_x1 = (int)coord_2.X1();
    temp_x2 = (int)coord_2.X2();
    pixel_aux_2 = (PrecisionType)( ( ( bicotiImageImplementation3D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1, temp_x2 ) );
    
    temp_x0 = (int)coord_3.X0(); 
    temp_x1 = (int)coord_3.X1();
    temp_x2 = (int)coord_3.X2();
    pixel_aux_3 = (PrecisionType)( ( ( bicotiImageImplementation3D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1, temp_x2 ) );
    
    temp_x0 = (int)coord_4.X0(); 
    temp_x1 = (int)coord_4.X1();
    temp_x2 = (int)coord_4.X2();
    pixel_aux_4 = (PrecisionType)( ( ( bicotiImageImplementation3D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1, temp_x2 ) );
    
    temp_x0 = (int)coord_5.X0(); 
    temp_x1 = (int)coord_5.X1();
    temp_x2 = (int)coord_5.X2();
    pixel_aux_5 = (PrecisionType)( ( ( bicotiImageImplementation3D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1, temp_x2 ) );
    
    temp_x0 = (int)coord_6.X0(); 
    temp_x1 = (int)coord_6.X1();
    temp_x2 = (int)coord_6.X2();
    pixel_aux_6 = (PrecisionType)( ( ( bicotiImageImplementation3D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1, temp_x2 ) );
    
    temp_x0 = (int)coord_7.X0(); 
    temp_x1 = (int)coord_7.X1();
    temp_x2 = (int)coord_7.X2();
    pixel_aux_7 = (PrecisionType)( ( ( bicotiImageImplementation3D< PixelType> *)ptr_image_imp)->GetPixel( temp_x0, temp_x1, temp_x2 ) );
    
    pixel_inter = pixel_aux_0 * w0 + pixel_aux_1 * w1 + pixel_aux_2 * w2 + pixel_aux_3 * w3 + pixel_aux_4 * w4 + pixel_aux_5 * w5 + pixel_aux_6 * w6 + pixel_aux_7 * w7;}
  else
    pixel_inter = ( PrecisionType)( ptr_image_imp->GetDefaultPixel() );
  
  return( (PixelType)pixel_inter );
};



#endif
