#ifndef _bicoti_interaction_menu
#define _bicoti_interaction_menu

#include <stl.h>
#include <source/bicoti_basic.hpp>
#include <source/actor/actor_basic.hpp>

//--------------------DECLARATIONS---------------------
//-----------------bicotiListenerMenu----------------

/** 
 * Listener de eventos de Men.
 *
 * Extienda esta clase en su aplicacin implementando el mtodo ItemSelected para que haga lo que usted desea.
 *Ejemplo:
 *<pre>
 *@@  class MyListenerMenu : public bicotiListenerMenu;
 *@@  { void ItemSelected( bicotiActor * ptr_actor, char * item ); }
 *</pre>
 *@see bicotiInteractionSourceMenu
 */ 
class bicotiListenerMenu
{
public:

  /**
   * Este mtodo se ejecuta cuando se selecciona una opcin del men.
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerMenu :: ItemSelected( bicotiActor * ptr_actor, char * item )
   *@@  { cout << " Se seleccion la opcin: " << item << endl; }
   *</pre>
   *@param ptr_actor puntero al actor que produjo el evento
   *@param item nombre del item seleccionado 
   */
  virtual void ItemSelected( bicotiActor * ptr_actor, char * item )=0;
};



//-----------bicotiInteractionSourceMenu-----

/**
 * Fuente de eventos de men.
 *
 * clase base para bicotiActorGraphicMenu.
 *@see bicotiActorGraphicMenu
 */
class bicotiInteractionSourceMenu
{
private:
  
  class InternalPair
  {
  public:
    bicotiListenerMenu * listener;
    char * item_name;

    InternalPair( bicotiListenerMenu * = NULL, char * = NULL );
    
    bool operator==( const InternalPair& ) const;
  };

protected:
  vector< InternalPair > listener_menu;
  
  /**
   * Verifica que el item pertenezca al men.
   *
   *@param item nombre de una opcin.
   *@returns true si item pertenece al men o false en caso contrario.
   */
  virtual bool CheckItem( char * item )=0;
  
public:
 /**
   * Registra un listener de eventos de men.
   *
   *@param ptr_listener listener de eventos a agregar.
   *@param item nombre de la opcin que el listener quiere escuchar.
   *@exception bicotiException si item no es una opcin del men.
   */ 
  void AddListenerMenu( bicotiListenerMenu * ptr_listener, char * item);
 
    /**
   * Quita un listener de eventos de scroll bar.
   *
   *@param ptr_listener listener de eventos a quitar.
   *@param item opcin que el listener se registro para escuchar.
   */
  void RemoveListenerMenu( bicotiListenerMenu * ptr_listener, char * item );

  /**
   * Dispara eventos de men ( item seleccionado ).
   *
   * Este mtodo avisa a todo los listener registrados que ha ocurrido un evento de men (  item seleccionado ).
   *@param ptr_actor actor que dispara el evento.
   *@param item opcin que el listener se registro para escuchar.
   *@exception bicotiException si item no es una opcin del men.
   */ 
  void FireItemSelected( bicotiActor * ptr_actor, char * item );
};

#endif

