#ifndef _bicoti_interaction_push_button
#define _bicoti_interaction_push_button

#include <stl.h>
#include <source/actor/actor_basic.hpp>

//--------------------DECLARATIONS---------------------
//-----------------bicotiListenerPushButton----------------

/**
 * Listener de eventos de botn
 *
 * Extienda esta clase en su aplicacin implementando el mtodo TimerTic para que haga lo que usted desea.
 *Ejemplo:
 *<pre>
 *@@  class MyListenerPushButton : public bicotiListenerPushButton;
 *@@  { void PushButtonClicked( bicotiActor * ptr_actor ); }
 *</pre>
 *@see bicotiInteractionSourcePushButton  
 */ 
class bicotiListenerPushButton
{
public:
 
  /** 
  * Este mtodo se ejecuta cuando ocurre un evento de botn
  *
  * Debe reescribir este mtodo en su aplicacin
  * Ejemplo:
  *<pre>
  *@@  void MyListenerPushButton :: PushButtonClicked( bicotiActor * ptr_actor )
  *@@  { cout << " Botn clickeado " << endl; }
  *<pre>
  *@param ptr_actor puntero al actor que produjo el evento
  */
  virtual void PushButtonClicked( bicotiActor * ptr_actor )=0;
};



//-----------bicotiInteractionSourcePushButton-----

/**
 * Fuente de eventos de botn
 *
 * Clase base para bicotiActorGraphic2DPushButton
 *@see bicotiActorGraphic2DPushButton
 */
class bicotiInteractionSourcePushButton
{
protected:
  vector< bicotiListenerPushButton * > listener_button;
  
public:
  
  /**
   * Registra un listener de eventos de botn
   *
   *@param ptr_listener listener para agregar
   */ 
  void AddListenerPushButton( bicotiListenerPushButton * ptr_listener);
  
  /**
   * Quita un listener de eventos de botn
   *
   *@param ptr_listener listener para quitar
   */
  void RemoveListenerPushButton( bicotiListenerPushButton * ptr_listener );

  
  /**
   * Dispara eventos de botn
   *
   * Avisa a todos los listener registrados que se ha ocurrido un evento de botn
   *@param ptr_actor actor que dispara los eventos
   *@see bicotiListenerPushButton::PushButtonClicked
   */
  void FirePushButtonClicked( bicotiActor * ptr_actor);
};

#endif

