#ifndef _bicoti_interaction_timer
#define _bicoti_interaction_timer

#include <stl.h>
#include <source/actor/actor_basic.hpp>

//--------------------DECLARATIONS---------------------
//-----------------bicotiListenerTimer---------------

/** 
 * Listener de eventos de reloj
 *
 * Extienda esta clase en su aplicacin implementando el mtodo TimerTic para que haga lo que usted desea.
 *Ejemplo:
 *<pre>
 *@@  class MyListenerTimer : public bicotiListenerTimer;
 *@@  { void TimerTic( bicotiActor * ptr_actor ); }
 *</pre>
 *@see bicotiInteractionSourceTimer  
 */ 
class bicotiListenerTimer
{
public:
  /**
   * Este mtodo se ejecuta cuando ocurre un evento de reloj (timer)
   *
   * Debe reescribir este mtodo en su aplicacin
   * Ejemplo:
   *<pre>
   *@@  void MyListenerTimer :: TimerTic( bicotiActor * ptr_actor )
   *@@  { cout << " Ocurri un evento de reloj " << endl; }
   *</pre>
   *@param ptr_actor puntero al actor que produjo el evento
   */
  virtual void TimerTic( bicotiActor * ptr_actor )=0;
};


//------------bicotiInteractionSourceTimer-----------

/**
 * Fuente de eventos de reloj
 *
 * Clase base para bicotiActorTimer
 *@see bicotiActorTimer
 */
class bicotiInteractionSourceTimer
{
protected:
  vector< bicotiListenerTimer * > listener_timer;

public:
  /** 
   * Registra un listener de eventos de reloj 
   *
   *@param ptr_listener listener a registrar
   */ 
  void AddListenerTimer( bicotiListenerTimer * ptr_listener );
  
  /**
   * Quita un listener de eventos de reloj
   *
   *@param ptr_listener listener a quitar
   */
  void RemoveListenerTimer( bicotiListenerTimer * ptr_listener);
  
  /** 
   * Dispara eventos de reloj
   *
   * Este mtodo avisa a todos los listener registrados que ha ocurrido un evento de reloj.
   *@param ptr_actor puntero al actor que dispara el evento 
   *@see bicotiListenerTimer::TimerTic
   */ 
  void FireTimerTic( bicotiActor * ptr_actor);
};



#endif
