//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _neighbour_filter
#define _neighbour_filter


#include <source/neighbour_filter/neighbour_filter_basic.hpp>



//***********************DECLARATIONS*****************************
//----------------bicotiNeighbourFilterFacade---------------------
template< class PixelType >
class bicotiNeighbourFilterFacade
{
protected:
  bicotiImageImplementation< PixelType > * ptr_image_imp_src;
  bicotiImageImplementation< PixelType > * ptr_image_imp_dest;

  bicotiNeighbourFilter< PixelType > * ptr_neigh_filter;
  bicotiNeighbourFunction< PixelType > * ptr_neigh_funct;
  bicotiImageExtrapolationStrategy< PixelType > * ptr_strategy;

  virtual void DeleteMe() = 0;
  virtual void ResetPointers() = 0;

public:
  virtual ~bicotiNeighbourFilterFacade(){};
  
  virtual void BuildLinearFilter( unsigned ) = 0;
  virtual void BuildMedianFilter( unsigned ) = 0;
  virtual void BuildMaximumFilter( unsigned ) = 0;
  virtual void BuildMinimumFilter( unsigned ) = 0;

  void DestroyFilter();
  
  virtual void UseExtrapolationStrategyConstant( PixelType ) = 0;
  virtual void UseExtrapolationStrategyEqual() = 0;
  virtual void UseExtrapolationStrategySimetric() = 0;
  virtual void UseExtrapolationStrategyCiclic() = 0;
  
  void Filter();

  bicotiImageImplementation< PixelType > * GetResult();
  bicotiImageImplementation< PixelType > * LoadResult();
};



//--------------bicotiNeighbourFilterFacade2D---------------------
template< class PixelType , class PrecisionType = PixelType >
class bicotiNeighbourFilterFacade2D : public bicotiNeighbourFilterFacade< PixelType >
{
protected:
  bicotiFrame2DInternal< PrecisionType > * ptr_filter_kernel;

  void DeleteMe();
  void ResetPointers();

public:
  virtual ~bicotiNeighbourFilterFacade2D(){};
  
  void BuildLinearFilter( unsigned );

  void UseExtrapolationStrategyConstant( PixelType );
  void UseExtrapolationStrategyEqual();
  void UseExtrapolationStrategySimetric();
  void UseExtrapolationStrategyCiclic();

  bicotiFrame2DInternal< PrecisionType > * GetKernel();
};



//---------------bicotiNeighbourFilterFacade3D--------------------
template< class PixelType , class PrecisionType = PixelType >
class bicotiNeighbourFilterFacade3D : public bicotiNeighbourFilterFacade< PixelType >
{
protected:
  bicotiFrame3DInternal< PrecisionType > * ptr_filter_kernel;

  void DeleteMe();
  void ResetPointers();

public:
  virtual ~bicotiNeighbourFilterFacade3D(){};
  
  void BuildLinearFilter( unsigned );
  
  void UseExtrapolationStrategyConstant( PixelType );
  void UseExtrapolationStrategyEqual();
  void UseExtrapolationStrategySimetric();
  void UseExtrapolationStrategyCiclic();

  bicotiFrame3DInternal< PrecisionType > * GetKernel();
};



//-------------bicotiNeighbourFilterFacade2DSingle----------------
template< class PixelType , class PrecisionType = PixelType >
class bicotiNeighbourFilterFacade2DSingle : public bicotiNeighbourFilterFacade2D< PixelType, PrecisionType >
{
public:
  bicotiNeighbourFilterFacade2DSingle( bicotiImageImplementation< PixelType > * );
  ~bicotiNeighbourFilterFacade2DSingle();
  
  void BuildMedianFilter( unsigned );
  void BuildMaximumFilter( unsigned );
  void BuildMinimumFilter( unsigned );
};



//-----------bicotiNeighbourFilterFacade2DVectorial---------------
template< class ColorType, class PixelType , class PrecisionType = PixelType >
class bicotiNeighbourFilterFacade2DVectorial : public bicotiNeighbourFilterFacade2D< PixelType, PrecisionType >
{
public:
  bicotiNeighbourFilterFacade2DVectorial( bicotiImageImplementation< PixelType > * );
  ~bicotiNeighbourFilterFacade2DVectorial();
  
  void BuildMedianFilter( unsigned );
  void BuildMaximumFilter( unsigned );
  void BuildMinimumFilter( unsigned );
};



//-------------bicotiNeighbourFilterFacade3DSingle----------------
template< class PixelType, class PrecisionType = PixelType >
class bicotiNeighbourFilterFacade3DSingle : public bicotiNeighbourFilterFacade3D< PixelType, PrecisionType >
{
public:
  bicotiNeighbourFilterFacade3DSingle( bicotiImageImplementation< PixelType > * );
  ~bicotiNeighbourFilterFacade3DSingle();
  
  void BuildMedianFilter( unsigned );
  void BuildMaximumFilter( unsigned );
  void BuildMinimumFilter( unsigned );
};



//------------bicotiNeighbourFilterFacade3DVectorial--------------
template< class ColorType, class PixelType, class PrecisionType = PixelType >
class bicotiNeighbourFilterFacade3DVectorial : public bicotiNeighbourFilterFacade3D< PixelType, PrecisionType >
{
public:
  bicotiNeighbourFilterFacade3DVectorial( bicotiImageImplementation< PixelType > * );
  ~bicotiNeighbourFilterFacade3DVectorial();
  
  void BuildMedianFilter( unsigned );
  void BuildMaximumFilter( unsigned );
  void BuildMinimumFilter( unsigned );
};




//*********************IMPLEMENTATIONS****************************
//--------------bicotiNeighbourFilterFacade-----------------------
template< class PixelType >
void bicotiNeighbourFilterFacade< PixelType > :: DestroyFilter()
{
  DeleteMe();
};


template< class PixelType >
void bicotiNeighbourFilterFacade< PixelType > :: Filter()
{
  if ( ptr_neigh_filter == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade", "Filter()", 6 );
  
  if ( ptr_strategy == NULL )
    UseExtrapolationStrategyEqual();

  ptr_neigh_filter->Apply();
};


template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiNeighbourFilterFacade< PixelType > :: GetResult()
{
  bicotiImageImplementation< PixelType > * ptr_aux;

  if( ptr_image_imp_dest == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade", "GetResult()", 5 );

  ptr_aux = ptr_image_imp_dest; 
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter->SetImagesImplementations( ptr_image_imp_dest, ptr_image_imp_src );

  return ( ptr_aux );
};


template< class PixelType >
bicotiImageImplementation< PixelType > * bicotiNeighbourFilterFacade< PixelType > :: LoadResult()
{
  if( ptr_image_imp_dest == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade", "LoadResult()", 5 );

  ptr_image_imp_src = ptr_image_imp_dest;
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();
  
  ptr_neigh_filter->SetImagesImplementations( ptr_image_imp_dest, ptr_image_imp_src );

  return( ptr_image_imp_src );
};



//---------------bicotiNeighbourFilterFacade2D--------------------
template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > :: DeleteMe()
{
  if ( ptr_filter_kernel != NULL )
    delete ptr_filter_kernel;
  
  if ( ptr_neigh_funct != NULL )
    delete ptr_neigh_funct;
  
  if ( ptr_neigh_filter != NULL )
    delete ptr_neigh_filter;
  
  if ( ptr_image_imp_dest != NULL )
    delete ptr_image_imp_dest;
  
  if ( ptr_strategy != NULL )
    delete ptr_strategy;
  
  ResetPointers();
}


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > :: ResetPointers()
{
  ptr_filter_kernel = NULL;
  ptr_neigh_funct = NULL;
  ptr_neigh_filter = NULL;
  ptr_image_imp_dest = NULL;
  ptr_strategy = NULL;
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > :: BuildLinearFilter( unsigned frame_size )
{
  DeleteMe();
  
  PrecisionType def_pix;
  def_pix *= 0;

  ptr_filter_kernel = new bicotiFrame2DInternal< PrecisionType >( def_pix, frame_size );
  
  ptr_neigh_funct = new bicotiNeighbourFunctionLinear< PixelType, PrecisionType >( ptr_filter_kernel );
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();
  
  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > :: UseExtrapolationStrategyConstant( PixelType default_pixel )
{
  if ( ptr_neigh_filter == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade2D", "UseExtrapolationStrategyConstant()", 6 );
  
  if ( ptr_strategy != NULL )
    delete ptr_strategy;
  
  ptr_strategy = new bicotiImageExtrapolationStrategy2DConstant< PixelType >( default_pixel );
  
  ptr_neigh_filter->SetExtrapolationStrategy( ptr_strategy );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > :: UseExtrapolationStrategyEqual()
{
  if ( ptr_neigh_filter == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade2D", "UseExtrapolationStrategyEqual()", 6 );
  
  if ( ptr_strategy != NULL )
    delete ptr_strategy;

  ptr_strategy = new bicotiImageExtrapolationStrategy2DEqual< PixelType >;
  
  ptr_neigh_filter->SetExtrapolationStrategy( ptr_strategy );
};

  
template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > :: UseExtrapolationStrategySimetric()
{
  if ( ptr_neigh_filter == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade2D", "UseExtrapolationStrategySimetric()", 6 );

  if ( ptr_strategy != NULL )
    delete ptr_strategy;

  ptr_strategy = new bicotiImageExtrapolationStrategy2DSimetric< PixelType >;
  
  ptr_neigh_filter->SetExtrapolationStrategy( ptr_strategy );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > :: UseExtrapolationStrategyCiclic()
{
  if ( ptr_neigh_filter == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade2D", "UseExtrapolationStrategyCiclic()", 6 );

  if ( ptr_strategy != NULL )
    delete ptr_strategy;

  ptr_strategy = new bicotiImageExtrapolationStrategy2DCiclic< PixelType >;
  
  ptr_neigh_filter->SetExtrapolationStrategy( ptr_strategy );
};


template< class PixelType, class PrecisionType >
bicotiFrame2DInternal< PrecisionType > * bicotiNeighbourFilterFacade2D< PixelType, PrecisionType > :: GetKernel()
{
  if ( ptr_filter_kernel == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade2D", "Kernel()", 6 );
  
  return ( ptr_filter_kernel );
};



//---------------bicotiNeighbourFilterFacade3D--------------------
template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > :: DeleteMe()
{
  if ( ptr_filter_kernel != NULL )
    delete ptr_filter_kernel;
  ptr_filter_kernel = NULL;
  
  if ( ptr_neigh_funct != NULL )
    delete ptr_neigh_funct;
  ptr_neigh_funct = NULL;
  
  if ( ptr_neigh_filter != NULL )
    delete ptr_neigh_filter;
  ptr_neigh_filter = NULL;
  
  if ( ptr_image_imp_dest != NULL )
    delete ptr_image_imp_dest;
  ptr_image_imp_dest = NULL;
  
  if ( ptr_strategy != NULL )
    delete ptr_strategy;
  ptr_strategy = NULL;
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > :: ResetPointers()
{
  ptr_filter_kernel = NULL;
  ptr_neigh_funct = NULL;
  ptr_neigh_filter = NULL;
  ptr_image_imp_dest = NULL;
  ptr_strategy = NULL;
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > :: BuildLinearFilter( unsigned frame_size )
{
  DeleteMe();

  PrecisionType def_pix;
  def_pix *= 0;

  ptr_filter_kernel = new bicotiFrame3DInternal< PrecisionType >( def_pix, frame_size );
  
  ptr_neigh_funct = new bicotiNeighbourFunctionLinear< PixelType, PrecisionType >( ptr_filter_kernel );
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();
  
  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > :: UseExtrapolationStrategyConstant( PixelType default_pixel )
{
  if ( ptr_neigh_filter == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade3D", "UseExtrapolationStrategyConstant()", 6 );
  
  if ( ptr_strategy != NULL )
    delete ptr_strategy;

  ptr_strategy = new bicotiImageExtrapolationStrategy3DConstant< PixelType >( default_pixel );
  
  ptr_neigh_filter->SetExtrapolationStrategy( ptr_strategy );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > :: UseExtrapolationStrategyEqual()
{
  if ( ptr_neigh_filter == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade3D", "UseExtrapolationStrategyEqual()", 6 );

  if ( ptr_strategy != NULL )
    delete ptr_strategy;

  ptr_strategy = new bicotiImageExtrapolationStrategy3DEqual< PixelType >;
  
  ptr_neigh_filter->SetExtrapolationStrategy( ptr_strategy );
};

  
template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > :: UseExtrapolationStrategySimetric()
{
  if ( ptr_neigh_filter == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade3D", "UseExtrapolationStrategySimetric()", 6 );

  if ( ptr_strategy != NULL )
    delete ptr_strategy;

  ptr_strategy = new bicotiImageExtrapolationStrategy3DSimetric< PixelType >;
  
  ptr_neigh_filter->SetExtrapolationStrategy( ptr_strategy );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > :: UseExtrapolationStrategyCiclic()
{
  if ( ptr_neigh_filter == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade3D", "UseExtrapolationStrategyCiclic()", 6 );

  if ( ptr_strategy != NULL )
    delete ptr_strategy;

  ptr_strategy = new bicotiImageExtrapolationStrategy3DCiclic< PixelType >;
  
  ptr_neigh_filter->SetExtrapolationStrategy( ptr_strategy );
};


template< class PixelType, class PrecisionType >
bicotiFrame3DInternal< PrecisionType > * bicotiNeighbourFilterFacade3D< PixelType, PrecisionType > :: GetKernel()
{
  if ( ptr_filter_kernel == NULL )
    throw bicotiException( "bicotiNeighbourFilterFacade3D", "Kernel()", 6 );
  
  return ( ptr_filter_kernel );
};



//-------------bicotiNeighbourFilterFacade2DSingle----------------
template< class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade2DSingle< PixelType, PrecisionType > :: bicotiNeighbourFilterFacade2DSingle( bicotiImageImplementation< PixelType > * ptr_ima_imp )
{
  ptr_image_imp_src = ptr_ima_imp;

  ResetPointers();
};


template< class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade2DSingle< PixelType, PrecisionType > :: ~bicotiNeighbourFilterFacade2DSingle()
{
  DeleteMe();
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2DSingle< PixelType, PrecisionType > :: BuildMedianFilter( unsigned frame_size )
{
  DeleteMe();
  
  ptr_neigh_funct = new bicotiNeighbourFunctionMedianSingle< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2DSingle< PixelType, PrecisionType > :: BuildMaximumFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMaximumSingle< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2DSingle< PixelType, PrecisionType > :: BuildMinimumFilter( unsigned local_frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMinimumSingle< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, local_frame_size, ptr_neigh_funct );
};



//------------bicotiNeighbourFilterFacade2DVectorial--------------
template< class ColorType, class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade2DVectorial< ColorType, PixelType, PrecisionType > :: bicotiNeighbourFilterFacade2DVectorial( bicotiImageImplementation< PixelType > * ptr_ima_imp )
{
  ptr_image_imp_src = ptr_ima_imp;

  ResetPointers();
};


template< class ColorType, class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade2DVectorial< ColorType, PixelType, PrecisionType > :: ~bicotiNeighbourFilterFacade2DVectorial()
{
  DeleteMe();
};


template< class ColorType, class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2DVectorial< ColorType, PixelType, PrecisionType > :: BuildMedianFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMedianVectorial< ColorType, PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class ColorType, class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2DVectorial< ColorType, PixelType, PrecisionType > :: BuildMaximumFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMaximumVectorial< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class ColorType, class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade2DVectorial< ColorType, PixelType, PrecisionType > :: BuildMinimumFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMinimumVectorial< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};



//-------------bicotiNeighbourFilterFacade3DSingle----------------
template< class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade3DSingle< PixelType, PrecisionType > :: bicotiNeighbourFilterFacade3DSingle( bicotiImageImplementation< PixelType > * ptr_ima_imp )
{
  ptr_image_imp_src = ptr_ima_imp;

  ResetPointers();
};


template< class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade3DSingle< PixelType, PrecisionType > :: ~bicotiNeighbourFilterFacade3DSingle()
{
  DeleteMe();
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3DSingle< PixelType, PrecisionType > :: BuildMedianFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMedianSingle< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3DSingle< PixelType, PrecisionType > :: BuildMaximumFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMaximumSingle< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3DSingle< PixelType, PrecisionType > :: BuildMinimumFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMinimumSingle< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};



//------------bicotiNeighbourFilterFacade3DVectorial--------------
template< class ColorType, class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade3DVectorial< ColorType, PixelType, PrecisionType > :: bicotiNeighbourFilterFacade3DVectorial( bicotiImageImplementation< PixelType > * ptr_ima_imp )
{
  ptr_image_imp_src = ptr_ima_imp;

  ResetPointers();
};


template< class ColorType, class PixelType, class PrecisionType >
bicotiNeighbourFilterFacade3DVectorial< ColorType, PixelType, PrecisionType > :: ~bicotiNeighbourFilterFacade3DVectorial()
{
  DeleteMe();
};


template< class ColorType, class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3DVectorial< ColorType, PixelType, PrecisionType > :: BuildMedianFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMedianVectorial< ColorType, PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class ColorType, class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3DVectorial< ColorType, PixelType, PrecisionType > :: BuildMaximumFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMaximumVectorial< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};


template< class ColorType, class PixelType, class PrecisionType >
void bicotiNeighbourFilterFacade3DVectorial< ColorType, PixelType, PrecisionType > :: BuildMinimumFilter( unsigned frame_size )
{
  DeleteMe();

  ptr_neigh_funct = new bicotiNeighbourFunctionMinimumVectorial< PixelType >;
  
  ptr_image_imp_dest = ptr_image_imp_src->CreateImageImplementation();

  ptr_neigh_filter = new bicotiNeighbourFilter< PixelType >( ptr_image_imp_dest, ptr_image_imp_src, frame_size, ptr_neigh_funct );
};



#endif
