//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################




#ifndef _neighbour_filter_basic
#define _neighbour_filter_basic



#include <source/neighbour_filter/neighbour_function.hpp>



//*******************DECLARATIONS*********************************
//----------------bicotiNeighbourFilter---------------------------
template< class PixelType >
class bicotiNeighbourFilter
{
protected:
  unsigned frame_size;
  bicotiImageImplementation< PixelType > * ptr_image_src;
  bicotiImageImplementation< PixelType > * ptr_image_dest;
  bicotiNeighbourFunction< PixelType > * ptr_function;
  bicotiImageExtrapolationStrategy< PixelType > * ptr_strategy;
    
public:
  bicotiNeighbourFilter();  
  bicotiNeighbourFilter( bicotiImageImplementation< PixelType > *, bicotiImageImplementation< PixelType > *, unsigned, bicotiNeighbourFunction< PixelType > * );  
  
  void SetImagesImplementations( bicotiImageImplementation< PixelType > *, bicotiImageImplementation< PixelType > * );
  void SetExtrapolationStrategy( bicotiImageExtrapolationStrategy< PixelType > * );
  void SetFunction( bicotiNeighbourFunction< PixelType > * , unsigned );
  
  void Apply();
}; 




//*********************IMPLEMENTATIONS****************************
//-------------------bicotiNeighbourFilter------------------------
template< class PixelType >
bicotiNeighbourFilter< PixelType > :: bicotiNeighbourFilter()
{
  ptr_image_src = NULL;
  ptr_image_dest = NULL;
  ptr_function = NULL;
  ptr_strategy = NULL;
  frame_size = 0;
}; 


template< class PixelType >
bicotiNeighbourFilter< PixelType > :: bicotiNeighbourFilter( bicotiImageImplementation< PixelType > * ptr_ima_dest, bicotiImageImplementation< PixelType >* ptr_ima_src, unsigned frm_size, bicotiNeighbourFunction< PixelType > * ptr_neigh_fun ) 
{  
  unsigned index;

  bicotiImageDescriptor * ptr_desc_src, * ptr_desc_dest;
  
  ptr_desc_src = ptr_ima_src->CreateImageDescriptor();
  ptr_desc_dest = ptr_ima_dest->CreateImageDescriptor();

  if ( ( ptr_desc_src->GetDimension() ) != ( ptr_desc_dest->GetDimension() ) )
    {  delete ptr_desc_src;
    delete ptr_desc_dest;
    throw bicotiException( "bicotiNeighbourFilter", "Constructor", 3 );}
  else
    {for ( index=0 ; index<ptr_desc_src->GetDimension() ; index++ )
      if ( ( ptr_desc_src->Size( index ) ) != ( ptr_desc_dest->Size( index ) ) )
	{ delete ptr_desc_src;
	delete ptr_desc_dest;
	throw bicotiException( "bicotiNeighbourFilter", "Constructor", 3 );}};
  
  for ( index=0 ; index<ptr_desc_src->GetDimension() ; index++ )
    if ( ( ptr_desc_src->Size( index ) ) < ( frm_size ) )
      { delete ptr_desc_src;
      delete ptr_desc_dest;
      throw bicotiException( "bicotiNeighbourFilter", "Constructor", 2 );};
  
  if ( ( frm_size % 2 ) == 0 )
    throw bicotiException( "bicotiNeighbourFilter", "Constructor", 2 );

  ptr_image_src = ptr_ima_src;
  ptr_image_dest = ptr_ima_dest;
  ptr_function = ptr_neigh_fun;
  frame_size = frm_size;
 
  delete ptr_desc_src;
  delete ptr_desc_dest;
};


template< class PixelType >
void bicotiNeighbourFilter< PixelType > ::SetImagesImplementations( bicotiImageImplementation< PixelType > * ptr_ima_dest, bicotiImageImplementation< PixelType > * ptr_ima_src )
{
  unsigned index;

  bicotiImageDescriptor * ptr_desc_src, * ptr_desc_dest;
  
  ptr_desc_src = ptr_ima_src->CreateImageDescriptor();
  ptr_desc_dest = ptr_ima_dest->CreateImageDescriptor();

  if ( ( ptr_desc_src->GetDimension() ) != ( ptr_desc_dest->GetDimension() ) )
    {  delete ptr_desc_src;
    delete ptr_desc_dest;
    throw bicotiException( "bicotiNeighbourFilter", "SetImagesImplementations", 3 );}
  else
    {for ( index = 0 ; index < ptr_desc_src->GetDimension() ; index++ )
      if ( ( ptr_desc_src->Size( index ) ) != ( ptr_desc_dest->Size( index ) ) )
	{  delete ptr_desc_src;
	delete ptr_desc_dest;
	throw bicotiException( "bicotiNeighbourFilter", "SetImagesImplementations", 3 );}};
  
  
  for ( index = 0 ; index < ptr_desc_src->GetDimension() ; index++ )
    if ( ( ptr_desc_src->Size( index ) ) < ( frame_size ) )
      {  delete ptr_desc_src;
      delete ptr_desc_dest;
      throw bicotiException( "bicotiNeighbourFilter", "SetImagesImplementations", 2 );};
  
  ptr_image_src = ptr_ima_src;
  ptr_image_dest = ptr_ima_dest;
  
  delete ptr_desc_src;
  delete ptr_desc_dest;
};


template< class PixelType >
void bicotiNeighbourFilter< PixelType > :: SetExtrapolationStrategy( bicotiImageExtrapolationStrategy< PixelType > * local_ptr_strategy )
{
  ptr_strategy = local_ptr_strategy;
};


template< class PixelType >
void bicotiNeighbourFilter< PixelType > :: SetFunction( bicotiNeighbourFunction< PixelType > * local_ptr_function, unsigned local_frame_size )
{
  ptr_function = local_ptr_function;

  frame_size = local_frame_size;
  
  if ( ( frame_size % 2 ) == 0 )
    throw bicotiException( "bicotiNeighbourFilter", "SetFunction()", 2 );
};


template< class PixelType >
void bicotiNeighbourFilter< PixelType > :: Apply()
{

  if ( ( ptr_image_src ==  NULL ) || ( ptr_image_dest ==  NULL ) )
    throw bicotiException( "bicotiNeighbourFilter", "Apply()", 6 );

  if ( ptr_function ==  NULL )
    throw bicotiException( "bicotiNeighbourFilter", "Apply()", 6 );

  if ( ptr_strategy ==  NULL )
    throw bicotiException( "bicotiNeighbourFilter", "Apply()", 6 );

  bicotiImageFrameIterator< PixelType >* ptr_frame_iter;
  bicotiImageIterator< PixelType >* ptr_iter;
  
  ptr_frame_iter = ptr_image_src->CreateFrameIterator( frame_size, ptr_strategy );
  ptr_iter = ptr_image_dest->CreateIterator();
  
  for( ptr_frame_iter->First(); !ptr_frame_iter->IsDone(); (*ptr_frame_iter)++ , (*ptr_iter)++ )
    {(**ptr_iter)=(*ptr_function)(ptr_frame_iter->GetCurrentFrame());};
};                     



#endif
