//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _pixel_complex
#define _pixel_complex

#include <source/bicoti_basic.hpp>



//*************************DECLARATIONS****************************
template< class PixelType, class PixelPrecisionType = PixelType >
class bicotiPixelComplex
{
private:
  PixelType real, imaginary;

public:
  bicotiPixelComplex( PixelType, PixelType );
  bicotiPixelComplex(){};
  bicotiPixelComplex( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );
  void operator=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );

  bool operator==( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );

  PixelType& Real();
  PixelType& Imaginary();

  void One();
  
  bicotiPixelComplex< PixelType, PixelPrecisionType > Conjugate();

  PixelType Module();
  PixelType Argument();
 
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator+( double );
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator-( double );
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator*( double );
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator/( double );

#if PixelType != double
  void operator+=( double );
  void operator-=( double );
  void operator*=( double );
  void operator/=( double );
#endif

  operator bicotiPixelComplex< PixelPrecisionType, PixelType >();

  
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator+( const PixelType& );
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator-( const PixelType& );
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator*( const PixelType& );
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator/( const PixelType& );

  void operator+=( const PixelType& );
  void operator-=( const PixelType& );
  void operator*=( const PixelType& );
  void operator/=( const PixelType& );

  bicotiPixelComplex< PixelType, PixelPrecisionType > operator+( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator-( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator*( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );
  bicotiPixelComplex< PixelType, PixelPrecisionType > operator/( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );

  void operator+=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );
  void operator-=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );
  void operator*=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );
  void operator/=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& );
};




//***********************IMPLEMENTATIONS***************************
template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > :: bicotiPixelComplex( PixelType rea , PixelType ima )
{
  real = rea;
  imaginary = ima;
};


template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > :: bicotiPixelComplex( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  real = compl_src.real;
  imaginary = compl_src.imaginary;
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  real = compl_src.real;
  imaginary = compl_src.imaginary;
};


template< class PixelType, class PixelPrecisionType >
bool bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator==( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  return( ( real == compl_src.real ) && ( imaginary == compl_src.imaginary ) );
};


template< class PixelType, class PixelPrecisionType >
PixelType& bicotiPixelComplex< PixelType, PixelPrecisionType > :: Real()
{
  return( real );
};


template< class PixelType, class PixelPrecisionType >
PixelType& bicotiPixelComplex< PixelType, PixelPrecisionType > :: Imaginary()
{
  return( imaginary );
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: One()
{
  real = real * 0 + 1;
  imaginary = imaginary * 0;
};


template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: Conjugate()
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( real, -imaginary ) );
};


template< class PixelType, class PixelPrecisionType >
PixelType bicotiPixelComplex< PixelType, PixelPrecisionType > :: Module()
{
  return( (PixelType)sqrt( real * real + imaginary * imaginary ) );
};


template< class PixelType, class PixelPrecisionType >
PixelType bicotiPixelComplex< PixelType, PixelPrecisionType > :: Argument()
{
  return( atan( imaginary / real ) + ( ( fabs( real ) - real ) * imaginary * pi ) / ( fabs( real ) * fabs( imaginary ) * 2 ) );
};

 
template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator+( const PixelType& scalar )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( real + scalar, imaginary ) );
};


template< class PixelType, class PixelPrecisionType>
bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator-( const PixelType& scalar )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( real - scalar, imaginary ) );
};


template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator*( const PixelType& scalar )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( real * scalar, imaginary * scalar ) );
};


template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator/( const PixelType& scalar )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( real / scalar, imaginary / scalar ) );
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator+=( const PixelType& scalar )
{
  real += scalar;
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator-=( const PixelType& scalar )
{
  real -= scalar;
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator*=( const PixelType& scalar )
{
  real *= scalar;
  imaginary *= scalar;
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator/=( const PixelType& scalar )
{
  real /= scalar;
  imaginary /= scalar;
};


template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator+( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( real + compl_src.real, imaginary + compl_src.imaginary ) );
};


template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator-( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( real - compl_src.real, imaginary - compl_src.imaginary ) );
};


template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator*( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  bicotiPixelComplex< PixelType, PixelPrecisionType > aux_compl;

  aux_compl.real = real * compl_src.real - imaginary * compl_src.imaginary;
  aux_compl.imaginary = real * compl_src.imaginary + imaginary * compl_src.real;

  return( aux_compl );
};


template< class PixelType, class PixelPrecisionType >
bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator/( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  bicotiPixelComplex< PixelType, PixelPrecisionType > aux_compl;
  PixelType aux_real = compl_src.real * compl_src.real + compl_src.imaginary * compl_src.imaginary;

  aux_compl.real = ( real * compl_src.real + imaginary * compl_src.imaginary ) / aux_real;
  aux_compl.imaginary = ( imaginary * compl_src.real - real * compl_src.imaginary ) / aux_real;

  return( aux_compl );
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator+=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  real += compl_src.real;
  imaginary += compl_src.imaginary;
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator-=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  real -= compl_src.real;
  imaginary -= compl_src.imaginary;
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator*=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  PixelType aux_real = real;

  real = real * compl_src.real - imaginary * compl_src.imaginary;
  imaginary = aux_real * compl_src.imaginary + imaginary * compl_src.real;
};


template< class PixelType, class PixelPrecisionType >
void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator/=( const bicotiPixelComplex< PixelType, PixelPrecisionType >& compl_src )
{
  PixelType mod_aux = compl_src.real * compl_src.real + compl_src.imaginary * compl_src.imaginary;
  PixelType aux_real = real;

  real = ( real * compl_src.real + imaginary * compl_src.imaginary ) / mod_aux;
  imaginary = ( imaginary * compl_src.real - aux_real * compl_src.imaginary ) / mod_aux;
};


template< class PixelType, class PixelPrecisionType >
inline bicotiPixelComplex< PixelType, PixelPrecisionType > fabs( bicotiPixelComplex< PixelType, PixelPrecisionType >& pixel )
{
  PixelType zero;
  zero = zero * 0;
  bicotiPixelComplex< PixelType, PixelPrecisionType > pixel_temp( pixel.Module(), zero );

  return( pixel_temp );
};


template< class PixelType, class PixelPrecisionType > 
inline bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator+( double scalar )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( (PixelType)(real + scalar), (PixelType)(imaginary + scalar) ) );
};


template< class PixelType, class PixelPrecisionType > 
inline bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator-( double scalar )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( (PixelType)(real - scalar), (PixelType)(imaginary - scalar) ) );
};


template< class PixelType, class PixelPrecisionType > 
inline bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator*( double scalar )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( (PixelType)(real * scalar), (PixelType)(imaginary * scalar) ) );
};


template< class PixelType, class PixelPrecisionType > 
inline bicotiPixelComplex< PixelType, PixelPrecisionType > bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator/( double scalar )
{
  return( bicotiPixelComplex< PixelType, PixelPrecisionType >( (PixelType)(real / scalar), (PixelType)(imaginary / scalar) ) );
};

#if PixelType != double
template< class PixelType, class PixelPrecisionType > 
inline void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator+=( double scalar )
{
  real = (PixelType)( real + scalar );
  imaginary = (PixelType)( imaginary + scalar );
};


template< class PixelType, class PixelPrecisionType > 
inline void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator-=( double scalar )
{
  real = (PixelType)( real - scalar );
  imaginary = (PixelType)( imaginary - scalar );
};


template< class PixelType, class PixelPrecisionType > 
inline void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator*=( double scalar )
{
  real = (PixelType)( real * scalar );
  imaginary = (PixelType)( imaginary * scalar );
};


template< class PixelType, class PixelPrecisionType > 
inline void bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator/=( double scalar )
{
  real = (PixelType)( real / scalar );
  imaginary = (PixelType)( imaginary / scalar );
};
#endif

template< class PixelType, class PixelPrecisionType > 
inline bicotiPixelComplex< PixelType, PixelPrecisionType > :: operator bicotiPixelComplex< PixelPrecisionType, PixelType >()
{
  return(  bicotiPixelComplex< PixelPrecisionType, PixelType >( (PixelPrecisionType)real, (PixelPrecisionType)imaginary ) );
};


template< class PixelType, class PrecisionType >
inline bicotiPixelComplex< PixelType, PrecisionType > sqrt( bicotiPixelComplex< PixelType, PrecisionType >& pixel )
{
  bicotiPixelComplex< PixelType, PrecisionType > pixel_temp;

  pixel_temp.Real() = (PixelType)sqrt( pixel.Module() )*sin( pixel.Argument()/2 );
  pixel_temp.Imaginary() = (PixelType)sqrt( pixel.Module() )*cos( pixel.Argument()/2 );

  return( pixel_temp );
};


template< class PixelType, class PrecisionType >
inline bicotiPixelComplex< PixelType, PrecisionType > log( bicotiPixelComplex< PixelType, PrecisionType >& pixel )
{
  bicotiPixelComplex< PixelType, PrecisionType > pixel_temp;

  pixel_temp.Real() = log( pixel.Module() );
  pixel_temp.Imaginary() = pixel.Argument();

  return( pixel_temp );
};            

#endif
