#include <source/pixel/pixel_converter_rgba.hpp>
#include <source/pixel/pixel_rgb_tools.hpp>
#include <source/bicoti_basic.hpp>

const bicotiPixelRGBA bicotiPixelConverterInt2RGBA :: Convert( const unsigned int pixel )
{
  return( bicotiRGBTools :: CreateRGBA( pixel, pixel, pixel ) );
}

const unsigned int bicotiPixelConverterInt2RGBA :: Unconvert( const bicotiPixelRGBA pixel )
{
  return( (unsigned int)(0.299 * (bicotiRGBTools :: GetRed( pixel )) + 0.587 * (bicotiRGBTools :: GetGreen( pixel )) + 0.114 * (bicotiRGBTools :: GetBlue( pixel ))) );
}


const bicotiPixelRGBA bicotiPixelConverterChar2RGBA :: Convert( const unsigned char pixel )
{
  return( bicotiRGBTools :: CreateRGBA( pixel, pixel, pixel ) );
}

const unsigned char bicotiPixelConverterChar2RGBA :: Unconvert( const bicotiPixelRGBA pixel )
{
  return( (unsigned char)(0.299 * (bicotiRGBTools :: GetRed( pixel )) + 0.587 * (bicotiRGBTools :: GetGreen( pixel )) + 0.114 * (bicotiRGBTools :: GetBlue( pixel ))) );
}


const bicotiPixelRGBA bicotiPixelConverterFloat2RGBA :: Convert( const float pixel )
{
  return( bicotiRGBTools :: CreateRGBA( pixel, pixel, pixel ) );
}

const float bicotiPixelConverterFloat2RGBA :: Unconvert( const bicotiPixelRGBA pixel )
{
  return( (float)(0.299 * (bicotiRGBTools :: GetRed( pixel )) + 0.587 * (bicotiRGBTools :: GetGreen( pixel )) + 0.114 * (bicotiRGBTools :: GetBlue( pixel ))) );
}



const bicotiPixelRGBA bicotiPixelConverterDouble2RGBA :: Convert( const double pixel )
{
  return( bicotiRGBTools :: CreateRGBA( pixel, pixel, pixel ) );
}

const double bicotiPixelConverterDouble2RGBA :: Unconvert( const bicotiPixelRGBA pixel )
{
  return( (double)(0.299 * (bicotiRGBTools :: GetRed( pixel )) + 0.587 * (bicotiRGBTools :: GetGreen( pixel )) + 0.114 * (bicotiRGBTools :: GetBlue( pixel ))) );
}



const bicotiPixelRGBA bicotiPixelConverterRGBChar2RGBA :: Convert( const bicotiRGB< unsigned char , REAL >  pixel )
{
  return( bicotiRGBTools :: CreateRGBA( pixel.Red() , pixel.Green() , pixel.Blue() ) );
}

const bicotiRGB< unsigned char, REAL > bicotiPixelConverterRGBChar2RGBA :: Unconvert( const bicotiPixelRGBA pixel )
{
  return( bicotiRGB< unsigned char, REAL >( (bicotiRGBTools :: GetRed( pixel )), (bicotiRGBTools :: GetGreen( pixel )), (bicotiRGBTools :: GetBlue( pixel ))) );
}



const bicotiPixelRGBA bicotiPixelConverterRGBFloat2RGBA :: Convert( const bicotiRGB< float , REAL >  pixel )
{
  return( bicotiRGBTools :: CreateRGBA( pixel.Red() , pixel.Green() , pixel.Blue() ) );
}

const bicotiRGB< float, REAL > bicotiPixelConverterRGBFloat2RGBA :: Unconvert( const bicotiPixelRGBA pixel )
{
  return( bicotiRGB< float, REAL >( (bicotiRGBTools :: GetRed( pixel )), (bicotiRGBTools :: GetGreen( pixel )), (bicotiRGBTools :: GetBlue( pixel ))) );
}



unsigned int bicotiPixelConverterHSVInt2RGBA :: Maximum( unsigned int red, unsigned int green, unsigned int blue )
{
  if ( red > green )
    {if ( red > blue ) 
      return( red );
    else
      return( blue );}
  else
    {if ( green > blue )
      return( green );
    else
      return( blue );}
}

const bicotiPixelRGBA bicotiPixelConverterHSVInt2RGBA :: Convert( const bicotiHSV< unsigned int , REAL >  pixel )
{
  unsigned int red, green, blue;

  if( pixel.Saturation() == 0 )
    {red = pixel.Value();
    green = pixel.Value();
    blue = pixel.Value();}
  else
    {REAL phi = (REAL)( pixel.Saturation() * cos( pi * pixel.Hue() / 180 ) );
    REAL psi = (REAL)( pixel.Saturation() * sin( pi * pixel.Hue() / 180 ) / sqrt(3) );
    
    if( ( 60 <= pixel.Hue() ) && ( pixel.Hue() <= 180 ) )
      {red = (unsigned int)(pixel.Value() + phi - psi );
      green = (unsigned int)pixel.Value();
      blue = (unsigned int)( pixel.Value() - 2 * psi );}
    else
      {if( ( 180 < pixel.Hue() ) && ( pixel.Hue() <= 300 ) )
	{red = (unsigned int)( pixel.Value() + phi + psi );
	green = (unsigned int)( pixel.Value() + 2 * psi );
	blue = (unsigned int)pixel.Value();}
      else
	{red = (unsigned int)pixel.Value();
	green = (unsigned int)( pixel.Value() + psi - phi );
	blue = (unsigned int)( pixel.Value() - phi - psi);}}}
  
  return( bicotiRGBTools :: CreateRGBA( red, green, blue ) );
}

const bicotiHSV< unsigned int, REAL > bicotiPixelConverterHSVInt2RGBA :: Unconvert( const bicotiPixelRGBA pixel )
{
  bicotiHSV< unsigned int, REAL > pixel_hsv;
  pixel_hsv.Value() = Maximum( bicotiRGBTools :: GetRed( pixel ), bicotiRGBTools :: GetGreen( pixel ), bicotiRGBTools :: GetBlue( pixel ) );
  
  REAL phi = (REAL)( (REAL)(bicotiRGBTools :: GetGreen( pixel )) - (REAL)(bicotiRGBTools :: GetBlue( pixel )) );
  REAL psi = (REAL)( bicotiRGBTools :: GetGreen( pixel ) + bicotiRGBTools :: GetBlue( pixel ) );
  REAL lambda = (REAL)( (REAL)(bicotiRGBTools :: GetRed( pixel )) - (REAL)(.5 * psi) );

  pixel_hsv.Saturation() = (unsigned int)sqrt( lambda * lambda + .75 * phi * phi );
  
  if ( lambda > 0 )
    {if ( phi >= 0 )
      pixel_hsv.Hue() = (unsigned int)( 180.0 * atan( sqrt(3.0) * phi / ( 2.0 * lambda ) ) / pi );
    else
      pixel_hsv.Hue() = (unsigned int)( 360.0 + 180.0 * atan( sqrt(3.0) * phi / ( 2.0 * lambda ) ) / pi );} //MAL
  else
    {if ( lambda < 0 )
      pixel_hsv.Hue() = 180 + (unsigned int)( 180.0 * atan( sqrt(3.0) * phi / ( 2.0 * lambda ) ) / pi ); // MAL
    else
      {if ( phi > 0 )
	pixel_hsv.Hue() = 90;
      else
	{if ( phi < 0 )
	  pixel_hsv.Hue() = 270;
	else
	  pixel_hsv.Undefined() = true;}}}
  
  return( pixel_hsv );
}
