//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _pixel_implementation_vectorial_dynamic
#define _pixel_implementation_vectorial_dynamic

#include <source/pixel/pixel_implementation_basic.hpp>



//**********************DECLRATIONS********************************
template< class ColorType, class PrecisionType = ColorType > 
class bicotiPixelVectorialDynamic
{
private:
  ColorType * ptr_colors;
  unsigned colors_number;

public:
  bicotiPixelVectorialDynamic( int = 0 );
  ~bicotiPixelVectorialDynamic();
  bicotiPixelVectorialDynamic( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );
  
  void operator=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );
  
  bool operator==( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );

  bicotiPixelVectorialDynamic< ColorType, PrecisionType > operator+( double );
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > operator-( double );
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > operator*( double );
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > operator/( double );

  void operator+=( double );
  void operator-=( double );
  void operator*=( double );
  void operator/=( double );

  bicotiPixelVectorialDynamic< ColorType, PrecisionType > operator+( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > operator-( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > operator*( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > operator/( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );

  void operator+=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );
  void operator-=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );
  void operator*=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );
  void operator/=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& );

  ColorType& operator() ( int );
  unsigned GetColorsNumber();

  operator bicotiPixelVectorialDynamic< PrecisionType, ColorType >();
};




//*********************IMPLEMENTATIONS*****************************
template< class ColorType, class PrecisionType > 
bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: bicotiPixelVectorialDynamic( int cols_num )
{
#ifdef TOTAL_CHECK
  if ( cols_num < 0 )
    throw bicotiException( "bicotiPixelVectorialDynamic", "Constructor", 6 );
#endif

  colors_number = ( unsigned ) cols_num;
  
  if ( colors_number > 0 ) 
    ptr_colors = new ColorType [ colors_number ];
};


template< class ColorType, class PrecisionType > 
bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: ~bicotiPixelVectorialDynamic()
{
  if ( colors_number > 0 ) 
    delete [] ptr_colors;
};


template< class ColorType, class PrecisionType > 
bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: bicotiPixelVectorialDynamic( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
  colors_number = pixel_src.colors_number;
  
  ptr_colors = new ColorType [ colors_number ];
  
  for( unsigned index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = pixel_src.ptr_colors[ index ];
};


template< class ColorType, class PrecisionType > 
void bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
  unsigned index;
  
  if ( colors_number = 0 ) 
    {colors_number = pixel_src.colors_number;
    ptr_colors = new ColorType [ colors_number ];}
  else
    {if( colors_number != pixel_src.colors_number ) 
      {delete [] ptr_colors;
      colors_number = pixel_src.colors_number;
      ptr_colors = new ColorType [ colors_number ];}}
  
  for( index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = pixel_src.ptr_colors[ index ];
};


template< class ColorType, class PrecisionType > 
bool bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator==( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
  bool is_equal;
  
  if ( colors_number != pixel_src.colors_number )
    is_equal = false;
  else
    {is_equal = true;
    for( unsigned index = 0; ( (index < colors_number ) && is_equal ); index++ )
      is_equal = ( ptr_colors[ index ] == pixel_src.ptr_colors[ index ] );}
  
  return is_equal;
};


template< class ColorType, class PrecisionType > 
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator+( double scalar )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp( colors_number );

  for ( unsigned index = 0; index < colors_number; index++ )
    pixel_temp( index ) = (ColorType) ( scalar + ptr_colors[ index ] );

  return ( pixel_temp );
};


template< class ColorType, class PrecisionType > 
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator-( double scalar )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp( colors_number );

  for ( unsigned index = 0; index < colors_number; index++ )
    pixel_temp( index ) = (ColorType) ( ptr_colors[ index ] - scalar );

  return ( pixel_temp );
};


template< class ColorType, class PrecisionType > 
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator*( double scalar )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp( colors_number );

  for ( unsigned index = 0; index < colors_number; index++ )
    pixel_temp( index ) = (ColorType) ( scalar * ptr_colors[ index ] );

  return ( pixel_temp );
};


template< class ColorType, class PrecisionType > 
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator/( double scalar )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp( colors_number );
  
  for ( unsigned index = 0; index < colors_number; index++ )
    pixel_temp( index ) = (ColorType) ( ptr_colors[ index ] / scalar );
  
  return ( pixel_temp );
};


template< class ColorType, class PrecisionType > 
inline void bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator+=( double scalar )
{
  for ( unsigned index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = (ColorType) ( ptr_colors[ index ] + scalar );
};


template< class ColorType, class PrecisionType > 
inline void bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator-=( double scalar )
{
  for ( unsigned index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = (ColorType) ( ptr_colors[ index ] - scalar );
};


template< class ColorType, class PrecisionType > 
inline void bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator*=( double scalar )
{
  for ( unsigned index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = (ColorType) ( ptr_colors[ index ] * scalar );
};


template< class ColorType, class PrecisionType > 
inline void bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator/=( double scalar )
{
  for ( unsigned index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = (ColorType) ( ptr_colors[ index ] / scalar );
};


template< class ColorType, class PrecisionType > 
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator+( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp( colors_number );
  
#ifdef TOTAL_CHECK
  if ( colors_number != pixel_src.colors_number )
    throw bicotiException( "bicotiPixelVectorialDynamic", "operator+", 2 );
#endif
  
  for( unsigned index = 0; index < colors_number; index++ )
    pixel_temp.ptr_colors[ index ] = ptr_colors[ index ] + pixel_src.ptr_colors[ index ];
  
  return( pixel_temp );
};


template< class ColorType, class PrecisionType > 
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator-( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp( colors_number );
  
#ifdef TOTAL_CHECK
  if ( colors_number != pixel_src.colors_number )
    throw bicotiException( "bicotiPixelVectorialDynamic", "operator-", 2 );
#endif
  
  for( unsigned index = 0; index < colors_number; index++ )
    pixel_temp.ptr_colors[ index ] = ptr_colors[ index ] - pixel_src.ptr_colors[ index ];
  
  return( pixel_temp );
};


template< class ColorType, class PrecisionType > 
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator*( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp( colors_number );
  
#ifdef TOTAL_CHECK
  if ( colors_number != pixel_src.colors_number )
    throw bicotiException( "bicotiPixelVectorialDynamic", "operator*", 2 );
#endif
  
  for( unsigned index = 0; index < colors_number; index++ )
    pixel_temp.ptr_colors[ index ] = ptr_colors[ index ] * pixel_src.ptr_colors[ index ];
  
  return( pixel_temp );
};


template< class ColorType, class PrecisionType > 
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator/( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp( colors_number );
  
#ifdef TOTAL_CHECK
  if ( colors_number != pixel_src.colors_number )
    throw bicotiException( "bicotiPixelVectorialDynamic", "operator/", 2 );
#endif
  
  for( unsigned index = 0; index < colors_number; index++ )
    pixel_temp.ptr_colors[ index ] = ptr_colors[ index ] / pixel_src.ptr_colors[ index ];
  
  return( pixel_temp );
};


template< class ColorType, class PrecisionType > 
inline void bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator+=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
#ifdef TOTAL_CHECK
  if ( colors_number != pixel_src.colors_number )
    throw bicotiException( "bicotiPixelVectorialDynamic", "operator+=", 2 );
#endif
  
  for( unsigned index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = ptr_colors[ index ] + pixel_src.ptr_colors[ index ];
};


template< class ColorType, class PrecisionType > 
inline void bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator-=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
#ifdef TOTAL_CHECK
  if ( colors_number != pixel_src.colors_number )
    throw bicotiException( "bicotiPixelVectorialDynamic", "operator-=", 2 );
#endif
  
  for( unsigned index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = ptr_colors[ index ] - pixel_src.ptr_colors[ index ];
};


template< class ColorType, class PrecisionType > 
inline void bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator*=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
#ifdef TOTAL_CHECK
  if ( colors_number != pixel_src.colors_number )
    throw bicotiException( "bicotiPixelVectorialDynamic", "operator*=", 2 );
#endif
  
  for( unsigned index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = ptr_colors[ index ] * pixel_src.ptr_colors[ index ];
};


template< class ColorType, class PrecisionType > 
inline void bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator/=( const bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel_src )
{
#ifdef TOTAL_CHECK
  if ( colors_number != pixel_src.colors_number )
    throw bicotiException( "bicotiPixelVectorialDynamic", "operator/=", 2 );
#endif

  for( unsigned index = 0; index < colors_number; index++ )
    ptr_colors[ index ] = ptr_colors[ index ] / pixel_src.ptr_colors[ index ];
};


template< class ColorType, class PrecisionType > 
inline ColorType& bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator() ( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( colors_number <= index) )
    throw bicotiException( "bicotiPixelVectorialDynamic", "operator()", 2 );
#endif
  
  return ( ptr_colors[ index ] );
};


template< class ColorType, class PrecisionType > 
inline unsigned bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: GetColorsNumber()
{
  return ( colors_number );
};


template< class ColorType, class PrecisionType >
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > fabs( bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp;
  
  for ( unsigned col_index = 0; col_index < dimension; col_index++ )
    pixel_temp( col_index ) = (ColorType) fabs( pixel( col_index ) );

  return( pixel_temp );
};


template< class ColorType, class PrecisionType >
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > sqrt( bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp;

  for ( unsigned col_index = 0; col_index < dimension; col_index++ )
    pixel_temp( col_index ) = (ColorType) sqrt( pixel( col_index ) );

  return( pixel_temp );
};


template< class ColorType, class PrecisionType >
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > atan( bicotiPixelVectorialDynamic< ColorType, PrecisionType >& pixel )
{
  bicotiPixelVectorialDynamic< ColorType, PrecisionType > pixel_temp;

  for ( unsigned col_index = 0; col_index < dimension; col_index++ )
    pixel_temp( col_index ) = (ColorType) atan( pixel( col_index ) );

  return( pixel_temp );
};


template< class ColorType, class PrecisionType > 
inline bicotiPixelVectorialDynamic< ColorType, PrecisionType > :: operator bicotiPixelVectorialDynamic< PrecisionType, ColorType >()
{
  bicotiPixelVectorialDynamic< PrecisionType, ColorType > aux_pix( colors_number );
  
  for( unsigned index = 0; index < colors_number; index++ )
    aux_pix( index ) = ptr_colors[ index ];

  return( aux_pix );
};

#endif
