//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################


#ifndef _pixel_implementation_vectorial_static
#define _pixel_implementation_vectorial_static

#include <source/pixel/pixel_implementation_basic.hpp>



//**********************DECLARATIONS*******************************
template< class ColorType, unsigned colors_number = 0, class PrecisionType = ColorType > 
class bicotiPixelVectorialStatic
{
private:
  ColorType colors[ colors_number ];
  
public:
  bicotiPixelVectorialStatic(){};
  virtual ~bicotiPixelVectorialStatic(){};
  bicotiPixelVectorialStatic( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );
  
  void operator=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );
  
  bool operator==( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );

  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > operator+( double );
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > operator-( double );
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > operator*( double );
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > operator/( double );

  void operator+=( double );
  void operator-=( double );
  void operator*=( double );
  void operator/=( double );

  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > operator+( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > operator-( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > operator*( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > operator/( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );

  void operator+=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );
  void operator-=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );
  void operator*=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );
  void operator/=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& );

  ColorType& operator() ( int );
  unsigned GetColorsNumber();

  operator bicotiPixelVectorialStatic< PrecisionType, colors_number, ColorType >();
};




//*********************IMPLEMENTATIONS*****************************
template< class ColorType, unsigned colors_number, class PrecisionType > 
bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: bicotiPixelVectorialStatic( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  unsigned index;

  for ( index = 0; index < colors_number; index++ )
    colors[ index ] = pixel_src.colors[ index ];
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
void bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  unsigned index;

  for( index = 0; index < colors_number; index++ )
    colors[ index ] = pixel_src.colors[ index ];
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
bool bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator==( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  bool is_equal = true;
  
  for( unsigned index = 0; ( (index < colors_number ) && is_equal ); index++ )
    is_equal = ( colors[ index ] == pixel_src.colors[ index ] );
  
  return is_equal;
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator+( double scalar )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_temp;

  for ( unsigned index = 0; index < colors_number; index++ )
    pixel_temp.colors[ index ] = (ColorType) ( scalar + colors[ index ] );

  return ( pixel_temp );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator-( double scalar )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_temp;

  for ( unsigned index = 0; index < colors_number; index++ )
    pixel_temp.colors[ index ] = (ColorType) ( colors[ index ] - scalar );

  return ( pixel_temp );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator*( double scalar )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_temp;

  for ( unsigned index = 0; index < colors_number; index++ )
    pixel_temp.colors[ index ] = (ColorType) ( scalar * colors[ index ] );

  return ( pixel_temp );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator/( double scalar )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_temp;

  for ( unsigned index = 0; index < colors_number; index++ )
    pixel_temp.colors[ index ] = (ColorType) ( colors[ index ] / scalar );

  return ( pixel_temp );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline void bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator+=( double scalar )
{
  for ( unsigned index = 0; index < colors_number; index++ )
    colors[ index ] = (ColorType) ( colors[ index ] + scalar );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline void bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator-=( double scalar )
{
  for ( unsigned index = 0; index < colors_number; index++ )
    colors[ index ] = (ColorType) ( colors[ index ] - scalar );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline void bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator*=( double scalar )
{
  for ( unsigned index = 0; index < colors_number; index++ )
    colors[ index ] = (ColorType) ( colors[ index ] * scalar );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline void bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator/=( double scalar )
{
  for ( unsigned index = 0; index < colors_number; index++ )
    colors[ index ] = (ColorType) ( colors[ index ] / scalar );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator+( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_res;

  for( unsigned index = 0; index < colors_number; index++ )
    pixel_res.colors[ index ] = colors[ index ] + pixel_src.colors[ index ];

  return( pixel_res );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator-( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_res;

  for( unsigned index = 0; index < colors_number; index++ )
    pixel_res.colors[ index ] = colors[ index ] - pixel_src.colors[ index ];

  return( pixel_res );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator*( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_res;

  for( unsigned index = 0; index < colors_number; index++ )
    pixel_res.colors[ index ] = colors[ index ] * pixel_src.colors[ index ];
  
  return( pixel_res );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator/( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_res;

  for( unsigned index = 0; index < colors_number; index++ )
    pixel_res.colors[ index ] = colors[ index ] / pixel_src.colors[ index ];

  return( pixel_res );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
void bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator+=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  for( unsigned index = 0; index < colors_number; index++ )
    colors[ index ] = colors[ index ] + pixel_src.colors[ index ];
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
void bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator-=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  for( unsigned index = 0; index < colors_number; index++ )
    colors[ index ] = colors[ index ] - pixel_src.colors[ index ];
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
void bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator*=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  for( unsigned index = 0; index < colors_number; index++ )
    colors[ index ] = colors[ index ] * pixel_src.colors[ index ];
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
void bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator/=( const bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel_src )
{
  for( unsigned index = 0; index < colors_number; index++ )
    colors[ index ] = colors[ index ] / pixel_src.colors[ index ];
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline ColorType& bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator() ( int index )
{
#ifdef TOTAL_CHECK
  if ( ( index < 0 ) || ( colors_number <= index) )
    throw bicotiException( "bicotiPixelVectorialStatic", "operator()", 2 );
#endif
  
  return ( colors[ index ] );
};


template< class ColorType, unsigned colors_number, class PrecisionType > 
inline unsigned bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: GetColorsNumber()
{
  return ( colors_number );
};


template< class ColorType, unsigned colors_number, class PrecisionType >
inline bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > fabs( bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_temp;
  
  for ( unsigned col_index = 0; col_index < colors_number; col_index++ )
    pixel_temp( col_index ) = (ColorType) fabs( pixel( col_index ) );

  return( pixel_temp );
};


template< class ColorType, unsigned colors_number, class PrecisionType >
inline bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > sqrt( bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_temp;
  
  for ( unsigned col_index = 0; col_index < colors_number; col_index++ )
    pixel_temp( col_index ) = (ColorType) sqrt( pixel( col_index ) );

  return( pixel_temp );
};


template< class ColorType, unsigned colors_number, class PrecisionType >
inline bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > atan( bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType >& pixel )
{
  bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > pixel_temp;
  
  for ( unsigned col_index = 0; col_index < colors_number; col_index++ )
    pixel_temp( col_index ) = (ColorType) atan( pixel( col_index ) );

  return( pixel_temp );
};


template< class ColorType, unsigned colors_number, class PrecisionType >
inline bicotiPixelVectorialStatic< ColorType, colors_number, PrecisionType > :: operator bicotiPixelVectorialStatic< PrecisionType, colors_number, ColorType >()
{
  bicotiPixelVectorialStatic< PrecisionType, colors_number, ColorType > aux_pix;
  
  for( unsigned index = 0; index < colors_number; index++ )
    aux_pix( index ) = colors[ index ];

  return( aux_pix );
};

#endif
