//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _property
#define _property


#include <source/property/histogram_equalizator.hpp>
#include <source/property/property_statistical.hpp>


//**********************DECLARATIONS***********************
//---------------------bicotiPropertyFacade----------------------
template<class CoordType, class ColorType, class PixelType = ColorType, class PrecisionType = PixelType >
class bicotiPropertyFacade
{
protected:
  bicotiImageImplementation< PixelType > * ptr_image_imp;
  
  virtual vector< bicotiPixelAttributeSelector< ColorType, PixelType > * > CreateAttributeSelectorList() = 0;

  virtual vector< bicotiPixelOperatorUnary< ColorType > * > CreateUniformOperatorsList() = 0;
  
public:
  bicotiPropertyHistogram< ColorType, PixelType > * ptr_histogram;
  bicotiPropertyStatistical< CoordType, ColorType, PixelType, PrecisionType > * ptr_statistical;

  virtual ~bicotiPropertyFacade(){};

  virtual void BuildStatistical()=0;
  void DestroyStatistical();

  bicotiPropertyStatistical< CoordType, ColorType, PixelType, PrecisionType > * GetStatistical();

  virtual void BuildHistogramUsual()=0;
  virtual void BuildHistogramExtended()=0;
  void DestroyHistogram();

  void EqualizateHistogramUniform();
  virtual void EqualizateHistogramSegmentation( PixelType, PixelType, PixelType )=0;

  bicotiPropertyHistogram< ColorType, PixelType > * GetHistogram();
};



//------------------bicotiPropertyFacadeSingle--------------------
template< class CoordType, class PixelType >
class bicotiPropertyFacadeSingle : public bicotiPropertyFacade< CoordType, PixelType, PixelType, REAL >
{
private:
  bicotiPixelAttributeSelector< PixelType, PixelType > * ptr_col_sel;
  
  vector< bicotiPixelAttributeSelector< PixelType, PixelType > * > CreateAttributeSelectorList();

  vector< bicotiPixelOperatorUnary< PixelType > * > CreateUniformOperatorsList();

public:
  bicotiPropertyFacadeSingle( bicotiImageImplementation< PixelType > * );
  ~bicotiPropertyFacadeSingle();
  
  void BuildStatistical();

  void BuildHistogramUsual();
  void BuildHistogramExtended();
  void EqualizateHistogramSegmentation( PixelType, PixelType, PixelType );
};



//-------------------bicotiPropertyFacadeVectorial---------------------
template<class CoordType, class ColorType, class PixelType, class PrecisionType = PixelType >
class bicotiPropertyFacadeVectorial : public bicotiPropertyFacade< CoordType, ColorType, PixelType, PrecisionType >
{
private:
  vector< bicotiPixelAttributeSelector< ColorType, PixelType > * > list_col_sel;

  vector< bicotiPixelAttributeSelector< ColorType, PixelType > * > CreateAttributeSelectorList();

  vector< bicotiPixelOperatorUnary< ColorType > * > CreateUniformOperatorsList();

public:
  bicotiPropertyFacadeVectorial( bicotiImageImplementation< PixelType > * );
  ~bicotiPropertyFacadeVectorial();
  
  void BuildStatistical();

  void BuildHistogramUsual();
  void BuildHistogramExtended();

  void EqualizateHistogramSegmentation( PixelType, PixelType, PixelType );
};




//********************IMPLEMENTATIONS**********************
//---------------------bicotiPropertyFacade----------------------
template< class CoordType, class ColorType, class PixelType, class PrecisionType >
bicotiPropertyHistogram< ColorType, PixelType > * bicotiPropertyFacade< CoordType, ColorType, PixelType, PrecisionType > :: GetHistogram()
{
  return ( ptr_histogram );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
bicotiPropertyStatistical< CoordType, ColorType, PixelType, PrecisionType > * bicotiPropertyFacade< CoordType, ColorType, PixelType, PrecisionType > :: GetStatistical()
{
  return ( ptr_statistical );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
void bicotiPropertyFacade< CoordType, ColorType, PixelType, PrecisionType > :: EqualizateHistogramUniform()
{
  vector< bicotiPixelOperatorUnary< ColorType > * > operators_list;
  
  operators_list = CreateUniformOperatorsList();
  
  bicotiHistogramEqualizator< CoordType, ColorType, PixelType >  equalizator( (bicotiPropertyHistogramExtended< CoordType, ColorType, PixelType >* )ptr_histogram , operators_list );
  
  equalizator();

  for( unsigned col_index = 0; col_index < operators_list.size(); col_index++ )
    delete operators_list[ col_index ];
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
void bicotiPropertyFacade< CoordType, ColorType, PixelType, PrecisionType > :: DestroyStatistical()
{
  if ( ptr_statistical != NULL )
    {delete ptr_statistical;
    ptr_statistical = NULL;}
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
void bicotiPropertyFacade< CoordType, ColorType, PixelType, PrecisionType > :: DestroyHistogram()
{
  if (ptr_histogram != NULL)
    {delete ptr_histogram;
    ptr_histogram = NULL;}
};



//------------------bicotiPropertyFacadeSingle-------------------
template< class CoordType, class PixelType >
bicotiPropertyFacadeSingle< CoordType, PixelType > :: bicotiPropertyFacadeSingle( bicotiImageImplementation< PixelType > * ptr_ima_imp )
{
  ptr_image_imp = ptr_ima_imp;

  ptr_histogram = NULL;
  ptr_statistical = NULL;

  ptr_col_sel = new bicotiPixelAttributeSelectorRound< PixelType >;
};


template< class CoordType, class PixelType >
bicotiPropertyFacadeSingle< CoordType, PixelType > ::~ bicotiPropertyFacadeSingle()
{
  if ( ptr_histogram != NULL )
    delete ptr_histogram;
  
  if ( ptr_statistical != NULL)
    delete ptr_statistical;

  if ( ptr_col_sel != NULL )
    delete ptr_col_sel;
};


template< class CoordType, class PixelType  >
vector< bicotiPixelAttributeSelector< PixelType, PixelType > * > bicotiPropertyFacadeSingle< CoordType, PixelType > :: CreateAttributeSelectorList()
{
  vector< bicotiPixelAttributeSelector< PixelType, PixelType > * > pix_att_sel_list;
  
  pix_att_sel_list.push_back( ptr_col_sel );  

  return( pix_att_sel_list );
};


template< class CoordType, class PixelType >
vector< bicotiPixelOperatorUnary< PixelType  > * > bicotiPropertyFacadeSingle< CoordType, PixelType > :: CreateUniformOperatorsList()
{
  vector< bicotiPixelOperatorUnary< PixelType > * > operators_list;
  
  bicotiPixelOperatorUnary< PixelType > * ptr_operator;
  ptr_operator = new bicotiPixelOperatorUnaryUniform< CoordType, PixelType, PixelType >( ( ( bicotiPropertyHistogramExtended< CoordType, PixelType, PixelType > * )ptr_histogram)->GetDexel()); 

  operators_list.push_back( ptr_operator );
  
  return( operators_list );
};


template< class CoordType, class PixelType >
void bicotiPropertyFacadeSingle< CoordType, PixelType > :: BuildStatistical()
{
  vector< bicotiPixelAttributeSelector< PixelType > * > pix_att_sel_list;
  pix_att_sel_list = CreateAttributeSelectorList();
  
  if ( ptr_statistical != NULL )
    delete ptr_statistical;
  
  ptr_statistical = new bicotiPropertyStatistical< CoordType, PixelType, PixelType, REAL >( ptr_image_imp, pix_att_sel_list );
};


template< class CoordType, class PixelType >
void bicotiPropertyFacadeSingle< CoordType, PixelType > :: BuildHistogramUsual()
{
  vector< bicotiPixelAttributeSelector< PixelType, PixelType > * > pix_att_sel_list;
  pix_att_sel_list = CreateAttributeSelectorList();

  if (ptr_histogram != NULL)
    delete ptr_histogram;
  
  ptr_histogram = new  bicotiPropertyHistogramUsual< PixelType, PixelType >( ptr_image_imp, pix_att_sel_list );
};


template< class CoordType, class PixelType >
void bicotiPropertyFacadeSingle< CoordType, PixelType > :: BuildHistogramExtended()
{
  vector< bicotiPixelAttributeSelector< PixelType, PixelType > * > pix_att_sel_list;
  pix_att_sel_list = CreateAttributeSelectorList();

  if (ptr_histogram != NULL)
    delete ptr_histogram;
  
  ptr_histogram = new  bicotiPropertyHistogramExtended< CoordType, PixelType, PixelType >( ptr_image_imp, pix_att_sel_list );
};


template< class CoordType, class PixelType >
void bicotiPropertyFacadeSingle< CoordType, PixelType > :: EqualizateHistogramSegmentation( PixelType umb_val, PixelType low_val, PixelType upp_val )
{
  vector< bicotiPixelOperatorUnary< PixelType > * > operators_list;
  
  bicotiPixelOperatorUnary< PixelType > * ptr_operator;
  ptr_operator = new bicotiPixelOperatorUnarySegmentation< PixelType >( umb_val, low_val, upp_val);

  operators_list.push_back( ptr_operator );
  
  bicotiHistogramEqualizator< CoordType, PixelType, PixelType >  equalizator( (bicotiPropertyHistogramExtended< CoordType, PixelType, PixelType >* )ptr_histogram , operators_list);
  
  equalizator();

  delete ptr_operator;
};




//-------------------bicotiPropertyFacadeVectorial---------------------
template< class CoordType, class ColorType, class PixelType, class PrecisionType >
bicotiPropertyFacadeVectorial< CoordType, ColorType, PixelType, PrecisionType > :: bicotiPropertyFacadeVectorial( bicotiImageImplementation< PixelType > * ptr_ima_imp )
{
  ptr_image_imp = ptr_ima_imp;
  
  ptr_histogram = NULL;
  ptr_statistical = NULL;
  
  for( unsigned col_index = 0; col_index < ptr_image_imp->GetDefaultPixel().GetColorsNumber(); col_index++ )
    list_col_sel.push_back( new bicotiPixelAttributeSelectorColorVectorial< ColorType, PixelType >( col_index ) );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
bicotiPropertyFacadeVectorial< CoordType, ColorType, PixelType, PrecisionType > ::~ bicotiPropertyFacadeVectorial()
{
  if ( ptr_histogram != NULL )
    delete ptr_histogram;

  if ( ptr_statistical != NULL )
    delete ptr_statistical;

  if ( list_col_sel[ 0 ] != NULL)
    {for( unsigned col_index = 0; col_index < list_col_sel.size(); col_index++ )
      delete list_col_sel[ col_index ];}
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
vector< bicotiPixelAttributeSelector< ColorType, PixelType > * > bicotiPropertyFacadeVectorial< CoordType, ColorType, PixelType, PrecisionType > :: CreateAttributeSelectorList()
{
  return( list_col_sel );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
vector< bicotiPixelOperatorUnary< ColorType > * > bicotiPropertyFacadeVectorial< CoordType, ColorType, PixelType, PrecisionType > :: CreateUniformOperatorsList()
{
  vector< bicotiPixelOperatorUnary< ColorType > * > operators_list;

  bicotiPixelOperatorUnary< ColorType >  * ptr_operator;
  
  for( unsigned col_index = 0; col_index < ptr_image_imp->GetDefaultPixel().GetColorsNumber(); col_index++ )
    {  ptr_operator = new bicotiPixelOperatorUnaryUniform< CoordType, ColorType, PixelType >( ( ( ( bicotiPropertyHistogramExtended< CoordType, ColorType, PixelType > * )ptr_histogram)->GetDexel( col_index ) ) ); 
    operators_list.push_back( ptr_operator );}
    
  return( operators_list );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
void bicotiPropertyFacadeVectorial< CoordType, ColorType, PixelType, PrecisionType > :: BuildStatistical()
{
  vector< bicotiPixelAttributeSelector< ColorType, PixelType > * > pix_att_sel_list;
  pix_att_sel_list = CreateAttributeSelectorList();
  
  if ( ptr_statistical != NULL )
    delete ptr_statistical;
  
  ptr_statistical = new bicotiPropertyStatistical< CoordType, ColorType, PixelType, PrecisionType >( ptr_image_imp, pix_att_sel_list );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
void bicotiPropertyFacadeVectorial< CoordType, ColorType, PixelType, PrecisionType > :: BuildHistogramUsual()
{
  vector< bicotiPixelAttributeSelector< ColorType, PixelType > * > pix_att_sel_list;
  pix_att_sel_list = CreateAttributeSelectorList();
  
  if (ptr_histogram != NULL)
    {delete ptr_histogram;
    ptr_histogram = NULL;}
  
  ptr_histogram = new  bicotiPropertyHistogramUsual< ColorType, PixelType >( ptr_image_imp, pix_att_sel_list );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
void bicotiPropertyFacadeVectorial< CoordType, ColorType, PixelType, PrecisionType > :: BuildHistogramExtended()
{
  vector< bicotiPixelAttributeSelector< ColorType, PixelType > * > pix_att_sel_list;
  pix_att_sel_list = CreateAttributeSelectorList();
  
  if (ptr_histogram != NULL)
    {delete ptr_histogram;
    ptr_histogram = NULL;}
  
  ptr_histogram = new  bicotiPropertyHistogramExtended< CoordType, ColorType, PixelType >( ptr_image_imp, pix_att_sel_list );
};


template< class CoordType, class ColorType, class PixelType, class PrecisionType >
void bicotiPropertyFacadeVectorial< CoordType, ColorType, PixelType, PrecisionType > :: EqualizateHistogramSegmentation( PixelType umb_val, PixelType low_val, PixelType upp_val )
{
  vector< bicotiPixelOperatorUnary< ColorType > * > operators_list;
  unsigned col_index;

  for( col_index = 0; col_index < ptr_image_imp->GetDefaultPixel().GetColorsNumber(); col_index++ )
    operators_list.push_back( new bicotiPixelOperatorUnarySegmentation< ColorType  >( umb_val( col_index ), low_val( col_index ), upp_val( col_index )));
  
  bicotiHistogramEqualizator< CoordType, ColorType, PixelType >  equalizator( (bicotiPropertyHistogramExtended< CoordType, ColorType, PixelType > * )ptr_histogram , operators_list);
  
  equalizator();

  for( unsigned col_index = 0; col_index < operators_list.size(); col_index++ )
    delete operators_list[ col_index ];
};



#endif
