//###########################################################
//#            Proyecto de fin de carrera BICOTI-I          #
//#   BIblioteca de COmponentes de Tratamiento de Imgenes  #
//#             Instituto de Ingeniera Elctrica           #
//#   Desarrollado por:                                     #
//#                     Claudio Risso                       #
//#                     Rodolfo Rodriguez                   #
//#                     Alvaro Valdes                       #
//#                                                         #
//#   Director:                                             #
//#                     Pablo Darscht                       #
//#                                                         #
//#   Asesoramiento en Tratamiento de Imgenes:             #
//#                     Alicia Fernandez                    #
//#                     Gregory Randall                     #
//#                                                         #
//#   Thanks To:                                            #
//#                     Hans Schmid                         #
//#                     Alberto Bartesaghi                  #
//#                     Facundo Memoli                      #
//#                     Alvaro Gomez                        #
//#                     Leonardo Steinfield                 #
//#                                                         #
//###########################################################



#ifndef _property_histogram_usual
#define _property_histogram_usual


#include <source/property/property_histogram_basic.hpp>


//---------------DECLARATIONS-----------------
//----------------HistoData-------------------
template< class ColorType >
class HistoData
{
public:
  ColorType pixel_color;
  unsigned pixel_number;
};


//------HistoDataSearchCriterioum------
template< class ColorType >
class HistoDataSearchCriterioum
{
public:
  bool operator() ( HistoData< ColorType >& );
  ColorType& ColorValue();
  
private:
  ColorType color_value;
};


//-----------Comparation Criterium------------
template< class PixelType >
class HistoDataCompare
{
public:
  bool operator() ( HistoData<PixelType>&, HistoData<PixelType>& );
};


//--------PropertyHistogramUsual-------------
template< class ColorType, class PixelType = ColorType >
class bicotiPropertyHistogramUsual : public bicotiPropertyHistogram< ColorType, PixelType >
{
private:
  unsigned lists_number;

  list< HistoData< ColorType > > * ptr_histo_data;
  list< HistoData< ColorType > > :: iterator * ptr_first;
  list< HistoData< ColorType > > :: iterator * ptr_last;

  void Append( unsigned, const ColorType& );

public:
  bicotiPropertyHistogramUsual( bicotiImageImplementation< PixelType > *, const vector< bicotiPixelAttributeSelector< ColorType, PixelType > * >& );
  ~bicotiPropertyHistogramUsual();
  bicotiPropertyHistogramUsual( const bicotiPropertyHistogramUsual< ColorType, PixelType >& );
  
  void operator=( const bicotiPropertyHistogramUsual< ColorType, PixelType >& );

  void UpdateImageImplementation();
  
  void First( unsigned = 0 );
  void Next( unsigned = 0 );
  ColorType GetCurrentColor( unsigned = 0 );
  unsigned GetCurrentPointsNumber( unsigned = 0 );
  bool IsDone( unsigned = 0 );
  
  unsigned GetHistogramsNumber();
};


//-------------IMPLEMENTATIONS----------------
//------HistoDataSearchCriterioum------
template< class ColorType > 
ColorType& HistoDataSearchCriterioum< ColorType > :: ColorValue()
{
  return ( color_value );
};


template< class ColorType > 
bool HistoDataSearchCriterioum< ColorType > :: operator() (HistoData< ColorType >& hist_val)
{
  return ( color_value == hist_val.pixel_color );
};



//-----------Comparation Criterium------------
template< class PixelType >
bool HistoDataCompare< PixelType > :: operator() ( HistoData< PixelType >& histo_data1, HistoData< PixelType >& histo_data2 )
{
  return ( histo_data1.pixel_color <= histo_data2.pixel_color );
};



//-------bicotiPropertyHistogramUsual------------
template< class ColorType, class PixelType >
bicotiPropertyHistogramUsual< ColorType, PixelType > :: bicotiPropertyHistogramUsual( bicotiImageImplementation< PixelType >* ptr_ima_imp, const vector< bicotiPixelAttributeSelector< ColorType, PixelType > * >& color_sel_list )
{
  ptr_image_imp = ptr_ima_imp;
  lists_number = color_sel_list.size();

  ptr_histo_data = new list< HistoData< ColorType > > [ lists_number ];
  ptr_first = new ( list< HistoData<ColorType> > :: iterator ) [ lists_number ];
  ptr_last = new ( list< HistoData<ColorType> > :: iterator ) [ lists_number ];

  unsigned index;

  bicotiImageIterator< PixelType > * ptr_iter;
  
  ptr_iter = ptr_image_imp->CreateIterator();
  for( ptr_iter->First(); !ptr_iter->IsDone(); (*ptr_iter)++ )
    {for( index = 0; index < lists_number; index++ )
      Append( index,  ( *( color_sel_list[index] ) )( **ptr_iter ) );}

  HistoDataCompare< ColorType > compar_criterium;
  for( index = 0; index < lists_number; index++ )
    {ptr_histo_data[ index ].sort( compar_criterium );
    ptr_first[ index ] = ptr_histo_data[ index ].begin();
    ptr_last[ index ] = ptr_histo_data[ index ].end();}
 
  delete ptr_iter;
};


template< class ColorType, class PixelType >
bicotiPropertyHistogramUsual< ColorType, PixelType > :: ~bicotiPropertyHistogramUsual()
{
  delete [] ptr_histo_data;
  delete [] ptr_first;
  delete [] ptr_last;
};


template< class ColorType, class PixelType >
bicotiPropertyHistogramUsual< ColorType, PixelType > :: bicotiPropertyHistogramUsual( const bicotiPropertyHistogramUsual< ColorType, PixelType >& proper_histo_usual_src )
{
  lists_number = proper_histo_usual_src.lists_number;
  ptr_image_imp = proper_histo_usual_src.ptr_image_imp;

  for( unsigned index=0; index < lists_number; index++ )
    {ptr_histo_data[ index ] = proper_histo_usual_src.ptr_histo_data[ index ];
    ptr_first[ index ] = ptr_histo_data[ index ].begin() + ( proper_histo_usual_src[ index ].first - proper_histo_usual_src[ index ].begin());
    ptr_last[ index ] = ptr_last[ index ].end() - (proper_histo_usual_src[ index ].end() - proper_histo_usual_src[ index ].last);}
};


template< class ColorType, class PixelType >
void bicotiPropertyHistogramUsual< ColorType, PixelType > :: Append( unsigned index, const ColorType& col_val )
{
  HistoDataSearchCriterioum< ColorType > criterioum;
  criterioum.ColorValue() = col_val;
  list< HistoData< ColorType > > :: iterator first = ptr_histo_data[ index ].begin();
  list< HistoData< ColorType > > :: iterator last = ptr_histo_data[ index ].end();
  
  first = find_if( first, last, criterioum );
  
  if ( first != last )
    ( *first ).pixel_number++;
  else
    {HistoData< ColorType > histo_data_aux;
    histo_data_aux.pixel_color = col_val;
    histo_data_aux.pixel_number = 1;
    ptr_histo_data[ index ].push_back( histo_data_aux );}
};


template< class ColorType, class PixelType >
void bicotiPropertyHistogramUsual< ColorType, PixelType > :: operator=( const bicotiPropertyHistogramUsual< ColorType, PixelType >& proper_histo_usual_src )
{
  lists_number = proper_histo_usual_src.lists_number;
  ptr_image_imp = proper_histo_usual_src.ptr_image_imp;

  for( unsigned index=0; index < lists_number; index++ )
    {ptr_histo_data[ index ] = proper_histo_usual_src.ptr_histo_data[ index ];
    ptr_first[ index ] = ptr_histo_data[ index ].begin() + ( proper_histo_usual_src[ index ].first - proper_histo_usual_src[ index ].begin() );
    ptr_last[ index ] = ptr_last[ index ].end() - ( proper_histo_usual_src[ index ].end() - proper_histo_usual_src[ index ].last );}
};


template< class ColorType, class PixelType >
void bicotiPropertyHistogramUsual< ColorType, PixelType > :: UpdateImageImplementation()
{
}; //Pra faer


template< class ColorType, class PixelType >
void bicotiPropertyHistogramUsual< ColorType, PixelType > :: First( unsigned col_index )
{
  if ( col_index >= lists_number )
    throw bicotiException( "bicotiPropertyHistogramUsual", "First()", 1 );

    ptr_first[ col_index ] = ptr_histo_data[ col_index ].begin();
    ptr_last[ col_index ] = ptr_histo_data[ col_index ].end();
};


template< class ColorType, class PixelType >
void bicotiPropertyHistogramUsual< ColorType, PixelType > :: Next( unsigned col_index )
{
  if ( col_index >= lists_number )
    throw bicotiException( "bicotiPropertyHistogramUsual", "Next()", 1 );

  ptr_first[ col_index ]++;
};


template< class ColorType, class PixelType >
ColorType bicotiPropertyHistogramUsual< ColorType, PixelType > :: GetCurrentColor( unsigned col_index )
{
  if ( col_index >= lists_number )
    throw bicotiException( "bicotiPropertyHistogramUsual", "GetCurrentColor()", 1 );

  return (  (*(ptr_first[col_index])).pixel_color );
};


template< class ColorType, class PixelType >
unsigned bicotiPropertyHistogramUsual< ColorType, PixelType > :: GetCurrentPointsNumber( unsigned col_index )
{
  if ( col_index >= lists_number )
    throw bicotiException( "bicotiPropertyHistogramUsual", "GetCurrentPointsNumber()", 1 );

  return ( (*( ptr_first[col_index] )).pixel_number );
};


template< class ColorType, class PixelType >
bool bicotiPropertyHistogramUsual< ColorType, PixelType > :: IsDone( unsigned col_index ) 
{
  if ( col_index >= lists_number )
    throw bicotiException( "bicotiPropertyHistogramUsual", "IsDone()", 1 );

  return ( ptr_first[col_index] == ptr_last[col_index] );
};


template< class ColorType, class PixelType >
unsigned  bicotiPropertyHistogramUsual< ColorType, PixelType > :: GetHistogramsNumber()
{
  return ( lists_number );
};      


#endif
